Package: vcd
Version: 1.4-13
Title: Visualizing Categorical Data
Authors@R: c(person(given = "David", family = "Meyer", role = c("aut", "cre"),
                    email = "David.Meyer@R-project.org",
		    comment = c(ORCID = "0000-0002-5196-3048")),	    
             person(given = "Achim", family = "Zeileis", role = "aut",
	            email = "Achim.Zeileis@R-project.org",
                    comment = c(ORCID = "0000-0003-0918-3766")),
             person(given = "Kurt", family = "Hornik", role = "aut",
	            email = "Kurt.Hornik@R-project.org",
                    comment = c(ORCID = "0000-0003-4198-9911")),
             person(given = "Florian", family = "Gerber", role = "ctb"),
             person(given = "Michael", family = "Friendly", role = "aut",
	            email = "friendly@yorku.ca",
		    comment = c(ORCID = "0000-0002-3237-0941")))
Description: Visualization techniques, data sets, summary and inference
        procedures aimed particularly at categorical data. Special
        emphasis is given to highly extensible grid graphics. The
        package was package was originally inspired by the book 
	"Visualizing Categorical Data" by Michael Friendly and is 
	now the main support package for a new book, 
	"Discrete Data Analysis with R" by Michael Friendly and 
	David Meyer (2015).
LazyLoad: yes
LazyData: yes
Depends: R (>= 2.4.0), grid
Suggests: KernSmooth, mvtnorm, kernlab, HSAUR3, coin
Imports: stats, utils, MASS, grDevices, colorspace, lmtest
License: GPL-2
NeedsCompilation: no
Packaged: 2024-09-16 09:01:34 UTC; meyer
Author: David Meyer [aut, cre] (<https://orcid.org/0000-0002-5196-3048>),
  Achim Zeileis [aut] (<https://orcid.org/0000-0003-0918-3766>),
  Kurt Hornik [aut] (<https://orcid.org/0000-0003-4198-9911>),
  Florian Gerber [ctb],
  Michael Friendly [aut] (<https://orcid.org/0000-0002-3237-0941>)
Maintainer: David Meyer <David.Meyer@R-project.org>
Repository: CRAN
Date/Publication: 2024-09-16 10:13:35 UTC
Built: R 4.6.0; ; 2026-04-25 23:39:27 UTC; unix

import(MASS)
import(grid)
import(stats)
import(grDevices)
import(colorspace)
importFrom("graphics", "pairs", "par")
importFrom("utils","head","str","tail")
importFrom("lmtest","coeftest","coeftest.default")

export(
## generic functions
       "agreementplot",
       "assoc",
       "cd_plot",
       "cotabplot",
       "distplot",
       "doubledecker",
       "fourfold",
       "goodfit",
       "mosaic",
       "oddsratio",
       "odds",
       "rootogram",
       "sieve",
       "spine",
       "tile",
       "structable",
       "loddsratio",
       "lodds",

## spacings
       "spacing_conditional",
       "spacing_equal",
       "spacing_dimequal",
       "spacing_increase",
       "spacing_highlighting",

## labelings
       "labeling_lboxed",
       "labeling_border",
       "labeling_cboxed",
       "labeling_left",
       "labeling_cells",
       "labeling_conditional",
       "labeling_list",
       "labeling_doubledecker",
       "labeling_values",
       "labeling_residuals",

## legends
       "legend_resbased",
       "legend_fixed",

# shadings
       "shading_binary",
       "shading_hcl",
       "shading_hsv",
       "shading_max",
       "shading_Friendly",
       "shading_Friendly2",
       "shading_diagonal",
       "shading_Marimekko",
       "shading_sieve",
       "hcl2hex",

# core functions
       "struc_mosaic",
       "struc_assoc",
       "struc_sieve",

## panel functions
       "pairs_barplot",
       "pairs_text",
       "pairs_diagonal_text",
       "pairs_diagonal_mosaic",
       "pairs_strucplot",
       "pairs_mosaic",
       "pairs_assoc",
       "pairs_sieve",
       "cotab_mosaic",
       "cotab_assoc",
       "cotab_sieve",
       "cotab_loddsratio",
       "cotab_agreementplot",
       "cotab_fourfold",
       "cotab_coindep",

## `normal' functions
       "Kappa",
       "assocstats",
       "table2d_summary",
       "co_table",
       "coindep_test",
       "grid_barplot",
       "hls",
       "is.structable",
       "independence_table",
       "mar_table",
       "Ord_estimate",
       "Ord_plot",
       "strucplot",
       "ternaryplot",
       "binreg_plot",
       "mplot",
       "grid_legend",
       "grid_abline",
       "woolf_test")

S3method("[", "structable")
S3method("[[", "structable")
S3method("[<-", "structable")
S3method("[[<-", "structable")
## We cannot do the following:
S3method("rbind", "structable")
S3method("cbind", "structable")
## Instead, we currently have to use:
##export("rbind.structable")
##export("cbind.structable")

S3method("str", "structable")
S3method("is.na", "structable")
S3method("length", "structable")
S3method("as.table", "structable")
S3method("as.matrix", "structable")
S3method("as.vector", "structable")
S3method("dim", "structable")
S3method("t", "structable")
S3method("dimnames", "structable")


S3method("agreementplot", "default")
S3method("agreementplot", "formula")

S3method("assoc", "default")
S3method("assoc", "formula")
S3method("assoc", "loglm")

S3method("cd_plot", "default")
S3method("cd_plot", "formula")

S3method("cotabplot", "default")
S3method("cotabplot", "formula")

S3method("doubledecker", "default")
S3method("doubledecker", "formula")

S3method("mosaic", "default")
S3method("mosaic", "formula")
S3method("mosaic", "loglm")

S3method("tile", "default")
S3method("tile", "formula")

S3method("rootogram", "default")
S3method("rootogram", "goodfit")

S3method("sieve", "default")
S3method("sieve", "formula")
S3method("sieve", "loglm")

S3method("structable", "default")
S3method("structable", "formula")

S3method("spine", "default")
S3method("spine", "formula")

S3method("pairs", "table")
S3method("pairs", "structable")

S3method("fitted", "goodfit")
S3method("fitted", "coindep_test")

S3method("residuals", "goodfit")

S3method("predict", "goodfit")

S3method("confint", "Kappa")

S3method("plot", "goodfit")
S3method("plot", "loglm")
S3method("plot", "structable")

S3method("print", "Kappa")
S3method("print", "summary.Kappa")
S3method("print", "goodfit")
S3method("print", "assocstats")
S3method("print", "summary.assocstats")
S3method("print", "table2d_summary")
S3method("print", "structable")

S3method("summary", "Kappa")
S3method("summary", "assocstats")
S3method("summary", "goodfit")

# loddsratio related methods
S3method("loddsratio", "default")
S3method("loddsratio", "formula")
S3method("coef", "loddsratio")
S3method("confint", "loddsratio")
S3method("dim", "loddsratio")
S3method("dimnames", "loddsratio")
S3method("print", "loddsratio")
S3method("plot", "loddsratio")
S3method("lines", "loddsratio")
S3method("summary", "loddsratio")
S3method("vcov", "loddsratio")
S3method("as.matrix", "loddsratio")
S3method("as.array", "loddsratio")
S3method("as.data.frame", "loddsratio")
S3method("aperm", "loddsratio")
S3method("t", "loddsratio")
S3method("image", "loddsratio")
S3method("tile", "loddsratio")

# loddsratio related methods
S3method("lodds", "default")
S3method("lodds", "formula")
S3method("coef", "lodds")
S3method("confint", "lodds")
S3method("dim", "lodds")
S3method("dimnames", "lodds")
S3method("print", "lodds")
#S3method("plot", "lodds")
#S3method("lines", "lodds")
S3method("summary", "lodds")
S3method("vcov", "lodds")
S3method("as.matrix", "lodds")
S3method("as.array", "lodds")
S3method("as.data.frame", "lodds")
S3method("aperm", "lodds")
S3method("t", "lodds")

[![PyPI - Python Version](https://img.shields.io/pypi/pyversions/vcd)](https://pypi.org/project/vcd/)
[![PyPI Release](https://img.shields.io/pypi/v/vcd)](https://pypi.org/project/vcd/)
[![PyPI - Downloads](https://img.shields.io/pypi/dm/vcd)](https://pypi.org/project/vcd/)
[![Pre-commit](https://img.shields.io/badge/pre--commit-enabled-brightgreen?logo=pre-commit)](https://github.com/pre-commit/pre-commit)
[![Code style: black](https://img.shields.io/badge/code_style-black-000000.svg)](https://github.com/psf/black)
[![Code style: flake8](https://img.shields.io/badge/code_style-flake8-orange)](https://github.com/PyCQA/flake8)
[![Linting: pylint](https://img.shields.io/badge/linting-pylint-yellowgreen)](https://github.com/PyCQA/pylint)

![VCD Video Content Description](https://vicomtech.gitlab.io/v4/libraries/vcd/vcd-python/resources/VCD_logo_2020.png)
# Video Content Description (VCD)

VCD is a metadata format designed to enable the description of scene information, particularly efficient for discrete data series, such as image or point-cloud sequences from sensor data.
Originally, VCD focused on video content data, but has been extended to provide structures to describe, potentially, any type of information of a scene.

VCD is defined as a structure of data, and as such, can be represented as a JSON Schema.

The syntax (see [openlabel_json_schema-v1.0.0.json](https://vcd.vicomtech.org/documentation/documentation-vcd#shema)), as a JSON Schema file, contains the full description of the VCD structure. This schema follows the ASAM OpenLABEL standard ([OpenLABEL Concept Paper](https://www.asam.net/index.php?eID=dumpFile&t=f&f=3876&token=413e8c85031ae64cc35cf42d0768627514868b2f)).

![VCD](https://vicomtech.gitlab.io/v4/libraries/vcd/vcd-python/resources/image.svg)

## Details

More details can be found at the project's website: https://vcd.vicomtech.org

## Install

### Python

Using pip (Python >3.7)):

```
pip install vcd
```
## Usage

### Python

VCD Python API exposes functions to load, create, manipulate and serialize VCD content.

The recommended way to learn VCD is throuhg the samples at the [test folder](https://github.com/Vicomtech/video-content-description-VCD/blob/master/tests).

As a basic example, VCD can be used in a Python script as follows:

```python
import vcd.core as core
import vcd.types as types

# Create a VCD instance
myVCD = core.VCD()

# Add Objects, Actions, etc.
uid1 = myVCD.add_object(name='ped1', semantic_type='#Pedestrian')
myVCD.add_object_data(uid=uid1, object_data=types.bbox(name="head", val=[0, 0, 100, 200]))

...

# Serialize
myVCD.stringify(pretty=False, validation=True)
```

The API contains useful functions that ensures the produced content is compliant with the syntax. Nevertheless, the VCD class allows the user to access directly the content, in the form of a Python dictionary.

```python
import vcd.core as core

# Create empty vcd object
myVCD = core.vcd()

# Load a VCD file
myVCD.load_from_file('./tests/etc/openlabel100_test_scene_KITTI_Tracking_3.json')

# Access data directly
metadata = myVCD.data['openlabel']['metadata']

# Modify data directly
myVCD['openlabel']['objects'][3]['type'] = "#Car"
...

# Serialize
stringified_vcd = myVCD.stringify(pretty=False, validation=True)
```

This can be useful some times, but it is not recommended, as the dictionary may deviate from a valid VCD syntax. To check if a content is valid, the API exposes a validation function:

```python
# Validate
myVCD.validate(stringified_vcd)
```

This validation function is optionally called when saving to JSON files.

## Versions

VCD is a toolkit with APIs in various programming languages (Python, Typescript, C++) which allows anyone to create, read, update and delete labels that follow the ASAM OpenLABEL standard v1.0.0.

Last version is VCD 6.0.0 compliant with OpenLABEL 1.0.0.

VCD has evolved as follows:

* VCD 1.0 (2013)
* VCD 2.0 (2014)
    * Integrated into Viulib library (module viulib_evaluation)
    * Element-wise and Frame-wise modes
    * XML and JSON serialization via ASL library
* VCD 3.0 (2018)
    * Independent C++ library
    * Element-wise and Frame-wise modes
    * Multi-sensor support
    * JSON serialization via ASL library
    * Pixel-wise loss-less compression modes
    * Comparison routines
* VCD 4.0 (2019)
    * Python API
    * Element and Frame-wise mode simultaneously
    * Multi-sensor and multi-interval
    * Native Python JSON serialization
    * Google's Protocol Buffer serialization
    * Object data 'num' for single numbers, 'vec' for arrays of numbers
* VCD 4.1.3 (2020-2021)
    * Explicit definition of intrinsics, extrinsics and odometry
    * Enhanced timestamping and sync information
    * Enhanced semantics management (RDF triplets)
    * Integrated SCL and complex calibration set-ups
    * Drawing functions
    * Preliminar work on Ontology and Neo4j connection
    * Multi-value attributes ('vec' of strings)
    * Typescript API
    * NPM and Pypi packages
* VCD 5.0.0 (2021)
    * VCD as toolkit to produce OpenLABEL compliant labels
    * Addition of C++ lite version
    * General improvements and consistency (Python, Typescript)
    * Removed support for protobuf serialization
* VCD 6.0.0 (2023)
    * VCD Library for Python separated from other language developments
    * Major Python API update

## Related projects

VCD has been used in the following projects: Cloud-LSVA, VI-DAS, inLane, P-REACT, EWISA, Viulib, begirale, SmaCS, HEADSTART, ACCURATE.

The [DMD](https://dmd.vicomtech.org/) (Driver Monitoring Dataset) project also uses VCD!

If your project also uses VCD, let us know!

## OpenLABEL

Along with the development of VCD, we are participating in the definition of the incoming labeling standard for the automotive sector: ASAM OpenLABEL.

https://www.asam.net/project-detail/asam-openlabel-v100

Since v5.0.0 VCD is shaped to be compliant with the format defined in OpenLABEL v1.0.0.
VCD is the first labeling toolset compliant with the standard and used during the elaboration of the standard to produce samples and create the JSON schema.

## Credits

Vicomtech created VCD in 2013, and since, has maintained VCD syntax and libraries. Developments of VCD were supported and funded by the European Commission (EC) Horizon 2020 programme (project [Cloud-LSVA] (http://cloud-lsva.eu), grant agreement 688099).

VCD was registered at the "Registro territorial de la propiedad intelectual de la comunidad autonoma del Pais Vasco", under number 55-354-17, by the Basque Governement, at 2017/07/07.

Main developers:
* Marcos Nieto, Orti Senderos, Jon Goenetxea

Contributors:
Thanks to Andoni Mujika, Paola Cañas, Eider Irigoyen, Juan Diego Ortega, Peter Leskovsky, Mikel Garcia, Gonzalo Pierola, Stefano Masneri, Lorena Garcia, Itziar Urbieta and many others in Vicomtech.

Also thanks to Nicola Croce (Deepen.ai), Jason Zhang (Warwick University), Tim Raedsch (Understand.ai) and other colleagues in ASAM for their ideas and comments during the ellaboration of the OpenLABEL v1.0.0 standard.

Finally, special thanks to Oihana Otaegui, as head of the ITS & Engineering department in Vicomtech. Without her lead this project would have never been possible. She believed in the VCD idea and supported me to carry on. Thanks Oihana! ; )


## License

Copyright (c) 2023 Vicomtech

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.

## Citation

Part of the work carried out to make VCD a reality has been published in Elsevier SoftwareX journal. If you find VCD useful and want to cite it in your publications, please use the following citation (the paper pdf can be accessed [here](https://www.sciencedirect.com/science/article/pii/S2352711020303666)).

M. Nieto, O. Senderos, and O. Otaegui, "Boosting AI applications: Labeling format for complex datasets," SoftwareX, 2021, p. 100653, vol. 13 (https://doi.org/10.1016/j.softx.2020.100653).

# How to generate the VCD Changelog

A "Changelog" is a record of all notable changes made to a project. Such a changelog, in our case the `CHANGELOG.md`, is read by the users. Therefore, any description should be aimed to users instead of describing internal changes which are only relevant to developers.

To avoid merge conflicts, we use the [Towncrier](https://github.com/twisted/towncrier) package to manage our changelog.

The directory [.changelog.d](../.changelog.d) contains "newsfragments" which are short Markdown-formatted files. On release, those news fragments are compiled into the [CHANGELOG.md](../CHANGELOG.md) file.

You don't need to install towncrier yourself, use the `tox` command to call the tool.

We recommend to follow the steps to make a smooth integration of your changes:

1. After you have created a GitLab Issue, add a new file into the directory [.changelog.d](../.changelog.d). Each filename follows the syntax:

    ```
    <ISSUE>.<TYPE>.md
    ```

    where `<ISSUE>` is the GitLab issue number. `<TYPE>` is one of:

    - `added`: for new features.
    - `changed`: for changes in existing functionality.
    - `deprecated`: for soon-to-be removed features.
    - `removed`: for now removed features.
    - `fixed`: for any bug fixes.
    - `security`: in case of vulnerabilities.
    - `other`: for other changes.

    For example: `123.added.md`, `233.removed.md`, `456.fixed.md` etc.

2. Create the new file with the command:

    ```bash
    tox -e changelog -- create 123.added.md
    ```

    The file is created int the [.changelog.d](../.changelog.d) directory.

3. Open the file and describe your changes in Markdown format.

    - Wrap symbols like modules, functions, or classes into double backticks so they are rendered in a monospace font.
    - Prefer simple past tense or constructions with "now".

4. Check your changes with:

    ```bash
    tox -e changelog -- check
    ```

5. Optionally, build a draft version of the changelog file with the command:

    ```bash
    tox -e changelog
    ```
6. Commit all your changes and push it.

## Release Changelog

On release, the maintainer compiles a new `CHANGELOG.md` file by running:

```bash
tox -e changelog -- build  --version <new_version>
```

This will remove all newsfragments inside the [.changelog.d](../.changelog.d) directory, making it ready for the next release.

## Modifications:
## -- return a dnames component, containing dimnames for the array version of coef
## -- added dim methods: dim.loddsratio, dimnames.loddsratio
## -- added print.loddsratio
## -- handle strata:  result computed correctly, but structure of coef() loses names
##    and confint doesn't work in the 2x2xk or RxCxk case

## -- Fixed problem with strata by setting rownames and colnames for contrast matrix
## DONE: handle multiple strata (|foo:bar)
## -- print.loddsratio now uses drop() for dimensions of length 1
## -- made generic, anticipating a formula method, maybe structable or ftable methods
## DONE: decide which methods should allow a log=FALSE argument to provide exp(lor)

## -- Now handle any number of strata
## -- Added log= argument to print, coef methods, and added confint.loddsratio method,
##    allowing log=FALSE

## -- Incorporated Z code additions, fixing some <FIXME>s
## -- Added as.matrix and as.array methods; had to make as.array S3 generic
## -- Added header to print method
## -- Added as.data.frame method (for use in plots)
## --   "LOR" is renamed "OR" if log=FALSE
## -- Revised as.matrix to drop leading 1:2 dimensions of length 1
## -- Removed as.array generic, now in base

## -- DM: added plot.oddsratio method
## -- DM: added formula interface
## -- DM: add t() and aperm() methdos

loddsratio <- function(x, ...)
    UseMethod("loddsratio")

loddsratio.formula <-
    function(formula, data = NULL, ..., subset = NULL, na.action = NULL)
{
    m <- match.call(expand.dots = FALSE)
    edata <- eval(m$data, parent.frame())

    fstr <- strsplit(paste(deparse(formula), collapse = ""), "~")
    vars <- strsplit(strsplit(gsub(" ", "", fstr[[1]][2]), "\\|")[[1]], "\\+")
    varnames <- vars[[1]]

    condnames <- if (length(vars) > 1) vars[[2]] else NULL

    dep <- gsub(" ", "", fstr[[1]][1])
    if (!dep %in% c("","Freq")) {
        if (all(varnames == ".")) {
            varnames <- if (is.data.frame(data))
                            colnames(data)
                        else
                            names(dimnames(as.table(data)))
            varnames <- varnames[-which(varnames %in% dep)]
        }

        varnames <- c(dep, varnames)
    }


    if (inherits(edata, "ftable") || inherits(edata, "table") || length(dim(edata)) > 2) {
        condind <- NULL
        dat <- as.table(data)
        if(all(varnames != ".")) {
            ind <- match(varnames, names(dimnames(dat)))
            if (any(is.na(ind)))
                stop(paste("Can't find", paste(varnames[is.na(ind)], collapse=" / "), "in", deparse(substitute(data))))

            if (!is.null(condnames)) {
                condind <- match(condnames, names(dimnames(dat)))
                if (any(is.na(condind)))
                    stop(paste("Can't find", paste(condnames[is.na(condind)], collapse=" / "), "in", deparse(substitute(data))))
                ind <- c(ind, condind)
            }
            dat <- margin.table(dat, ind)
        }
        loddsratio.default(dat, strata = if (is.null(condind)) NULL else match(condnames, names(dimnames(dat))), ...)
    } else {
        m <- m[c(1, match(c("formula", "data", "subset", "na.action"), names(m), 0))]
        m[[1]] <- as.name("xtabs")
        m$formula <-
            formula(paste(if("Freq" %in% colnames(data)) "Freq",
                          "~",
                          paste(c(varnames, condnames), collapse = "+")))
        tab <- eval(m, parent.frame())
        loddsratio.default(tab, ...)
    }
}

loddsratio.default <- function(x, strata = NULL, log = TRUE,
                               ref = NULL, correct = any(x == 0), ...)
{
    ## check dimensions
    L <- length(d <- dim(x))
    if(any(d < 2L)) stop("All table dimensions must be 2 or greater")
    if(L > 2L && is.null(strata)) strata <- 3L:L
    if(is.character(strata)) strata <- which(names(dimnames(x)) == strata)
    if(L - length(strata) != 2L) stop("All but 2 dimensions must be specified as strata.")

    ## dimensions of primary R x C table
    dp <- if (length(strata)) d[-strata] else d
    dn <- if (length(strata)) dimnames(x)[-strata] else dimnames(x)
    R <- dp[1]
    C <- dp[2]
                                        # shadow matrix with proper dimnames
    X <- matrix(0, R, C, dimnames=dn)

    ## process reference categories (always return list of length
    ## two with reference for rows/cols, respectively)
    if(is.null(ref)) {
        ref <- list(NULL, NULL)
    } else if(is.character(ref)) {
        if(length(ref) != 2L) stop("'ref' must specify both reference categories")
        ref <- list(match(ref[1L], rownames(x)), match(ref[2L], colnames(x)))
    } else if(is.numeric(ref)) {
        ref <- as.integer(rep(ref, length.out = 2L))
        ref <- list(ref[1L], ref[2L])
    }

    ## compute corresponding indices
    compute_index <- function(n, ref) {
        if(is.null(ref)) return(cbind(1:(n-1), 2:n))
        rval <- cbind(ref, 1:n)
        d <- rval[,2L] - rval[,1L]
        rval <- rbind(
            rval[d > 0, 1:2],
            rval[d < 0, 2:1]
        )
        return(rval[order(rval[,1L]),,drop = FALSE])
    }
    Rix <- compute_index(R, ref[[1L]])
    Cix <- compute_index(C, ref[[2L]])

    ## set up contrast matrix for the primary R x C table
    contr <- matrix(0L, nrow = (R-1) * (C-1), ncol = R * C)
    colnames(contr) <- paste(rownames(X)[as.vector(row(X))], colnames(X)[as.vector(col(X))], sep = ":")
    rownames(contr) <- rep("", (R-1) * (C-1))
    for(i in 1:(R-1)) for(j in 1:(C-1)) {
        rix <- (j-1) * (R-1) + i
        cix <- rep(Rix[i,], 2L) + R * (rep(Cix[j,], each = 2L) - 1L)
        contr[rix, cix] <- c(1L, -1L, -1L, 1L)
        if (R > 2 || C > 2 || is.null(strata))
            rownames(contr)[rix] <- sprintf("%s/%s",
                                            paste(rownames(X)[Rix[i,]], collapse = ":"),
                                            paste(colnames(X)[Cix[j,]], collapse = ":"))
    }

                                        # handle strata
    if (!is.null(strata)) {
  	if (length(strata)==1) {
            sn <- dimnames(x)[[strata]]
        }
        else {
            sn <- apply(expand.grid(dimnames(x)[strata]), 1, paste, collapse = ":")
        }
        SEP <- if (R > 2 || C > 2 || is.null(strata)) "|" else ""
        rn <- as.vector(outer( dimnames(contr)[[1]], sn, paste, sep=SEP))
        cn <- as.vector(outer( dimnames(contr)[[2]], sn, paste, sep=SEP))
        contr <- kronecker(diag(prod(dim(x)[strata])), contr)
        rownames(contr) <- rn
        colnames(contr) <- cn
    }

    ## dimnames for array version
    dn <- list(rep("", R-1), rep("", C-1))
    for(i in 1:(R-1)) dn[[1]][i] <- paste(rownames(x)[Rix[i,]], collapse = ":")
    for(j in 1:(C-1)) dn[[2]][j] <- paste(colnames(x)[Cix[j,]], collapse = ":")
    if (!is.null(strata)) dn <- c(dn, dimnames(x)[strata])
    if (!is.null(names(dimnames(x)))) names(dn) <- names(dimnames(x))

    ## point estimates

    if (is.logical(correct)) {
        add <- if(correct) 0.5 else 0
    }
    else if(is.numeric(correct)) {
        add <- as.vector(correct)
        if (length(add) != length(x))
            stop("array size of 'correct' does not conform to the data")
    }
    else stop("correct is not valid")

    ##coef <- drop(contr %*% log(as.vector(x) + add))
    ##FIXME: 0 cells mess up the matrix product, try workaround:
    mat <- log(as.vector(x) + add) * t(contr)
    nas <- apply(contr != 0 & is.na(t(mat)), 1, any)
    coef <- apply(mat, 2, sum, na.rm = TRUE)
    coef[nas] <- NA
    ## covariances
    ##vcov <- crossprod(diag(sqrt(1/(as.vector(x) + add))) %*% t(contr))
    tmp <- sqrt(1/(as.vector(x) + add)) * t(contr)
    tmp[is.na(tmp)] <- 0
    vcov <- crossprod(tmp)
    vcov[nas,] <- NA
    vcov[,nas] <- NA

    rval <- structure(list(
        coefficients = coef,
        dimnames = dn,
        dim = as.integer(sapply(dn, length)),
        vcov = vcov,
        contrasts = contr,
        log = log
    ), class = "loddsratio")
    rval
}

## dim methods
dimnames.loddsratio <- function(x, ...) x$dimnames
dim.loddsratio <- function(x, ...) x$dim

## t/aperm-methods
t.loddsratio <- function(x)
    aperm(x)

aperm.loddsratio <- function(a, perm = NULL, ...)
{
    d <- length(a$dim)
    if(is.null(perm)) {
        perm <- if (d < 3) 2L : 1L else c(2L : 1L, d : 3L)
    } else {
        if (any(perm[1:2] > 2L) || (d > 2L) && any(perm[-c(1:2)] < 2L))
            stop("Mixing of strata and non-strata variables not allowed!")
    }
    nams <- names(a$coefficients)
    a$coefficients <- as.vector(aperm(array(a$coef, dim = a$dim),
                                      perm, ...))
    nams <- as.vector(aperm(array(nams, dim = a$dim), perm, ...))
    names(a$coefficients) <- nams
    a$dimnames <- a$dimnames[perm]
    a$dim <- a$dim[perm]
    a$vcov <- a$vcov[nams, nams]
    a$contrasts <- a$contrasts[nams,]
    a
}

## straightforward methods
coef.loddsratio <- function(object, log = object$log, ...)
    if(log) object$coefficients else exp(object$coefficients)

vcov.loddsratio <- function(object, log = object$log, ...)
    if(log) object$vcov else `diag<-`(object$vcov, diag(object$vcov) * exp(object$coefficients)^2)

confint.loddsratio <-
    function(object, parm, level = 0.95, log = object$log, ...) {
        if (log) confint.default(object, parm = parm, level = level, ... )
        else {
            object$log = TRUE
            exp(confint.default(object, parm = parm, level = level, ... ))
        }
    }


make_header <- function(x)
{
    vn <- names(dimnames(x))
    header <- c(if(x$log) "log" else "",
                "odds ratios for", vn[1], "and", vn[2],
                if (length(vn) > 2) c("by", paste(vn[-(1:2)], collapse=', ')), "\n\n")
    paste(header, sep = " ")
}

## print method
print.loddsratio <- function(x, log = x$log, ...) {
    cat(make_header(x))
    print(drop(array(coef(x, log = log), dim = dim(x), dimnames = dimnames(x)), ...))
    invisible(x)
}

summary.loddsratio <- function(object, ...)
    lmtest::coeftest(object, ...)

## reshape coef() methods
as.matrix.loddsratio <- function (x, log=x$log, ...) {
    Coef <- coef(x, log = log)
    if (length(dim(x)) == 2L) matrix(Coef, ncol = dim(x)[2], dimnames=dimnames(x))
    else {  # drop leading dimensions with length 1, then reshape
        ddim <- which(dim(x)[1:2]==1)
        dim(Coef) <- dim(x)[-ddim]
        dimnames(Coef) <- dimnames(x)[-ddim]
        if (length(dim(Coef)) == 1L) Coef
        else
            matrix(Coef, ncol = prod(dim(Coef)[-1]),
                   dimnames=list(dimnames(Coef)[[1]], apply(expand.grid(dimnames(Coef)[[-1]]), 1, paste, collapse = ":")))
    }
}


as.array.loddsratio <- function (x, log=x$log, ...) {
    res <- array(coef(x, log = log), dim = dim(x), dimnames=dimnames(x))
    drop(res)
}

as.data.frame.loddsratio <- function(x, row.names = NULL, optional, log=x$log, ...) {
    df <-data.frame(expand.grid(dimnames(x)),
                    LOR = coef(x, log=log),
                    ASE = sqrt(diag(vcov(x, log=log))), row.names=row.names,  ...
                    )
    if (!log) colnames(df)[ncol(df)-1] <- "OR"
    df
}

image.loddsratio <-
    function(x, interpolate = NULL, legend = legend_fixed,
             gp = shading_Friendly, gp_args = NULL,
             labeling = labeling_values("residuals", suppress = 0),
             perm = NULL, ...)
{
    a <- as.array(x)
    if (!is.null(dim(a))) {
        if (is.null(perm)) {
            d <- seq_along(dim(a))
            perm <- c(d[-c(1:2)], 1:2)
        }
        a <- aperm(a, perm)
    } else {
        a <- as.table(a)
        names(dimnames(a)) <- names(dimnames(x))[1]
    }
    if (is.null(interpolate))
        interpolate <- seq(0.1, max(abs(a), length.out = 4L))
    if (is.null(gp_args))
        gp_args <- list(interpolate = interpolate)
    tmp <- a
    tmp[] <- 1
    mosaic(tmp, type = "expected",
           residuals = a, shade = TRUE,
           gp = shading_Friendly, gp_args = gp_args,
           legend = legend,
           labeling = labeling, ...)
}

tile.loddsratio <-
    function(x, interpolate = NULL, legend = legend_fixed,
             gp = shading_Friendly, gp_args = NULL,
             labeling = labeling_values("residuals", suppress = 0),
             halign = "center", valign = "center", perm = NULL, ...)
{
    a <- as.array(x)
    if (!is.null(dim(a))) {
        if (is.null(perm)) {
            d <- seq_along(dim(a))
            perm <- c(d[-c(1:2)], 1:2)
        }
        a <- aperm(a, perm)
    } else {
        a <- as.table(a)
        names(dimnames(a)) <- names(dimnames(x))[1]
    }

    if (is.null(interpolate))
        interpolate <- seq(0.1, max(abs(a), length.out = 4L))
    if (is.null(gp_args))
        gp_args <- list(interpolate = interpolate)

    tile(abs(a), halign = halign, valign = valign,
         residuals = a, shade = TRUE,
         gp = shading_Friendly, gp_args = gp_args,
         legend = legend,
         labeling = labeling, ...)
}

"plot.loddsratio" <-
    function(x,
             baseline = TRUE,
             gp_baseline = gpar(lty = 2),
             lines = TRUE,
             lwd_lines = 3,
             confidence = TRUE,
             conf_level = 0.95,
             lwd_confidence = 2,
             whiskers = 0,
             transpose = FALSE,
             col = NULL,
             cex = 0.8,
             pch = NULL,
             bars = NULL,
             gp_bars = gpar(fill = "lightgray", alpha = 0.5),
             bar_width = unit(0.05, "npc"),

             legend = TRUE, legend_pos = "topright", legend_inset = c(0, 0),
             legend_vgap = unit(0.5, "lines"),
             gp_legend_frame = gpar(lwd = 1, col = "black"),
             gp_legend_title = gpar(fontface = "bold"),
             gp_legend = gpar(), legend_lwd = 1,
             legend_size = 1,

             xlab = NULL,
             ylab = NULL,
             xlim = NULL,
             ylim = NULL,

             main = NULL,
             gp_main = gpar(fontsize = 12, fontface = "bold"),
             newpage = TRUE, pop = FALSE, return_grob = FALSE,
             add = FALSE,
             prefix = "",
             ...)
{
    ## handle default values, limits etc.
    LOG <- x$log
    values <- as.array(x)
    d <- dim(values)
    if (is.null(bars))
        bars <- is.null(d)
    oddsrange <- range(values, na.rm = TRUE)

    if(confidence) {
        CI  <- confint(x, log = LOG, level = conf_level)
        lwr <- CI[,1]
        upr <- CI[,2]
        oddsrange <- if (baseline)
                         c(min(0, lwr, na.rm = TRUE), max(0, upr, na.rm = TRUE))
                     else
                         c(min(lwr, na.rm = TRUE), max(upr, na.rm = TRUE))
    }

    if (is.null(main))
        main <- paste(make_header(x), collapse = " ")

    if (is.null(xlim))
        xlim <- if (is.null(d))
                    c(1, length(values))
                else
                    c(1, d[1])

    if (is.null(ylim))
        ylim <- oddsrange

    ylimaxis <- ylim + c(-1, 1) * diff(ylim) * 0.04
    xlimaxis <- xlim + c(-1, 1) * diff(xlim) * 0.04

    ncols <- if (is.null(d)) 1 else prod(d[-1])
    if (is.null(col))
        col <- rainbow_hcl(ncols, l = 50)

    if (is.null(pch))
        pch <- c(19,15,17, 1:14, 16, 18, 20:25)

    labs <- if (is.null(d)) names(values) else dimnames(values)[[1]]
    if (is.null(xlab))
        xlab <- if (is.null(d)) names(dimnames(x))[3] else names(dimnames(values))[1]
    if (is.null(ylab))
        ylab <- paste(if (LOG) "L" else "",
                      "OR(", paste(names(dimnames(x))[1:2], collapse = " / "), ")", sep = "")


    if (newpage) grid.newpage()
    if (transpose) {
        if (!add) {
            ## set up plot region, similar to plot.xy()
            pushViewport(plotViewport(xscale = ylimaxis, yscale = xlimaxis,
                                      default.units = "native",
                                      name = paste(prefix,"oddsratio_plot")))
            grid.yaxis(name = "yaxis", seq_along(labs), labs,
                       edits = gEdit("labels", rot = 90, hjust = .5, vjust = 0))
            grid.xaxis()
            grid.text(ylab, y = unit(-3.5, "lines"))
            grid.text(xlab, x = unit(-3, "lines"), rot = 90)
            grid.text(main, y = unit(1, "npc") + unit(1, "lines"), gp = gp_main)
            pushViewport(viewport(xscale = ylimaxis, yscale = xlimaxis,
                                  default.units = "native", clip = "on"))

            ## baseline
            if (baseline)
                grid.lines(unit(c(1,1) - LOG, "native"),
                           unit(c(0,1), "npc"),
                           gp = gp_baseline)
        }

        # workhorse for one stratum
        draw_one_stratum <- function(vals, pch = "o", col = "black", offset = 0,
                                     jitter = 0) {
            if (bars) {
                if (any(vals > !LOG))
                    grid.rect(unit(vals[vals > !LOG], "native"),
                              unit(seq_along(vals)[vals > !LOG], "native"),
                              height = bar_width,
                              width = unit(vals[vals > !LOG] - !LOG, "native"),
                              just = "right",
                              gp = gp_bars
                              )
                if (any(vals < !LOG))
                    grid.rect(unit(vals[vals < !LOG], "native"),
                              unit(seq_along(vals)[vals < !LOG], "native"),
                              height = bar_width,
                              width = unit(abs(vals[vals < !LOG] - !LOG), "native"),
                              just = "left",
                              gp = gp_bars
                              )
            }
            if (lines)
                grid.lines(unit(vals, "native"),
                           unit(seq_along(vals), "native"),
                           gp = gpar(col = col, lwd = lwd_lines),
                           default.units = "native"
                           )
            grid.points(unit(vals, "native"),
                        unit(seq_along(vals), "native"),
                        pch = pch,
                        size = unit(cex, "char"),
                        gp = gpar(col = col, lwd = lwd_lines),
                        default.units = "native"
                        )
            if (confidence)
                for (i in seq_along(vals)) {
                    ii <- i + jitter
                    grid.lines(unit(c(lwr[offset + i], upr[offset + i]), "native"),
                               unit(c(ii, ii), "native"),
                               gp = gpar(col = col, lwd = lwd_confidence))
                    grid.lines(unit(c(lwr[offset + i], lwr[offset + i]), "native"),
                               unit(c(ii - whiskers/2, ii + whiskers/2), "native"),
                               gp = gpar(col = col, lwd = lwd_confidence))
                    grid.lines(unit(c(upr[offset + i], upr[offset + i]), "native"),
                               unit(c(ii - whiskers/2, ii + whiskers/2), "native"),
                               gp = gpar(col = col, lwd = lwd_confidence))
                }
        }
    } else {
        if (!add) {
            ## set up plot region
            pushViewport(plotViewport(xscale = xlimaxis, yscale = ylimaxis,
                                      default.units = "native",
                                      name = "oddsratio_plot"))
            grid.xaxis(seq_along(labs), labs)
            grid.yaxis()
            grid.text(xlab, y = unit(-3.5, "lines"))
            grid.text(ylab, x = unit(-3, "lines"), rot = 90)
            grid.text(main, y = unit(1, "npc") + unit(1, "lines"), gp = gp_main)
            pushViewport(viewport(xscale = xlimaxis, yscale = ylimaxis,
                                  default.units = "native", clip = "on"))

            ## baseline
            if (baseline)
                grid.lines(unit(c(0,1), "npc"),
                           unit(c(1,1) - LOG, "native"),
                           gp = gp_baseline)
        }

        ## workhorse for one stratum
        draw_one_stratum <- function(vals, pch = "o", col = "black", offset = 0,
                                     jitter = 0) {
            if (bars) {
                if (any(vals > !LOG))
                    grid.rect(unit(seq_along(vals)[vals > !LOG], "native"),
                              unit(vals[vals > !LOG], "native"),
                              width = bar_width,
                              height = unit(vals[vals > !LOG] - !LOG, "native"),
                              just = "top",
                              gp = gp_bars
                              )
                if (any(vals < !LOG))
                    grid.rect(unit(seq_along(vals)[vals < !LOG], "native"),
                              unit(vals[vals < !LOG], "native"),
                              width = bar_width,
                              height = unit(abs(vals[vals < !LOG] - !LOG), "native"),
                              just = "bottom",
                              gp = gp_bars
                              )
            }
            if (lines)
                grid.lines(unit(seq_along(vals), "native"),
                           unit(vals, "native"),
                           gp = gpar(col = col, lwd = lwd_lines),
                           default.units = "native"
                           )
            grid.points(unit(seq_along(vals), "native"),
                        unit(vals, "native"),
                        pch = pch,
                        size = unit(cex, "char"),
                        gp = gpar(col = col, lwd = lwd_lines),
                        default.units = "native"
                        )
            if (confidence)
                for (i in seq_along(vals)) {
                    ii <- i + jitter
                    grid.lines(unit(c(ii, ii), "native"),
                               unit(c(lwr[offset + i], upr[offset + i]), "native"),
                               gp = gpar(col = col, lwd = lwd_confidence))
                    grid.lines(unit(c(ii - whiskers/2, ii + whiskers/2), "native"),
                               unit(c(lwr[offset + i], lwr[offset + i]), "native"),
                               gp = gpar(col = col, lwd = lwd_confidence))
                    grid.lines(unit(c(ii - whiskers/2, ii + whiskers/2), "native"),
                               unit(c(upr[offset + i], upr[offset + i]), "native"),
                               gp = gpar(col = col, lwd = lwd_confidence))
                }
        }

    }

    if (is.null(d))
        draw_one_stratum(values, pch[1], col[1])
    else {
        jitt <- scale(seq_len(prod(d[-1])), scale = 25 * prod(d[-1]))
        for (i in 1 : prod(d[-1]))
            draw_one_stratum(values[(i - 1) * d[1] + seq(d[1])],
                             pch[(i - 1 ) %% length(pch) + 1],
                             col[i],
                             offset = (i - 1) * d[1],
                             jitt[i])
        if (legend)
            grid_legend(legend_pos,
                        labels = apply(expand.grid(dimnames(values)[-1]),
                                       1, paste, collapse = "|"),
                        pch = pch[1 : prod(d[-1])],
                        col = col,
                        lwd = legend_lwd,
                        lty = "solid",
                        size = legend_size,
                        vgap = legend_vgap,
                        gp = gp_legend,
                        gp_frame = gp_legend_frame,
                        inset = legend_inset,
                        title = paste(names(dimnames(values)[-1]), collapse = " x "),
                        gp_title = gp_legend_title, ...)
    }

    grid.rect(gp = gpar(fill = "transparent"))
    if (!add && pop) popViewport(2)
    if (return_grob)
        invisible(grid.grab())
    else
        invisible(NULL)
}

lines.loddsratio <-
function(x, legend = FALSE, confidence = FALSE, cex = 0, ...)
{
    plot(x, add = TRUE, newpage = FALSE, legend = legend, confidence = confidence, cex = cex, ...)
}

legend_resbased <- function(fontsize = 12,
                            fontfamily = "",
                            x = unit(1, "lines"),
                            y = unit(0.1, "npc"),
                            height = unit(0.8, "npc"),
                            width = unit(0.7, "lines"),
			    digits = 2,
                            pdigits = max(1, getOption("digits") - 2),
			    check_overlap = TRUE,
                            text = NULL,
                            steps = 200,
                            ticks = 10,
                            pvalue = TRUE,
                            range = NULL) {

  if(!is.unit(x)) x <- unit(x, "native")
  if(!is.unit(y)) y <- unit(y, "npc")
  if(!is.unit(width)) width <- unit(width, "lines")
  if(!is.unit(height)) height <- unit(height, "npc")

  function(residuals, shading, autotext) {
    res <- as.vector(residuals)

    if(is.null(text)) text <- autotext
    p.value <- attr(shading, "p.value")
    legend <- attr(shading, "legend")

    if (all(residuals == 0)) {
      pushViewport(viewport(x = x, y = y, just = c("left", "bottom"),
                            default.units = "native",
                            height = height, width = width))
      grid.lines(y = 0.5)
      grid.text(0, x = unit(1, "npc") + unit(0.8, "lines"),  y = 0.5,
                gp = gpar(fontsize = fontsize, fontfamily = fontfamily))
      warning("All residuals are zero.")

    } else {
        if (is.null(range))
            range <- range(res)
        if (length(range) != 2)
            stop("Range must have length two!")
        if (is.na(range[1]))
            range[1] <- min(res)
        if (is.na(range[2]))
            range[2] <- max(res)

        pushViewport(viewport(x = x, y = y, just = c("left", "bottom"),
                              yscale = range, default.units = "native",
                              height = height, width = width))


      if(is.null(legend$col.bins)) {
        col.bins <- seq(range[1], range[2], length.out = steps)
        at <- NULL
      } else {
        col.bins <- sort(unique(c(legend$col.bins, range)))
        col.bins <- col.bins[col.bins <= range[2] & col.bins >= range[1]]
        at <- col.bins
      }
      y.pos <- col.bins[-length(col.bins)]
      y.height <- diff(col.bins)

      grid.rect(x = unit(rep.int(0, length(y.pos)), "npc"),
                y = y.pos,
                height = y.height, default.units = "native",
                gp = gpar(fill = shading(y.pos + 0.5 * y.height)$fill, col = 0),
                just = c("left", "bottom"))

      grid.rect(gp = gpar(fill = "transparent"))

      if(is.null(at))
            at <- seq(from = head(col.bins, 1), to = tail(col.bins, 1),
                      length.out = ticks)
      lab <- format(round(at, digits = digits), nsmall = digits)
      tw <- lab[which.max(nchar(lab))]

      ## if(is.null(at))
      ##   at <- seq(from = head(col.bins, 1), to = tail(col.bins, 1), length = ticks)
      ## tw <- paste(rep("4", digits), collapse = "")
      ## if (any(trunc(at) != at))
      ##   tw <- paste(tw, ".", sep = "")
      ## if (any(at < 0))
      ##   tw <- paste(tw, "-", sep = "")

      grid.text(format(signif(at, digits = digits)),
                x = unit(1, "npc") + unit(0.8, "lines") + unit(1, "strwidth", tw),
                y = at,
                default.units = "native",
                just = c("right", "center"),
                gp = gpar(fontsize = fontsize, fontfamily = fontfamily),
                check.overlap = check_overlap)
      grid.segments(x0 = unit(1, "npc"), x1 = unit(1,"npc") + unit(0.5, "lines"),
                    y0 = at, y1 = at, default.units = "native")

    }

    popViewport(1)

    grid.text(text, x = x, y = unit(1, "npc") - y + unit(1, "lines"),
              gp = gpar(fontsize = fontsize, fontfamily = fontfamily,
              lineheight = 0.8),
              just = c("left", "bottom")
              )
    if(!is.null(p.value) && pvalue) {
        grid.text(paste("p-value =\n",
                        format.pval(p.value, digits = pdigits), sep = ""),
                x = x,
                y = y - unit(1, "lines"),
                gp = gpar(fontsize = fontsize, fontfamily = fontfamily,
                lineheight = 0.8),
                just = c("left", "top"))
    }
  }
}
class(legend_resbased) <- "grapcon_generator"

legend_fixed <- function(fontsize = 12,
                         fontfamily = "",
                         x = unit(1, "lines"),
                         y = NULL,
                         height = NULL,
                         width = unit(1.5, "lines"),
                         steps = 200,
			 digits = 1,
                         space = 0.05,
                         text = NULL,
                         range = NULL) {

  if(!is.unit(x)) x <- unit(x, "native")
  if(!is.unit(y) && !is.null(y)) y <- unit(y, "npc")
  if(!is.unit(width)) width <- unit(width, "lines")
  if(!is.unit(height) && !is.null(height)) height <- unit(height, "npc")

  function(residuals, shading, autotext) {
    res <- as.vector(residuals)

    if(is.null(text)) text <- autotext

    if (is.null(y)) y <- unit(1, "strwidth", text) + unit(1, "lines")
    if (is.null(height)) height <- unit(1, "npc") - y

    pushViewport(viewport(x = x, y = y, just = c("left", "bottom"),
                          yscale = c(0,1), default.units = "npc",
                          height = height, width = width))

    p.value <- attr(shading, "p.value")
    legend <- attr(shading, "legend")

        if (is.null(range))
            range <- range(res)
        if (length(range) != 2)
            stop("Range must have length two!")
        if (is.na(range[1]))
            range[1] <- min(res)
        if (is.na(range[2]))
            range[2] <- max(res)

    if(is.null(legend$col.bins)) {
      col.bins <- seq(range[1], range[2], length.out = steps)
    } else {
      col.bins <- sort(unique(c(legend$col.bins, range)))
      col.bins <- col.bins[col.bins <= range[2] & col.bins >= range[1]]
    }
    l <- length(col.bins)
    y.height <- (1 - (l - 2) * space) / (l - 1)
    y.pos <- cumsum(c(0, rep(y.height + space, l - 2)))
    res <- col.bins[-l] + diff(col.bins) / 2

    grid.rect(x = unit(rep.int(0, length(y.pos)), "npc"),
              y = y.pos,
              height = y.height,
              default.units = "npc",
              gp = shading(res),
              just = c("left", "bottom"))
    numbers <- format(col.bins, nsmall = digits, digits = digits)
    wid <- unit(1, "strwidth", format(max(abs(col.bins)),
                                      nsmall = digits, digits = digits))
    grid.text(numbers[-l],
              x = unit(1, "npc") + unit(0.6, "lines") + wid,
              y = y.pos, gp = gpar(fontsize = fontsize, fontfamily = fontfamily),
              default.units = "npc", just = c("right", "bottom"))
    grid.text(numbers[-1],
              x = unit(1, "npc") + unit(0.6, "lines") + wid,
              y = y.pos + y.height, gp = gpar(fontsize = fontsize, fontfamily = fontfamily),
              default.units = "npc", just = c("right", "top"))
    wid2 <- unit(1, "strwidth", format(max(abs(trunc(col.bins))))) +
      unit(0.3, "strwidth", ".")
    grid.segments(x0 = unit(1, "npc") + wid2 + unit(0.6, "lines"),
                  x1 = unit(1, "npc") + wid2 + unit(0.6, "lines"),
                  y0 = unit(y.pos, "npc") + 1.5 * unit(1, "strheight", "-44.4"),
                  y1 = unit(y.pos + y.height, "npc") - 1.5 * unit(1, "strheight", "-44.4")
                  )

    popViewport(1)
    grid.text(text, x = x + 0.5 * width, y = 0,
              gp = gpar(fontsize = fontsize, fontfamily = fontfamily,
              lineheight = 0.8),
              just = c("left", "top"),
              rot = 90
              )
  }
}
class(legend_fixed) <- "grapcon_generator"

if(require("tcltk"))
{
    hue  <- tclVar("hue")
    chroma  <- tclVar("chroma")
    luminance  <- tclVar("luminance")
    fixup <- tclVar("fixup")
    hue <- tclVar(230)
    hue.sav <- 230
    chroma <- tclVar(55)
    chroma.sav <- 55
    luminance <- tclVar(75)
    luminance.sav <- 75
    fixup <- tclVar(FALSE)

    replot <- function(...) {
        hue.sav <- my.h <- as.numeric(tclvalue(hue))
        chroma.sav <- my.c <- as.numeric(tclvalue(chroma))
        luminance.sav <- my.l <- as.numeric(tclvalue(luminance))
	my.fixup <- as.logical(as.numeric(tclvalue(fixup)))
	barplot(1, col = hcl2hex(my.h, my.c, my.l, fixup = my.fixup), axes = FALSE)
    }

    replot.maybe <- function(...)
    {
        if(!((as.numeric(tclvalue(hue)) == hue.sav) &&
	     (as.numeric(tclvalue(chroma)) == chroma.sav) &&
	     (as.numeric(tclvalue(luminance)) == luminance.sav))) replot()
    }


    base <- tktoplevel()
    tkwm.title(base, "HCL Colors")

    spec.frm <- tkframe(base, borderwidth = 2)
    hue.frm <- tkframe(spec.frm, relief = "groove", borderwidth = 2)
    chroma.frm <- tkframe(spec.frm, relief = "groove", borderwidth = 2)
    luminance.frm <- tkframe(spec.frm, relief = "groove", borderwidth = 2)
    fixup.frm <- tkframe(spec.frm, relief = "groove", borderwidth = 2)

    tkpack(tklabel(hue.frm, text = "Hue"))
    tkpack(tkscale(hue.frm, command = replot.maybe, from = 0, to = 360,
                   showvalue = TRUE, variable = hue,
                   resolution = 1, orient = "horiz"))

    tkpack(tklabel(chroma.frm, text = "Chroma"))
    tkpack(tkscale(chroma.frm, command = replot.maybe, from = 0, to = 100,
                   showvalue = TRUE, variable = chroma,
                   resolution = 5, orient = "horiz"))

    tkpack(tklabel(luminance.frm, text = "Luminance"))
    tkpack(tkscale(luminance.frm, command = replot.maybe, from = 0, to = 100,
                   showvalue = TRUE, variable = luminance,
                   resolution = 5, orient = "horiz"))
    tkpack(tklabel(fixup.frm, text="Fixup"))
    for (i in c("TRUE", "FALSE") ) {
        tmp <- tkradiobutton(fixup.frm, command = replot,
                             text = i, value = as.logical(i), variable = fixup)
        tkpack(tmp, anchor="w")
    }

    tkpack(hue.frm, chroma.frm, luminance.frm, fixup.frm, fill="x")

    ## Bottom frame on base:
    q.but <- tkbutton(base, text = "Quit",
                      command = function() tkdestroy(base))

    tkpack(spec.frm, q.but)

    replot()
}

### R code from vignette source 'residual-shadings.Rnw'

###################################################
### code chunk number 1: preliminaries
###################################################
library("grid")
library("vcd")
rseed <- 1071


###################################################
### code chunk number 2: Arthritis-data
###################################################
data("Arthritis", package = "vcd")
(art <- xtabs(~ Treatment + Improved, data = Arthritis, subset = Sex == "Female"))


###################################################
### code chunk number 3: Arthritis-classic (eval = FALSE)
###################################################
## mosaic(art)
## assoc(art)


###################################################
### code chunk number 4: Arthritis-classic1
###################################################
grid.newpage()
pushViewport(viewport(layout = grid.layout(1, 2)))
pushViewport(viewport(layout.pos.col=1, layout.pos.row=1))
mosaic(art, newpage = FALSE, margins = c(2.5, 4, 2.5, 3))
popViewport()
pushViewport(viewport(layout.pos.col=2, layout.pos.row=1))
assoc(art, newpage = FALSE, margins = c(5, 2, 5, 4))
popViewport(2)


###################################################
### code chunk number 5: Arthritis-max
###################################################
set.seed(rseed)
(art_max <- coindep_test(art, n = 5000))


###################################################
### code chunk number 6: Arthritis-sumsq
###################################################
ss <- function(x) sum(x^2)
set.seed(rseed)
coindep_test(art, n = 5000, indepfun = ss)


###################################################
### code chunk number 7: Arthritis-extended (eval = FALSE)
###################################################
## mosaic(art, gp = shading_Friendly(lty = 1, eps = NULL))
## mosaic(art, gp = shading_hsv, gp_args = list(
##   interpolate = art_max$qdist(c(0.9, 0.99)), p.value = art_max$p.value))
## set.seed(rseed)
## mosaic(art, gp = shading_max, gp_args = list(n = 5000))


###################################################
### code chunk number 8: pistonrings-data
###################################################
data("pistonrings", package = "HSAUR3")
pistonrings


###################################################
### code chunk number 9: shadings
###################################################
mymar <- c(1.5, 0.5, 0.5, 2.5)
grid.newpage()
pushViewport(viewport(layout = grid.layout(2, 3)))
pushViewport(viewport(layout.pos.row = 1, layout.pos.col = 1))
mosaic(art, margins = mymar, newpage = FALSE,
  gp = shading_Friendly(lty = 1, eps = NULL))
popViewport()
pushViewport(viewport(layout.pos.row = 1, layout.pos.col = 2))
mosaic(art, gp = shading_hsv, margins = mymar, newpage = FALSE,
  gp_args = list(interpolate = art_max$qdist(c(0.9, 0.99)), p.value = art_max$p.value))
popViewport()
pushViewport(viewport(layout.pos.row = 1, layout.pos.col = 3))
set.seed(rseed)
mosaic(art, gp = shading_max, margins = mymar, newpage = FALSE,
  gp_args = list(n = 5000))
popViewport()

pushViewport(viewport(layout.pos.row = 2, layout.pos.col = 1))
mosaic(pistonrings, margins = mymar, newpage = FALSE,
  gp = shading_Friendly(lty = 1, eps = NULL, interpolate = c(1, 1.5)))
popViewport()
pushViewport(viewport(layout.pos.row = 2, layout.pos.col = 2))
mosaic(pistonrings, gp = shading_hsv, margins = mymar, newpage = FALSE,
  gp_args = list(p.value = 0.069, interpolate = c(1, 1.5)))
popViewport()
pushViewport(viewport(layout.pos.row = 2, layout.pos.col = 3))
mosaic(pistonrings, gp = shading_hcl, margins = mymar, newpage = FALSE,
  gp_args = list(p.value = 0.069, interpolate = c(1, 1.5)))
popViewport(2)


###################################################
### code chunk number 10: pistonrings-inference
###################################################
set.seed(rseed)
coindep_test(pistonrings, n = 5000)
set.seed(rseed)
(pring_ss <- coindep_test(pistonrings, n = 5000, indepfun = ss))


###################################################
### code chunk number 11: pistonrings-plot (eval = FALSE)
###################################################
## mosaic(pistonrings, gp = shading_Friendly(lty = 1, eps = NULL, interpolate = c(1, 1.5)))
## mosaic(pistonrings, gp = shading_hsv, gp_args = list(p.value = pring_ss$p.value, interpolate = c(1, 1.5)))
## mosaic(pistonrings, gp = shading_hcl, gp_args = list(p.value = pring_ss$p.value, interpolate = c(1, 1.5)))


###################################################
### code chunk number 12: alzheimer-data
###################################################
data("alzheimer", package = "coin")
alz <- xtabs(~ smoking + disease + gender, data = alzheimer)
alz


###################################################
### code chunk number 13: alzheimer-plot1
###################################################
set.seed(rseed)
cotabplot(~ smoking + disease | gender, data = alz, panel = cotab_coindep, panel_args = list(n = 5000))


###################################################
### code chunk number 14: alzheimer-inference
###################################################
set.seed(rseed)
coindep_test(alz, 3, n = 5000)
set.seed(rseed)
coindep_test(alz, 3, n = 5000, indepfun = ss)
set.seed(rseed)
coindep_test(alz, 3, n = 5000, indepfun = ss, aggfun = sum)


###################################################
### code chunk number 15: alzheimer-plot (eval = FALSE)
###################################################
## set.seed(rseed)
## cotabplot(~ smoking + disease | gender, data = alz, panel = cotab_coindep, panel_args = list(n = 5000))


###################################################
### code chunk number 16: Punishment-data
###################################################
data("Punishment", package = "vcd")
pun <- xtabs(Freq ~ memory + attitude + age + education, data = Punishment)
ftable(pun, row.vars = c("age", "education", "memory"))


###################################################
### code chunk number 17: Punishment-assoc1
###################################################
set.seed(rseed)
cotabplot(~ memory + attitude | age + education, data = pun, panel = cotab_coindep,
  n = 5000, type = "assoc", test = "maxchisq", interpolate = 1:2)


###################################################
### code chunk number 18: Punishment-mosaic1
###################################################
set.seed(rseed)
cotabplot(~ memory + attitude | age + education, data = pun, panel = cotab_coindep,
  n = 5000, type = "mosaic", test = "maxchisq", interpolate = 1:2)


###################################################
### code chunk number 19: Punishment-inference
###################################################
set.seed(rseed)
coindep_test(pun, 3:4, n = 5000)
set.seed(rseed)
coindep_test(pun, 3:4, n = 5000, indepfun = ss)
set.seed(rseed)
coindep_test(pun, 3:4, n = 5000, indepfun = ss, aggfun = sum)


###################################################
### code chunk number 20: Punishment-assoc (eval = FALSE)
###################################################
## set.seed(rseed)
## cotabplot(~ memory + attitude | age + education, data = pun, panel = cotab_coindep,
##   n = 5000, type = "assoc", test = "maxchisq", interpolate = 1:2)


###################################################
### code chunk number 21: Punishment-mosaic (eval = FALSE)
###################################################
## set.seed(rseed)
## cotabplot(~ memory + attitude | age + education, data = pun, panel = cotab_coindep,
##   n = 5000, type = "mosaic", test = "maxchisq", interpolate = 1:2)

  #####################
  ## Fourfold tables ##
  #####################

  ### Berkeley Admission Data ###
  ###############################
  data(UCBAdmissions)

  ## unstratified
  ### no margin is standardized
  x <- margin.table(UCBAdmissions, 2:1)
  fourfold(x, std = "i", extended = FALSE)
  ### std. for gender
  fourfold(x, margin = 1, extended = FALSE)
  ### std. for both
  fourfold(x, extended = FALSE)

  ## stratified
  fourfold(UCBAdmissions, extended = FALSE)
  fourfold(UCBAdmissions) ## extended plots

  ## using cotabplot
  cotabplot(UCBAdmissions, panel = function(x, condlevels, ...)
              fourfold(co_table(x, names(condlevels))[[paste(condlevels, collapse = ".")]],
                       newpage = F, return_grob = FALSE, ...)
            )

  ### Coal Miners Lung Data ###
  #############################
  data(CoalMiners)

  ## Fourfold display, both margins equated
  fourfold(CoalMiners, mfcol = c(3,3))

  ## Log Odds Ratio Plot
  data(CoalMiners, package = "vcd")
  lor_CM <- loddsratio(CoalMiners)
  plot(lor_CM)
  lor_CM_df <- as.data.frame(lor_CM)

  # fit linear models using WLS
  age <- seq(20, 60, by = 5)
  lmod <- lm(LOR ~ age, weights = 1 / ASE^2, data = lor_CM_df)
  grid.lines(age, fitted(lmod), gp = gpar(col = "blue"))
  qmod <- lm(LOR ~ poly(age, 2), weights = 1 / ASE^2, data = lor_CM_df)
  grid.lines(age, fitted(qmod), gp = gpar(col = "red"))

  ## Fourfold display, strata equated
  fourfold(CoalMiners, std = "ind.max", mfcol = c(3,3))

  ####################
  ## Sieve Diagrams ##
  ####################

  ### Hair Eye Color ###
  ######################
  data(HairEyeColor)

  ## aggregate over `sex':
  (tab <- margin.table(HairEyeColor, 1:2))
  ## plot expected values:
  sieve(t(tab), sievetype = "expected", shade = TRUE)

  ## plot sieve diagram:
  sieve(t(tab), shade = TRUE)

  ### Visual Acuity ###
  #####################
  data(VisualAcuity)
  attach(VisualAcuity)
  sieve(Freq ~ right + left,
        data = VisualAcuity,
        subset = gender == "female",
        main = "Unaided distant vision data",
        labeling_args = list(set_varnames = c(left = "Left Eye Grade",
                               right = "Right Eye Grade")),
        shade = TRUE
        )
  detach(VisualAcuity)

  ### Berkeley Admission ###
  ##########################

  ## -> Larger tables: e.g., Cross factors
  ### Cross Gender and Admission
  data(UCBAdmissions)

  (tab <- xtabs(Freq ~ Dept + I(Gender : Admit), data = UCBAdmissions))
  sieve(tab,
        labeling_args = list(set_varnames = c("I(Gender:Admit)" = "Gender:Admission",
                               Dept = "Department")),
        main = "Berkeley Admissions Data",
        shade = TRUE
        )

  ## or use extended sieve plots:
  sieve(UCBAdmissions, shade = TRUE)

  ######################
  ## Association Plot ##
  ######################

  ### Hair Eye Color ###
  ######################
  data(HairEyeColor)
  assoc(margin.table(HairEyeColor, 1:2),
        labeling_args = list(set_varnames = c(Hair = "Hair Color", Eye = "Eye Color")),
        main = "Association Plot")

  ####################
  ## Agreement Plot ##
  ####################

  ### Sexual Fun ###
  ##################
  data(SexualFun)

  ## Kappa statistics
  Kappa(SexualFun)

  ## Agreement Chart
  agreementplot(t(SexualFun), weights = 1)
  ## Partial Agreement Chart and B-Statistics
  (agreementplot(t(SexualFun),
                      xlab = "Husband's Rating",
                      ylab = "Wife's Rating",
                      main = "Husband's and Wife's Sexual Fun")
   )

  ### MS Diagnosis data ###
  #########################
  data(MSPatients)
  ## use e.g., X11(width = 12), or expand graphics device
  agreementplot(t(MSPatients[,,1]), main = "Winnipeg Patients")
  agreementplot(t(MSPatients[,,2]), main = "New Orleans Patients")

  ##################
  ## Ternary Plot ##
  ##################

  ### sample data ###
  ###################
  (x <- rbind(c(A=10,B=10,C=80),
              c(40,30,30),
              c(20,60,20)
              )
   )
  ternaryplot(x,
              cex = 2,
              col = c("black", "blue", "red"),
              coordinates = TRUE
              )

  ### Arthritis Treatment Data ###
  ################################
  data(Arthritis)

  ## Build table by crossing Treatment and Sex
  (tab <- as.table(xtabs(~ I(Sex:Treatment) + Improved, data = Arthritis)))

  ## Mark groups
  col <- c("red", "red", "blue", "blue")
  pch <- c(1, 19, 1, 19)

  ## plot
  ternaryplot(
              tab,
              col = col,
              pch = pch,
              cex = 2,
              bg = "lightgray",
              grid_color = "white",
              labels_color = "white",
              main = "Arthritits Treatment Data"
              )
  ## legend
  grid_legend(0.8, 0.7, pch, col, rownames(tab), title = "GROUP")

  ### Baseball Hitters Data ###
  #############################
  data(Hitters)
  attach(Hitters)

  colors <- c("black","red","green","blue","red","black","blue")
  pch <- substr(levels(Positions), 1, 1)
  ternaryplot(
              Hitters[,2:4],
              pch = as.character(Positions),
              col = colors[as.numeric(Positions)],
              main = "Baseball Hitters Data"
              )
  grid_legend(0.8, 0.9, pch, colors, levels(Positions), title = "POSITION(S)")

  detach(Hitters)

  ### Lifeboats on the Titanic ###
  ################################
  data(Lifeboats)
  attach(Lifeboats)

  ternaryplot(
              Lifeboats[,4:6],
              pch = ifelse(side=="Port", 1, 19),
              col = ifelse(side=="Port", "red", "blue"),
              id  = ifelse(men/total > 0.1, as.character(boat), NA),
              dimnames_position = "edge",
              dimnames = c("Men of Crew", "Men passengers", "Women and Children"),
              main = "Lifeboats on the Titanic"
              )
  grid_legend(0.8, 0.9, c(1, 19), c("red", "blue"), c("Port", "Starboard"), title = "SIDE")

  ## Load against time for Port/Starboard boats
  plot(launch, total,
       pch = ifelse(side == "Port", 1, 19),
       col = ifelse(side == "Port", "red", "darkblue"),
       xlab = "Launch Time",
       ylab = "Total loaded",
       main = "Lifeboats on the Titanic"
       )
  legend(as.POSIXct("1912-04-15 01:48:00"), 70,
         legend = c("SIDE","Port","Starboard"),
         pch = c(NA, 1, 19),
         col = c(NA, "red", "darkblue")
         )
  text(as.POSIXct(launch),
       total,
       labels = as.character(boat),
       pos = 3,
       offset = 0.3
       )
  abline(lm(total ~ as.POSIXct(launch),
            subset = side == "Port"),
         col = "red")
  abline(lm(total ~ as.POSIXct(launch),
            subset = side == "Starboard"),
         col = "darkblue")

  detach(Lifeboats)

if(require("tcltk"))
{
    hue  <- tclVar("hue")
    luminance  <- tclVar("luminance")
    saturation  <- tclVar("saturation")
    hue <- tclVar(0)
    hue.sav <- 0
    luminance <- tclVar(0.5)
    luminance.sav <- 0.5
    saturation <- tclVar(1)
    saturation.sav <- 1

    replot <- function(...) {
        hue.sav <- my.h <- as.numeric(tclvalue(hue))
        saturation.sav <- my.s <- as.numeric(tclvalue(saturation))
        luminance.sav <- my.l <- as.numeric(tclvalue(luminance))
	barplot(1, col = hls(my.h, my.l, my.s), axes = FALSE)
    }

    replot.maybe <- function(...)
    {
        if(!((as.numeric(tclvalue(hue)) == hue.sav) &&
	     (as.numeric(tclvalue(saturation)) == saturation.sav) &&
	     (as.numeric(tclvalue(luminance)) == luminance.sav))) replot()
    }


    base <- tktoplevel()
    tkwm.title(base, "HLS Colors")

    spec.frm <- tkframe(base, borderwidth = 2)
    hue.frm <- tkframe(spec.frm, relief = "groove", borderwidth = 2)
    saturation.frm <- tkframe(spec.frm, relief = "groove", borderwidth = 2)
    luminance.frm <- tkframe(spec.frm, relief = "groove", borderwidth = 2)

    tkpack(tklabel(hue.frm, text = "Hue"))
    tkpack(tkscale(hue.frm, command = replot.maybe, from = 0, to = 1,
                   showvalue = TRUE, variable = hue,
                   resolution = 0.01, orient = "horiz"))

    tkpack(tklabel(luminance.frm, text = "Luminance"))
    tkpack(tkscale(luminance.frm, command = replot.maybe, from = 0, to = 1,
                   showvalue = TRUE, variable = luminance,
                   resolution = 0.01, orient = "horiz"))

    tkpack(tklabel(saturation.frm, text = "Saturation"))
    tkpack(tkscale(saturation.frm, command = replot.maybe, from = 0, to = 1,
                   showvalue = TRUE, variable = saturation,
                   resolution = 0.01, orient = "horiz"))

    tkpack(hue.frm, luminance.frm, saturation.frm, fill="x")

    ## Bottom frame on base:
    q.but <- tkbutton(base, text = "Quit",
                      command = function() tkdestroy(base))

    tkpack(spec.frm, q.but)

    replot()
}

#####################
## Mosaic Displays ##
#####################

#########################
## Hair Eye Color Data ##
#########################

data(HairEyeColor)

## Basic Mosaic Display ##

HairEye <- margin.table(HairEyeColor, c(1,2))

mosaic(HairEye, main = "Basic Mosaic Display of Hair Eye Color data")

## Hair Mosaic Display with Pearson residuals ##
Hair <- margin.table(HairEyeColor,1)
Hair
mHair <- as.table(rep(mean(margin.table(HairEyeColor, 1)), 4))
names(mHair) <- names(Hair)
mHair

## Pearson residuals from Equiprobability model ##

resid <- (Hair - mHair) / sqrt(mHair)
resid

## First Step in a Mosaic Display ##

mosaic(Hair, residuals = resid, main = "Hair Color Proportions")

## Hair Eye Mosais Display with Pearson residuals ##

mosaic(HairEye, main = " Hair Eye Color with Pearson residuals")

## Show Pearson Residuals ##

(HairEye - loglin(HairEye, c(1, 2), fit = TRUE)$fit) /
  sqrt(loglin(HairEye, c(1, 2), fit = TRUE)$fit)

###################
## UKSoccer Data ##
###################

data(UKSoccer)

## UKSoccer Mosaic Display ##

mosaic(UKSoccer, main = "UK Soccer Scores")

###############################
## Repeat Victimization Data ##
###############################

data(RepVict)

## mosaic(RepVict[-c(4, 7), -c(4, 7)], main = "Repeat Victimization Data")


##################
## 3-Way Tables ##
##################

## Hair Eye Sex Mosais Display with Pearson residuals ##
mosaic(HairEyeColor, main = "Hair Eye Color Sex" )

mosaic(HairEyeColor, expected = ~ Hair * Eye + Sex,
                main = "Model: (Hair Eye) (Sex)" )

mosaic(HairEyeColor, expected = ~ Hair * Sex + Eye*Sex,
               main = "Model: (Hair Sex) (Eye Sex)")


####################
## Premarital Sex ##
####################

data(PreSex)

## Mosaic display for Gender and Premarital Sexual Expirience ##

## (Gender Pre) ##
mosaic(margin.table(PreSex, c(3, 4)), legend = FALSE,
                main = "Gender and Premarital Sex")

## (Gender Pre)(Extra) ##
mosaic(margin.table(PreSex,c(2,3,4)), legend = FALSE,
                expected = ~ Gender * PremaritalSex + ExtramaritalSex ,
                main = "(PreMaritalSex Gender) (Sex)")

## (Gender Pre Extra)(Marital) ##
mosaic(PreSex,
       expected = ~ Gender * PremaritalSex * ExtramaritalSex + MaritalStatus,
       legend = FALSE,
       main = "(PreMarital ExtraMarital) (MaritalStatus)")

## (GPE)(PEM) ##
mosaic(PreSex,
       expected = ~ Gender * PremaritalSex * ExtramaritalSex
       + MaritalStatus * PremaritalSex * ExtramaritalSex,
       legend = FALSE,
       main = "(G P E) (P E M)")

############################
## Employment Status Data ##
############################

data(Employment)

## Employment Status ##
# mosaic(Employment,
#        expected = ~ LayoffCause * EmploymentLength + EmploymentStatus,
#        main = "(Layoff Employment) + (EmployStatus)")


# mosaic(Employment,
#        expected = ~ LayoffCause * EmploymentLength +
#                 LayoffCause * EmploymentStatus,
#        main = "(Layoff EmpL) (Layoff EmplS)")

# ## Closure ##
# mosaic(Employment[,,1], main = "Layoff : Closure")

# ## Replaced ##
# mosaic(Employment[,,2], main = "Layoff : Replaced")


#####################
## Mosaic Matrices ##
#####################

data(UCBAdmissions)

pairs(PreSex)

pairs(UCBAdmissions)

pairs(UCBAdmissions, type = "conditional")

pairs(UCBAdmissions, type = "pairwise", gp = shading_max)

if(require("tcltk"))
{
    hue  <- tclVar("hue")
    saturation  <- tclVar("saturation")
    value  <- tclVar("value")
    hue <- tclVar(0)
    hue.sav <- 0
    saturation <- tclVar(1)
    saturation.sav <- 1
    value <- tclVar(1)
    value.sav <- 1

    replot <- function(...) {
        hue.sav <- my.h <- as.numeric(tclvalue(hue))
        saturation.sav <- my.s <- as.numeric(tclvalue(saturation))
        value.sav <- my.v <- as.numeric(tclvalue(value))
	barplot(1, col = hsv(my.h, my.s, my.v), axes = FALSE)
    }

    replot.maybe <- function(...)
    {
        if(!((as.numeric(tclvalue(hue)) == hue.sav) &&
	     (as.numeric(tclvalue(saturation)) == saturation.sav) &&
	     (as.numeric(tclvalue(value)) == value.sav))) replot()
    }


    base <- tktoplevel()
    tkwm.title(base, "HSV Colors")

    spec.frm <- tkframe(base, borderwidth = 2)
    hue.frm <- tkframe(spec.frm, relief = "groove", borderwidth = 2)
    saturation.frm <- tkframe(spec.frm, relief = "groove", borderwidth = 2)
    value.frm <- tkframe(spec.frm, relief = "groove", borderwidth = 2)

    tkpack(tklabel(hue.frm, text = "Hue"))
    tkpack(tkscale(hue.frm, command = replot.maybe, from = 0, to = 1,
                   showvalue = TRUE, variable = hue,
                   resolution = 0.01, orient = "horiz"))

    tkpack(tklabel(saturation.frm, text = "Saturation"))
    tkpack(tkscale(saturation.frm, command = replot.maybe, from = 0, to = 1,
                   showvalue = TRUE, variable = saturation,
                   resolution = 0.01, orient = "horiz"))

    tkpack(tklabel(value.frm, text = "Value"))
    tkpack(tkscale(value.frm, command = replot.maybe, from = 0, to = 1,
                   showvalue = TRUE, variable = value,
                   resolution = 0.01, orient = "horiz"))

    tkpack(hue.frm, saturation.frm, value.frm, fill="x")

    ## Bottom frame on base:
    q.but <- tkbutton(base, text = "Quit",
                      command = function() tkdestroy(base))

    tkpack(spec.frm, q.but)

    replot()
}

data("Titanic")
data("UCBAdmissions")
data("HairEyeColor")
data("PreSex")
 
mosaic(Titanic)
mosaic(Titanic, shade = TRUE)
mosaic(~ Sex + Class, data = Titanic, shade = TRUE)
mosaic(~ Sex + Class + Survived, data = Titanic, shade = TRUE)
mosaic(~ PremaritalSex + ExtramaritalSex | MaritalStatus + Gender, data = PreSex)
mosaic(~ PremaritalSex + ExtramaritalSex | MaritalStatus + Gender, data = PreSex, labeling = labeling_conditional(abbreviate_varnames = TRUE))
mosaic(Titanic, spacing = spacing_increase())
mosaic(Titanic, spacing = spacing_equal())
mosaic(Titanic, labeling = labeling_border())
mosaic(Titanic, labeling = labeling_cells())
mosaic(Titanic, labeling = labeling_cells(abbreviate_labels = TRUE))
mosaic(Titanic, labeling = labeling_cells(abbreviate_varnames = TRUE))
mosaic(Titanic, labeling = labeling_cells(abbreviate_varnames = TRUE, abbreviate_labels = TRUE))
mosaic(Titanic, labeling = labeling_border(abbreviate = TRUE))
mosaic(Titanic, labeling = labeling_border(abbreviate = c(Survived = TRUE)))
mosaic(Titanic, labeling = labeling_border(rot_labels = c(bottom = 45)))
mosaic(Titanic, labeling = labeling_border(tl_labels = TRUE))
mosaic(Titanic, labeling = labeling_border(tl_labels = TRUE, tl_varnames = FALSE))
mosaic(Titanic, labeling = labeling_border(tl_labels = TRUE, tl_varnames = c(TRUE,TRUE,FALSE,FALSE), boxes = TRUE))
mosaic(Titanic, labeling = labeling_cboxed())
mosaic(Titanic, labeling = labeling_lboxed())
mosaic(Titanic, labeling = labeling_left())
mosaic(Titanic, labeling = labeling_list(), mar = c(2,2,4,2))
mosaic(Titanic, labeling = labeling_border(rep = FALSE))
mosaic(Titanic, labeling = labeling_border(labbl_varnames = c(TRUE,TRUE,FALSE,FALSE)))
mosaic(~ Gender + Admit | Dept, data = UCBAdmissions, labeling = labeling_conditional(labels_varnames = TRUE, varnames = FALSE), keep_aspect_ratio = FALSE, split_vertical = c(Dept = TRUE))
doubledecker(Titanic)
assoc(Hair ~ Eye, data = HairEyeColor)
assoc(Hair ~ Eye, data = HairEyeColor, compress = FALSE)
assoc(HairEyeColor, labeling = labeling_lboxed())
pairs(Titanic, shade = TRUE)
pairs(Titanic, panel_upper = pairs_assoc, shade = TRUE)

######################################################
#### ternary plot demo
#### Task: plotting data point hulls in a ternary plot
#### data provided by Manuel Dominguez-Rodrigo
######################################################

library(vcd)

## data

humans=matrix(c(18,19,17,21,7,9,8,62,70,53,69,81,73,71,20,10,30,10,12,18,19),
ncol=3)
colnames(humans)=c("young", "adult", "old")
lions=matrix(c(41,59,62,49,45,21,12,5,11,13,38,29,33,40,42), ncol=3)
colnames(lions)=c("young", "adult", "old")
site=matrix(c(9,12,15,11,70,62,69,68,21,26,16,21), ncol=3)
colnames(site)=c("young", "adult", "old")
humans=matrix(c(18,19,17,21,7,9,8,62,70,53,69,81,73,71,20,10,30,10,12,18,19),
ncol=3)

## regular ternary plot

data = rbind(humans, lions, site)
count = c(nrow(humans), nrow(lions), nrow(site))
rownames(data) = rep(c("humans", "lions", "site"), count)
cols = rep(c("red", "green", "blue"), count)

ternaryplot(data, col = cols)

## now try to draw hull

prop2xy <- function(x) {
  x <- as.matrix(x)
  x <- x / rowSums(x)
  xp <- x[,2] + x[,3] / 2
  yp <- x[,3] * sqrt(3) / 2
  cbind(x = xp, y = yp)
}

hullpoints <- function(x) {
    ind <- chull(x)
    ind <- c(ind, ind[1])
    x[ind,]
}

drawhull <- function(data, color) {
    hp <- hullpoints(prop2xy(data))
    grid.lines(hp[,"x"], hp[,"y"], gp = gpar(col = color))
}

## setup plot region without data points
ternaryplot(data, col = NA, pop = FALSE)

## grab plot viewport
downViewport("plot")

## now plot hulls
drawhull(humans, "blue")
drawhull(site, "red")
drawhull(lions, "green")

  #################################################
  ## Fitting and Graphing Discrete Distributions ##
  #################################################

  data(HorseKicks)
  barplot(HorseKicks, col = 2,
          xlab = "Number of Deaths", ylab = "Number of Corps-Years",
          main = "Deaths by Horse Kicks")

  data(Federalist)
  barplot(Federalist, col = 2,
          xlab = "Occurrences of 'may'", ylab = "Number of Blocks of Text",
          main = "'may' in Federalist papers")

  data(WomenQueue)
  barplot(WomenQueue, col = 2,
          xlab = "Number of women", ylab = "Number of queues",
          main = "Women in queues of length 10")

  data(WeldonDice)
  barplot(WeldonDice, names = c(names(WeldonDice)[-11], "10+"), col = 2,
          xlab = "Number of 5s and 6s", ylab = "Frequency",
	  main = "Weldon's dice data")

  data(Butterfly)
  barplot(Butterfly, col = 2,
          xlab = "Number of individuals", ylab = "Number of Species",
  	main = "Butterfly species im Malaya")


  ############################
  ## Binomial distributions ##
  ############################

  par(mfrow = c(1,2))
  barplot(dbinom(0:10, p = 0.15, size = 10), names = 0:10, col = grey(0.7),
          main = "p = 0.15", ylim = c(0,0.35))
  barplot(dbinom(0:10, p = 0.35, size = 10), names = 0:10, col = grey(0.7),
          main = "p = 0.35", ylim = c(0,0.35))
  par(mfrow = c(1,1))
  mtext("Binomial distributions", line = 2, cex = 1.5)

  plot(0:10, dbinom(0:10, p = 0.15, size = 10), type = "b", ylab = "Density",
       ylim = c(0, 0.4), main = "Binomial distributions, N = 10", pch = 19)
  lines(0:10, dbinom(0:10, p = 0.35, size = 10), type = "b", col = 2, pch = 19)
  lines(0:10, dbinom(0:10, p = 0.55, size = 10), type = "b", col = 4, pch = 19)
  lines(0:10, dbinom(0:10, p = 0.75, size = 10), type = "b", col = 3, pch = 19)
  legend(3, 0.4, c("p", "0.15", "0.35", "0.55", "0.75"), lty = rep(1,5), col =
         c(0,1,2,4,3), bty = "n")

  ###########################
  ## Poisson distributions ##
  ###########################

  par(mfrow = c(1,2))
  dummy <- barplot(dpois(0:12, 2), names = 0:12, col = grey(0.7), ylim = c(0,0.3),
        main = expression(lambda == 2))
  abline(v = dummy[3], col = 2)
  diff <- (dummy[3] - dummy[2]) * sqrt(2)/2
  lines(c(dummy[3] - diff, dummy[3] + diff), c(0.3, 0.3), col = 2)
  dummy <- barplot(dpois(0:12, 5), names = 0:12, col = grey(0.7), ylim = c(0,0.3),
          main = expression(lambda == 5))
  abline(v = dummy[6], col = 2)
  diff <- (dummy[6] - dummy[5]) * sqrt(5)/2
  lines(c(dummy[6] - diff, dummy[6] + diff), c(0.3, 0.3), col = 2)
  par(mfrow = c(1,1))
  mtext("Poisson distributions", line = 2, cex = 1.5)

  #####################################
  ## Negative binomial distributions ##
  #####################################

  nbplot <- function(p = 0.2, size = 2, ylim = c(0, 0.2))
  {
    plot(0:20, dnbinom(0:20, p = p, size = size), type = "h", col = grey(0.7),
         xlab = "Number of failures (k)", ylab = "Density", ylim = ylim,
         yaxs = "i", bty = "L")
    nb.mean <- size * (1-p)/p
    nb.sd <- sqrt(nb.mean/p)
    abline(v = nb.mean, col = 2)
    lines(nb.mean + c(-nb.sd, nb.sd), c(0.01, 0.01), col = 2)
    legend(14, 0.2, c(paste("p = ", p), paste("n = ", size)), bty = "n")
  }
  par(mfrow = c(3,2))
  nbplot()
  nbplot(size = 4)
  nbplot(p = 0.3)
  nbplot(p = 0.3, size = 4)
  nbplot(p = 0.4, size = 2)
  nbplot(p = 0.4, size = 4)
  par(mfrow = c(1,1))
  mtext("Negative binomial distributions for the number of trials to observe n = 2 or n = 4 successes", line = 3)

  #####################
  ## Goodness of fit ##
  #####################

  p <- weighted.mean(as.numeric(names(HorseKicks)), HorseKicks)
  p.hat <- dpois(0:4, p)
  expected <- sum(HorseKicks) * p.hat
  chi2 <- sum((HorseKicks - expected)^2/expected)
  pchisq(chi2, df = 3, lower = FALSE)
  ## or:
  HK.fit <- goodfit(HorseKicks)
  summary(HK.fit)

  ## Are the dice fair?
  p.hyp <- 1/3
  p.hat <- dbinom(0:12, prob = p.hyp, size = 12)
  expected <- sum(WeldonDice) * p.hat
  expected <- c(expected[1:10], sum(expected[11:13]))
  chi2 <- sum((WeldonDice - expected)^2/expected)
  G2 <- 2*sum(WeldonDice*log(WeldonDice/expected))
  pchisq(chi2, df = 10, lower = FALSE)
  ## Are the data from a binomial distribution?
  p <- weighted.mean(as.numeric(names(WeldonDice))/12, WeldonDice)
  p.hat <- dbinom(0:12, prob = p, size = 12)
  expected <- sum(WeldonDice) * p.hat
  expected <- c(expected[1:10], sum(expected[11:13]))
  chi2 <- sum((WeldonDice - expected)^2/expected)
  G2 <- 2*sum(WeldonDice*log(WeldonDice/expected))
  pchisq(chi2, df = 9, lower = FALSE)
  ## or:
  WD.fit1 <- goodfit(WeldonDice, type = "binomial", par = list(prob = 1/3, size = 12))
  WD.fit1$fitted[11] <- sum(predict(WD.fit1, newcount = 10:12))
  WD.fit2 <- goodfit(WeldonDice, type = "binomial", par = list(size = 12), method = "MinChisq")
  summary(WD.fit1)
  summary(WD.fit2)

  F.fit1 <- goodfit(Federalist)
  F.fit2 <- goodfit(Federalist, type = "nbinomial")
  summary(F.fit1)
  par(mfrow = c(2,2))
  plot(F.fit1, scale = "raw", type = "standing")
  plot(F.fit1, type = "standing")
  plot(F.fit1)
  plot(F.fit1, type = "deviation")
  par(mfrow = c(1,1))
  plot(F.fit2, type = "deviation")
  summary(F.fit2)

  data(Saxony)
  S.fit <- goodfit(Saxony, type = "binomial", par = list(size = 12))
  summary(S.fit)
  plot(S.fit)

  ###############
  ## Ord plots ##
  ###############

  par(mfrow = c(2,2))
  Ord_plot(HorseKicks, main = "Death by horse kicks")
  Ord_plot(Federalist, main = "Instances of 'may' in Federalist papers")
  Ord_plot(Butterfly, main = "Butterfly species collected in Malaya")
  Ord_plot(WomenQueue, main = "Women in queues of length 10")
  par(mfrow = c(1,1))

  ###############
  ## Distplots ##
  ###############

  distplot(HorseKicks, type = "poisson")
  distplot(HorseKicks, type = "poisson", lambda = 0.61)
  distplot(Federalist, type = "poisson")
  distplot(Federalist, type = "nbinomial")
  distplot(Saxony, type = "binomial", size = 12)

library(vcd)

## shape
foo1 <- c(3, 7, 3, 1.5)
foo2 <- c(2, 6.5, 1.5)
foo <- outer(foo1/sum(foo1), foo2/sum(foo2), "*")

## color
mondrian <- rep("#EAE6E3", 12)
mondrian[1] <- "#DE1024"
mondrian[3] <- "#FFD83B"
mondrian[12] <- "#032349"

## plot
## best visualized with resized display, e.g. using:
## get(getOption("device"))(width = 4.9, height = 7.5)
grid.newpage()
grid.rect(gp = gpar(fill = 1))

mondrianMosaic <- function(x, fill)
  mosaic(x, gp = gpar(col = rep(0, length(fill)), fill = fill),
         legend = FALSE, margins = 0,  newpage = FALSE, keep_aspect_ratio = FALSE)

mondrianMosaic(foo, mondrian)

#################################################################333
## assocplot

assoc <- function(x, ...)
    UseMethod("assoc")

assoc.formula <-
    function(formula, data = NULL, ..., subset = NULL, na.action = NULL,
             main = NULL, sub = NULL)
{
    if (is.logical(main) && main)
        main <- deparse(substitute(data))
    else if (is.logical(sub) && sub)
        sub <- deparse(substitute(data))

    assoc.default(structable(formula, data, subset = subset,
                             na.action = na.action),
                  main = main, sub = sub, ...)
}

assoc.default <- function(x,
                          row_vars = NULL, col_vars = NULL,
                          compress = TRUE, xlim = NULL, ylim = NULL,
                          spacing = spacing_conditional(sp = 0),
                          spacing_args = list(),
                          split_vertical = NULL,
                          keep_aspect_ratio = FALSE,
                          xscale = 0.9, yspace = unit(0.5, "lines"),
                          main = NULL,
                          sub = NULL,
                          ...,
			  residuals_type = "Pearson",
                          gp_axis = gpar(lty = 3)
                          ) {

    if (is.logical(main) && main)
        main <- deparse(substitute(x))
    else if (is.logical(sub) && sub)
        sub <- deparse(substitute(x))

    if (!inherits(x, "ftable"))
        x <- structable(x)

    tab <- as.table(x)
    dl <- length(dim(tab))

    ## spacing
    cond <- rep(TRUE, dl)
    cond[length(attr(x, "row.vars")) + c(0, length(attr(x, "col.vars")))] <- FALSE
    if (inherits(spacing, "grapcon_generator"))
        spacing <- do.call("spacing", spacing_args)
    spacing <- spacing(dim(tab), condvars = which(cond))

    ## splitting arguments
    if (is.null(split_vertical))
        split_vertical <- attr(x, "split_vertical")

    if(match.arg(tolower(residuals_type), "pearson") != "pearson")
        warning("Only Pearson residuals can be visualized with association plots.")

    strucplot(tab,
              spacing = spacing,
              split_vertical = split_vertical,
              core = struc_assoc(compress = compress, xlim = xlim, ylim = ylim,
              yspace = yspace, xscale = xscale, gp_axis = gp_axis),
              keep_aspect_ratio = keep_aspect_ratio,
              residuals_type = "Pearson",
              main = main,
              sub = sub,
              ...)
}

## old code: more elegant conceptually, but less performant
##
## struc_assoc2 <- function(compress = TRUE, xlim = NULL, ylim = NULL,
##                         yspace = unit(0.5, "lines"), xscale = 0.9,
##                         gp_axis = gpar(lty = 3))
##   function(residuals, observed = NULL, expected, spacing, gp, split_vertical, prefix = "") {
##     dn <- dimnames(expected)
##     dnn <- names(dn)
##     dx <- dim(expected)
##     dl <- length(dx)

##     ## axis limits
##     resid <- structable(residuals, split_vertical = split_vertical)
##     sexpected <- structable(sqrt(expected), split_vertical = split_vertical)
##     rfunc <- function(x) c(min(x, 0), max(x, 0))
##     if (is.null(ylim))
##       ylim <- if (compress)
##         matrix(apply(as.matrix(resid), 1, rfunc), nrow = 2)
##       else
##         rfunc(as.matrix(resid))
##     if (!is.matrix(ylim))
##       ylim <- matrix(as.matrix(ylim), nrow = 2, ncol = nrow(as.matrix(resid)))

##     attr(ylim, "split_vertical") <- rep(TRUE, sum(!split_vertical))
##     attr(ylim, "dnames") <- dn[!split_vertical]
##     class(ylim) <- "structable"

##     if(is.null(xlim))
##       xlim <- if (compress)
##         matrix(c(-0.5, 0.5) %o% apply(as.matrix(sexpected), 2, max), nrow = 2)
##       else
##         c(-0.5, 0.5) * max(sexpected)
##     if (!is.matrix(xlim))
##       xlim <- matrix(as.matrix(xlim), nrow = 2, ncol = ncol(as.matrix(resid)))
##     attr(xlim, "split_vertical") <- rep(TRUE, sum(split_vertical))
##     attr(xlim, "dnames") <- dn[split_vertical]
##     class(xlim) <- "structable"

##     ## split workhorse
##     split <- function(res, sexp, i, name, row, col) {
##       v <- split_vertical[i]
##       splitbase <- if (v) sexp else res
##       splittab <- lapply(seq(dx[i]), function(j) splitbase[[j]])
##       len <- sapply(splittab, function(x) sum(unclass(x)[1,] - unclass(x)[2,]))

##       d <- dx[i]

##       ## compute total cols/rows and build split layout
##       dist <- unit.c(unit(len, "null"), spacing[[i]]  + (1 * !v) * yspace)
##       idx <- matrix(1:(2 * d), nrow = 2, byrow = TRUE)[-2 * d]
##       layout <- if (v)
##         grid.layout(ncol = 2 * d - 1, widths = dist[idx])
##       else
##         grid.layout(nrow = 2 * d - 1, heights = dist[idx])
##       vproot <- viewport(layout.pos.col = col, layout.pos.row = row,
##                          layout = layout, name = remove_trailing_comma(name))

##       ## next level: either create further splits, or final viewports
##       name <- paste(name, dnn[i], "=", dn[[i]], ",", sep = "")
##       rows <- cols <- rep.int(1, d)
##       if (v) cols <- 2 * 1:d - 1 else rows <- 2 * 1:d - 1

##       f <- if (i < dl) {
##         if (v)
##           function(m) split(res, splittab[[m]], i + 1, name[m], rows[m], cols[m])
##         else
##           function(m) split(splittab[[m]], sexp, i + 1, name[m], rows[m], cols[m])
##       } else {
##         if (v)
##           function(m) viewport(layout.pos.col = cols[m], layout.pos.row = rows[m],
##                                name = remove_trailing_comma(name[m]),
##                                yscale = unclass(res)[,1],
##                                xscale = unclass(sexp)[,m], default.units = "null")
##         else
##           function(m) viewport(layout.pos.col = cols[m], layout.pos.row = rows[m],
##                                name = remove_trailing_comma(name[m]),
##                                yscale = unclass(res)[,m],
##                                xscale = unclass(sexp)[,1], default.units = "null")
##       }
##       vpleaves <- structure(lapply(1:d, f), class = c("vpList", "viewport"))

##       vpTree(vproot, vpleaves)
##     }

##     ## start spltting on top, creates viewport-tree
##     pushViewport(split(ylim, xlim, i = 1, name = paste(prefix, "cell:", sep = ""),
##                        row = 1, col = 1))

##     ## draw tiles
##     mnames <- paste(apply(expand.grid(dn), 1,
##                           function(i) paste(dnn, i, collapse = ",", sep = "=")
##                           )
##                     )
##     for (i in seq_along(mnames)) {
##       seekViewport(paste(prefix, "cell:", mnames[i], sep = ""))
##       grid.lines(y = unit(0, "native"), gp = gp_axis)
##       grid.rect(y = 0, x = 0,
##                 height = residuals[i],
##                 width = xscale * unit(sqrt(expected[i]), "native"),
##                 default.units = "native",
##                 gp = structure(lapply(gp, function(x) x[i]), class = "gpar"),
##                 just = c("center", "bottom"),
##                 name = paste(prefix, "rect:", mnames[i], sep = "")
##                 )
##     }

##   }
## class(struc_assoc2) <- "grapcon_generator"


struc_assoc <- function(compress = TRUE, xlim = NULL, ylim = NULL,
                        yspace = unit(0.5, "lines"), xscale = 0.9,
                        gp_axis = gpar(lty = 3))
    function(residuals, observed = NULL, expected, spacing,
             gp, split_vertical, prefix = "") {
        if(is.null(expected)) stop("Need expected values.")
        dn <- dimnames(expected)
        dnn <- names(dn)
        dx <- dim(expected)
        dl <- length(dx)
        if (is.null(dnn))
            dnn <- names(dn) <- names(dimnames(expected)) <- LETTERS[1:dl]

        ## axis limits
        resid <- structable(residuals, split_vertical = split_vertical)
        sexpected <- structable(sqrt(expected), split_vertical = split_vertical)
        rfunc <- function(x) c(min(x, 0), max(x, 0))
        if (is.null(ylim))
            ylim <- if (compress)
                matrix(apply(as.matrix(resid), 1, rfunc), nrow = 2)
            else
                rfunc(as.matrix(resid))
        if (!is.matrix(ylim))
            ylim <- matrix(as.matrix(ylim), nrow = 2, ncol = nrow(as.matrix(resid)))
        ylim[2,] <- ylim[2,] + .Machine$double.eps

        attr(ylim, "split_vertical") <- rep(TRUE, sum(!split_vertical))
        attr(ylim, "dnames") <- dn[!split_vertical]
        class(ylim) <- "structable"

        if(is.null(xlim))
            xlim <- if (compress)
                matrix(c(-0.5, 0.5) %o% apply(as.matrix(sexpected), 2, max), nrow = 2)
            else
                c(-0.5, 0.5) * max(sexpected)
        if (!is.matrix(xlim))
            xlim <- matrix(as.matrix(xlim), nrow = 2, ncol = ncol(as.matrix(resid)))
        attr(xlim, "split_vertical") <- rep(TRUE, sum(split_vertical))
        attr(xlim, "dnames") <- dn[split_vertical]
        class(xlim) <- "structable"

        ## split workhorse
        split <- function(res, sexp, i, name, row, col, index) {
            v <- split_vertical[i]
            d <- dx[i]

            splitbase <- if (v) sexp else res
            splittab <- lapply(seq(d), function(j) splitbase[[j]])
            len <- abs(sapply(splittab, function(x) sum(unclass(x)[1,] - unclass(x)[2,])))

            ## compute total cols/rows and build split layout
            dist <- if (d > 1)
                unit.c(unit(len, "null"), spacing[[i]]  + (1 * !v) * yspace)
            else
                unit(len, "null")
            idx <- matrix(1:(2 * d), nrow = 2, byrow = TRUE)[-2 * d]
            layout <- if (v)
                grid.layout(ncol = 2 * d - 1, widths = dist[idx])
            else
                grid.layout(nrow = 2 * d - 1, heights = dist[idx])
            pushViewport(viewport(layout.pos.col = col, layout.pos.row = row,
                                  layout = layout, name = paste(prefix, "cell:",
                                                   remove_trailing_comma(name), sep = "")))

            ## next level: either create further splits, or final viewports
            rows <- cols <- rep.int(1, d)
            if (v) cols <- 2 * 1:d - 1 else rows <- 2 * 1:d - 1

            for (m in 1:d) {
                nametmp <- paste(name, dnn[i], "=", dn[[i]][m], ",", sep = "")
                if (i < dl) {
                    if (v) sexp <- splittab[[m]] else res <- splittab[[m]]
                    split(res, sexp, i + 1, nametmp, rows[m], cols[m], cbind(index, m))
                } else {
                    pushViewport(viewport(layout.pos.col = cols[m], layout.pos.row = rows[m],
                                          name = paste(prefix, "cell:",
                                          remove_trailing_comma(nametmp), sep = ""),
                                          yscale = unclass(res)[,if (v) 1 else m],
                                          xscale = unclass(sexp)[,if (v) m else 1],
                                          default.units = "npc")
                                 )

                    ## draw tiles
                    grid.lines(y = unit(0, "native"), gp = gp_axis)
                    grid.rect(y = 0, x = 0,
                              height = residuals[cbind(index, m)],
                              width = xscale * unit(sqrt(expected[cbind(index, m)]), "native"),
                              default.units = "native",
                              gp = structure(lapply(gp, function(x) x[cbind(index,m)]),
                              class = "gpar"),
                              just = c("center", "bottom"),
                              name = paste(prefix, "rect:", remove_trailing_comma(nametmp), sep = "")
                              )
                }
                upViewport(1)
            }
        }
        split(ylim, xlim, i = 1, name = "", row = 1, col = 1, index = cbind())
    }
class(struc_assoc) <- "grapcon_generator"

################################################################
## labeling

pexpand <- function(par, len, default_value, default_names, choices = NULL) {
  if (is.null(par))
      par <- default_value
  nam <- names(par)
  if (!is.null(choices))
      par <- sapply(par, match.arg, choices)
  if (is.null(nam)) {
      default_value <- par
      par <- rep(par, length.out = len)
      nam <- names(par) <- default_names
  } else if (length(nam[nam == ""])) {
      default_value <- par[nam == ""]
      nam <- nam[nam != ""]
  }
  ret <- rep(default_value, length.out = len)
  if (!is.null(nam)) {
      names(ret) <- default_names
      ret[nam] <- par[nam]
  }
  ret
}

labeling_list <- function(gp_text = gpar(),
                          just = "left",
                          pos = "left",
                          lsep = ": ", sep = " ",
                          offset = unit(c(2, 2), "lines"),
                          varnames = TRUE,
                          cols = 2,
                          ...) {
  function(d, split_vertical, condvars, prefix = "") {
    if (is.table(d) || is.structable(d))
      d <- dimnames(d)
    ld <- length(d)
    labeling_border(labels = FALSE, varnames = varnames)(d, split_vertical, condvars, prefix)
    seekViewport(paste(prefix, "margin_bottom", sep = ""))
    pos <- unit(switch(pos, left = 0, center = 0.5, 1) / cols, "npc")
    ind <- split(seq(ld), rep.int(seq(cols), ceiling(ld / cols))[seq(ld)])

    for (i in seq_along(ind))
      grid.text(x = offset[1] + pos + unit((i - 1) / cols, "npc"),
                y = unit(1, "npc") - offset[2],
                paste(names(d[ind[[i]]]),
                      sapply(d[ind[[i]]], paste, collapse = sep),
                      sep = lsep,
                      collapse = "\n"
                      ),
                just = c(just, "top"),
                gp = gp_text
                )
  }
}
class(labeling_list) <- "grapcon_generator"

labeling_conditional <- function(...) {
  function (d, split_vertical, condvars, prefix = "") {
    if (is.table(d) || is.structable(d))
      d <- dimnames(d)
    v <- rep.int(TRUE, length(d))
    v[seq(condvars)] <- FALSE
    labeling_border(labels = !v, ...)(d, split_vertical, condvars, prefix)
    labeling_cells(labels = v, ...)(d, split_vertical, condvars, prefix)
  }
}
class(labeling_conditional) <- "grapcon_generator"

labeling_cells <- function(labels = TRUE, varnames = TRUE,
                         abbreviate_labels = FALSE, abbreviate_varnames = FALSE,
                         gp_text = gpar(), lsep = ": ", lcollapse = "\n",
                         just = "center", pos = "center", rot = 0,
                         margin = unit(0.5, "lines"), clip_cells = TRUE,
                         text = NULL, ...) {
  function(d, split_vertical, condvars, prefix = "") {
    if (is.table(d) || is.structable(d))
      d <- dimnames(d)
    dn <- names(d)
    ld <- length(d)

    ## expand parameters
    if (length(pos) < 2) pos <- c(pos, pos)
    labels <- pexpand(labels, ld, TRUE, dn)
    varnames <- pexpand(varnames, ld, TRUE, dn)
    abbreviate_labels <- pexpand(abbreviate_labels, ld, FALSE, dn)
    abbreviate_varnames <- pexpand(abbreviate_varnames, ld, FALSE, dn)

    ## margin
    if (!is.unit(margin))
      margin <- unit(margin, "lines")

    prvars <- ifelse(abbreviate_varnames,
                     sapply(seq_along(dn),
                            function(i) abbreviate(dn[i], abbreviate_varnames[i])),
                     dn)
    prvars <- ifelse(varnames, paste(prvars, lsep, sep = ""), "")

    if (is.structable(text))
        text <- as.table(text)

    ## draw labels
    split <- function(vind = 1, labs = c()) {
      n <- d[[vind]]
      for (labind in seq_along(n)) {
        lab <- c(labs, n[labind])
        names(lab) <- names(d)[1:vind]
        mlab <- paste(prefix, "cell:", paste(dn[1:vind], lab, sep = "=", collapse = ","),
                      sep = "")

        if (vind < ld)
          split(vind + 1, lab)
        else {
          seekViewport(mlab)
          pushViewport(viewport(width = max(unit(0, "npc"), unit(1, "npc") - 2 * margin),
                                height = unit(1, "npc") - 2 * margin,
                                clip = clip_cells))
          txt <- if (!is.null(text)) {
            lab <- lab[names(dimnames(text))]
            do.call("[", c(list(text), as.list(lab)))
          } else {
            prlab <- ifelse(abbreviate_labels,
                            sapply(seq_along(lab),
                                   function(i) abbreviate(lab[i], abbreviate_labels[i])),
                            lab)
            prlab <- prlab[labels[1:ld]]
            paste(prvars[labels[1:ld]], prlab, sep = "", collapse = lcollapse)
          }

          grid.text(if(!is.na(txt)) txt,
                    x = switch(pos[1], left =, top = 0, center = 0.5, 1),
                    y = switch(pos[2], left =, top = 1, center = 0.5, 0),
                    gp = gp_text, just = just, rot = rot)
          popViewport()
        }
      }
    }
    split()
    seekViewport(paste(prefix, "base", sep = ""))
    upViewport(1)
  }
}
class(labeling_cells) <- "grapcon_generator"

labeling_border <- function(labels = TRUE, varnames = labels,
                            set_labels = NULL, set_varnames = NULL,
                            tl_labels = NULL, alternate_labels = FALSE,
                            tl_varnames = NULL,
                            gp_labels = gpar(fontsize = 12),
                            gp_varnames = gpar(fontsize = 12, fontface = 2),
                            rot_labels = c(0, 90, 0, 90),
                            rot_varnames = c(0, 90, 0, 90),
                            pos_labels = "center", pos_varnames = "center",
                            just_labels = "center", just_varnames = pos_varnames,
                            boxes = FALSE, fill_boxes = FALSE,
                            offset_labels = c(0, 0, 0, 0),
                            offset_varnames = offset_labels,

                            labbl_varnames = NULL,
                            labels_varnames = FALSE, sep = ": ",

                            abbreviate_labs = FALSE, rep = TRUE,
                            clip = FALSE, ...
                            ) {
    ## expand parameters that apply to the four table margins
    pos_labels <- pexpand(pos_labels, 4, "center", c("top", "right", "bottom", "left"),
                          c("left", "center", "right"))
    just_labels <- pexpand(just_labels, 4, "center", c("top", "right", "bottom", "left"),
                           c("left", "center", "right"))
    offset_varnames <- if (!is.unit(offset_varnames))
      unit(pexpand(offset_varnames, 4,
                   rep.int(0, 4), c("top","right","bottom","left")), "lines")
    else
      rep(offset_varnames, length.out = 4)

    offset_labels <- if (!is.unit(offset_labels))
      unit(pexpand(offset_labels, 4,
                   rep.int(0, 4), c("top","right","bottom","left")), "lines")
    else
      rep(offset_labels, length.out = 4)

    rot_labels <- pexpand(rot_labels, 4, c(0, 90, 0, 90),
                          c("top", "right", "bottom", "left"))

    if (inherits(gp_varnames, "gpar"))
      gp_varnames <- list(gp_varnames)
    gp_varnames <- pexpand(gp_varnames, 4, list(gpar(fontsize = 12, fontface = 2)),
                           c("top", "right", "bottom", "left"))

    rot_varnames <- pexpand(rot_varnames, 4, c(0, 90, 0, 90),
                          c("top", "right", "bottom", "left"))

    pos_varnames <- pexpand(pos_varnames, 4, "center",
                           c("top", "right", "bottom", "left"),
                            c("left", "center", "right"))

    just_varnames <- pexpand(just_varnames, 4, pos_varnames,
                             c("top", "right", "bottom", "left"),
                             c("left", "center", "right"))

  function(d, split_vertical, condvars, prefix = "") {
    if (is.table(d) || is.structable(d))
      d <- dimnames(d)
    dn <- names(d)
    ld <- length(d)

    ## expand table- (i.e., dimensionality)-dependent parameters
    clip <- pexpand(clip, ld, TRUE, dn)
    labels <- pexpand(labels, ld, TRUE, dn)
    labels_varnames <- pexpand(labels_varnames, ld, FALSE, dn)

    ## tl_labels
    def <- logical()
    def[split_vertical] <- rep(c(TRUE, FALSE), length.out = sum(split_vertical))
    def[!split_vertical] <- rep(c(TRUE, FALSE), length.out = sum(!split_vertical))
    tl_labels <- if (is.null(tl_labels))
      def
    else
      pexpand(tl_labels, ld, def, dn)

    ## rep labels
    rep <- pexpand(rep, ld, TRUE, dn)
    printed <- lapply(d, function(i) rep.int(FALSE, length(i)))

    ## alternate labels
    alternate_labels <- pexpand(alternate_labels, ld, FALSE, dn)

    ## abbreviate
    abbreviate_labs <- pexpand(abbreviate_labs, ld, FALSE, dn)
    labs <- d
    for (i in seq_along(d))
      if (abbreviate_labs[i])
        labs[[i]] <- abbreviate(labs[[i]], abbreviate_labs[i])

    ## gp_labels
    if (inherits(gp_labels, "gpar"))
      gp_labels <- list(gp_labels)
    gp_labels <- pexpand(gp_labels, ld, list(gpar(fontsize = 12)), dn)

    ## varnames
    varnames <- pexpand(varnames, ld, labels, dn)

    ## tl_varnames
    if (is.null(tl_varnames) && is.null(labbl_varnames))
      tl_varnames <- tl_labels
    tl_varnames <- pexpand(tl_varnames, ld, tl_labels, dn)

    ## labbl_varnames
    if (!is.null(labbl_varnames))
      labbl_varnames <- pexpand(labbl_varnames, ld, TRUE, dn)

    ## boxes
    boxes <- pexpand(boxes, ld, FALSE, dn)

    ## fill_boxes
    dnl <- sapply(d, length)
    fill_boxes <- if (is.atomic(fill_boxes)) {
      fill_boxes <- if (is.logical(fill_boxes))
        ifelse(pexpand(fill_boxes, ld, FALSE, dn), "grey", NA)
      else
        pexpand(fill_boxes, ld, "grey", dn)
      col <- rgb2hsv(col2rgb(fill_boxes))
      lapply(seq(along.with = dnl),
             function(i) if (is.na(fill_boxes[i])) "white" else
                         hsv(h = col["h",i],
                             s = col["s",i],
                             v = seq(from = col["v",i],
                                     to = 0.5 * col["v",i],
                                     length.out = dnl[i])
                             )
             )
    } else {
      fill_boxes <- pexpand(fill_boxes, ld, "white", dn)
      lapply(seq(ld),
             function(i) pexpand(fill_boxes[[i]], dnl[i], "white", d[[i]])
             )
    }


    ## precompute spaces
    lsp <- tsp <- bsp <- rsp <- 0
    labsp <- rep.int(0, ld)
    for (i in seq_along(dn)[tl_labels & labels])
      labsp[i] <- if (split_vertical[i]) {
        if (alternate_labels[i]) bsp <- bsp - 1
        tsp <- tsp + 1
      } else {
        if (alternate_labels[i]) rsp <- rsp + 1
        lsp <- lsp - 1
      }
    for (i in rev(seq_along(dn)[!tl_labels & labels]))
      labsp[i] <- if (split_vertical[i]) {
        if (alternate_labels[i]) tsp <- tsp + 1
        bsp <- bsp - 1
      } else {
        if (alternate_labels[i]) lsp <- lsp - 1
        rsp <- rsp + 1
      }

    if(is.null(labbl_varnames)) {
    ## varnames in the outer margin
      ## compute axis names
      tt <- bt <- lt <- rt <- ""
      for (i in seq_along(dn)) {
        var <- if (!is.null(set_varnames) && !is.na(set_varnames[dn[i]]))
          set_varnames[dn[i]]
        else
          dn[i]
        if (varnames[i]) {
          if (split_vertical[i]) {
            if (tl_varnames[i])
              tt <- paste(tt, var, sep = if (tt == "") "" else " / ")
            else
              bt <- paste(bt, var, sep = if (bt == "") "" else " / ")
          } else {
            if (tl_varnames[i])
              lt <- paste(lt, var, sep = if (lt == "") "" else " / ")
            else
              rt <- paste(rt, var, sep = if (rt == "") "" else " / ")
          }
        }
      }
      ## draw axis names
      if (tt != "")
        grid.text(tt, y = unit(1, "npc") + unit(tsp + 1, "lines") + offset_varnames[1],
                  x = switch(pos_varnames[1], left =, bottom = 0, center =, centre = 0.5, 1),
                  rot = rot_varnames[1], just = just_varnames[1], gp = gp_varnames[[1]])
      if (bt != "")
        grid.text(bt, y = unit(bsp - 1, "lines") + -1 * offset_varnames[3],
                  x = switch(pos_varnames[3], left =, bottom = 0, center =, centre = 0.5, 1),
                  rot = rot_varnames[3], just = just_varnames[3], gp = gp_varnames[[3]])
      if (lt != "")
        grid.text(lt, x = unit(lsp - 1, "lines") + -1 * offset_varnames[4],
                  y = switch(pos_varnames[4], left =, bottom = 0, center =, centre = 0.5, 1),
                  rot = rot_varnames[4], just = just_varnames[4], gp = gp_varnames[[4]])
      if (rt != "")
        grid.text(rt, x = unit(1, "npc") + unit(rsp + 1, "lines") + offset_varnames[2],
                  y = switch(pos_varnames[2], left =, bottom = 0, center =, centre = 0.5, 1),
                  rot = rot_varnames[2], just = just_varnames[2], gp = gp_varnames[[2]])
    } else {
    ## varnames beneath labels
      for (i in seq_along(dn)) {
        var <- if (!is.null(set_varnames) && !is.na(set_varnames[dn[i]]))
          set_varnames[dn[i]]
        else
          dn[i]
        if (varnames[i]) {
          if (split_vertical[i]) {
            if (tl_labels[i]) {
              if (labbl_varnames[i]) {
                grid.text(var,
                          y = unit(1, "npc") + unit(1 + tsp - labsp[i], "lines") + offset_varnames[1],
                          x = unit(-0.5, "lines"),
                          just = "right", gp = gp_varnames[[4]])
              } else {
                grid.text(var, y = unit(1, "npc") + unit(1 + tsp - labsp[i], "lines") + offset_varnames[1],
                          x = unit(1, "npc") + unit(0.5, "lines"),
                          just = "left", gp = gp_varnames[[2]])
              }
            } else {
              if (labbl_varnames[i]) {
                grid.text(var, y = unit(labsp[i], "lines") + -1 * offset_varnames[3],
                          x = unit(-0.5, "lines"), just = "right",
                          gp = gp_varnames[[4]])
              } else {
                grid.text(var, y = unit(labsp[i], "lines") + -1 * offset_varnames[3],
                          x = unit(1, "npc") + unit(0.5, "lines"),
                          just = "left", gp = gp_varnames[[2]])
              }
            }
          } else {
            if (tl_labels[i]) {
              if (labbl_varnames[i]) {
                grid.text(var, x = unit(lsp - 1 - labsp[i], "lines") + -1 * offset_varnames[4],
                          y = unit(-0.5, "lines"), just = "right", rot = 90,
                          gp = gp_varnames[[4]])
              } else {
                grid.text(var, x = unit(lsp - 1 - labsp[i], "lines") + -1 * offset_varnames[4],
                          y = unit(1, "npc") + unit(0.5, "lines"),
                          just = "left", rot = 90, gp = gp_varnames[[2]])
              }
            } else {
              if (labbl_varnames[i]) {
                grid.text(var, x = unit(1, "npc") + unit(labsp[i], "lines") + offset_varnames[2],
                          y = unit(-0.5, "lines"),
                          just = "right", rot = 90, gp = gp_varnames[[4]])
              } else {
                grid.text(var, x = unit(1, "npc") + unit(labsp[i], "lines") + offset_varnames[2],
                          y = unit(1, "npc") + unit(0.5, "lines"),
                          just = "left", rot = 90, gp = gp_varnames[[2]])
              }
            }
          }
        }
      }
    }

    ## draw labels
    split <- function(vind = 1, root = paste(prefix, "cell:", sep = ""),
                      left = TRUE, right = TRUE, top = TRUE, bottom = TRUE) {
      n <- d[[vind]]
      vl <- length(n)
      sp <- split_vertical[vind]
      labseq <- seq_along(n)
      if (!sp) labseq <- rev(labseq)

      for (labind in labseq) {
        mlab <- paste(root, dn[vind], "=", n[labind], sep = "")
        if (labels[vind] && (rep[vind] || !printed[[vind]][labind])) {
          lab <- if (!is.null(set_labels) && !is.null(set_labels[[dn[vind]]]))
            set_labels[[dn[vind]]][labind]
          else
            labs[[vind]][labind]
          if (labels_varnames[vind])
            lab <- if (!is.null(set_varnames) && !is.na(set_varnames[dn[vind]]))
              paste(set_varnames[dn[vind]], lab, sep = sep)
          else
              paste(dn[vind], lab, sep = sep)
          if (sp) {
            if (tl_labels[vind]) {
              if (top) {
                seekViewport(mlab)
                if (clip[vind])
                  pushViewport(viewport(height = unit(1, "npc") + 2 * offset_labels[1] +
                                        unit(2 * (2 + tsp - labsp[vind]), "lines"),
                                        clip = "on"))
                if (boxes[vind])
                  grid.rect(height = unit(0.8, "lines"),
                            y = unit(1, "npc") + offset_labels[1] +
                                unit(1 + tsp - labsp[vind] - (2 + as.numeric(offset_labels[1]) + tsp - labsp[vind]) * clip[vind], "lines"),
                            gp = gpar(fill = fill_boxes[[vind]][labind]))
                grid.text(lab,
                          y = unit(1, "npc") + offset_labels[1] +
                              unit(1 + tsp - labsp[vind] - (2 + as.numeric(offset_labels[1]) + tsp - labsp[vind]) * clip[vind], "lines"),
                          x = unit(0.15 * switch(pos_labels[1], left =, bottom = 1, center =, centre = 0, -1) * boxes[vind], "lines") +
                              unit(switch(pos_labels[1], left =, bottom = 0, center =, centre = 0.5, 1), "npc"),
                          rot = rot_labels[1], just = just_labels[1],
                          gp = gp_labels[[vind]])
                if (clip[vind]) popViewport()
                printed[[vind]][labind] <<- TRUE
              }
            } else {
              if (bottom) {
                seekViewport(mlab)
                if (clip[vind])
                  pushViewport(viewport(height = unit(1, "npc") + 2 * offset_labels[3] +
                                        unit(2 * (1 + abs(labsp[vind])), "lines"),
                                        clip = "on"))
###
                if (boxes[vind])
                  grid.rect(height = unit(0.8, "lines"),
                            y = -1 * offset_labels[3] + unit(labsp[vind] + (1 + as.numeric(offset_labels[3]) + abs(labsp[vind])) * clip[vind], "lines"),
                            gp = gpar(fill = fill_boxes[[vind]][labind]))

                grid.text(lab,
                          y = -1 * offset_labels[3] + unit(labsp[vind] + (1 + as.numeric(offset_labels[3]) + abs(labsp[vind])) * clip[vind], "lines"),
                          x = unit(0.15 * switch(pos_labels[3], left =, bottom = 1, center =, centre = 0, -1) * boxes[vind], "lines") +
                              unit(switch(pos_labels[3], left =, bottom = 0, center =, centre = 0.5, 1), "npc"),
                          rot = rot_labels[3], just = just_labels[3],
                          gp = gp_labels[[vind]])
                if (clip[vind]) popViewport()
                printed[[vind]][labind] <<- TRUE
              }
            }
          } else {
            if (tl_labels[vind]) {
              if (left) {
                seekViewport(mlab)
                if (clip[vind])
                  pushViewport(viewport(width = unit(1, "npc") + 2 * offset_labels[4] +
                                        unit(2 * (2 - lsp + labsp[vind]), "lines"),
                                        clip = "on"))
                if (boxes[vind])
                  grid.rect(width = unit(0.8, "lines"),
                            x = -1 * offset_labels[4] + unit(lsp - 1 - labsp[vind] + (2 - lsp + as.numeric(offset_labels[4]) + labsp[vind]) * clip[vind], "lines"),
                            gp = gpar(fill = fill_boxes[[vind]][labind]))

                grid.text(lab,
                          x = -1 * offset_labels[4] + unit(lsp - 1 - labsp[vind] + (2 - lsp + as.numeric(offset_labels[4]) + labsp[vind]) * clip[vind], "lines"),
                          y = unit(0.15 * switch(pos_labels[4], left =, bottom = 1, centre = 0, -1) * boxes[vind], "lines") +
                          unit(switch(pos_labels[4], left =, bottom = 0, center =, centre = 0.5, 1), "npc"),
                          rot = rot_labels[4], just = just_labels[4],
                          gp = gp_labels[[vind]])

                if (clip[vind]) popViewport()
                printed[[vind]][labind] <<- TRUE
              }
            } else {
              if (right) {
                seekViewport(mlab)
                if (clip[vind])
                  pushViewport(viewport(width = unit(1, "npc") + 2 * offset_labels[2] +
                                        unit(2 * (1 + abs(labsp[vind])), "lines"),
                                        clip = "on"))
                if (boxes[vind])
                  grid.rect(width = unit(0.8, "lines"),
                            x = offset_labels[2] + unit(1, "npc") +
                                unit(labsp[vind] - (1 + as.numeric(offset_labels[2]) + abs(labsp[vind])) * clip[vind], "lines"),
                            gp = gpar(fill = fill_boxes[[vind]][labind]))
                grid.text(lab,
                          x = offset_labels[2] + unit(1, "npc") + unit(0.1, "lines") +
                              unit(labsp[vind] - (1 + as.numeric(offset_labels[2]) + abs(labsp[vind])) * clip[vind], "lines"),
                          y = unit(0.15 * switch(pos_labels[2], left =, bottom = 1, center =, centre = 0, -1) * boxes[vind], "lines") +
                              unit(switch(pos_labels[2], left =, bottom = 0, center =, centre = 0.5, 1), "npc"),
                          rot = rot_labels[2], just = just_labels[2],
                          gp = gp_labels[[vind]])

                if (clip[vind]) popViewport()
                printed[[vind]][labind] <<- TRUE
              }
            }
          }
        }

        if (vind < ld) Recall(vind + 1, paste(mlab, ",", sep = ""),
                              if (sp) left && labind == 1 else left,
                              if (sp) right && labind == vl else right,
                              if (!sp) top && labind == 1 else top,
                              if (!sp) bottom && labind == vl else bottom)
      }
    }
    ## patch for alternating labels, part 1
    if (any(alternate_labels)) {
      ## save set_labels
      set_labels_hold <- set_labels

      ## create vanilla set_labels-object
      set_labels <- d

      ## copy old set_labels
      if (!is.null(set_labels_hold))
        set_labels[names(set_labels_hold)] <- set_labels_hold

      ## mask half of the labels
      for (i in which(alternate_labels))
        if (length(d[[i]]) > 1)
          set_labels[[i]][seq(2, length(d[[i]]), 2)] <- ""
    }

    split()

    ## patch for alternating labels, part 2
    if (any(alternate_labels)) {
      ## create again vanilla set_labels-object
      set_labels <- d

      ## copy again old set_labels
      if (!is.null(set_labels_hold))
        set_labels[names(set_labels_hold)] <- set_labels_hold

      ## clear all non-alternated labels
      labels[!alternate_labels] <- FALSE

      ## mask other half of alternated labels
      for (i in which(alternate_labels))
        set_labels[[i]][seq(1, length(d[[i]]), 2)] <- ""

      ## invert tl_labels and labsp
      tl_labels <- ! tl_labels
      labsp <- -labsp

      ## label again
      split()
    }
    seekViewport(paste(prefix, "base", sep = ""))
    upViewport(1)
  }
}
class(labeling_border) <- "grapcon_generator"

labeling_doubledecker <- function(lab_pos = c("bottom", "top"),
                                  dep_varname = TRUE,
                                  boxes = NULL,
                                  clip = NULL,
                                  labbl_varnames = FALSE,
                                  rot_labels = rep.int(0, 4),
                                  pos_labels = c("left", "center", "left", "center"),
                                  just_labels = c("left", "left", "left", "center"),
                                  varnames = NULL,
                                  gp_varnames = gpar(fontsize = 12, fontface = 2),
                                  offset_varnames = c(0, -0.6, 0, 0),
                                  tl_labels = NULL,
                                  ...) {
  lab_pos <- match.arg(lab_pos)

  if (inherits(gp_varnames, "gpar"))
      gp_varnames <- list(gp_varnames)
  gp_varnames <- pexpand(gp_varnames, 4, list(gpar(fontsize = 12, fontface = 2)), c("top", "right", "bottom", "left"))

  function(d, split_vertical, condvars, prefix = "") {
    if (is.table(d) || is.structable(d))
      d <- dimnames(d)
    ld <- length(d)
    dn <- names(d)

    ## expand dimension parameters
    boxes <- pexpand(boxes, ld, c(rep.int(TRUE, ld - 1), FALSE), dn)
    clip <- pexpand(clip, ld, c(rep.int(TRUE, ld - 1), FALSE), dn)
    varnames <- pexpand(varnames, ld, c(rep.int(TRUE, ld - 1), FALSE), dn)
    tl_labels <- pexpand(tl_labels, ld, c(rep.int(lab_pos == "top", ld - 1), FALSE), dn)
    if (!is.null(labbl_varnames))
        labbl_varnames <- pexpand(labbl_varnames, ld, FALSE, dn)

    ## expand side parameters
    rot_labels <- pexpand(rot_labels, 4, c(0, 0, 0, 0),
                          c("top", "right", "bottom", "left"))
    pos_labels <- pexpand(pos_labels, 4,
                          c("left", "center", "left", "center"),
                          c("top", "right", "bottom", "left"),
                          c("left", "center", "right"))
    just_labels <- pexpand(just_labels, 4,
                           c("left", "left", "left", "center"),
                           c("top", "right", "bottom", "left"),
                           c("left", "center", "right"))
    offset_varnames <- if (!is.unit(offset_varnames))
      unit(pexpand(offset_varnames, 4,
                   c(0, -0.6, 0, 0),
                   c("top","right","bottom","left")), "lines")
    else
      rep(offset_varnames, length.out = 4)

    labeling_border(boxes = boxes,
                    clip = clip,
                    labbl_varnames = labbl_varnames,
                    rot_labels = rot_labels,
                    pos_labels = pos_labels,
                    just_labels = just_labels,
                    varnames = varnames,
                    gp_varnames = gp_varnames,
                    offset_varnames = offset_varnames,
                    tl_labels = tl_labels,
                    ...
                    )(d, split_vertical, condvars, prefix)
    if (!(is.logical(dep_varname) && !dep_varname)) {
        if (is.null(dep_varname) || is.logical(dep_varname))
            dep_varname <- names(d)[length(d)]
        seekViewport(paste(prefix, "margin_right", sep = ""))
        grid.text(dep_varname,
                  x = unit(0.5, "lines"), y = unit(1, "npc"), just = c("left","top"),
                  gp = gp_varnames[[2]])
    }
  }
}
class(labeling_doubledecker) <- "grapcon_generator"

labeling_left <- function(rep = FALSE, pos_varnames = "left",
                        pos_labels = "left", just_labels = "left", ...)
  labeling_border(rep = rep, pos_varnames = pos_varnames,
              pos_labels = pos_labels, just_labels = just_labels, ...)
class(labeling_left) <- "grapcon_generator"

labeling_left2 <- function(tl_labels = TRUE, clip = TRUE, pos_varnames = "left",
                        pos_labels = "left", just_labels = "left", ...)
  labeling_border(tl_labels = tl_labels, clip = clip, pos_varnames = pos_varnames,
              pos_labels = pos_labels, just_labels = just_labels, ...)
class(labeling_left2) <- "grapcon_generator"

labeling_cboxed <- function(tl_labels = TRUE, boxes = TRUE, clip = TRUE, pos_labels = "center", ...)
  labeling_border(tl_labels = tl_labels, boxes = boxes, clip = clip, pos_labels = pos_labels, ...)
class(labeling_cboxed) <- "grapcon_generator"

labeling_lboxed <- function(tl_labels = FALSE, boxes = TRUE, clip = TRUE, pos_labels = "left", just_labels = "left", labbl_varnames = FALSE, ...)
  labeling_border(tl_labels = tl_labels, boxes = boxes, clip = clip, pos_labels = pos_labels, labbl_varnames = labbl_varnames, just_labels = just_labels, ...)
class(labeling_lboxed) <- "grapcon_generator"

labeling_values <-
function(value_type = c("observed", "expected", "residuals"),
         suppress = NULL, digits = 1, clip_cells = FALSE, ...)
{
   value_type <- match.arg(value_type)
   if (value_type == "residuals" && is.null(suppress))
       suppress <- 2
   if (is.null(suppress))
       suppress <- 0
   if (length(suppress) == 1)
       suppress <- c(-suppress, suppress)

   function(d, split_vertical, condvars, prefix) {
       lookup <- if (value_type == "observed") "x" else value_type
       if (!exists(lookup, envir = parent.frame()))
           stop(paste("Could not find", dQuote(value_type), "object."))

       values <- get(lookup, envir = parent.frame())
       values <- ifelse((values > suppress[2]) | (values < suppress[1]),
                        round(values, digits), NA)
       labeling_border(...)(d, split_vertical, condvars, prefix)
       labeling_cells(text = values, clip_cells = clip_cells, ...)(d, split_vertical, condvars, prefix)
   }
}
class(labeling_values) <- "grapcon_generator"

labeling_residuals <-
function(suppress = NULL, digits = 1, clip_cells = FALSE, ...)
    labeling_values(value_type = "residuals", suppress = suppress,
                    digits = digits, clip_cells = clip_cells, ...)
class(labeling_residuals) <- "grapcon_generator"

## convenience function for interfacing
## HCL colors as implemented in colorspace
hcl2hex <- function(h = 0, c = 35, l = 85, fixup = TRUE)
{
  colorspace::hex(polarLUV(l, c, h), fixup = fixup)
}

## shading-generating functions should take at least the arguments
##   observed, residuals, expected, df
## and return a function which takes a single argument (interpreted
## to be a vector of residuals).

shading_hsv <- function(observed, residuals = NULL, expected = NULL, df = NULL,
  h = c(2/3, 0), s = c(1, 0), v = c(1, 0.5),
  interpolate = c(2, 4), lty = 1, eps = NULL, line_col = "black",
  p.value = NULL, level = 0.95, ...)
{
  ## get h/s/v and lty
  my.h <- rep(h, length.out = 2L)  ## positive and negative hue
  my.s <- rep(s, length.out = 2L)  ## maximum and minimum saturation
  my.v <- rep(v, length.out = 2L)  ## significant and non-significant value
  lty <- rep(lty, length.out = 2L) ## positive and negative lty

  ## model fitting (if necessary)
  if(is.null(expected) && !is.null(residuals)) stop("residuals without expected values specified")
  if(!is.null(expected) && is.null(df) && is.null(p.value)) {
    warning("no default inference available without degrees of freedom")
    p.value <- NA
  }
  if(is.null(expected) && !is.null(observed)) {
    expected <- loglin(observed, 1:length(dim(observed)), fit = TRUE, print = FALSE)
    df <- expected$df
    expected <- expected$fit
  }
  if(is.null(residuals) && !is.null(observed)) residuals <- (observed - expected)/sqrt(expected)

  ## conduct significance test (if specified)
  if(is.null(p.value)) p.value <- function(observed, residuals, expected, df)
    pchisq(sum(as.vector(residuals)^2), df, lower.tail = FALSE)
  if(!is.function(p.value) && is.na(p.value)) {
    v <- my.v[1]
    p.value <- NULL
  } else {
    if(is.function(p.value)) p.value <- p.value(observed, residuals, expected, df)
    v <- if(p.value < (1-level)) my.v[1] else my.v[2]
  }

  ## set up function for interpolation of saturation
  if(!is.function(interpolate)) {
    col.bins <- sort(interpolate)
    interpolate <- stepfun(col.bins,  seq(my.s[2], my.s[1], length.out = length(col.bins) + 1))
    col.bins <- sort(unique(c(col.bins, 0, -col.bins)))
  } else {
    col.bins <- NULL
  }

  ## store color and lty information for legend
  legend <- NULL
  if(!is.null(col.bins)) {
    res2 <- col.bins
    res2 <- c(head(res2, 1) - 1, res2[-1] - diff(res2)/2, tail(res2, 1) + 1)
    legend.col <- hsv(ifelse(res2 > 0, my.h[1], my.h[2]),
                      pmax(pmin(interpolate(abs(res2)), 1), 0),
		      v, ...)
    lty.bins <- 0
    legend.lty <- lty[2:1]
    legend <- list(col = legend.col, col.bins = col.bins,
                   lty = legend.lty, lty.bins = lty.bins)
  }

  ## set up function that computes color/lty from residuals
  rval <- function(x) {
    res <- as.vector(x)

    fill <- hsv(ifelse(res > 0, my.h[1], my.h[2]),
                pmax(pmin(interpolate(abs(res)), 1), 0),
	        v, ...)
    dim(fill) <- dim(x)

    col <- rep(line_col, length.out = length(res))
    if(!is.null(eps)) {
      eps <- abs(eps)
      col[res > eps] <- hsv(my.h[1], 1, v, ...)
      col[res < -eps] <- hsv(my.h[2], 1, v, ...)
    }
    dim(col) <- dim(x)

	# line type should be solid if abs(resid) < eps
	ltytmp <- ifelse(x > 0, lty[1], lty[2])
	if(!is.null(eps))
		ltytmp[abs(x) < abs(eps)] <- lty[1]
	dim(ltytmp) <- dim(x)

	return(structure(list(col = col, fill = fill, lty = ltytmp), class = "gpar"))
}
  attr(rval, "legend") <- legend
  attr(rval, "p.value") <- p.value
  return(rval)
}
class(shading_hsv) <- "grapcon_generator"


shading_hcl <- function(observed, residuals = NULL, expected = NULL, df = NULL,
  h = NULL, c = NULL, l = NULL,
  interpolate = c(2, 4), lty = 1, eps = NULL, line_col = "black",
  p.value = NULL, level = 0.95, ...)
{
  ## set defaults
  if(is.null(h)) h <- c(260, 0)
  if(is.null(c)) c <- c(100, 20)
  if(is.null(l)) l <- c(90, 50)

  ## get h/c/l and lty
  my.h <- rep(h, length.out = 2L)  ## positive and negative hue
  my.c <- rep(c, length.out = 2L)  ## significant and non-significant maximum chroma
  my.l <- rep(l, length.out = 2L)  ## maximum and minimum luminance
  lty <- rep(lty, length.out = 2L) ## positive and negative lty

  ## model fitting (if necessary)
  if(is.null(expected) && !is.null(residuals)) stop("residuals without expected values specified")
  if(!is.null(expected) && is.null(df) && is.null(p.value)) {
    warning("no default inference available without degrees of freedom")
    p.value <- NA
  }
  if(is.null(expected) && !is.null(observed)) {
    expected <- loglin(observed, 1:length(dim(observed)), fit = TRUE, print = FALSE)
    df <- expected$df
    expected <- expected$fit
  }
  if(is.null(residuals) && !is.null(observed)) residuals <- (observed - expected)/sqrt(expected)

  ## conduct significance test (if specified)
  if(is.null(p.value)) p.value <- function(observed, residuals, expected, df)
    pchisq(sum(as.vector(residuals)^2), df, lower.tail = FALSE)
  if(!is.function(p.value) && is.na(p.value)) {
    max.c <- my.c[1]
    p.value <- NULL
  } else {
    if(is.function(p.value)) p.value <- p.value(observed, residuals, expected, df)
    max.c <- ifelse(p.value < (1-level), my.c[1], my.c[2])
  }

  ## set up function for interpolation of saturation
  if(!is.function(interpolate)) {
    col.bins <- sort(interpolate)
    interpolate <- stepfun(col.bins,  seq(0, 1, length.out = length(col.bins) + 1))
    col.bins <- sort(unique(c(col.bins, 0, -col.bins)))
  } else {
    col.bins <- NULL
  }

  ## store color and lty information for legend
  legend <- NULL
  if(!is.null(col.bins)) {
    res2 <- col.bins
    res2 <- c(head(res2, 1) - 1, res2[-1] - diff(res2)/2, tail(res2, 1) + 1)
    legend.col <- hcl2hex(ifelse(res2 > 0, my.h[1], my.h[2]),
                      max.c * pmax(pmin(interpolate(abs(res2)), 1), 0),
	              my.l[1] + diff(my.l) * pmax(pmin(interpolate(abs(res2)), 1), 0),
		      ...)
    lty.bins <- 0
    legend.lty <- lty[2:1]
    legend <- list(col = legend.col, col.bins = col.bins,
                   lty = legend.lty, lty.bins = lty.bins)
  }

  ## set up function that computes color/lty from residuals
  rval <- function(x) {
    res <- as.vector(x)

    fill <- hcl2hex(ifelse(res > 0, my.h[1], my.h[2]),
                max.c * pmax(pmin(interpolate(abs(res)), 1), 0),
	        my.l[1] + diff(my.l) * pmax(pmin(interpolate(abs(res)), 1), 0),
	        ...)
    dim(fill) <- dim(x)

    col <- rep(line_col, length.out = length(res))
    if(!is.null(eps)) {
      eps <- abs(eps)
      col[res > eps] <- hcl2hex(my.h[1], max.c, my.l[2], ...)
      col[res < -eps] <- hcl2hex(my.h[2], max.c, my.l[2], ...)
    }
    dim(col) <- dim(x)

    ltytmp <- ifelse(x > 0, lty[1], lty[2])
    if(!is.null(eps))
        ltytmp[abs(x) < abs(eps)] <- lty[1]
    dim(ltytmp) <- dim(x)

    return(structure(list(col = col, fill = fill, lty = ltytmp), class = "gpar"))
  }
  attr(rval, "legend") <- legend
  attr(rval, "p.value") <- p.value
  return(rval)
}
class(shading_hcl) <- "grapcon_generator"

shading_Friendly <- function(observed = NULL, residuals = NULL, expected = NULL, df = NULL,
  h = c(2/3, 0), lty = 1:2, interpolate = c(2, 4), eps = 0.01, line_col = "black", ...)
{
  shading_hsv(observed = NULL, residuals = NULL, expected = NULL, df = NULL,
              h = h, v = 1, lty = lty, interpolate = interpolate,
	      eps = eps, line_col = line_col, p.value = NA, ...)
}
class(shading_Friendly) <- "grapcon_generator"

shading_Friendly2 <- function(observed = NULL, residuals = NULL, expected = NULL, df = NULL, lty = 1:2, interpolate = c(2, 4), eps = 0.01, line_col = "black", ...)
{
  shading_hcl(observed = NULL, residuals = NULL, expected = NULL, df = NULL,
              lty = lty, interpolate = interpolate,
	      eps = eps, line_col = line_col, p.value = NA, ...)
}
class(shading_Friendly2) <- "grapcon_generator"

shading_sieve <-
    function(observed = NULL, residuals = NULL, expected = NULL, df = NULL,
             h = c(260, 0), lty = 1:2, interpolate = c(2, 4), eps = 0.01,
             line_col = "black", ...)
{
    shading_hcl(observed = NULL, residuals = NULL, expected = NULL,
                df = NULL, h = h, c = 100, l = 50, lty = lty,
                interpolate = interpolate,
                eps = eps, line_col = line_col, p.value = NA, ...)
}
class(shading_sieve) <- "grapcon_generator"

shading_max <- function(observed = NULL, residuals = NULL, expected = NULL, df = NULL,
  h = NULL, c = NULL, l = NULL, lty = 1, eps = NULL, line_col = "black", level = c(0.9, 0.99), n = 1000, ...)
{
  stopifnot(length(dim(observed)) == 2L)

  ## set defaults
  if(is.null(h)) h <- c(260, 0)
  if(is.null(c)) c <- c(100, 20)
  if(is.null(l)) l <- c(90, 50)

  obs.test <- coindep_test(observed, n = n)
  col.bins <- obs.test$qdist(sort(level))
  rval <- shading_hcl(observed = NULL, residuals = NULL, expected = NULL, df = NULL,
                        h = h, c = c, l = l, interpolate = col.bins, lty = lty,
			eps = eps, line_col = line_col, p.value = obs.test$p.value, ...)
  return(rval)
}
class(shading_max) <- "grapcon_generator"

shading_binary <- function(observed = NULL, residuals = NULL, expected = NULL, df = NULL,
  col = NULL)
{
  ## check col argument
  if(is.null(col)) col <- hcl2hex(c(260, 0), 50, 70)
  col <- rep(col, length.out = 2)

  ## store color information for legend
  legend <- list(col = col[2:1], col.bins = 0, lty = NULL, lty.bins = NULL)

  ## set up function that computes color/lty from residuals
  rval <- function(x)
    gpar(fill = ifelse(x > 0, col[1], col[2]))

  ## add meta information for legend
  attr(rval, "legend") <- legend
  attr(rval, "p.value") <- NULL

  rval
}
class(shading_binary) <- "grapcon_generator"

shading_Marimekko <-
function(x, fill = NULL, byrow = FALSE)
{
    if (is.null(fill)) fill <- colorspace::rainbow_hcl
    d <- dim(x)
    l1 <- if (length(d) > 1L) d[2] else d
    l2 <- if (length(d) > 1L) d[1] else 1L
    if (is.function(fill)) fill <- fill(l1)
    fill <- if (byrow) rep(fill, l2) else rep(fill, each = l2)
    gpar(col = NA, lty = "solid",
         fill = array(fill, dim = d))
}

shading_diagonal <-
function(x, fill = NULL)
{
    if (is.null(fill)) fill <- colorspace::rainbow_hcl
    d <- dim(x)
    if (length(d) < 1L)
        stop("Need matrix or array!")
    if (d[1] != d[2])
        stop("First two dimensions need to be of same length!")
    if (is.function(fill)) fill <- fill(d[1])
    tp = toeplitz(seq_len(d[1]))
    gpar(col = NA, lty = "solid",
         fill = array(rep(fill[tp],  d[1]), dim = d))
}

coindep_test <- function(x, margin = NULL, n = 1000,
  indepfun = function(x) max(abs(x)), aggfun = max,
  alternative = c("greater", "less"),
  pearson = TRUE)
{
  DNAME <- deparse(substitute(x))
  alternative <- match.arg(alternative)

  if(is.null(margin)) {
    rs <- rowSums(x)
    cs <- colSums(x)
    expctd <- rs %o% cs / sum(rs)
    Pearson <- function(x) (x - expctd)/sqrt(expctd)      
    resids <- Pearson(x)
    
    ff <- if(!is.null(aggfun)) {
      if(pearson) function(x) aggfun(indepfun(Pearson(x)))
        else function(x) aggfun(indepfun(x))
    } else {
      if(pearson) function(x) indepfun(Pearson(x))
        else function(x) indepfun(x)    
    }
       
    if(length(dim(x)) > 2) stop("currently only implemented for (conditional) 2d tables")
    dist <- sapply(r2dtable(n, rowSums(x), colSums(x)), ff)
    STATISTIC <- ff(x)   
  } else {
    ff <- if(pearson) function(x) indepfun(Pearson(x))
      else function(x) indepfun(x)

    cox <- co_table(x, margin)
    nc <- length(cox)
    if(length(dim(cox[[1]])) > 2) stop("currently only implemented for conditional 2d tables")
  
    dist <- matrix(rep(0, n * nc), ncol = nc)
    for(i in 1:nc) {
      coxi <- cox[[i]]    
      cs <- colSums(coxi)
      rs <- rowSums(coxi)
      expctd <- rs %o% cs / sum(rs)
      Pearson <- function(x) (x - expctd)/sqrt(expctd)      
      if(any(c(cs, rs) < 1)) warning("structural zeros") ## FIXME
      dist[, i] <- sapply(r2dtable(n, rs, cs), ff)
    }
    dist <- apply(dist, 1, aggfun)

    Pearson <- function(x) {
      expctd <- rowSums(x) %o% colSums(x) / sum(x)
      return((x - expctd)/sqrt(expctd))
    }
    STATISTIC <- aggfun(sapply(cox, ff))

    ## just for returning nicely formatted fitted values
    ## and residuals: fit once more with loglm()
    vars <- names(dimnames(x))
    condvars <- if(is.numeric(margin)) vars[margin] else margin
    indvars <- vars[!(vars %in% condvars)]
    coind.form <- as.formula(paste("~ (", paste(indvars, collapse = " + "),
				   ") * ",
				   paste(condvars, collapse = " * "),
				   sep = ""))
    fm <- loglm(coind.form, data = x, fitted = TRUE)
    expctd <- fitted(fm)
    resids <- residuals(fm, type = "pearson")
  }

  pdist <- function(x) sapply(x, function(y) mean(dist <= y))
  qdist <- function(p) quantile(dist, p)
  PVAL <- switch(alternative,
                 greater = mean(dist >= STATISTIC),
                 less = mean(dist <= STATISTIC))
  METHOD <- "Permutation test for conditional independence"
  names(STATISTIC) <- "f(x)"

  rval <- list(statistic = STATISTIC,
               p.value = PVAL,
	       method = METHOD,
	       data.name = DNAME,
	       observed = x,
	       expected = expctd,
	       residuals = resids,
	       margin = margin,	       
	       dist = dist,
	       qdist = qdist,
	       pdist = pdist)
  class(rval) <- c("coindep_test", "htest")
  return(rval)
}

fitted.coindep_test <- function(object, ...)
  object$expected

## plot.coindep_test
## mosaic.coindep_test
## assoc.coindep_test
## difficult, depends on functionals...

"oddsratio" <- function(x, stratum = NULL, log = TRUE)
    loddsratio(x, strata = stratum, log = log)

## "oddsratio" <-
## function (x, stratum = NULL, log = TRUE) {
##   l <- length(dim(x))
##   if (l > 2 && is.null(stratum))
##     stratum <- 3:l
##   if (l - length(stratum) > 2)
##     stop("All but 2 dimensions must be specified as strata.")
##   if (l == 2 && dim(x) != c(2, 2))
##     stop("Not a 2x2 table.")
##   if (!is.null(stratum) && dim(x)[-stratum] != c(2,2))
##     stop("Need strata of 2x2 tables.")

##   lor <- function (y) {
##     if (any(y == 0))
##       y <- y + 0.5
##     y <- log(y)
##     or <- y[1,1] + y[2,2] - y[1,2] - y[2,1]
##     if (log) or else exp(or)
##   }

##   ase <- function(y) {
##     if (any(y == 0))
##         y <- y + 0.5
##     sqrt(sum(1/y))
##   }

##   if(is.null(stratum)) {
##     LOR <- lor(x)
##     ASE <- ase(x)
##   } else {
##     LOR <- apply(x, stratum, lor)
##     ASE <- apply(x, stratum, ase)
##   }

##   structure(LOR,
##             ASE = ASE,
##             log = log,
##             class = "oddsratio"
##             )}

## "print.oddsratio" <-
## function(x, ...) {
##   if (length(dim(x)) > 1)
##     print(cbind(unclass(x)), ...)
##   else
##     print(c(x), ...)
##   invisible(x)
## }

## "summary.oddsratio" <-
## function(object, ...) {
##   if(!is.null(dim(object)))
##     ret <- object
##   else {
##     LOG <- attr(object, "log")
##     ASE <- attr(object, "ASE")
##     Z <- object / ASE

##     ret <- cbind("Estimate"   = object,
##                  "Std. Error" = if (LOG) ASE,
##                  "z value"    = if (LOG) Z,
##                  "Pr(>|z|)"   = if (LOG) 2 * pnorm(-abs(Z))
##                  )
##     colnames(ret)[1] <- if (LOG) "Log Odds Ratio" else "Odds Ratio"
##   }

##   class(ret) <- "summary.oddsratio"
##   ret
## }


## "print.summary.oddsratio" <-
## function(x, ...) {
##   if(!is.null(attr(x, "log"))) {
##     cat("\n")
##     cat(if(attr(x, "log")) "Log Odds Ratio(s):" else "Odds Ratio(s):", "\n\n")
##     print(as.data.frame(unclass(x)), ...)
##     cat("\nAsymptotic Standard Error(s):\n\n")
##     print(attr(x, "ASE"), ...)
##     cat("\n")
##   } else printCoefmat(unclass(x), ...)
##   invisible(x)
## }

## "plot.oddsratio" <-
## function(x,
##          conf_level = 0.95,
##          type = "o",
##          xlab = NULL,
##          ylab = NULL,
##          xlim = NULL,
## 	 ylim = NULL,
##          whiskers = 0.1,
##          baseline = TRUE,
##          transpose = FALSE,
##          ...)
## {
##   if (length(dim(x)) > 1)
##     stop ("Plot function works only on vectors.")

##   LOG <- attr(x, "log")
##   confidence <- !(is.null(conf_level) || conf_level == FALSE)
##   oddsrange <- range(x)

##   if(confidence) {
##     CI  <- confint(x, level = conf_level)
##     lwr <- CI[,1]
##     upr <- CI[,2]
##     oddsrange[1] <- trunc(min(oddsrange[1], min(lwr)))
##     oddsrange[2] <- ceiling(max(oddsrange[2], max(upr)))
##   }

##   if (transpose) {
##     plot(x = unclass(x),
##          y = 1:length(x),
##          ylab = if (!is.null(ylab)) ylab else "Strata",
##          xlab = if (!is.null(xlab)) xlab else if (LOG) "Log Odds Ratio" else "Odds Ratio",
##          type = type,
##          yaxt = "n",
##          xlim = if(is.null(xlim)) oddsrange else xlim,
##          ...)
##     axis (2, at = 1:length(x), names(x))

##     if (baseline)
##       lines(c(1,1) - LOG, c(0,length(x) + 1), lty = 2, col = "red")

##     if (confidence)
##       for (i in 1:length(x)) {
##         lines(c(lwr[i], upr[i]), c(i, i))
##         lines(c(lwr[i], lwr[i]), c(i - whiskers/2, i + whiskers/2))
##         lines(c(upr[i], upr[i]), c(i - whiskers/2, i + whiskers/2))
##       }
##   } else {
##     plot(unclass(x),
##          xlab = if (!is.null(xlab)) xlab else "Strata",
##          ylab = if(!is.null(ylab)) ylab else if(LOG) "Log Odds Ratio" else "Odds Ratio",
##          type = type,
##          xaxt = "n",
##          ylim = if(is.null(ylim)) oddsrange else ylim,
##          ...)
##     axis (1, at = 1:length(x), names(x))

##     if (baseline)
##       lines(c(0,length(x) + 1), c(1,1) - LOG, lty = 2, col = "red")

##     if (confidence)
##       for (i in 1:length(x)) {
##         lines(c(i, i), c(lwr[i], upr[i]))
##         lines(c(i - whiskers/2, i + whiskers/2), c(lwr[i], lwr[i]))
##         lines(c(i - whiskers/2, i + whiskers/2), c(upr[i], upr[i]))
##       }
##   }

## }

## "confint.oddsratio" <-
## function(object, parm, level = 0.95, ...) {
##   ASE <- attr(object, "ASE")
##   LOG <- attr(object, "log")
##   I <- ASE * qnorm((1 + level) / 2)
##   cbind(
##         lwr = if (LOG) object - I else exp(log(object) - I),
##         upr = if (LOG) object + I else exp(log(object) + I)
##         )
## }

cotabplot <- function(x, ...)
{
  UseMethod("cotabplot")
}

cotabplot.formula <- function(formula, data = NULL, ...)
{
  m <- match.call()
  edata <- eval(m$data, parent.frame())

  fstr <- deparse(formula)
  fstr <- gsub("*", "+", fstr,  fixed = TRUE)
  fstr <- gsub("/", "+", fstr,  fixed = TRUE)
  fstr <- gsub("(", "", fstr,  fixed = TRUE)
  fstr <- gsub(")", "", fstr,  fixed = TRUE)

  fstr <- strsplit(paste(fstr, collapse = ""), "~")
  vars <- strsplit(strsplit(gsub(" ", "", fstr[[1]][2]), "\\|")[[1]], "\\+")
  varnames <- vars[[1]]
  condnames <- if(length(vars) > 1) vars[[2]] else NULL

  if (inherits(edata, "ftable") || inherits(edata, "table") || length(dim(edata)) > 2) {
    tab <- as.table(data)
    if(all(varnames != ".")) {
      ind <- match(varnames, names(dimnames(tab)))
      if (any(is.na(ind)))
        stop(paste("Can't find", paste(varnames[is.na(ind)], collapse=" / "), "in", deparse(substitute(data))))

      if (!is.null(condnames)) {
        condind <- match(condnames, names(dimnames(tab)))
        if (any(is.na(condind)))
          stop(paste("Can't find", paste(condnames[is.na(condind)], collapse=" / "), "in", deparse(substitute(data))))
        ind <- c(condind, ind)
      }
      tab <- margin.table(tab, ind)
    }
  } else {
    tab <- if ("Freq" %in% colnames(data))
      xtabs(formula(paste("Freq~", paste(c(condnames, varnames), collapse = " + "))),
            data = data)
    else
      xtabs(formula(paste("~", paste(c(condnames, varnames), collapse = " + "))),
            data = data)
  }
  tab <- margin.table(tab, match(c(varnames, condnames), names(dimnames(tab))))
  cotabplot(tab, cond = condnames, ...)
}


cotabplot.default <- function(x, cond = NULL,
  panel = cotab_mosaic, panel_args = list(),
  margins = rep(1, 4), layout = NULL,
  text_gp = gpar(fontsize = 12), rect_gp = gpar(fill = grey(0.9)),
  pop = TRUE, newpage = TRUE, return_grob = FALSE, ...)
{
  ## coerce to table
  x <- as.table(x)

  ## initialize newpage
  if(newpage) grid.newpage()

  ## process default option
  ldx <- length(dim(x))
  if(is.null(cond)) {
    indep <- if(ldx > 1) 1:2 else 1
    if(ldx > 2) cond <- 3:ldx
  } else {
    if(is.character(cond)) cond <- match(cond, names(dimnames(x)))
    cond <- as.integer(cond)
    indep <- (1:ldx)[!(1:ldx %in% cond)]
  }

  ## sort margins
  x <- margin.table(x, c(indep, cond))

  ## convenience variables that describe conditioning variables
  if(is.null(cond)) {
    cond.n <- 0
    cond.num <- cond.dnam <- cond.char <- NULL
  } else {
    cond.n <- length(cond)               ## number of variables
    cond.num <- (length(indep) + 1):ldx  ## position in x
    cond.dnam <- dimnames(x)[cond.num]   ## corresponding dimnames
    cond.char <- names(cond.dnam)        ## names of variables
  }

  ## create panel function (if necessary)
  if(inherits(panel, "grapcon_generator"))
    panel <- do.call("panel", c(list(x, cond.char), as.list(panel_args), list(...)))

  if(cond.n < 1) panel(x, NULL) ## no conditioning variables
  else {

    cond.nlevels <- sapply(cond.dnam, length)
    nplots <- prod(cond.nlevels)
    condition <- as.matrix(expand.grid(cond.dnam))

    ## compute layout
    #Z# needs fixing for more than two conditioning variables
    if(is.null(layout)) {
      layout <- c(1,1,1) ## rows, cols, pages
      if(cond.n == 1) {
        layout[2] <- ceiling(sqrt(floor(cond.nlevels)))
        layout[1] <- ceiling(cond.nlevels/layout[2])
      } else {
        layout[1] <- cond.nlevels[1]
        layout[2] <- cond.nlevels[2]
        if(cond.n >= 3) layout[3] <- nplots/prod(cond.nlevels[1:2]) #Z# FIXME
        if(layout[3] > 1) stop("multiple pages not supported yet")
      }
    } else {
      layout <- c(rep(layout, length.out = 2), 1)
      if(layout[1] * layout[2] < nplots) stop("number of panels specified in 'layout' is too small")
    }
    layout <- expand.grid(lapply(layout, function(x) 1:x))[1:nplots,]

    ## push basic grid of nr x nc cells
    nr <- max(layout[,1])
    nc <- max(layout[,2])
    pushViewport(plotViewport(margins))
    pushViewport(viewport(layout = grid.layout(nr, nc, widths = unit(1/nc, "npc"))))

    strUnit <- unit(2 * ncol(condition), "strheight", "A")
    cellport <- function(name) viewport(layout = grid.layout(2, 1,
                                          heights = unit.c(strUnit, unit(1, "npc") - strUnit)),
                                        name = name)

    ## go through each conditioning combination
    for(i in 1:nrow(condition)) {

      ## conditioning information in ith cycle
      condi <- as.vector(condition[i,])
      names(condi) <- colnames(condition)
      condistr <- paste(condi, collapse = ".")
      condilab <- paste(cond.char, condi, sep = " = ")

      ## header
      pushViewport(viewport(layout.pos.row = layout[i,1], layout.pos.col = layout[i,2]))
      pushViewport(cellport(paste("cell", condistr, sep = ".")))
      pushViewport(viewport(layout.pos.row = 1, name = paste("lab", condistr, sep = ".")))
      grid.rect(gp = rect_gp)
      grid.text(condilab, y = cond.n:1/cond.n - 1/(2*cond.n), gp = text_gp)
      grid.segments(0, 0:cond.n/cond.n, 1, 0:cond.n/cond.n)
      upViewport()

      ## main plot
      pushViewport(viewport(layout.pos.row = 2, name = paste("plot", condistr, sep = ".")))
      panel(x, condi)
      upViewport(2)
      grid.rect(gp = gpar(fill = "transparent"))
      upViewport()
    }

  upViewport()
  if(pop) popViewport() else upViewport()
}

  if (return_grob)
      invisible(structure(x, grob = grid.grab()))
  else
      invisible(x)

}

cotab_mosaic <- function(x = NULL, condvars = NULL, ...) {
  function(x, condlevels) {
    if(is.null(condlevels)) mosaic(x, newpage = FALSE, pop = FALSE, return_grob = FALSE, ...)
      else mosaic(co_table(x, names(condlevels))[[paste(condlevels, collapse = ".")]],
                  newpage = FALSE, pop = FALSE, return_grob = FALSE,
                  prefix = paste("panel:", paste(names(condlevels), condlevels, sep = "=", collapse = ","), "|", sep = ""), ...)
  }
}
class(cotab_mosaic) <- "grapcon_generator"

cotab_sieve <- function(x = NULL, condvars = NULL, ...) {
  function(x, condlevels) {
    if(is.null(condlevels)) sieve(x, newpage = FALSE, pop = FALSE, return_grob = FALSE, ...)
      else sieve(co_table(x, names(condlevels))[[paste(condlevels, collapse = ".")]],
                 newpage = FALSE, pop = FALSE, return_grob = FALSE,
                 prefix = paste("panel:", paste(names(condlevels), condlevels, sep = "=", collapse = ","), "|", sep = ""), ...)
  }
}
class(cotab_sieve) <- "grapcon_generator"

cotab_assoc <- function(x = NULL, condvars = NULL, ylim = NULL, ...) {
  if(!is.null(x)) {
    fm <- coindep_test(x, condvars, n = 1)
    if(is.null(ylim)) ylim <- range(residuals(fm))
  }

  function(x, condlevels) {
    if(is.null(condlevels)) assoc(x, newpage = FALSE, pop = FALSE, ylim = ylim, return_grob = FALSE, ...)
      else assoc(co_table(x, names(condlevels))[[paste(condlevels, collapse = ".")]],
                  newpage = FALSE, pop = FALSE, return_grob = FALSE, ylim = ylim,
                 prefix = paste("panel:", paste(names(condlevels), condlevels, sep = "=", collapse = ","), "|", sep = ""), ...)
  }
}
class(cotab_assoc) <- "grapcon_generator"

cotab_fourfold <- function (x = NULL, condvars = NULL, ...) {
  function(x, condlevels) {
    if (is.null(condlevels))
      fourfold(x, newpage = FALSE, return_grob = FALSE, ...)
    else
      fourfold(co_table(x, names(condlevels))[[paste(condlevels, collapse = ".")]],
               newpage = FALSE, return_grob = FALSE, ...)
  }
}
class(cotab_fourfold) <- "grapcon_generator"

cotab_loddsratio <- function(x = NULL, condvars = NULL, ...) {
    function(x, condlevels) {
        if(is.null(condlevels)) {
            plot(loddsratio(x, ...), newpage = FALSE, pop = FALSE, return_grob = FALSE, ...)
        } else {
            plot(loddsratio(co_table(x, names(condlevels))[[paste(condlevels, collapse = ".")]], ...),
                 newpage = FALSE, pop = FALSE, return_grob = FALSE,
                 prefix = paste("panel:", paste(names(condlevels), condlevels, sep = "=", collapse = ","), "|", sep = ""), ...)
        }
        upViewport(2)
    }
}
class(cotab_loddsratio) <- "grapcon_generator"

cotab_agreementplot <- function(x = NULL, condvars = NULL, ...) {
  function(x, condlevels) {
    if(is.null(condlevels)) agreementplot(x, newpage = FALSE, pop = FALSE, return_grob = FALSE, ...)
      else agreementplot(co_table(x, names(condlevels))[[paste(condlevels, collapse = ".")]], newpage = FALSE, pop = FALSE, return_grob = FALSE,
                         prefix = paste("panel:", paste(names(condlevels), condlevels, sep = "=", collapse = ","), "|", sep = ""), ...)
  }
}
class(cotab_agreementplot) <- "grapcon_generator"

cotab_coindep <- function(x, condvars,
  test = c("doublemax", "maxchisq", "sumchisq"),
  level = NULL, n = 1000, interpolate = c(2, 4),
  h = NULL, c = NULL, l = NULL, lty = 1,
  type = c("mosaic", "assoc"),
  legend = FALSE, ylim = NULL, ...)
{
  if(is.null(condvars))
    stop("at least one conditioning variable is required")

  ## set color defaults
  if(is.null(h)) h <- c(260, 0)
  if(is.null(c)) c <- c(100, 20)
  if(is.null(l)) l <- c(90, 50)

  ## process conditional variables and get independent variables
  ## store some convenience information
  ldx <- length(dim(x))
  if(is.character(condvars)) condvars <- match(condvars, names(dimnames(x)))
  condvars <- as.integer(condvars)
  indep <- (1:ldx)[!(1:ldx %in% condvars)]

  ## sort margins
  x <- margin.table(x, c(indep, condvars))

  ind.n <- length(indep)
  ind.num <- 1:ind.n
  ind.dnam <- dimnames(x)[ind.num]
  ind.char <- names(ind.dnam)
  cond.n <- length(condvars)
  cond.num <- (ind.n + 1):length(dim(x))
  cond.dnam <- dimnames(x)[cond.num]
  cond.char <- names(cond.dnam)

  test <- match.arg(test)
  switch(test,
  "doublemax" = {
    if(is.null(level)) level <- c(0.9, 0.99)
    fm <- coindep_test(x, cond.num, n = n)
    resids <- residuals(fm)

    col.bins <- fm$qdist(sort(level))
    gpfun <- shading_hcl(observed = NULL, residuals = NULL, expected = NULL, df = NULL,
      h = h, c = c, l = l, interpolate = col.bins, lty = lty, p.value = fm$p.value)
  },
  "maxchisq" = {
    if(is.null(level)) level <- 0.95
    level <- level[1]
    fm <- coindep_test(x, cond.num, n = n, indepfun = function(x) sum(x^2))
    resids <- residuals(fm)

    chisqs <- sapply(co_table(residuals(fm), fm$margin), function(x) sum(x^2))
    pvals <- 1 - fm$pdist(chisqs)
    gpfun <- sapply(pvals, function(p)
      shading_hcl(observed = NULL, residuals = NULL, expected = NULL, df = NULL,
      h = h, c = c, l = l, interpolate = interpolate, lty = lty, level = level, p.value = p))
  },
  "sumchisq" = {
    if(is.null(level)) level <- 0.95
    level <- level[1]
    fm <- coindep_test(x, cond.num, n = n, indepfun = function(x) sum(x^2), aggfun = sum)
    resids <- residuals(fm)

    gpfun <- shading_hcl(observed = NULL, residuals = NULL, expected = NULL, df = NULL,
      h = h, c = c, l = l, interpolate = interpolate, lty = lty, level = level, p.value = fm$p.value)
  })

  type <- match.arg(type)
  if(type == "mosaic") {
    rval <- function(x, condlevels) {
      if(is.null(condlevels)) {
        tab <- x
        gp <- if(is.list(gpfun)) gpfun[[1]] else gpfun
      } else {
        tab <- co_table(x, names(condlevels))[[paste(condlevels, collapse = ".")]]
        gp <- if(is.list(gpfun)) gpfun[[paste(condlevels, collapse = ".")]] else gpfun
      }
      mosaic(tab, newpage = FALSE, pop = FALSE, return_grob = FALSE, gp = gp, legend = legend,
             prefix = paste("panel:", paste(names(condlevels), condlevels, sep = "=", collapse = ","), "|", sep = ""), ...)
    }
  } else {
    if(is.null(ylim)) ylim <- range(resids)

    rval <- function(x, condlevels) {
      if(is.null(condlevels)) {
        tab <- x
        gp <- if(is.list(gpfun)) gpfun[[1]] else gpfun
      } else {
        tab <- co_table(x, names(condlevels))[[paste(condlevels, collapse = ".")]]
        gp <- if(is.list(gpfun)) gpfun[[paste(condlevels, collapse = ".")]] else gpfun
      }
      assoc(tab, newpage = FALSE, pop = FALSE, return_grob = FALSE, gp = gp, legend = legend, ylim = ylim,
            prefix = paste("panel:", paste(names(condlevels), condlevels, sep = "=", collapse = ","), "|", sep = ""), ...)
    }
  }

  return(rval)
}
class(cotab_coindep) <- "grapcon_generator"

## Modifications - MF - 1 Dec 2010
#  -- change default colors to more distinguishable values
#  -- allow to work with >3 dimensional arrays
#  -- modified defaults for mfrow/mfcol to give landscape display, nr <= nc, rather than nr >= nc

# Take a 2+D array and return a 3D array, with dimensions 3+ as a single dimension
# Include as a separate function, since it is useful in other contexts
array3d <- function(x, sep=':') {
	if(length(dim(x)) == 2) {
        x <- if(is.null(dimnames(x)))
            array(x, c(dim(x), 1))
        else
            array(x, c(dim(x), 1), c(dimnames(x), list(NULL)))
        return(x)
    }
  else if(length(dim(x))==3) return(x)
  else {
	  x3d <- array(x, c(dim(x)[1:2], prod(dim(x)[-(1:2)])))
	  if (!is.null(dimnames(x))) {
		  n3d <- paste(names(dimnames(x))[-(1:2)], collapse=sep)
		  d3d <- apply(expand.grid(dimnames(x)[-(1:2)]), 1, paste, collapse=sep)
		  dimnames(x3d) <- c(dimnames(x)[1:2], list(d3d))
		  names(dimnames(x3d))[3] <- n3d
  	}
  	return(x3d)
  }
}

"fourfold" <-
function(x,
#        color = c("#99CCFF","#6699CC","#FF5050","#6060A0", "#FF0000", "#000080"),
         color = c("#99CCFF","#6699CC","#FFA0A0","#A0A0FF", "#FF0000", "#000080"),
         conf_level = 0.95,
         std = c("margins", "ind.max", "all.max"), margin = c(1, 2),
         space = 0.2, main = NULL, sub = NULL, mfrow = NULL, mfcol = NULL, extended = TRUE,
         ticks = 0.15, p_adjust_method = p.adjust.methods, newpage = TRUE,
         fontsize = 12, default_prefix = c("Row", "Col", "Strata"), sep = ": ", varnames = TRUE,
         return_grob = FALSE)

{
    ## Code for producing fourfold displays.
    ## Reference:
    ##   Friendly, M. (1994).
    ##   A fourfold display for 2 by 2 by \eqn{k} tables.
    ##   Technical Report 217, York University, Psychology Department.
    ##   http://datavis.ca/papers/4fold/4fold.pdf
    ##
    ## Implementation notes:
    ##
    ##   We need plots with aspect ratio FIXED to 1 and glued together.
    ##   Hence, even if k > 1 we prefer keeping everything in one plot
    ##   region rather than using a multiple figure layout.
    ##   Each 2 by 2 pie is is drawn into a square with x/y coordinates
    ##   between -1 and 1, with row and column labels in [-1-space, -1]
    ##   and [1, 1+space], respectively.  If k > 1, strata labels are in
    ##   an area with y coordinates in [1+space, 1+(1+gamma)*space],
    ##   where currently gamma=1.25.  The pies are arranged in an nr by
    ##   nc layout, with horizontal and vertical distances between them
    ##   set to space.
    ##
    ##   The drawing code first computes the complete are of the form
    ##     [0, totalWidth] x [0, totalHeight]
    ##   needed and sets the world coordinates using plot.window().
    ##   Then, the strata are looped over, and the corresponding pies
    ##   added by filling rows or columns of the layout as specified by
    ##   the mfrow or mfcol arguments.  The world coordinates are reset
    ##   in each step by shifting the origin so that we can always plot
    ##   as detailed above.

    if(!is.array(x))
        stop("x must be an array")
    dimx <- dim(x)   # save original dimensions for setting default mfrow/mfcol when length(dim(x))>3
    x <- array3d(x)
    if(any(dim(x)[1:2] != 2))
        stop("table for each stratum must be 2 by 2")
    dnx <- dimnames(x)
    if(is.null(dnx))
        dnx <- vector("list", 3)
    for(i in which(sapply(dnx, is.null)))
        dnx[[i]] <- LETTERS[seq(length.out = dim(x)[i])]
    if(is.null(names(dnx)))
        i <- 1 : 3
    else
        i <- which(is.null(names(dnx)))
    if(any(i > 0))
        names(dnx)[i] <- default_prefix[i]
    dimnames(x) <- dnx
    k <- dim(x)[3]

    if(!((length(conf_level) == 1) && is.finite(conf_level) &&
         (conf_level >= 0) && (conf_level < 1)))
        stop("conf_level must be a single number between 0 and 1")
    if(conf_level == 0)
        conf_level <- FALSE

    std <- match.arg(std)

    findTableWithOAM <- function(or, tab) {
        ## Find a 2x2 table with given odds ratio `or' and the margins
        ## of a given 2x2 table `tab'.
        m <- rowSums(tab)[1]
        n <- rowSums(tab)[2]
        t <- colSums(tab)[1]
        if(or == 1)
            x <- t * n / (m + n)
        else if(or == Inf)
            x <- max(0, t - m)
        else {
            A <- or - 1
            B <- or * (m - t) + (n + t)
            C <- - t * n
            x <- (- B + sqrt(B ^ 2 - 4 * A * C)) / (2 * A)
        }
        matrix(c(t - x, x, m - t + x, n - x), nrow = 2)
    }

    drawPie <- function(r, from, to, n = 500, color = "transparent") {
        p <- 2 * pi * seq(from, to, length.out = n) / 360
        x <- c(cos(p), 0) * r
        y <- c(sin(p), 0) * r
        grid.polygon(x, y, gp = gpar(fill = color), default.units = "native")
        invisible(NULL)
    }

    stdize <- function(tab, std, x) {
        ## Standardize the 2 x 2 table `tab'.
        if(std == "margins") {
            if(all(sort(margin) == c(1, 2))) {
                ## standardize to equal row and col margins
                u <- sqrt(odds(tab)$or)
                u <- u / (1 + u)
                y <- matrix(c(u, 1 - u, 1 - u, u), nrow = 2)
            }
            else if(margin %in% c(1, 2))
                y <- prop.table(tab, margin)
            else
                stop("incorrect margin specification")
        }
        else if(std == "ind.max")
            y <- tab / max(tab)
        else if(std == "all.max")
            y <- tab / max(x)
        y
    }

    odds <- function(x) {
        ## Given a 2 x 2 or 2 x 2 x k table `x', return a list with
        ## components `or' and `se' giving the odds ratios and standard
        ## deviations of the log odds ratios.
        if(length(dim(x)) == 2) {
            dim(x) <- c(dim(x), 1)
            k <- 1
        }
        else
            k <- dim(x)[3]
        or <- double(k)
        se <- double(k)
        for(i in 1 : k) {
            f <- x[ , , i]
            if(any(f == 0))
                f <- f + 0.5
            or[i] <- (f[1, 1] * f[2, 2]) / (f[1, 2] * f[2, 1])
            se[i] <- sqrt(sum(1 / f))
        }
        list(or = or, se = se)
    }

    gamma <- 1.25                       # Scale factor for strata labels

    angle.f <- c( 90, 180,  0, 270)     # `f' for `from'
    angle.t <- c(180, 270, 90, 360)     # `t' for `to'

    byrow <- FALSE
    if(!is.null(mfrow)) {
        nr <- mfrow[1]
        nc <- mfrow[2]
    }
    else if(!is.null(mfcol)) {
        nr <- mfcol[1]
        nc <- mfcol[2]
        byrow <- TRUE
    }
    else if(length(dimx)>3) {
        nr <- dimx[3]
        nc <- prod(dimx[-(1:3)])
    }
    else {
#       nr <- ceiling(sqrt(k))
        nr <- round(sqrt(k))
        nc <- ceiling(k / nr)
    }
    if(nr * nc < k)
        stop("incorrect geometry specification")
    if(byrow)
        indexMatrix <- expand.grid(1 : nc, 1 : nr)[, c(2, 1)]
    else
        indexMatrix <- expand.grid(1 : nr, 1 : nc)

    totalWidth <- nc * 2 * (1 + space) + (nc - 1) * space
    totalHeight <- if(k == 1)
        2 * (1 + space)
    else
        nr * (2 + (2 + gamma) * space) + (nr - 1) * space
    xlim <- c(0, totalWidth)
    ylim <- c(0, totalHeight)
    if (newpage) grid.newpage()
    if (!is.null(main) || !is.null(sub))
      pushViewport(viewport(height = 1 - 0.1 * sum(!is.null(main), !is.null(sub)),
                            width = 0.9,
                            y = 0.5 - 0.05 * sum(!is.null(main), - !is.null(sub))
                            )
                   )

    pushViewport(viewport(xscale = xlim, yscale = ylim,
                           width = unit(min(totalWidth / totalHeight, 1), "snpc"),
                           height = unit(min(totalHeight / totalWidth, 1), "snpc")))
    o <- odds(x)

    ## perform logoddsratio-test for each stratum (H0: lor = 0) and adjust p-values
    if(is.numeric(conf_level) && extended)
      p.lor.test <- p.adjust(sapply(1 : k, function(i) {
                               u <- abs(log(o$or[i])) / o$se[i]
                               2 * (1 - pnorm(u))
                             }),
                             method = p_adjust_method
                             )

    scale <- space / (2 * convertY(unit(1, "strheight", "Ag"), "native", valueOnly = TRUE) )
    v <- 0.95 - max(convertX(unit(1, "strwidth", as.character(c(x))), "native", valueOnly = TRUE) ) / 2

    fontsize = fontsize * scale

    for(i in 1 : k) {

        tab <- x[ , , i]

        fit <- stdize(tab, std, x)

        xInd <- indexMatrix[i, 2]
        xOrig <- 2 * xInd - 1 + (3 * xInd - 2) * space
        yInd <- indexMatrix[i, 1]
        yOrig <- if(k == 1)
            (1 + space)
        else
            (totalHeight
             - (2 * yInd - 1 + ((3 + gamma) * yInd - 2) * space))
        pushViewport(viewport(xscale = xlim - xOrig, yscale = ylim - yOrig))

        ## drawLabels()
        u <- 1 + space / 2
        adjCorr <- 0.2
        grid.text(
                  paste(names(dimnames(x))[1],
                        dimnames(x)[[1]][1],
                        sep = sep),
                  0, u,
                  gp = gpar(fontsize = fontsize),
                  default.units = "native"
             )
        grid.text(
                  paste(names(dimnames(x))[2],
                        dimnames(x)[[2]][1],
                        sep = sep),
                  -u, 0,
                  default.units = "native",
                  gp = gpar(fontsize = fontsize),
                  rot = 90)
        grid.text(
                  paste(names(dimnames(x))[1],
                        dimnames(x)[[1]][2],
                        sep = sep),
                  0, -u,
                  gp = gpar(fontsize = fontsize),
                  default.units = "native"
             )
        grid.text(
                  paste(names(dimnames(x))[2],
                        dimnames(x)[[2]][2],
                        sep = sep),
                  u, 0,
                  default.units = "native",
                  gp = gpar(fontsize = fontsize),
                  rot = 90)
        if (k > 1) {
            grid.text(if (!varnames)
                          dimnames(x)[[3]][i]
                      else
                          paste(names(dimnames(x))[3],
                                dimnames(x)[[3]][i],
                                sep = sep),
                      0, 1 + (1 + gamma / 2) * space,
                      gp = gpar(fontsize = fontsize * gamma),
                      default.units = "native"
                      )
          }

        ## drawFrequencies()

        ### in extended plots, emphasize charts with significant logoddsratios
        emphasize <- if(extended && is.numeric(conf_level))
          2 * extended * (1 + (p.lor.test[i] < 1 - conf_level))
        else 0

        d <- odds(tab)$or
        drawPie(sqrt(fit[1,1]),  90, 180, color = color[1 + (d > 1) + emphasize])
        drawPie(sqrt(fit[2,1]), 180, 270, color = color[2 - (d > 1) + emphasize])
        drawPie(sqrt(fit[1,2]),   0,  90, color = color[2 - (d > 1) + emphasize])
        drawPie(sqrt(fit[2,2]), 270, 360, color = color[1 + (d > 1) + emphasize])

        u <- 1 - space / 2
        grid.text(as.character(c(tab))[1],
                  -v, u,
                  just = c("left", "top"),
                  gp = gpar(fontsize = fontsize),
                  default.units = "native")
        grid.text(as.character(c(tab))[2],
                  -v, -u,
                  just = c("left", "bottom"),
                  gp = gpar(fontsize = fontsize),
                  default.units = "native")
        grid.text(as.character(c(tab))[3],
                  v, u,
                  just = c("right", "top"),
                  gp = gpar(fontsize = fontsize),
                  default.units = "native")
        grid.text(as.character(c(tab))[4],
                  v, -u,
                  just = c("right", "bottom"),
                  gp = gpar(fontsize = fontsize),
                  default.units = "native")

        ## draw ticks
        if(extended && ticks)
          if(d > 1) {
            grid.lines(c(sqrt(fit[1,1])           * cos(3*pi/4),
                         (sqrt(fit[1,1]) + ticks) * cos(3*pi/4)),
                       c(sqrt(fit[1,1])           * sin(3*pi/4),
                         (sqrt(fit[1,1]) + ticks) * sin(3*pi/4)),
                       gp = gpar(lwd = 1),
                       default.units = "native"
                       )
            grid.lines(c(sqrt(fit[2,2])           * cos(-pi/4),
                         (sqrt(fit[2,2]) + ticks) * cos(-pi/4)),
                       c(sqrt(fit[2,2])           * sin(-pi/4),
                         (sqrt(fit[2,2]) + ticks) * sin(-pi/4)),
                       gp = gpar(lwd = 1),
                       default.units = "native"
                       )
          } else {
            grid.lines(c(sqrt(fit[1,2])           * cos(pi/4),
                         (sqrt(fit[1,2]) + ticks) * cos(pi/4)),
                       c(sqrt(fit[1,2])           * sin(pi/4),
                         (sqrt(fit[1,2]) + ticks) * sin(pi/4)),
                       gp = gpar(lwd = 1),
                       default.units = "native"
                       )
            grid.lines(c(sqrt(fit[2,1])           * cos(-3*pi/4),
                         (sqrt(fit[2,1]) + ticks) * cos(-3*pi/4)),
                       c(sqrt(fit[2,1])           * sin(-3*pi/4),
                         (sqrt(fit[2,1]) + ticks) * sin(-3*pi/4)),
                       gp = gpar(lwd = 1),
                       default.units = "native"
                       )
          }

        ## drawConfBands()
        if(is.numeric(conf_level)) {
            or <- o$or[i]
            se <- o$se[i]
            ## lower
            theta <- or * exp(qnorm((1 - conf_level) / 2) * se)
            tau <- findTableWithOAM(theta, tab)
            r <- sqrt(c(stdize(tau, std, x)))
            for(j in 1 : 4)
                drawPie(r[j], angle.f[j], angle.t[j])
            ## upper
            theta <- or * exp(qnorm((1 + conf_level) / 2) * se)
            tau <- findTableWithOAM(theta, tab)
            r <- sqrt(c(stdize(tau, std, x)))
            for(j in 1 : 4)
                drawPie(r[j], angle.f[j], angle.t[j])
        }

        ## drawBoxes()
        grid.polygon(c(-1,  1, 1, -1),
                     c(-1, -1, 1,  1),
                     default.units = "native",
                     gp = gpar(fill = "transparent")
                     )
        grid.lines(c(-1, 1), c(0, 0), default.units = "native")
        for(j in seq(from = -0.8, to = 0.8, by = 0.2))
            grid.lines(c(j, j), c(-0.02, 0.02), default.units = "native")
        for(j in seq(from = -0.9, to = 0.9, by = 0.2))
            grid.lines(c(j, j), c(-0.01, 0.01), default.units = "native")
        grid.lines(c(0, 0), c(-1, 1), default.units = "native")
        for(j in seq(from = -0.8, to = 0.8, by = 0.2))
            grid.lines(c(-0.02, 0.02), c(j, j), default.units = "native")
        for(j in seq(from = -0.9, to = 0.9, by = 0.2))
            grid.lines(c(-0.01, 0.01), c(j, j), default.units = "native")

        popViewport(1)

    }

    if(!is.null(main) || !is.null(sub)) {
        if (!is.null(main))
          grid.text(main,
                    y = unit(1, "npc") + unit(1, "lines"),
                    gp = gpar(fontsize = 20, fontface = 2))
        if (!is.null(sub))
          grid.text(sub,
                    y = unit(0, "npc") - unit(1, "lines"),
                    gp = gpar(fontsize = 20, fontface = 2))
        popViewport(1)
      }

    popViewport(1)

    if (return_grob)
        return(invisible(grid.grab()))
    else
        return(invisible(NULL))
}

independence_table <- function(x, frequency = c("absolute", "relative")) {
  if (!is.array(x))
    stop("Need array of absolute frequencies!")
  frequency <- match.arg(frequency)

  n <- sum(x)
  x <- x / n
  d <- dim(x)

  ## build margins
  margins <- lapply(1:length(d), function(i) apply(x, i, sum))

  ## multiply all combinations & reshape
  tab <- array(apply(expand.grid(margins), 1, prod), d, dimnames = dimnames(x))
  
  if (frequency == "relative") tab else tab * n
}

mar_table <- function(x) {
  if(!is.matrix(x))
    stop("Function only defined for 2-way tables.")
  tab <- rbind(cbind(x, TOTAL = rowSums(x)), TOTAL = c(colSums(x), sum(x)))
  names(dimnames(tab)) <- names(dimnames(x))
  tab
}

table2d_summary <- function(object,
                            margins = TRUE,
                            percentages = FALSE,
                            conditionals = c("none", "row", "column"),
                            chisq.test = TRUE,
                            ...
                            )
{
  ret <- list()
  if (chisq.test)
      ret$chisq <- summary.table(object, ...)
  
  if(is.matrix(object)) {
    
    conditionals <- match.arg(conditionals)
  
    tab <- array(0, c(dim(object) + margins, 1 + percentages + (conditionals != "none")))

    ## frequencies
    tab[,,1] <- if(margins) mar_table(object) else object

    ## percentages
    if(percentages) {
      tmp <- prop.table(object)
      tab[,,2] <- 100 * if(margins) mar_table(tmp) else tmp
    }

    ## conditional distributions
    if(conditionals != "none") {
      tmp <- prop.table(object, margin = 1 + (conditionals == "column"))
      tab[,,2 + percentages] <- 100 * if(margins) mar_table(tmp) else tmp
    }

    ## dimnames
    dimnames(tab) <- c(dimnames(if(margins) mar_table(object) else object),
                       list(c("freq",
                              if(percentages) "%",
                              switch(conditionals, row = "row%", column = "col%")
                              )
                            )
                       )

    ## patch row% / col% margins
    if(conditionals == "row") 
      tab[dim(tab)[1],,2 + percentages] <- NA
    
    if(conditionals == "column")
      tab[,dim(tab)[2],2 + percentages] <- NA
    
    ret$table <- tab
  }    

  class(ret) <- "table2d_summary"
  ret
}

print.table2d_summary <- 
function (x, digits = max(1, getOption("digits") - 3), ...) 
{
  if (!is.null(x$table))
    if(dim(x$table)[3] == 1)
      print(x$table[,,1], digits = digits, ...)
    else
      print(ftable(aperm(x$table, c(1,3,2))), 2, digits = digits, ...)
  
  cat("\n")
  
  if (!is.null(x$chisq))
    print.summary.table(x$chisq, digits, ...)
  invisible(x)
}

################################################################
### strucplot - generic plot framework for mosaic-like layouts
### 2 core functions are provided: struc_mosaic and struc_assoc
################################################################

strucplot <- function(## main parameters
                      x,
                      residuals = NULL,
                      expected = NULL,
		      condvars = NULL,
                      shade = NULL,
                      type = c("observed", "expected"),
                      residuals_type = NULL,
                      df = NULL,

                      ## layout
                      split_vertical = NULL,
                      spacing = spacing_equal,
                      spacing_args = list(),
                      gp = NULL,
		      gp_args = list(),
                      labeling = labeling_border,
                      labeling_args = list(),
                      core = struc_mosaic,
                      core_args = list(),
                      legend = NULL,
                      legend_args = list(),

                      main = NULL,
                      sub = NULL,
                      margins = unit(3, "lines"),
                      title_margins = NULL,
                      legend_width = NULL,

                      ## control parameters
                      main_gp = gpar(fontsize = 20),
                      sub_gp = gpar(fontsize = 15),
                      newpage = TRUE,
                      pop = TRUE,
                      return_grob = FALSE,
                      keep_aspect_ratio = NULL,
                      prefix = "",
                      ...
                      ) {
  ## default behaviour of shade
  if (is.null(shade)) shade <- !is.null(gp) || !is.null(expected)

  type <- match.arg(type)
  if (is.null(residuals)) {
      residuals_type <- if (is.null(residuals_type))
          "pearson"
       else
           match.arg(tolower(residuals_type),
                     c("pearson", "deviance", "ft"))
  } else {
      if (is.null(residuals_type))
          residuals_type <- ""
  }

  ## convert structable object
  if (is.structable(x)) {
    if (is.null(split_vertical))
      split_vertical <- attr(x, "split_vertical")
    x <- as.table(x)
  }
  if (is.null(split_vertical))
    split_vertical <- FALSE

  ## table characteristics
  d <- dim(x)
  dl <- length(d)
  dn <- dimnames(x)
  if (is.null(dn))
    dn <- dimnames(x) <- lapply(d, seq)
  dnn <- names(dimnames(x))
  if (is.null(dnn))
    dnn <- names(dn) <- names(dimnames(x)) <- LETTERS[1:dl]

  ## replace NAs by 0
  if (any(nas <- is.na(x))) x[nas] <- 0

  ## model fitting:
  ## calculate df and expected if needed
  ## (used for inference in some shading (generating) functions).
  ## note: will *not* be calculated if residuals are given
  if ((is.null(expected) && is.null(residuals)) ||
      !is.numeric(expected)) {
      if (!is.null(df))
          warning("Using calculated degrees of freedom.")
      if (inherits(expected, "formula")) {
          fm <- loglm(expected, x, fitted = TRUE)
          expected <- fitted(fm)
          df <- fm$df
      } else {
          if (is.null(expected))
              expected <- if (is.null(condvars))
                  as.list(1:dl)
              else
                  lapply((condvars + 1):dl, c, seq(condvars))

          fm <- loglin(x, expected, fit = TRUE, print = FALSE)
          expected <- fm$fit
          df <- fm$df
      }
  }

  ## compute residuals
  if (is.null(residuals))
    residuals <- switch(residuals_type,
                        pearson = (x - expected) / sqrt(ifelse(expected > 0, expected, 1)),
                        deviance = {
                          tmp <- 2 * (x * log(ifelse(x == 0, 1, x / ifelse(expected > 0, expected, 1))) - (x - expected))
                          tmp <- sqrt(pmax(tmp, 0))
                          ifelse(x > expected, tmp, -tmp)
                        },
                        ft = sqrt(x) + sqrt(x + 1) - sqrt(4 * expected + 1)
                        )
  ## replace NAs by 0
  if (any(nas <- is.na(residuals))) residuals[nas] <- 0

  ## splitting
  if (length(split_vertical) == 1)
    split_vertical <- rep(c(split_vertical, !split_vertical), length.out = dl)

  if (is.null(keep_aspect_ratio))
    keep_aspect_ratio <- dl < 3

  ## spacing
  if (is.function(spacing)) {
    if (inherits(spacing, "grapcon_generator"))
      spacing <- do.call("spacing", spacing_args)
    spacing <- spacing(d, condvars)
  }

  ## gp (color, fill, lty, etc.) argument
  if (shade) {
    if (is.null(gp)) gp <- shading_hcl
    if (is.function(gp)) {
      if (is.null(legend) || (is.logical(legend) && legend))
        legend <- legend_resbased
      gpfun <- if (inherits(gp, "grapcon_generator"))
        do.call("gp", c(list(x, residuals, expected, df), as.list(gp_args))) else gp
      gp <- gpfun(residuals)
    } else if (!is.null(legend) && !(is.logical(legend) && !legend))
      stop("gp argument must be a shading function for drawing a legend")
  } else {
    if(!is.null(gp)) {
      warning("gp parameter ignored since shade = FALSE")
      gp <- NULL
    }
  }

  ## choose gray when no shading is used
  if (is.null(gp)) gp <- gpar(fill = grey(0.8))

  ## recycle gpar values in the *first* dimension
  size <- prod(d)
  FUN <- function(par) {
    if (is.structable(par))
      par <- as.table(par)
    if (length(par) < size || is.null(dim(par)))
        array(par, dim = d)
    else
        par
  }
  gp <- structure(lapply(gp, FUN), class = "gpar")

  ## set up page
  if (newpage)
      grid.newpage()
  if (keep_aspect_ratio)
      pushViewport(viewport(width = 1, height = 1, default.units = "snpc"))

  pushViewport(vcdViewport(mar = margins,
                           oma = title_margins,
                           legend = shade && !(is.null(legend) || is.logical(legend) && !legend),
                           main = !is.null(main), sub = !is.null(sub),
                           keep_aspect_ratio = keep_aspect_ratio,
                           legend_width = legend_width,
                           prefix = prefix))

  ## legend
  if (inherits(legend, "grapcon_generator"))
    legend <- do.call("legend", legend_args)
  if (shade && !is.null(legend) && !(is.logical(legend) && !legend)) {
    seekViewport(paste(prefix, "legend", sep = ""))
    residuals_type <- switch(residuals_type,
                             deviance = "deviance\nresiduals:",
                             ft = "Freeman-Tukey\nresiduals:",
                             pearson = "Pearson\nresiduals:",
                             residuals_type)
    legend(residuals, gpfun, residuals_type)
  }

  ## titles
  if (!is.null(main)) {
    seekViewport(paste(prefix, "main", sep = ""))
    if (is.logical(main) && main)
      main <- deparse(substitute(x))
    grid.text(main, gp = main_gp)
  }

  if (!is.null(sub)) {
    seekViewport(paste(prefix, "sub", sep = ""))
    if (is.logical(sub) && sub && is.null(main))
      sub <- deparse(substitute(x))
    grid.text(sub, gp = sub_gp)
  }

  ## make plot
  seekViewport(paste(prefix, "plot", sep = ""))

  if (inherits(core, "grapcon_generator"))
    core <- do.call("core", core_args)
  core(residuals = residuals,
       observed = if (type == "observed") x else expected,
       expected = if (type == "observed") expected else x,
       spacing = spacing,
       gp = gp,
       split_vertical = split_vertical,
       prefix = prefix)

  upViewport(1)

  ## labels
  if (is.logical(labeling))
    labeling <- if (labeling) labeling_border else NULL
  if (!is.null(labeling)) {
    if (inherits(labeling, "grapcon_generator"))
      labeling <- do.call("labeling", c(labeling_args, list(...)))
    labeling(dn, split_vertical, condvars, prefix)
  }

  ## pop/move up viewport

  seekViewport(paste(prefix, "base", sep = ""))
  ## one more up if sandwich-mode
  if (pop)
      popViewport(1 + keep_aspect_ratio)
  else
      upViewport(1 + keep_aspect_ratio)

  ## return visualized table
  if (return_grob)
      invisible(structure(structable(if (type == "observed") x else expected,
                                     split_vertical = split_vertical),
                          grob = grid.grab()
                          )
                )
  else
      invisible(structable(if (type == "observed") x else expected,
                           split_vertical = split_vertical))
}

vcdViewport <- function(mar = rep.int(2.5, 4),
                        legend_width = unit(5, "lines"),
                        oma = NULL,
                        legend = FALSE, main = FALSE, sub = FALSE,
                        keep_aspect_ratio = TRUE,
                        prefix = "")
{
  ## process parameters
  if (is.null(legend_width))
    legend_width <- unit(5 * legend, "lines")
  if (!is.unit(legend_width))
    legend_width <- unit(legend_width, "lines")

  if (legend && !main && !sub && keep_aspect_ratio) main <- sub <- TRUE
  mar <- if (!is.unit(mar))
    unit(pexpand(mar, 4, rep.int(2.5, 4), c("top","right","bottom","left")), "lines")
  else
    rep(mar, length.out = 4)
  if (is.null(oma)) {
    space <- if (legend && keep_aspect_ratio)
      legend_width + mar[2] + mar[4] - mar[1] - mar[3]
    else unit(0, "lines")
    oma <- if (main && sub)
      max(unit(2, "lines"), 0.5 * space)
    else if (main)
      unit.c(max(unit(2, "lines"), space), unit(0, "lines"))
    else if (sub)
      unit.c(unit(0, "lines"), max(unit(2, "lines"), space))
    else
      0.5 * space
  }
  oma <- if (!is.unit(oma))
    unit(pexpand(oma, 2, rep.int(2, 2), c("top","bottom")), "lines")
  else
    rep(oma, length.out = 2)

  ## set up viewports
  vpPlot <- vpStack(viewport(layout.pos.col = 2, layout.pos.row = 3),
                    viewport(width = 1, height = 1, name = paste(prefix, "plot", sep = ""),
                             default.units = if (keep_aspect_ratio) "snpc" else "npc"))
  vpMarginBottom <- viewport(layout.pos.col = 2, layout.pos.row = 4,
                             name = paste(prefix, "margin_bottom", sep = ""))
  vpMarginLeft <- viewport(layout.pos.col = 1, layout.pos.row = 3,
                           name = paste(prefix, "margin_left", sep = ""))
  vpMarginTop <- viewport(layout.pos.col = 2, layout.pos.row = 2,
                          name = paste(prefix, "margin_top", sep = ""))
  vpMarginRight <- viewport(layout.pos.col = 3, layout.pos.row = 3,
                            name = paste(prefix, "margin_right", sep = ""))
  vpCornerTL <- viewport(layout.pos.col = 1, layout.pos.row = 2,
                         name = paste(prefix, "corner_top_left", sep = ""))
  vpCornerTR <- viewport(layout.pos.col = 3, layout.pos.row = 2,
                         name = paste(prefix, "corner_top_right", sep = ""))
  vpCornerBL <- viewport(layout.pos.col = 1, layout.pos.row = 4,
                         name = paste(prefix, "corner_bottom_left", sep = ""))
  vpCornerBR <- viewport(layout.pos.col = 3, layout.pos.row = 4,
                         name = paste(prefix, "corner_bottom_right", sep = ""))

  vpLegend <- viewport(layout.pos.col = 4, layout.pos.row = 3,
                       name = paste(prefix, "legend", sep = ""))
  vpLegendTop <- viewport(layout.pos.col = 4, layout.pos.row = 2,
                          name = paste(prefix, "legend_top", sep = ""))
  vpLegendSub <- viewport(layout.pos.col = 4, layout.pos.row = 4,
                          name = paste(prefix, "legend_sub", sep = ""))
  vpBase <- viewport(layout = grid.layout(5, 4,
                       widths = unit.c(mar[4], unit(1, "null"), mar[2], legend_width),
                       heights = unit.c(oma[1], mar[1], unit(1, "null"), mar[3], oma[2])),
                     name = paste(prefix, "base", sep = ""))
  vpMain <- viewport(layout.pos.col = 1:4, layout.pos.row = 1,
                     name = paste(prefix, "main", sep = ""))
  vpSub <- viewport(layout.pos.col = 1:4, layout.pos.row = 5,
                    name = paste(prefix, "sub", sep = ""))

  vpTree(vpBase, vpList(vpMain, vpMarginBottom, vpMarginLeft, vpMarginTop,
                        vpMarginRight, vpLegendTop, vpLegend,
                        vpLegendSub, vpCornerTL, vpCornerTR,
                        vpCornerBL, vpCornerBR, vpPlot, vpSub))
}

# added lwd arg, changed default point sizes

distplot <-
    function(x, type = c("poisson", "binomial", "nbinomial"),
             size = NULL, lambda = NULL, legend = TRUE, xlim = NULL, ylim = NULL,
             conf_int = TRUE, conf_level = 0.95, main = NULL,
             xlab = "Number of occurrences", ylab = "Distribution metameter",
             gp = gpar(cex = 0.8), lwd=2, gp_conf_int = gpar(lty = 2),
             name = "distplot", newpage = TRUE,
             pop = TRUE, return_grob = FALSE, ...)
{
    if(is.vector(x)) {
        x <- table(x)
    }
    if(is.table(x)) {
        if(length(dim(x)) > 1) stop ("x must be a 1-way table")
        freq <- as.vector(x)
        count <- as.numeric(names(x))
    } else {
        if(!(!is.null(ncol(x)) && ncol(x) == 2))
            stop("x must be a 2-column matrix or data.frame")
        freq <- as.vector(x[,1])
        count <- as.vector(x[,2])
    }

    myindex <- (1:length(freq))[freq > 0]
    mycount <- count[myindex]
    myfreq <- freq[myindex]

    switch(match.arg(type),

           "poisson" = {
               par.ml <- suppressWarnings(goodfit(x, type = type)$par$lambda)

               phi <- function(nk, k, N, size = NULL)
                   ifelse(nk > 0, lgamma(k + 1) + log(nk/N), NA)
               y <- phi(myfreq, mycount, sum(freq))
               if(!is.null(lambda)) y <- y + lambda - mycount * log(lambda)
               fm <- lm(y ~ mycount)
               par.estim <- exp(coef(fm)[2])
               names(par.estim) <- "lambda"
               txt <- "exp(slope)"
               if(!is.null(lambda)) {
                   par.estim <- par.estim * lambda
                   txt <- paste(txt, "x lambda")
               }
               legend.text <- paste(txt, "=", round(par.estim, digits = 3))
               if(is.null(main)) main <- "Poissoness plot"
  },

           "binomial" = {
               if(is.null(size)) {
                   size <- max(count)
                   warning("size was not given, taken as maximum count")
               }
               par.ml <- suppressWarnings(goodfit(x, type = type, par = list(size = size))$par$prob)

               phi <- function(nk, k, N, size = NULL)
                   log(nk) - log(N * choose(size, k))
               y <- phi(myfreq, mycount, sum(freq), size = size)
               fm <- lm(y ~ mycount)
               par.estim <- exp(coef(fm)[2])
               par.estim <- par.estim / (1 + par.estim)
               names(par.estim) <- "prob"
               legend.text <- paste("inv.logit(slope) =", round(par.estim, digits = 3))
               if(is.null(main)) main <- "Binomialness plot"
           },

           "nbinomial" = {
               if(is.null(size)) {
                   par.ml <- suppressWarnings(goodfit(x, type = type)$par)
                   size <- par.ml$size
                   par.ml <- par.ml$prob
               }else{
                   xbar <- weighted.mean(mycount, myfreq)
                   par.ml <- size / (size+xbar)
               }
               phi <- function(nk, k, N, size = NULL)
                   log(nk) - log(N * choose(size + k - 1, k))
               y <- phi(myfreq, mycount, sum(freq), size = size)
               fm <- lm(y ~ mycount)
               par.estim <- 1 - exp(coef(fm)[2])
               names(par.estim) <- "prob"
               legend.text <- paste("1-exp(slope) =", round(par.estim, digits = 3))
               if(is.null(main)) main <- "Negative binomialness plot"
           })

    yhat <- ifelse(myfreq > 1.5, myfreq - 0.67, 1/exp(1))
    yhat <- phi(yhat, mycount, sum(freq), size = size)
    if(!is.null(lambda)) yhat <- yhat + lambda - mycount * log(lambda)

    phat <- myfreq / sum(myfreq)
    ci.width <- qnorm(1-(1 - conf_level)/2) *
        sqrt(1-phat)/sqrt(myfreq - (0.25 * phat + 0.47)*sqrt(myfreq))

    RVAL <- cbind(count, freq, NA, NA, NA, NA, NA)
    RVAL[myindex,3:7] <- cbind(y,yhat,ci.width, yhat-ci.width, yhat + ci.width)
    RVAL <- as.data.frame(RVAL)
    names(RVAL) <- c("Counts", "Freq", "Metameter", "CI.center",
                     "CI.width", "CI.lower", "CI.upper")

    if(is.null(xlim)) xlim <- range(RVAL[,1])
    if(is.null(ylim)) ylim <- range(RVAL[,c(3,6,7)], na.rm = TRUE)
    xlim <- xlim + c(-1, 1) * diff(xlim) * 0.04
    ylim <- ylim + c(-1, 1) * diff(ylim) * 0.04

    if(newpage) grid.newpage()
    pushViewport(plotViewport(xscale = xlim, yscale = ylim, default.units = "native", name = name))
    grid.points(x = RVAL[,1], y = RVAL[,3], default.units = "native", gp = gp, ...)
    grid.lines(x = xlim, y = predict(fm, newdata = data.frame(mycount = xlim)),
               default.units = "native", gp = gpar(lwd=lwd, col = 2))
    grid.rect(gp = gpar(fill = "transparent"))
    grid.xaxis()
    grid.yaxis()
    grid.text(xlab, y = unit(-3.5, "lines"))
    grid.text(ylab, x = unit(-3, "lines"), rot = 90)
    grid.text(main, y = unit(1, "npc") + unit(2, "lines"), gp = gpar(fontface = "bold"))

    if(conf_int) {
        grid.points(x = RVAL[,1], y = RVAL[,4], pch = 19, gp = gpar(cex = 0.5))
        grid.segments(RVAL[,1], RVAL[,6], RVAL[,1], RVAL[,7], default.units = "native", gp = gp_conf_int)
    }

    if(legend) {
        mymin <- which.min(RVAL[,5])
        leg.x <- RVAL[mymin,1]
        if(RVAL[mymin,6] - ylim[1] > ylim[2] - RVAL[mymin,7])
            leg.y <- ylim[1] + 0.7 * (RVAL[mymin,6] - ylim[1])
        else leg.y <- ylim[2]

        legend.text <- c(paste("slope =", round(coef(fm)[2], digits = 3)),
                         paste("intercept =", round(coef(fm)[1], digits = 3)),
                         "", paste(names(par.estim),": ML =", round(par.ml, digits=3)),
                         legend.text)
        legend.text <- paste(legend.text, collapse = "\n")
        grid.text(legend.text, leg.x, leg.y - 0.05 * abs(leg.y), default.units = "native", just = c("left", "top"))
    }

    if(pop) popViewport() else upViewport()

    if (return_grob)
        structure(invisible(RVAL),
                  grob = grid.grab())
    else
        invisible(RVAL)
}

"ternaryplot" <-
function (x,
          scale = 1,
          dimnames = NULL,
          dimnames_position = c("corner", "edge", "none"),
          dimnames_color = "black",
          dimnames_rot = c(-60, 60, 0),
          id = NULL,
          id_color = "black",
          id_just = c("center", "center"),
          coordinates = FALSE,
          grid = TRUE,
          grid_color = "gray",
          labels = c("inside", "outside", "none"),
          labels_color = "darkgray",
          labels_rot = c(120, -120, 0),
          border = "black",
          bg = "white",
          pch = 19,
          cex = 1,
          prop_size = FALSE,
          col = "red",
          main = "ternary plot",
          newpage = TRUE,
	  pop = TRUE,
          return_grob = FALSE,
          ...)
{
  ## parameter handling
    labels <- match.arg(labels)
  if (grid == TRUE) grid <- "dotted"

  if (coordinates)
    id <- paste("(",round(x[,1] * scale, 1),",",
                    round(x[,2] * scale, 1),",",
                    round(x[,3] * scale, 1),")", sep="")

  dimnames_position <- match.arg(dimnames_position)
  if(is.null(dimnames) && dimnames_position != "none")
    dimnames <- colnames(x)

  if(is.logical(prop_size) && prop_size) prop_size <- 3

  ## some error handling
  if(ncol(x) != 3)
    stop("Need a matrix with 3 columns")
  if(any(x < 0)) stop("X must be non-negative")
  s <- rowSums(x)
  if(any(s <= 0)) stop("each row of X must have a positive sum")

  ## rescaling
  x <- x / s

  ## prepare plot
  top <- sqrt(3) / 2
  if (newpage) grid.newpage()
  xlim <- c(-0.03, 1.03)
  ylim <- c(-1, top)
  pushViewport(viewport(width = unit(1, "snpc")))
  if (!is.null(main))
    grid.text(main, y = 0.9, gp = gpar(fontsize = 18, fontstyle = 1))
  pushViewport(viewport(width = 0.8, height = 0.8,
                        xscale = xlim, yscale = ylim, name = "plot"))
  eps <- 0.01

  ## coordinates of point P(a,b,c): xp = b + c/2, yp = c * sqrt(3)/2

  ## triangle
  grid.polygon(c(0, 0.5, 1), c(0, top, 0), gp = gpar(fill = bg, col = border), ...)

  ## title, labeling
  if (dimnames_position == "corner") {
    grid.text(x = c(0, 1, 0.5), y = c(-0.02, -0.02, top + 0.02),
              label = dimnames, gp = gpar(fontsize = 12))
  }
  if (dimnames_position == "edge") {
    shift <- eps * if (labels == "outside") 8 else 0
    grid.text(x = 0.25 - 2 * eps - shift, y = 0.5 * top + shift,
              label = dimnames[2], rot = dimnames_rot[2], gp = gpar(col = dimnames_color))
    grid.text(x = 0.75 + 3 * eps + shift, y = 0.5 * top + shift,
              label = dimnames[1], rot = dimnames_rot[1], gp = gpar(col = dimnames_color))
    grid.text(x = 0.5, y = -0.02 - shift,
              label = dimnames[3], rot = dimnames_rot[3], gp = gpar(col = dimnames_color))
  }

  ## grid
  if (is.character(grid))
    for (i in 1:4 * 0.2) {
      ## a - axis
      grid.lines(c(1 - i , (1 - i) / 2), c(0, 1 - i) * top,
                 gp = gpar(lty = grid, col = grid_color))
      ## b - axis
      grid.lines(c(1 - i , 1 - i + i / 2), c(0, i) * top,
                 gp = gpar(lty = grid, col = grid_color))
      ## c - axis
      grid.lines(c(i / 2, 1 - i + i/2), c(i, i) * top,
                 gp = gpar(lty = grid, col = grid_color))

      ## grid labels
      if (labels == "inside") {
        grid.text(x = (1 - i) * 3 / 4 - eps, y = (1 - i) / 2 * top, label = i * scale,
              gp = gpar(col = labels_color), rot = labels_rot[1])
        grid.text(x = 1 - i + i / 4 + eps, y = i / 2 * top - eps, label = (1 - i) * scale,
              gp = gpar(col = labels_color), rot = labels_rot[2])
        grid.text(x = 0.5, y = i * top + eps, label = i * scale,
                  gp = gpar(col = labels_color), rot = labels_rot[3])
      }
      if (labels == "outside") {
        grid.text(x = (1 - i) / 2 - 6 * eps, y = (1 - i) * top,
                  label = (1 - i) * scale, rot = labels_rot[3], gp = gpar(col = labels_color))
        grid.text(x = 1 - (1 - i) / 2 + 3 * eps, y = (1 - i) * top + 5 * eps,
                  label = i * scale, rot = labels_rot[2], gp = gpar(col = labels_color))
        grid.text(x = i + eps, y = -0.05, label = (1 - i) * scale, vjust = 1,
                  rot = labels_rot[1], gp = gpar(col = labels_color))
      }
    }


  ## plot points
  xp <- x[,2] + x[,3] / 2
  yp <- x[,3] * top
  size = unit(if(prop_size) prop_size * (s / max(s)) else cex, "lines")
  grid.points(xp, yp, pch = pch, gp = gpar(col = col), default.units = "snpc",
              size = size, ...)

  ## plot
  if (!is.null(id))
    grid.text(x = xp,
              y = unit(yp - 0.015, "snpc") - 0.5 * size,
              label = as.character(id),
              just = id_just,
              gp = gpar(col = id_color, cex = cex))

  ## cleanup
  if(pop) popViewport(2) else upViewport(2)
  if (return_grob)
      invisible(grid.grab())
  else
      invisible(NULL)
}

odds <- function(x, log = FALSE, ...)
	lodds(x, log = log, ...)


lodds <- function(x, ...)
    UseMethod("lodds")

lodds.formula <-
    function(formula, data = NULL, ..., subset = NULL, na.action = NULL)
{
    m <- match.call(expand.dots = FALSE)
    edata <- eval(m$data, parent.frame())

    fstr <- strsplit(paste(deparse(formula), collapse = ""), "~")
    vars <- strsplit(strsplit(gsub(" ", "", fstr[[1]][2]), "\\|")[[1]], "\\+")
    varnames <- vars[[1]]

    condnames <- if (length(vars) > 1) vars[[2]] else NULL

    dep <- gsub(" ", "", fstr[[1]][1])
    if (!dep %in% c("","Freq")) {
        if (all(varnames == ".")) {
            varnames <- if (is.data.frame(data))
                            colnames(data)
                        else
                            names(dimnames(as.table(data)))
            varnames <- varnames[-which(varnames %in% dep)]
        }

        varnames <- c(dep, varnames)
    }


    if (inherits(edata, "ftable") || inherits(edata, "table") || length(dim(edata)) > 2) {
        condind <- NULL
        dat <- as.table(data)
        if(all(varnames != ".")) {
            ind <- match(varnames, names(dimnames(dat)))
            if (any(is.na(ind)))
                stop(paste("Can't find", paste(varnames[is.na(ind)], collapse=" / "), "in", deparse(substitute(data))))

            if (!is.null(condnames)) {
                condind <- match(condnames, names(dimnames(dat)))
                if (any(is.na(condind)))
                    stop(paste("Can't find", paste(condnames[is.na(condind)], collapse=" / "), "in", deparse(substitute(data))))
                ind <- c(ind, condind)
            }
            dat <- margin.table(dat, ind)
        }
        lodds.default(dat, strata = if (is.null(condind)) NULL else match(condnames, names(dimnames(dat))), ...)
    } else {
        m <- m[c(1, match(c("formula", "data", "subset", "na.action"), names(m), 0))]
        m[[1]] <- as.name("xtabs")
        m$formula <-
            formula(paste(if("Freq" %in% colnames(data)) "Freq",
                          "~",
                          paste(c(varnames, condnames), collapse = "+")))
        tab <- eval(m, parent.frame())
        lodds.default(tab, ...)
    }
}

lodds.default <- function(x, response = NULL, strata = NULL, log = TRUE,
                                  ref = NULL, correct = any(x == 0), ...)
{
    ## check dimensions
    L <- length(d <- dim(x))
    if(any(d < 2L)) stop("All table dimensions must be 2 or greater")

		## assign and check response and stata; convert variable names to indices
    if (is.null(response)) {
    	if (is.null(strata)) {
    		response <- 1
    		strata <- setdiff(1:L, response)
    	}
    	else {   # only strata was specified
    		if(L - length(strata) != 1L) stop("All but 1 dimension must be specified as strata.")
    		if(is.character(strata)) strata <- which(names(dimnames(x)) == strata)
    		response <- setdiff(1:L, strata)
    	}
    }
    else {  # response was specified; take strata as the complement
    	if(length(response) > 1) stop("Only 1 dimension can be specified as a response")
    	if(is.character(response)) response <- which(names(dimnames(x)) == response)
    	if (!is.null(strata))
	    	warning(paste("strata =", paste(strata, collapse=","), "ignored when response has been specified"))
    	strata <- setdiff(1:L, response)
	  }

    ## dimensions of primary R x 1 table  ### [Or should this just be a vector???]
    dp <- if (length(strata)) d[response] else d
    dn <- if (length(strata)) dimnames(x)[response] else dimnames(x)
    R <- dp[1]
    C <- 1
                                        # shadow matrix with proper dimnames
    X <- matrix(0, R, C, dimnames=dn)

    ## process reference category
    if(!is.null(ref)) {
        if(is.character(ref)) {
            ref <- match(ref, colnames(x))
        } else if(is.numeric(ref)) {
            ref <- as.integer(ref)
        } else {
            stop("Wrong 'ref=' argument!")
        }
    }

    ## compute corresponding indices
    compute_index <- function(n, ref) {
        if(is.null(ref)) return(cbind(1:(n-1), 2:n))
        rval <- cbind(ref, 1:n)
        d <- rval[,2L] - rval[,1L]
        rval <- rbind(
            rval[d > 0, 1:2],
            rval[d < 0, 2:1]
        )
        return(rval[order(rval[,1L]),,drop = FALSE])
    }
    Rix <- compute_index(R, ref[[1L]])
    contr <- matrix(0L, nrow = (R-1), ncol = R)
    colnames(contr) <- rownames(X)
    rownames(contr) <- rep("", R-1)
    for(i in 1:(R-1)) {
        rix <-  i
        cix <- Rix[i,]
        contr[rix, cix] <- c(1L, -1L)
        rownames(contr)[rix] <-  paste(rownames(X)[Rix[i,]], collapse = ":")
    }

    ## handle strata
    if (!is.null(strata)) {
  	if (length(strata)==1) {
            sn <- dimnames(x)[[strata]]
        }
        else {
            sn <- apply(expand.grid(dimnames(x)[strata]), 1, paste, collapse = ":")
        }
        rn <- as.vector(outer( dimnames(contr)[[1]], sn, paste, sep='|'))
        cn <- as.vector(outer( dimnames(contr)[[2]], sn, paste, sep='|'))
        contr <- kronecker(diag(prod(dim(x)[strata])), contr)
        rownames(contr) <- rn
        colnames(contr) <- cn
    }

    ## dimnames for array version
    dn <- list(rep("", R-1))
    for(i in 1:(R-1)) dn[[1]][i] <- paste(rownames(X)[Rix[i,]], collapse = ":")
    if (!is.null(strata)) dn <- c(dn, dimnames(x)[strata])
    ndn <- names(dimnames(x))
    if (!is.null(names(dimnames(x)))) names(dn) <- c(ndn[response], ndn[strata])

    ## point estimates

    if (is.logical(correct)) {
        add <- if(correct) 0.5 else 0
    }
    else if(is.numeric(correct)) {
        add <- as.vector(correct)
        if (length(add) != length(x))
            stop("array size of 'correct' does not conform to the data")
    }
    else stop("correct is not valid")

    ## reorder columns of contrast matrix to match original data
    contr <- contr[, order(as.vector(aperm(array(seq.int(prod(d)), d), c(response, strata))))]

    ##coef <- drop(contr %*% log(as.vector(x) + add))
    ##FIXME: 0 cells mess up the matrix product, try workaround:
    mat <- log(as.vector(x) + add) * t(contr)
    nas <- apply(contr != 0 & is.na(t(mat)), 1, any)
    coef <- apply(mat, 2, sum, na.rm = TRUE)
    coef[nas] <- NA
    ## covariances
    ##vcov <- crossprod(diag(sqrt(1/(as.vector(x) + add))) %*% t(contr))
    tmp <- sqrt(1/(as.vector(x) + add)) * t(contr)
    tmp[is.na(tmp)] <- 0
    vcov <- crossprod(tmp)
    vcov[nas,] <- NA
    vcov[,nas] <- NA

    rval <- structure(list(
        response = response,
        strata = strata,
        coefficients = coef,
        dimnames = dn,
        dim = as.integer(sapply(dn, length)),
        vcov = vcov,
        contrasts = contr,
        log = log
    ), class = "lodds")
    rval
}

## ----------------  Methods -------------------

summary.lodds <- function(object, ...)
    lmtest::coeftest(object, ...)

## dim methods
dimnames.lodds <- function(x, ...) x$dimnames
dim.lodds <- function(x, ...) x$dim

## t/aperm-methods
t.lodds <- function(x)
    aperm(x)

### FIXME
aperm.lodds <- function(a, perm = NULL, ...)
{
    d <- length(a$dim)
    if(is.null(perm)) {
        perm <- if (d < 3) 2L : 1L else c(2L : 1L, d : 3L)
    } else {
        if (any(perm[1:2] > 2L) || (d > 2L) && any(perm[-c(1:2)] < 2L))
            stop("Mixing of strata and non-strata variables not allowed!")
    }
    nams <- names(a$coefficients)
    a$coefficients <- as.vector(aperm(array(a$coef, dim = a$dim),
                                      perm, ...))
    nams <- as.vector(aperm(array(nams, dim = a$dim), perm, ...))
    names(a$coefficients) <- nams
    a$dimnames <- a$dimnames[perm]
    a$dim <- a$dim[perm]
    a$vcov <- a$vcov[nams, nams]
    a$contrasts <- a$contrasts[nams,]
    a
}


## straightforward methods
coef.lodds <- function(object, log = object$log, ...)
    if(log) object$coefficients else exp(object$coefficients)

vcov.lodds <- function(object, log = object$log, ...)
    if(log) object$vcov else `diag<-`(object$vcov, diag(object$vcov) * exp(object$coefficients)^2)

confint.lodds <-
    function(object, parm, level = 0.95, log = object$log, ...) {
        if (log) confint.default(object, parm = parm, level = level, ... )
        else {
            object$log = TRUE
            exp(confint.default(object, parm = parm, level = level, ... ))
        }
    }


### DONE:
##  The header should say:
#    (log) odds for vn[response] by ... all the rest (strata)
#   Fixed:  clash with make_header in loddsratio
make_header_odds <- function(x)
{
    vn <- names(dimnames(x))
    resp <- vn[x$response]
    strat <- paste(vn[x$strata], collapse=", ")
    header <- c(if(x$log) "log" else "",
                "odds for", resp, "by", strat,
#                if (length(vn)>2) c("by", paste(vn[-(1:2)], collapse=', ')),
                "\n\n")
    paste(header, sep = " ")
}

## print method
print.lodds <- function(x, log = x$log, ...) {
    cat(make_header_odds(x))
    print(drop(array(coef(x, log = log), dim = dim(x), dimnames = dimnames(x)), ...))
    invisible(x)
}

## as.data.frame
#as.data.frame.lodds <-
#    function(x, ...)
#        as.data.frame.table(vcd:::as.array.loddsratio(x), ...)

## Q:  I don't understand the purpose of the row.names and optional arguments
## DM: The generic has them, so each method must have them, too
as.data.frame.lodds <- function(x, row.names = NULL, optional = FALSE, log=x$log, ...) {
    df <-data.frame(expand.grid(dimnames(x)),
                    logodds = coef(x, log = log),
                    ASE = sqrt(diag(vcov(x, log = log))), row.names = row.names, ...
                    )
    if (!log) colnames(df)[ncol(df) - 1] <- "odds"
    df
}

## FIXME
## reshape coef() methods
as.matrix.lodds <- function (x, log=x$log, ...) {
    ## Coef <- coef(x, log = log)
    ## if (length(dim(x))==2) matrix(Coef, ncol = dim(x)[2], dimnames=dimnames(x))
    ## else {  # drop leading dimensions with length 1, then reshape
    ##     ddim <- which(dim(x)[1:2]==1)
    ##     dim(Coef) <- dim(x)[-ddim]
    ##     dimnames(Coef) <- dimnames(x)[-ddim]
    ##     if (length(dim(Coef))==1) Coef
    ##     else
    ##         matrix(Coef, ncol = prod(dim(Coef)[-1]),
    ##                dimnames=list(dimnames(Coef)[[1]], apply(expand.grid(dimnames(Coef)[[-1]]), 1, paste, collapse = ":")))
    ## }
    as.array(x, log = log, ...)
}


as.array.lodds <- function (x, log=x$log, ...) {
    res <- array(coef(x, log = log), dim = dim(x), dimnames=dimnames(x))
    drop(res)
}

###########################################################
## mosaicplot

mosaic <- function(x, ...)
    UseMethod("mosaic")

mosaic.formula <-
    function(formula, data = NULL, highlighting = NULL,
             ..., main = NULL, sub = NULL, subset = NULL, na.action = NULL)
{
    if (is.logical(main) && main)
        main <- deparse(substitute(data))
    else if (is.logical(sub) && sub)
        sub <- deparse(substitute(data))

    m <- match.call(expand.dots = FALSE)
    edata <- eval(m$data, parent.frame())

    fstr <- strsplit(paste(deparse(formula), collapse = ""), "~")
    vars <- strsplit(strsplit(gsub(" ", "", fstr[[1]][2]), "\\|")[[1]], "\\+")
    varnames <- vars[[1]]

    condnames <- if (length(vars) > 1) vars[[2]] else NULL

    dep <- gsub(" ", "", fstr[[1]][1])
    if (is.null(highlighting) && (!dep %in% c("","Freq"))) {
        if (all(varnames == ".")) {
            varnames <- if (is.data.frame(data))
                            colnames(data)
                        else
                            names(dimnames(as.table(data)))
            varnames <- varnames[-which(varnames %in% dep)]
        }

        varnames <- c(varnames, dep)
        highlighting <- length(varnames) + length(condnames)
    }


    if (inherits(edata, "ftable") || inherits(edata, "table") || length(dim(edata)) > 2) {
        condind <- NULL
        dat <- as.table(data)
        if(all(varnames != ".")) {
            ind <- match(varnames, names(dimnames(dat)))
            if (any(is.na(ind)))
                stop(paste("Can't find", paste(varnames[is.na(ind)], collapse=" / "), "in", deparse(substitute(data))))

            if (!is.null(condnames)) {
                condind <- match(condnames, names(dimnames(dat)))
                if (any(is.na(condind)))
                    stop(paste("Can't find", paste(condnames[is.na(condind)], collapse=" / "), "in", deparse(substitute(data))))
                ind <- c(condind, ind)
            }
            dat <- margin.table(dat, ind)
        }
        mosaic.default(dat, main = main, sub = sub, highlighting = highlighting,
                       condvars = if (is.null(condind)) NULL else match(condnames, names(dimnames(dat))), ...)
    } else {
        m <- m[c(1, match(c("formula", "data", "subset", "na.action"), names(m), 0))]
        m[[1]] <- as.name("xtabs")
        m$formula <-
            formula(paste(if("Freq" %in% colnames(data)) "Freq",
                          "~",
                          paste(c(condnames, varnames), collapse = "+")))
        tab <- eval(m, parent.frame())
        mosaic.default(tab, main = main, sub = sub, highlighting = highlighting, ...)
    }
}

mosaic.default <- function(x, condvars = NULL,
                           split_vertical = NULL, direction = NULL,
                           spacing = NULL, spacing_args = list(),
                           gp = NULL, expected = NULL, shade = NULL,
                           highlighting = NULL,
                           highlighting_fill = rev(gray.colors(tail(dim(x), 1))),
                           highlighting_direction = NULL,
                           zero_size = 0.5,
                           zero_split = FALSE,
                           zero_shade = NULL,
                           zero_gp = gpar(col = 0),
                           panel = NULL,
                           main = NULL, sub = NULL, ...) {
    zero_shade <- !is.null(shade) && shade || !is.null(expected) || !is.null(gp)
    if (!is.null(shade) && !shade) zero_shade = FALSE

    if (is.logical(main) && main)
        main <- deparse(substitute(x))
    else if (is.logical(sub) && sub)
        sub <- deparse(substitute(x))

    if (is.structable(x)) {
        if (is.null(direction) && is.null(split_vertical))
            split_vertical <- attr(x, "split_vertical")
        x <- as.table(x)
    }
    if (is.null(split_vertical))
        split_vertical <- FALSE

    d <- dim(x)
    dl <- length(d)

    ## splitting argument
    if (!is.null(direction))
        split_vertical <- direction == "v"
    if (length(split_vertical) == 1)
        split_vertical <- rep(c(split_vertical, !split_vertical), length.out = dl)
    if (length(split_vertical) < dl)
        split_vertical <- rep(split_vertical, length.out = dl)

    ## highlighting
    if (!is.null(highlighting)) {
        if (is.character(highlighting))
            highlighting <- match(highlighting, names(dimnames(x)))
        if (length(highlighting) > 0) {
            if (is.character(condvars))
                condvars <- match(condvars, names(dimnames(x)))
            perm <- if (length(condvars) > 0)
                     c(condvars, seq(dl)[-c(condvars,highlighting)], highlighting)
                 else
                     c(seq(dl)[-highlighting], highlighting)
            x <- aperm(x, perm)
            d <- d[perm]
            split_vertical <- split_vertical[perm]
            if (is.null(spacing))
                spacing <- spacing_highlighting
            if (is.function(highlighting_fill))
                highlighting_fill <- highlighting_fill(d[dl])
            if (is.null(gp))
                gp <- gpar(fill = aperm(array(highlighting_fill, dim = rev(d))))
            if (!is.null(highlighting_direction)) {
                split_vertical[dl] <- highlighting_direction %in% c("left", "right")
                if (highlighting_direction %in% c("left", "top")) {
                    ## ugly:
                    tmp <- as.data.frame.table(x)
                    tmp[,dl] <- factor(tmp[,dl], rev(levels(tmp[,dl])))
                    x <- xtabs(Freq ~ ., data = tmp)
                    gp <- gpar(fill = aperm(array(rev(highlighting_fill),
                                                  dim = rev(d))))
                }
            }
        }
    } else if (!is.null(condvars)) { # Conditioning only
        if (is.character(condvars))
            condvars <- match(condvars, names(dimnames(x)))
        if (length(condvars) > 0) {
            perm <- c(condvars, seq(dl)[-condvars])
            x <- aperm(x, perm)
            split_vertical <- split_vertical[perm]
        }
        if (is.null(spacing))
            spacing <- spacing_conditional
    }

    ## spacing argument
    if (is.null(spacing))
        spacing <- if (dl < 3) spacing_equal else spacing_increase

    strucplot(x,
              condvars = if (is.null(condvars)) NULL else length(condvars),
              core = struc_mosaic(zero_size = zero_size, zero_split = zero_split,
              zero_shade = zero_shade, zero_gp = zero_gp, panel = panel),
              split_vertical = split_vertical,
              spacing = spacing,
              spacing_args = spacing_args,
              gp = gp,
              expected = expected,
              shade = shade,
              main = main,
              sub = sub,
              ...)
}

## old code: more elegant, but less performant
##
## struc_mosaic2 <- function(zero_size = 0.5, zero_split = FALSE,
##                          zero_shade = TRUE, zero_gp = gpar(col = 0))
##   function(residuals, observed, expected = NULL, spacing, gp, split_vertical, prefix = "") {
##     dn <- dimnames(observed)
##     dnn <- names(dn)
##     dx <- dim(observed)
##     dl <- length(dx)

##     ## split workhorse
##     zerostack <- character(0)
##     split <- function(x, i, name, row, col, zero) {
##       cotab <- co_table(x, 1)
##       margin <- sapply(cotab, sum)
##       v <- split_vertical[i]
##       d <- dx[i]

##       ## compute total cols/rows and build split layout
##       dist <- unit.c(unit(margin, "null"), spacing[[i]])
##       idx <- matrix(1:(2 * d), nrow = 2, byrow = TRUE)[-2 * d]
##       layout <- if (v)
##         grid.layout(ncol = 2 * d - 1, widths = dist[idx])
##       else
##         grid.layout(nrow = 2 * d - 1, heights = dist[idx])
##       vproot <- viewport(layout.pos.col = col, layout.pos.row = row,
##                          layout = layout, name = remove_trailing_comma(name))

##       ## next level: either create further splits, or final viewports
##       name <- paste(name, dnn[i], "=", dn[[i]], ",", sep = "")
##       row <- col <- rep.int(1, d)
##       if (v) col <- 2 * 1:d - 1 else row <- 2 * 1:d - 1
##       f <- if (i < dl)
##         function(m) {
##           co <- cotab[[m]]
##           z <- mean(co) <= .Machine$double.eps
##           if (z && !zero && !zero_split) zerostack <<- c(zerostack, name[m])
##           split(co, i + 1, name[m], row[m], col[m], z && !zero_split)
##         }
##       else
##         function(m) {
##           if (cotab[[m]] <= .Machine$double.eps && !zero)
##             zerostack <<- c(zerostack, name[m])
##           viewport(layout.pos.col = col[m], layout.pos.row = row[m],
##                    name = remove_trailing_comma(name[m]))
##         }
##       vpleaves <- structure(lapply(1:d, f), class = c("vpList", "viewport"))

##       vpTree(vproot, vpleaves)
##     }

##     ## start spltting on top, creates viewport-tree
##     pushViewport(split(observed + .Machine$double.eps,
##                        i = 1, name = paste(prefix, "cell:", sep = ""),
##                        row = 1, col = 1, zero = FALSE))

##     ## draw rectangles
##     mnames <-  apply(expand.grid(dn), 1,
##                      function(i) paste(dnn, i, collapse=",", sep = "=")
##                      )
##     zeros <- observed <= .Machine$double.eps

##     ## draw zero cell lines
##     for (i in remove_trailing_comma(zerostack)) {
##       seekViewport(i)
##       grid.lines(x = 0.5)
##       grid.lines(y = 0.5)
##       if (!zero_shade && zero_size > 0) {
##         grid.points(0.5, 0.5, pch = 19, size = unit(zero_size, "char"),
##                     gp = zero_gp,
##                     name = paste(prefix, "disc:", mnames[i], sep = ""))
##         grid.points(0.5, 0.5, pch = 1, size = unit(zero_size, "char"),
##                     name = paste(prefix, "circle:", mnames[i], sep = ""))
##       }
##     }

##     # draw boxes
##     for (i in seq_along(mnames)) {
##       seekViewport(paste(prefix, "cell:", mnames[i], sep = ""))
##       gpobj <- structure(lapply(gp, function(x) x[i]), class = "gpar")
##       if (!zeros[i]) {
##         grid.rect(gp = gpobj, name = paste(prefix, "rect:", mnames[i], sep = ""))
##       } else {
##         if (zero_shade && zero_size > 0) {
##           grid.points(0.5, 0.5, pch = 19, size = unit(zero_size, "char"),
##                       gp = gpar(col = gp$fill[i]),
##                       name = paste(prefix, "disc:", mnames[i], sep = ""))
##           grid.points(0.5, 0.5, pch = 1, size = unit(zero_size, "char"),
##                       name = paste(prefix, "circle:", mnames[i], sep = ""))
##         }
##       }
##     }
##   }
## class(struc_mosaic2) <- "grapcon_generator"

struc_mosaic <- function(zero_size = 0.5, zero_split = FALSE,
                         zero_shade = TRUE, zero_gp = gpar(col = 0),
                         panel = NULL)
    function(residuals, observed, expected = NULL,
             spacing, gp, split_vertical, prefix = "") {
        dn <- dimnames(observed)
        dnn <- names(dn)
        dx <- dim(observed)
        dl <- length(dx)

        zeros <- function(gp, name) {
            grid.lines(x = 0.5)
            grid.lines(y = 0.5)
            if (zero_size > 0) {
                grid.points(0.5, 0.5, pch = 19, size = unit(zero_size, "char"),
                            gp = gp, name = paste(prefix, "disc:", name, sep = ""))
                grid.points(0.5, 0.5, pch = 1, size = unit(zero_size, "char"),
                            name = paste(prefix, "circle:", name, sep = ""))
            }
        }

        ## split workhorse
        zerostack <- character(0)
        split <- function(x, i, name, row, col, zero, index) {
            cotab <- co_table(x, 1)
            margin <- sapply(cotab, sum)
            margin[margin == 0] <- .Machine$double.eps
                                        #      margin <- margin + .Machine$double.eps
            v <- split_vertical[i]
            d <- dx[i]

            ## compute total cols/rows and build split layout
            dist <- if (d > 1)
                        unit.c(unit(margin, "null"), spacing[[i]])
                    else
                        unit(margin, "null")
            idx <- matrix(1:(2 * d), nrow = 2, byrow = TRUE)[-2 * d]
            layout <- if (v)
                          grid.layout(ncol = 2 * d - 1, widths = dist[idx])
                      else
                          grid.layout(nrow = 2 * d - 1, heights = dist[idx])
            pushViewport(viewport(layout.pos.col = col, layout.pos.row = row,
                                  layout = layout, name = paste(prefix, "cell:",
                                                   remove_trailing_comma(name),
                                                   sep = "")))

            ## next level: either create further splits, or final viewports
            row <- col <- rep.int(1, d)
            if (v) col <- 2 * 1:d - 1 else row <- 2 * 1:d - 1
            for (m in 1:d) {
                nametmp <- paste(name, dnn[i], "=", dn[[i]][m], ",", sep = "")
                if (i < dl) {
                    co <- cotab[[m]]

                    ## zeros
                    z <- mean(co) <= .Machine$double.eps
                    split(co, i + 1, nametmp, row[m], col[m],
                          z && !zero_split, cbind(index, m))
                    if (z && !zero && !zero_split && !zero_shade && (zero_size > 0))
                        zeros(zero_gp, nametmp)
                } else {
                    pushViewport(viewport(layout.pos.col = col[m],
                                          layout.pos.row = row[m],
                                          name = paste(prefix, "cell:",
                                          remove_trailing_comma(nametmp), sep = "")))

                    ## zeros
                    if (cotab[[m]] <= .Machine$double.eps && !zero) {
                        zeros(if (!zero_shade) zero_gp else gpar(col = gp$fill[cbind(index,m)]), nametmp)
                    } else {
                        ## rectangles
                        gpobj <- structure(lapply(gp, function(x) x[cbind(index, m)]),
                                           class = "gpar")
                        nam <- paste(prefix, "rect:",
                                     remove_trailing_comma(nametmp), sep = "")
                        if (!is.null(panel))
                            panel(residuals, observed, expected, c(cbind(index, m)),
                                  gpobj, nam)
                        else
                            grid.rect(gp = gpobj, name = nam)
                    }
                }
                upViewport(1)
            }
        }

        ## start splitting on top, creates viewport-tree
        split(observed, i = 1, name = "", row = 1, col = 1,
              zero = FALSE, index = cbind())
    }
class(struc_mosaic) <- "grapcon_generator"

###########################################################
## sieveplot

sieve <- function(x, ...)
    UseMethod("sieve")

sieve.formula <-
    function(formula, data = NULL, ..., main = NULL, sub = NULL, subset = NULL)
{
    if (is.logical(main) && main)
        main <- deparse(substitute(data))
    else if (is.logical(sub) && sub)
        sub <- deparse(substitute(data))

    m <- match.call(expand.dots = FALSE)
    edata <- eval(m$data, parent.frame())

    fstr <- strsplit(paste(deparse(formula), collapse = ""), "~")
    vars <- strsplit(strsplit(gsub(" ", "", fstr[[1]][2]), "\\|")[[1]], "\\+")
    varnames <- vars[[1]]
    condnames <- if (length(vars) > 1) vars[[2]] else NULL

    if (inherits(edata, "ftable") || inherits(edata, "table") || length(dim(edata)) > 2) {
        condind <- NULL
        dat <- as.table(data)
        if(all(varnames != ".")) {
            ind <- match(varnames, names(dimnames(dat)))
            if (any(is.na(ind)))
                stop(paste("Can't find", paste(varnames[is.na(ind)], collapse=" / "), "in", deparse(substitute(data))))

            if (!is.null(condnames)) {
                condind <- match(condnames, names(dimnames(dat)))
                if (any(is.na(condind)))
                    stop(paste("Can't find", paste(condnames[is.na(condind)], collapse=" / "), "in", deparse(substitute(data))))
                ind <- c(condind, ind)
            }
            dat <- margin.table(dat, ind)
        }
        sieve.default(dat, main = main, sub = sub,
                      condvars = if (is.null(condind)) NULL else match(condnames, names(dimnames(dat))), ...)
    } else {
        tab <- if ("Freq" %in% colnames(data))
            xtabs(formula(paste("Freq~", paste(c(condnames, varnames), collapse = "+"))),
                  data = data, subset = subset)
        else
            xtabs(formula(paste("~", paste(c(condnames, varnames), collapse = "+"))),
                  data = data, subset = subset)

        sieve.default(tab, main = main, sub = sub, ...)
    }
}

sieve.default <- function(x, condvars = NULL, gp = NULL,
                          shade = NULL, legend = FALSE,
                          split_vertical = NULL, direction = NULL,
                          spacing = NULL, spacing_args = list(),
                          sievetype = c("observed","expected"),
                          gp_tile = gpar(),
                          scale = 1,
                          main = NULL, sub = NULL, ...) {
    if (is.logical(main) && main)
        main <- deparse(substitute(x))
    else if (is.logical(sub) && sub)
        sub <- deparse(substitute(x))
    sievetype = match.arg(sievetype)
    if (is.logical(shade) && shade && is.null(gp))
        gp <- if (sievetype == "observed")
#            shading_sieve(interpolate = 0, lty = c("longdash", "solid"))
            shading_sieve(interpolate = 0, lty = c("solid", "longdash"))
	else
            shading_sieve(interpolate = 0, line_col = "darkgray", eps = Inf, lty = "dotted")

    if (is.structable(x)) {
        if (is.null(direction) && is.null(split_vertical))
            split_vertical <- attr(x, "split_vertical")
        x <- as.table(x)
    }
    if (is.null(split_vertical))
        split_vertical <- FALSE

    dl <- length(dim(x))

    ## splitting argument
    if (!is.null(direction))
        split_vertical <- direction == "v"
    if (length(split_vertical) == 1)
        split_vertical <- rep(c(split_vertical, !split_vertical), length.out = dl)
    if (length(split_vertical) < dl)
        split_vertical <- rep(split_vertical, length.out = dl)

    ## condvars
    if (!is.null(condvars)) {
        if (is.character(condvars))
            condvars <- match(condvars, names(dimnames(x)))
        x <- aperm(x, c(condvars, seq(dl)[-condvars]))
        if (is.null(spacing))
            spacing <- spacing_conditional
    }

    ## spacing argument
    if (is.null(spacing))
        spacing <- if (dl < 3) spacing_equal(sp = 0) else spacing_increase

    strucplot(x,
              condvars = if (is.null(condvars)) NULL else length(condvars),
              core = struc_sieve(sievetype = sievetype, gp_tile = gp_tile,
                                 scale = scale),
              split_vertical = split_vertical,
              spacing = spacing,
              spacing_args = spacing_args,
              main = main,
              sub = sub,
              shade = shade,
              legend = legend,
              gp = gp,
              ...)
}

## old version (not performant enough)
##
## struc_sieve <- function(sievetype = c("observed", "expected")) {
##   sievetype = match.arg(sievetype)
##   function(residuals, observed, expected, spacing, gp, split_vertical, prefix = "") {
##     dn <- dimnames(expected)
##     dnn <- names(dn)
##     dx <- dim(expected)
##     dl <- length(dx)
##     n <- sum(expected)

##     ## split workhorse
##     split <- function(x, i, name, row, col, rowmargin, colmargin) {
##       cotab <- co_table(x, 1)
##       margin <- sapply(cotab, sum)
##       v <- split_vertical[i]
##       d <- dx[i]

##       ## compute total cols/rows and build split layout
##       dist <- unit.c(unit(margin, "null"), spacing[[i]])
##       idx <- matrix(1:(2 * d), nrow = 2, byrow = TRUE)[-2 * d]
##       layout <- if (v)
##         grid.layout(ncol = 2 * d - 1, widths = dist[idx])
##       else
##         grid.layout(nrow = 2 * d - 1, heights = dist[idx])
##       vproot <- viewport(layout.pos.col = col, layout.pos.row = row,
##                          layout = layout, name = remove_trailing_comma(name))

##       ## next level: either create further splits, or final viewports
##       name <- paste(name, dnn[i], "=", dn[[i]], ",", sep = "")
##       row <- col <- rep.int(1, d)
##       if (v) col <- 2 * 1:d - 1 else row <- 2 * 1:d - 1
##       proptab <- function(x) x / max(sum(x), 1)
##       f <- if (i < dl) {
##         if (v)
##           function(m) split(cotab[[m]], i + 1, name[m], row[m], col[m],
##                             colmargin = colmargin * proptab(margin)[m],
##                             rowmargin = rowmargin)
##         else
##           function(m) split(cotab[[m]], i + 1, name[m], row[m], col[m],
##                             colmargin = colmargin,
##                             rowmargin = rowmargin * proptab(margin)[m])
##       } else {
##         if (v)
##           function(m) viewport(layout.pos.col = col[m], layout.pos.row = row[m],
##                                name = remove_trailing_comma(name[m]),
##                                yscale = c(0, rowmargin),
##                                xscale = c(0, colmargin * proptab(margin)[m]))
##         else
##           function(m) viewport(layout.pos.col = col[m], layout.pos.row = row[m],
##                                name = remove_trailing_comma(name[m]),
##                                yscale = c(0, rowmargin * proptab(margin)[m]),
##                                xscale = c(0, colmargin))
##       }
##       vpleaves <- structure(lapply(1:d, f), class = c("vpList", "viewport"))

##       vpTree(vproot, vpleaves)
##     }

##     ## start splitting on top, creates viewport-tree
##     pushViewport(split(expected + .Machine$double.eps,
##                        i = 1, name = paste(prefix, "cell:", sep = ""), row = 1, col = 1,
##                        rowmargin = n, colmargin = n))

##     ## draw rectangles
##     mnames <- apply(expand.grid(dn), 1,
##                     function(i) paste(dnn, i, collapse=",", sep = "=")
##                     )

##     for (i in seq_along(mnames)) {
##       seekViewport(paste(prefix, "cell:", mnames[i], sep = ""))
##       vp <- current.viewport()
##       gpobj <- structure(lapply(gp, function(x) x[i]), class = "gpar")

##       div <- if (sievetype == "observed") observed[i] else expected[i]
##       if (div > 0) {
##         square.side <- sqrt(vp$yscale[2] * vp$xscale[2] / div)

##         ii <- seq(0, vp$xscale[2], by = square.side)
##         jj <- seq(0, vp$yscale[2], by = square.side)

##         grid.segments(x0 = ii, x1 = ii, y0 = 0, y1 = vp$yscale[2],
##                       default.units = "native", gp = gpobj)
##         grid.segments(x0 = 0, x1 = vp$xscale[2], y0 = jj, y1 = jj,
##                       default.units = "native", gp = gpobj)
##       }
##       grid.rect(name = paste(prefix, "rect:", mnames[i], sep = ""),
##                 gp = gpar(fill = "transparent"))
##     }
##   }
## }
##class(struc_sieve) <- "grapcon_generator"

struc_sieve <- function(sievetype = c("observed", "expected"),
                        gp_tile = gpar(), scale = 1) {
    sievetype = match.arg(sievetype)
    function(residuals, observed, expected, spacing,
             gp, split_vertical, prefix = "") {
        observed <- scale * observed
        expected <- scale * expected
        if (is.null(expected)) stop("Need expected values.")
        dn <- dimnames(expected)
        dnn <- names(dn)
        dx <- dim(expected)
        dl <- length(dx)
        n <- sum(expected)

        ## split workhorse
        split <- function(x, i, name, row, col, rowmargin, colmargin, index) {
            cotab <- co_table(x, 1)
            margin <- sapply(cotab, sum)
            v <- split_vertical[i]
            d <- dx[i]

            ## compute total cols/rows and build split layout
            dist <- if (d > 1)
                unit.c(unit(margin, "null"), spacing[[i]])
            else
                unit(margin, "null")
            idx <- matrix(1:(2 * d), nrow = 2, byrow = TRUE)[-2 * d]
            layout <- if (v)
                grid.layout(ncol = 2 * d - 1, widths = dist[idx])
            else
                grid.layout(nrow = 2 * d - 1, heights = dist[idx])
            pushViewport(viewport(layout.pos.col = col, layout.pos.row = row,
                                  layout = layout, name = paste(prefix, "cell:",
                                                   remove_trailing_comma(name), sep = "")))

            ## next level: either create further splits, or final viewports
            row <- col <- rep.int(1, d)
            if (v) col <- 2 * 1:d - 1 else row <- 2 * 1:d - 1
            proptab <- function(x) x / max(sum(x), 1)
            for (m in 1:d) {
                nametmp <- paste(name, dnn[i], "=", dn[[i]][m], ",", sep = "")
                if (v) {
                    colmargintmp <- colmargin * proptab(margin)[m]
                    rowmargintmp <- rowmargin
                } else {
                    rowmargintmp <- rowmargin * proptab(margin)[m]
                    colmargintmp <- colmargin
                }
                if (i < dl)
                    split(cotab[[m]], i + 1, nametmp, row[m], col[m],
                          colmargin = colmargintmp,
                          rowmargin = rowmargintmp, index = cbind(index, m))
                else {
                    pushViewport(viewport(layout.pos.col = col[m],
                                          layout.pos.row = row[m],
                                          name = paste(prefix, "cell:",
                                              remove_trailing_comma(nametmp),
                                              sep = ""),
                                          yscale = c(0, rowmargintmp),
                                          xscale = c(0, colmargintmp)))

                    gpobj <- structure(lapply(gp,
                                              function(x) x[cbind(index, m)]),
                                       class = "gpar")

                    ## draw sieve
                    div <- if (sievetype == "observed")
                        observed[cbind(index, m)]
                    else
                        expected[cbind(index, m)]
                    gptmp <- gp_tile
                    gptmp$col <- "transparent"
                    grid.rect(name = paste(prefix, "rect:", nametmp, sep = ""),
                              gp = gptmp)
                    if (div > 0) {
                        square.side <- sqrt(colmargintmp * rowmargintmp / div)

                        ii <- seq(0, colmargintmp, by = square.side)
                        jj <- seq(0, rowmargintmp, by = square.side)

                        grid.segments(x0 = ii, x1 = ii, y0 = 0, y1 = rowmargintmp,
                                      default.units = "native", gp = gpobj)
                        grid.segments(x0 = 0, x1 = colmargintmp, y0 = jj, y1 = jj,
                                      default.units = "native", gp = gpobj)
                    }
                    gptmp <- gp_tile
                    gptmp$fill <- "transparent"
                    grid.rect(name = paste(prefix, "border:", nametmp, sep = ""),
                              gp = gptmp)

                }
                upViewport(1)
            }
        }

        ## start splitting on top, creates viewport-tree
        split(expected + .Machine$double.eps,
              i = 1, name = "", row = 1, col = 1,
              rowmargin = n, colmargin = n, index = cbind())
    }
}
class(struc_sieve) <- "grapcon_generator"

cd_plot <- function(x, ...) {
  UseMethod("cd_plot")
}

cd_plot.formula <- function(formula, data = list(),
  plot = TRUE, ylab_tol = 0.05,
  bw = "nrd0", n = 512, from = NULL, to = NULL,
  main = "", xlab = NULL, ylab = NULL, margins = c(5.1, 4.1, 4.1, 3.1),
  gp = gpar(), name = "cd_plot", newpage = TRUE, pop = TRUE,
  return_grob = FALSE, ...)
{
  ## extract x, y from formula
  mf <- model.frame(formula, data = data)
  if(NCOL(mf) != 2) stop("`formula' should specify exactly two variables")
  y <- mf[,1]
  if(!is.factor(y)) stop("dependent variable should be a factor")
  x <- mf[,2]
  if(!is.numeric(x)) stop("explanatory variable should be numeric")

  ## graphical parameters
  if(is.null(xlab)) xlab <- names(mf)[2]
  if(is.null(ylab)) ylab <- names(mf)[1]

  ## call default interface
  cd_plot(x, y,
    plot = plot, ylab_tol = ylab_tol,
    bw = bw, n = n, from = from, to = to,
    main = main, xlab = xlab, ylab = ylab, margins = margins,
    gp = gp, name = name, newpage = newpage, pop = pop, ...)
}

cd_plot.default <- function(x, y,
  plot = TRUE, ylab_tol = 0.05,
  bw = "nrd0", n = 512, from = NULL, to = NULL,
  main = "", xlab = NULL, ylab = NULL, margins = c(5.1, 4.1, 4.1, 3.1),
  gp = gpar(), name = "cd_plot", newpage = TRUE, pop = TRUE,
  return_grob = FALSE, ...)
{
  ## check x and y
  if(!is.numeric(x)) stop("explanatory variable should be numeric")
  if(!is.factor(y)) stop("dependent variable should be a factor")
  ny <- length(levels(y))

  ## graphical parameters
  if(is.null(xlab)) xlab <- deparse(substitute(x))
  if(is.null(ylab)) ylab <- deparse(substitute(y))
  if(is.null(gp$fill)) gp$fill <- gray.colors(ny)
  gp$fill <- rep(gp$fill, length.out = ny)

  ## unconditional density of x
  dx <- if(is.null(from) & is.null(to)) density(x, bw = bw, n = n, ...)
          else density(x, bw = bw, from = from, to = to, n = n, ...)
  x1 <- dx$x

  ## setup conditional values
  yprop <- cumsum(prop.table(table(y)))
  y1 <- matrix(rep(0, n*(ny-1)), nrow = (ny-1))

  ## setup return value
  rval <- list()

  for(i in 1:(ny-1)) {
    dxi <- density(x[y %in% levels(y)[1:i]], bw = dx$bw, n = n, from = min(dx$x), to = max(dx$x), ...)
    y1[i,] <- dxi$y/dx$y * yprop[i]
    rval[[i]] <- approxfun(x1, y1[i,], rule = 2)
  }
  names(rval) <- levels(y)[1:(ny-1)]

  ## use known ranges
  y1 <- rbind(0, y1, 1)
  y1 <- y1[,which(x1 >= min(x) & x1 <= max(x))]
  x1 <- x1[x1 >= min(x) & x1 <= max(x)]

  ## plot polygons
  if(plot) {
    ## setup
    if(newpage) grid.newpage()
    pushViewport(plotViewport(xscale = range(x1), yscale = c(0, 1),
      default.units = "native", name = name, margins = margins, ...))

    ## polygons
    for(i in 1:(NROW(y1)-1)) {
     gpi <- gp
     gpi$fill <- gp$fill[i]
     grid.polygon(x = c(x1, rev(x1)), y = c(y1[i+1,], rev(y1[i,])), default.units = "native", gp = gpi)
    }

    ## axes
    grid.rect(gp = gpar(fill = "transparent"))
    grid.xaxis()
    grid.yaxis(main = FALSE)
    equidist <- any(diff(y1[,1]) < ylab_tol)
    yat <- if(equidist) seq(1/(2*ny), 1-1/(2*ny), by = 1/ny) else (y1[-1,1] + y1[-NROW(y1), 1])/2
    grid.text(x = unit(-1.5, "lines"), y = unit(yat, "native"), label = levels(y),
      rot = 90, check.overlap = TRUE)

    ## annotation
    grid.text(xlab, y = unit(-3.5, "lines"))
    grid.text(ylab, x = unit(-3, "lines"), rot = 90)
    grid.text(main, y = unit(1, "npc") + unit(2, "lines"), gp = gpar(fontface = "bold"))

    ## pop
    if(pop) popViewport()
  }

  ## return conditional density functions
  if (plot && return_grob)
      invisible(structure(rval, grob = grid.grab()))
  else
      invisible(rval)

}

#########################################
## structable

structable <- function(x, ...)
  UseMethod("structable")

structable.formula <- function(formula, data = NULL, direction = NULL,
                               split_vertical = NULL, ..., subset, na.action) {
    if (missing(formula) || !inherits(formula, "formula"))
        stop("formula is incorrect or missing")

    m <- match.call(expand.dots = FALSE)
    edata <- eval(m$data, parent.frame())

    if (!is.null(direction))
      split_vertical <- direction == "v"

    if (is.structable(data)) {
      split_vertical <- attr(data, "split_vertical")
      data <- as.table(data)
    }

    if (is.null(split_vertical))
      split_vertical <- FALSE

    if (length(formula) == 3 && formula[[2]] == "Freq")
      formula[[2]] = NULL
    ## only rhs present without `.' in lhs => xtabs-interface
    if (length(formula) != 3) {
      if (formula[[1]] == "~") {
        if (inherits(edata, "ftable") || inherits(edata, "table") ||
            length(dim(edata)) > 2) {
          data <- as.table(data)
          varnames <- attr(terms(formula, allowDotAsName = TRUE), "term.labels")
          dnames <- names(dimnames(data))
          di <- match(varnames, dnames)
          if (any(is.na(di)))
            stop("incorrect variable names in formula")
          if (all(varnames != "."))
            data <- margin.table(data, di)
          return(structable(data, split_vertical = split_vertical, ...))
        }
        else if (is.data.frame(data)) {
          if ("Freq" %in% colnames(data))
            return(structable(xtabs(formula(paste("Freq", deparse(formula))),
                                    data = data),
                              split_vertical = split_vertical, ...))
          else
            return(structable(xtabs(formula, data),  split_vertical = split_vertical, ...))

        } else {
          if (is.matrix(edata))
            m$data <- as.data.frame(data)
          m$... <- m$split_vertical <- m$direction <- NULL
          m[[1L]] <- quote(stats::model.frame)
          mf <- eval(m, parent.frame())
          return(structable(table(mf), split_vertical = split_vertical, ...))
        }

     } else
        stop("formula must have both left and right hand sides")
    }

    ## `ftable' behavior
    if (any(attr(terms(formula, allowDotAsName = TRUE), "order") > 1))
        stop("interactions are not allowed")
    rvars <- attr(terms(formula[-2], allowDotAsName = TRUE), "term.labels")
    cvars <- attr(terms(formula[-3], allowDotAsName = TRUE), "term.labels")
    rhs.has.dot <- any(rvars == ".")
    lhs.has.dot <- any(cvars == ".")
    if (lhs.has.dot && rhs.has.dot)
        stop(paste("formula has", sQuote("."), "in both left and right hand side"))
    if (inherits(edata, "ftable") || inherits(edata, "table") ||
        length(dim(edata)) > 2) {
        if (inherits(edata, "ftable"))
            data <- as.table(data)

        dnames <- names(dimnames(data))
        rvars <- pmatch(rvars, dnames)
        cvars <- pmatch(cvars, dnames)
        if (rhs.has.dot)
            rvars <- seq_along(dnames)[-cvars]
        else if (any(is.na(rvars)))
          stop("incorrect variable names in rhs of formula")
        if (lhs.has.dot)
            cvars <- seq_along(dnames)[-rvars]
        else if (any(is.na(cvars)))
          stop("incorrect variable names in lhs of formula")
        split_vertical <- c(rep(FALSE, length(rvars)), rep(TRUE, length(cvars)))
        structable(margin.table(data, c(rvars, cvars)), split_vertical = split_vertical, ...)
    } else {
        if (is.matrix(edata))
            m$data <- as.data.frame(data)
        m$... <- m$split_vertical <- m$direction <- NULL
        if (!is.null(data) && is.environment(data)) {
            dnames <- names(data)
            if (rhs.has.dot)
                rvars <- seq_along(dnames)[-cvars]
            if (lhs.has.dot)
                cvars <- seq_along(dnames)[-rvars]
        }
        else {
            if (lhs.has.dot || rhs.has.dot)
                stop("cannot use dots in formula with given data")
        }
        if ("Freq" %in% colnames(m$data))
          m$formula <- formula(paste("Freq~", paste(c(rvars, cvars), collapse = "+")))
        else
          m$formula <- formula(paste("~", paste(c(rvars, cvars), collapse = "+")))
        m[[1]] <- as.name("xtabs")
        mf <- eval(m, parent.frame())
        split_vertical <- c(rep(FALSE, length(rvars)), rep(TRUE, length(cvars)))
        structable(mf, split_vertical = split_vertical, ...)
    }
}


structable.default <- function(..., direction = NULL, split_vertical = FALSE) {
  ## several checks & transformations for arguments
  args <- list(...)

  if (length(args) == 0)
    stop("Nothing to tabulate")

  x <- args[[1]]
  x <- if (is.list(x))
    table(x)
  else if (inherits(x, "ftable"))
    as.table(x)
  else if (!(is.array(x) && length(dim(x)) > 1 || inherits(x, "table")))
    do.call("table", as.list(substitute(list(...)))[-1])
  else
    x

  if (is.null(dimnames(x)))
      dimnames(x) <- lapply(dim(x), function(i) letters[seq_len(i)])
  if (is.null(names(dimnames(x))))
      names(dimnames(x)) <- LETTERS[seq_along(dim(x))]
  idx <- sapply(names(dimnames(x)), nchar) < 1
  if(any(idx))
      names(dimnames(x))[idx] <- LETTERS[seq_len(sum(idx))]

  ## splitting argument
  dl <- length(dim(x))
  if (!is.null(direction))
    split_vertical <- direction == "v"
  if (length(split_vertical) == 1)
    split_vertical <- rep(c(split_vertical, !split_vertical), length.out = dl)
  if (length(split_vertical) < dl)
    split_vertical <- rep(split_vertical, length.out = dl)


  ## permute & reshape
  ret <- base::aperm(x, c(rev(which(!split_vertical)), rev(which(split_vertical))))

  dn <- dimnames(x)
  rv <- dn[split_vertical]
  cv <- dn[!split_vertical]
  rl <- if (length(rv)) sapply(rv, length) else 1
  cl <- if (length(cv)) sapply(cv, length) else 1
  dim(ret) <- c(prod(cl), prod(rl))

  ## add dimnames
  attr(ret, "dnames") <- dn
  attr(ret, "split_vertical") <- split_vertical

  ## add dimension attributes in ftable-format
  attr(ret, "col.vars") <- rv
  attr(ret, "row.vars") <- cv

  class(ret) <- c("structable", "ftable")
  ret
}

"[[.structable" <- function(x, ...) {
  if(nargs() > 3)
    stop("Incorrect number of dimensions (max: 2).")
  args <- if (nargs() < 3)
    list(..1)
  else
    .massage_args(...)

  args <- lapply(args, function(x) if (is.logical(x)) which(x) else x)

  ## handle one-arg cases
  if (nargs() < 3)
    if (length(args[[1]]) > 1)
      ## resolve calls like x[[c(1,2)]]
      return(x[[ args[[1]][1] ]] [[ args[[1]][-1] ]])
    else
      ## resolve x[[foo]]
      return(if (attr(x, "split_vertical")[1]) x[[,args[[1]] ]] else x[[args[[1]],]])

  ## handle calls like x[[c(1,2), c(3,4)]]
  if (length(args[[1]]) > 1 && length(args[[2]]) > 1)
    return(x[[ args[[1]][1], args[[2]][1] ]] [[ args[[1]][-1], args[[2]][-1] ]])

  ## handle calls like x[[c(1,2), 3]]
  if (length(args[[1]]) > 1)
    return(x[[ args[[1]][1], args[[2]] ]] [[ args[[1]][-1], ]])

  ## handle calls like x[[1, c(1,3)]]
  if (length(args[[2]]) > 1)
    return(x[[ args[[1]], args[[2]][1] ]] [[ , args[[2]][-1] ]])

  ## final cases like x[[1,2]] or x[[1,]] or x[[,1]]
  dnames <- attr(x, "dnames")
  split <- attr(x, "split_vertical")
  rv <- dnames[!split]
  cv <- dnames[split]

  lsym <- is.symbol(args[[1]])
  rsym <- is.symbol(args[[2]])
  if (!lsym) {
    rstep <- dim(unclass(x))[1] / length(rv[[1]])
    if (is.character(args[[1]]))
      args[[1]] <- match(args[[1]], rv[[1]])
  }
  if (!rsym) {
    cstep <- dim(unclass(x))[2] / length(cv[[1]])
    if (is.character(args[[2]]))
      args[[2]] <- match(args[[2]], cv[[1]])
  }

  lind <- if (!lsym)
    (1 + (args[[1]] - 1) * rstep) : (args[[1]] * rstep)
  else
    1:nrow(unclass(x))
  rind <- if (!rsym)
    (1 + (args[[2]] - 1) * cstep) : (args[[2]] * cstep)
  else
    1:ncol(unclass(x))
  ret <- unclass(x)[lind, rind, drop = FALSE]

  if (!lsym) {
    i <- which(!split)[1]
    split <- split[-i]
    dnames <- dnames[-i]
  }

  if (!rsym) {
    i <- which(split)[1]
    split <- split[-i]
    dnames <- dnames[-i]
  }

  attr(ret, "split_vertical") <- split
  attr(ret, "dnames") <- dnames

  ## add dimension attributes in ftable-format
  attr(ret, "col.vars") <- dnames[split]
  attr(ret, "row.vars") <- dnames[!split]

  class(ret) <- class(x)
  ret
}

"[[<-.structable" <- function(x, ..., value) {
  args <- if (nargs() < 4)
    list(..1)
  else
    .massage_args(...)

  ## handle one-arg cases
  if (nargs() < 4)
    return(if (length(args[[1]]) > 1)
               ## resolve calls like x[[c(1,2)]]<-value
               Recall(x, args[[1]][1],
                      value = Recall(x[[ args[[1]][1] ]], args[[1]][-1], value = value))
           else
               ## resolve x[[foo]]<-value
               if (attr(x, "split_vertical")[1])
                   Recall(x,,args[[1]], value = value)
               else
                   Recall(x,args[[1]],, value = value)
           )

  ## handle calls like x[[c(1,2), c(3,4)]]<-value
  if (length(args[[1]]) > 1 && length(args[[2]]) > 1)
    return(Recall(x, args[[1]][1], args[[2]][1],
                  value = Recall(x[[ args[[1]][1], args[[2]][1] ]],
                    args[[1]][-1], args[[2]][-1], value = value)))

  ## handle calls like x[[c(1,2), 3]]<-value
  if (length(args[[1]]) > 1)
    return(Recall(x, args[[1]][1], args[[2]],
                  value = Recall(x[[ args[[1]][1], args[[2]] ]],
                    args[[1]][-1], ,value = value)))

  ## handle calls like x[[1, c(1,3)]]<-value
  if (length(args[[2]]) > 1)
    return(Recall(x, args[[1]], args[[2]][1],
                  value = Recall(x[[ args[[1]], args[[2]][1] ]],,
                    args[[2]][-1], value = value)))

  ## final cases like x[[1,2]]<-value or x[[1,]]<-value or x[[,1]]<-value
  dnames <- attr(x, "dnames")
  split <- attr(x, "split_vertical")
  rv <- dnames[!split]
  cv <- dnames[split]

  lsym <- is.symbol(args[[1]])
  rsym <- is.symbol(args[[2]])
  if (!lsym) {
    rstep <- dim(unclass(x))[1] / length(rv[[1]])
    if (is.character(args[[1]]))
      args[[1]] <- match(args[[1]], rv[[1]])
  }
  if (!rsym) {
    cstep <- dim(unclass(x))[2] / length(cv[[1]])
    if (is.character(args[[2]]))
      args[[2]] <- match(args[[2]], cv[[1]])
  }

  lind <- if (!lsym)
    (1 + (args[[1]] - 1) * rstep) : (args[[1]] * rstep)
  else
    1:nrow(unclass(x))
  rind <- if (!rsym)
    (1 + (args[[2]] - 1) * cstep) : (args[[2]] * cstep)
  else
    1:ncol(unclass(x))
  ret <- unclass(x)

  ret[lind, rind] <- value

  class(ret) <- class(x)
  ret
}

"[.structable" <- function(x, ...) {
  if(nargs() > 3)
        stop("Incorrect number of dimensions (max: 2).")
  args <- if (nargs() < 3)
    list(..1)
  else
    .massage_args(...)

  args <- lapply(args, function(x) if (is.logical(x)) which(x) else x)

  ## handle one-arg cases
  if (nargs() < 3)
    return(if (attr(x, "split_vertical")[1]) x[,args[[1]] ] else x[args[[1]],])

  ## handle calls like x[c(1,2), foo]
  if (length(args[[1]]) > 1)
    return(do.call(rbind, lapply(args[[1]], function(i) x[i, args[[2]]])))

  ## handle calls like x[foo, c(1,3)]
  if (length(args[[2]]) > 1)
    return(do.call(cbind, lapply(args[[2]], function(i) x[args[[1]], i])))

  ## final cases like x[1,2] or x[1,] or x[,1]
  dnames <- attr(x, "dnames")
  split <- attr(x, "split_vertical")
  rv <- dnames[!split]
  cv <- dnames[split]

  lsym <- is.symbol(args[[1]])
  rsym <- is.symbol(args[[2]])
  if (!lsym) {
    rstep <- dim(unclass(x))[1] / length(rv[[1]])
    if (is.character(args[[1]]))
      args[[1]] <- match(args[[1]], rv[[1]])
  }
  if (!rsym) {
    cstep <- dim(unclass(x))[2] / length(cv[[1]])
    if (is.character(args[[2]]))
      args[[2]] <- match(args[[2]], cv[[1]])
  }

  lind <- if (!lsym)
    (1 + (args[[1]] - 1) * rstep) : (args[[1]] * rstep)
  else
    1:nrow(unclass(x))
  rind <- if (!rsym)
    (1 + (args[[2]] - 1) * cstep) : (args[[2]] * cstep)
  else
    1:ncol(unclass(x))
  ret <- unclass(x)[lind, rind, drop = FALSE]

  if (!lsym) {
    i <- which(!split)[1]
    dnames[[i]] <- dnames[[i]][args[[1]]]
  }

  if (!rsym) {
    i <- which(split)[1]
    dnames[[i]] <- dnames[[i]][args[[2]]]
  }

  attr(ret, "split_vertical") <- split
  attr(ret, "dnames") <- dnames

  ## add dimension attributes in ftable-format
  attr(ret, "col.vars") <- dnames[split]
  attr(ret, "row.vars") <- dnames[!split]

  class(ret) <- class(x)
  ret
}

"[<-.structable" <- function(x, ..., value) {
  args <- if (nargs() < 4)
    list(..1)
  else
    .massage_args(...)

  ## handle one-arg cases
  if (nargs() < 4)
    return(## resolve x[foo]
           if (attr(x, "split_vertical")[1])
             Recall(x,,args[[1]], value = value)
           else
             Recall(x,args[[1]],, value = value)
           )

  ## handle calls like x[c(1,2), 3]
  if (length(args[[1]]) > 1) {
    for (i in seq_along(args[[1]]))
      x[ args[[1]][i], args[[2]] ] <- value[i,]
    return(x)
  }

  ## handle calls like x[1, c(2,3)]
  if (length(args[[2]]) > 1) {
    for (i in seq_along(args[[2]]))
      x[ args[[1]], args[[2]][i] ] <- value[,i]
    return(x)
  }

  ## final cases like x[1,2] or x[1,] or x[,1]
  dnames <- attr(x, "dnames")
  split <- attr(x, "split_vertical")
  rv <- dnames[!split]
  cv <- dnames[split]

  lsym <- is.symbol(args[[1]])
  rsym <- is.symbol(args[[2]])
  if (!lsym) {
    rstep <- dim(unclass(x))[1] / length(rv[[1]])
    if (is.character(args[[1]]))
      args[[1]] <- match(args[[1]], rv[[1]])
  }
  if (!rsym) {
    cstep <- dim(unclass(x))[2] / length(cv[[1]])
    if (is.character(args[[2]]))
      args[[2]] <- match(args[[2]], cv[[1]])
  }

  lind <- if (!lsym)
    (1 + (args[[1]] - 1) * rstep) : (args[[1]] * rstep)
  else
    1:nrow(unclass(x))
  rind <- if (!rsym)
    (1 + (args[[2]] - 1) * cstep) : (args[[2]] * cstep)
  else
    1:ncol(unclass(x))
  ret <- unclass(x)

  ret[lind, rind] <- value

  class(ret) <- class(x)
  ret
}


cbind.structable <- function(..., deparse.level = 1) {
  mergetables <- function(t1, t2) {
    ret <- cbind(unclass(t1),unclass(t2))
    class(ret) <- class(t1)
    attr(ret, "split_vertical") <- attr(t1, "split_vertical")
    attr(ret, "dnames") <- attr(t1, "dnames")
    attr(ret, "row.vars") <- attr(t1, "row.vars")
    attr(ret, "col.vars") <- attr(t1, "col.vars")
    attr(ret, "col.vars")[[1]] <- c(attr(t1, "col.vars")[[1]],attr(t2, "col.vars")[[1]])
    if (length(unique(attr(ret, "col.vars")[[1]])) != length(attr(ret, "col.vars")[[1]]))
      stop("Levels of factor(s) to be merged must be unique.")
    attr(ret, "dnames")[names(attr(ret, "col.vars"))] <- attr(ret, "col.vars")
    ret
  }
  args <- list(...)
  if (length(args) < 2)
    return(args[[1]])
  ret <- mergetables(args[[1]], args[[2]])
  if (length(args) > 2)
    do.call(cbind, c(list(ret), args[-(1:2)]))
  else
    ret
}

rbind.structable <- function(..., deparse.level = 1) {
  mergetables <- function(t1, t2) {
    ret <- rbind(unclass(t1),unclass(t2))
    class(ret) <- class(t1)
    attr(ret, "split_vertical") <- attr(t1, "split_vertical")
    attr(ret, "dnames") <- attr(t1, "dnames")
    attr(ret, "row.vars") <- attr(t1, "row.vars")
    attr(ret, "col.vars") <- attr(t1, "col.vars")
    attr(ret, "row.vars")[[1]] <- c(attr(t1, "row.vars")[[1]],attr(t2, "row.vars")[[1]])
    if (length(unique(attr(ret, "row.vars")[[1]])) != length(attr(ret, "row.vars")[[1]]))
      stop("Levels of factor(s) to be merged must be unique.")
    attr(ret, "dnames")[names(attr(ret, "row.vars"))] <- attr(ret, "row.vars")
    ret
  }
  args <- list(...)
  if (length(args) < 2)
    return(args[[1]])
  ret <- mergetables(args[[1]], args[[2]])
  if (length(args) > 2)
    do.call(rbind, c(list(ret), args[-(1:2)]))
  else
    ret
}

as.table.structable <- function(x, ...) {
  class(x) <- "ftable"
  ret <- NextMethod("as.table", object = x)
  structure(base::aperm(ret, match(names(attr(x, "dnames")),
                                   names(dimnames(ret)))),
            class = "table")
}

plot.structable <- function(x, ...)
  mosaic(x, ...)

t.structable <- function(x) {
  ret <- t.default(x)
  attr(ret, "split_vertical") <- !attr(ret, "split_vertical")
  hold <- attr(ret, "row.vars")
  attr(ret, "row.vars") = attr(ret, "col.vars")
  attr(ret, "col.vars") = hold
  ret
}

is.structable <- function(x)
  inherits(x, "structable")

dim.structable <- function(x)
  as.integer(sapply(attr(x, "dnames"), length))

print.structable <- function(x, ...) {
  class(x) <- "ftable"
  NextMethod("print", object = x)
}

dimnames.structable <- function(x) attr(x,"dnames")

as.vector.structable <- function(x, ...)
  as.vector(as.table(x), ...)

## FIXME: copy as.matrix.ftable, committed to R-devel on 2014/1/12
## replace by call to as.matrix.ftable when this becomes stable

as_matrix_ftable <-
function (x, sep = "_", ...)
{
    if (!inherits(x, "ftable"))
        stop("'x' must be an \"ftable\" object")
    make_dimnames <- function(vars) {
        structure(list(do.call(paste, c(rev(expand.grid(rev(vars))),
            list(sep = sep)))), names = paste(collapse = sep,
            names(vars)))
    }
    structure(unclass(x), dimnames = c(make_dimnames(attr(x,
        "row.vars")), make_dimnames(attr(x, "col.vars"))), row.vars = NULL,
        col.vars = NULL)
}

as.matrix.structable <- function(x, sep="_", ...) {
    structure(as_matrix_ftable(x, sep, ...),
              dnames = NULL,
              split_vertical = NULL
              )
}

length.structable <- function(x) dim(x)[1]

is.na.structable <- function(x)
  sapply(seq_along(x), function(sub) any(is.na(sub)))

str.structable <- function(object, ...)
    str(unclass(object), ...)

find.perm <- function(vec1, vec2) {
  unlist(Map(function(x) which(x == vec2), vec1))
}

aperm.structable <- function(a, perm, resize=TRUE, ...){
  newtable <- aperm(as.table(a), perm = perm, resize = resize, ...)
  if (!is.numeric(perm))
      perm <- find.perm(names(dimnames(newtable)), names(dimnames(a)))
  structable(newtable, split_vertical = attr(a, "split_vertical")[perm])
}

############# helper function
.massage_args <- function(...) {
    args <- vector("list", 2)
    args[[1]] <- if(missing(..1)) as.symbol("grrr") else ..1
    args[[2]] <- if(missing(..2)) as.symbol("grrr") else ..2
    args
}

mplot <- function(...,
                  .list = list(),
                  layout = NULL, cex = NULL,
                  main = NULL, gp_main = gpar(fontsize = 20),
                  sub = NULL, gp_sub = gpar(fontsize = 15),
                  keep_aspect_ratio = TRUE) {
    l <- c(list(...), .list)
    ll <- length(l)
    m <- !is.null(main)
    s <- !is.null(sub)

    ## calculate layout
    if (is.null(layout))
        layout <- c(trunc(sqrt(ll)), ceiling(ll / trunc(sqrt(ll))))

    ## push base layout
    grid.newpage()
    hts = unit(1 - 0.1 * m - 0.1 * s, "null")
    if (m)
        hts <- c(unit(0.1, "null"), hts)
    if (s)
        hts <- c(hts, unit(0.1, "null"))
    pushViewport(viewport(layout =
                              grid.layout(nrow = 1 + m + s,
                                          ncol = 1, heights = hts)
                          )
                 )

    ## push main, if any
    if (!is.null(main)) {
        pushViewport(viewport(layout.pos.row = 1,
                              layout.pos.col = NULL))

        grid.text(main, gp = gp_main)
        popViewport(1)
    }

    ## push strucplots
    if (is.null(cex))
        cex <- sqrt(1/layout[1])
    pushViewport(viewport(layout.pos.row = 1 + m,
                          layout.pos.col = NULL))
    pushViewport(viewport(layout = grid.layout(nrow = layout[1],
                          ncol = layout[2]),
                          gp = gpar(cex = cex)
                          )
                 )
    count <- 1
    for (i in seq_len(layout[1]))
        for (j in seq_len(layout[2]))
            if(count <= ll) {
                pushViewport(viewport(layout.pos.row = i,
                                      layout.pos.col = j))
                pushViewport(viewport(width = 1,
                                      height = 1,
                                      default.units = if (keep_aspect_ratio) "snpc" else "npc"))
                if (inherits(l[[count]], "grob"))
                    grid.draw(l[[count]])
                else
                    if (!is.null(attr(l[[count]], "grob")))
                        grid.draw(attr(l[[count]], "grob"))
                popViewport(2)
                count <- count + 1
            }
    popViewport(2)

    ## push sub, if any
    if (!is.null(sub)) {
        pushViewport(viewport(layout.pos.row = 1 + m + s,
                              layout.pos.col = NULL))
        grid.text(sub, gp = gp_sub)
        popViewport()
    }

    popViewport(1)
}

grid_legend <- function (x, y, pch = NA, col = par('col'), labels, frame = TRUE, hgap = unit(0.8, "lines"), vgap = unit(0.8, "lines"), default_units = "lines", gp = gpar(), draw = TRUE, title = NULL, just = 'center', lwd = NA, lty = NA, size = 1, gp_title = NULL, gp_labels = NULL, gp_frame = gpar(fill = "transparent"), inset = c(0, 0))

{
    inset <- rep(inset, length.out = 2)

    if((length(x) > 1) && missing(y)) {
        y <- x[2]
        x <- x[1]
    }

    if(is.character(x))
        switch(x,
               left = {x = unit(0 + inset[1],'npc'); y = unit(0.5 + inset[2],'npc'); just = c("left","center")},
               topleft = {x = unit(0 + inset[1],'npc'); y = unit(1 - inset[2],'npc'); just = c(0,1)},
               top = {x = unit(0.5 + inset[1],'npc'); y = unit(1 - inset[2],'npc'); just = c("center", "top")},
               topright = {x = unit(1 - inset[1],'npc'); y = unit(1 - inset[2],'npc'); just = c(1,1)},
               center = {x = unit(0.5 + inset[1],'npc'); y = unit(0.5 + inset[2],'npc'); just = c("center","center")},
               bottom = {x = unit(0.5 - inset[1],'npc'); y = unit(0 + inset[2],'npc'); just = c("center","bottom")},
               bottomright = {x = unit(1 - inset[1],'npc'); y = unit(0 + inset[2],'npc'); just = c(1,0)},
               right = {x = unit(1 - inset[1],'npc'); y = unit(0.5 + inset[2],'npc'); just = c("right","center")},
               bottomleft = {x = unit(0 + inset[1],'npc'); y = unit(0 + inset[2],'npc'); just = c(0,0)})

    labels <- as.character(labels)
    nlabs <- length(labels)

    if(length(pch) == 1)
        pch <- rep(pch, nlabs)
    if(length(lwd) == 1)
        lwd <- rep(lwd, nlabs)
    if(length(lty) == 1)
        lty <- rep(lty, nlabs)
    if(length(col) == 1)
        col <- rep(col, nlabs)
    if(length(gp_labels) == 1)
        gp_labels <- rep(list(gp_labels), nlabs)


    if (is.logical(title) && !title)
        title <- NULL
    if(is.null(title))
        tit <- 0
    else
        tit <- 1

    if (!is.unit(hgap))
        hgap <- unit(hgap, default_units)
    if (length(hgap) != 1)
        stop("hgap must be single unit")
    if (!is.unit(vgap))
        vgap <- unit(vgap, default_units)
    if (length(vgap) != 1)
        stop("vgap must be single unit")

    if(tit)
        legend.layout <- grid.layout(nlabs + tit, 3,
                                     widths = unit.c(unit(2, "lines"),
                                     max(unit(rep(1, nlabs), "strwidth", as.list(c(labels))),
                                         unit(1, "strwidth", title) - unit(2, "lines")), hgap),
                                     heights = unit.pmax(unit(1, "lines"),
                                     vgap + unit(rep(1, nlabs + tit ),
                                                 "strheight", as.list(c(labels,title)))))
    else
        legend.layout <- grid.layout(nlabs, 3,
                                     widths = unit.c(unit(2, "lines"),
                                     max(unit(rep(1, nlabs), "strwidth", as.list(labels))), hgap),
                                     heights = unit.pmax(unit(1, "lines"),
                                     vgap + unit(rep(1, nlabs), "strheight", as.list(labels))))

    fg <- frameGrob(layout = legend.layout, gp = gp)

    if (frame)
        fg <- placeGrob(fg, rectGrob(gp = gp_frame))

    if (tit)
        fg <- placeGrob(fg, textGrob(title, x = .2, y = 0.5, just = c("left", "center"), gp = gp_title), col = 1, row = 1)

    for (i in 1:nlabs) {
        if(!is.na(pch[i]))
            fg <- placeGrob(fg, pointsGrob(0.5, 0.5, pch = pch[i],
                                           size = unit(size, "char"),
                                           gp = gpar(col = col[i])), col = 1, row = i + tit)
        else if(!is.na(lwd[i]) || !is.na(lty[i]))
            fg <- placeGrob(fg, linesGrob( unit(c(0.2, .8), "npc"),  unit(c(.5), "npc"),
                                          gp = gpar(col = col[i], lwd = lwd[i], lty=lty[i])), col = 1, row = i + tit)

        fg <- placeGrob(fg, textGrob(labels[i], x = .1, y = 0.5, just = c("left", "center"), gp = gp_labels[[i]]), col = 2, row = i + tit)
    }

    pushViewport(viewport(x, y, height = grobHeight(fg), width = grobWidth(fg), just = just ))

    if (draw)
        grid.draw(fg)
    popViewport(1)
    invisible(fg)
}

assocstats <- function(x) {
  if(!is.matrix(x)) {
      l <- length(dim(x))
      str <- apply(x, 3 : l, FUN = assocstats)
      if (l == 3) {
          names(str) <- paste(names(dimnames(x))[3], names(str), sep = ":")
      } else {
          dn <- dimnames(str)
          dim(str) <- NULL
          names(str) <-
              apply(expand.grid(dn), 1,
                    function(x) paste(names(dn), x, sep = ":", collapse = "|"))
      }
      return(str)
  }

  tab    <- summary(loglm(~1+2, x))$tests
  phi    <- sqrt(tab[2,1] / sum(x))
  cont   <- sqrt(phi^2 / (1 + phi^2))
  cramer <- sqrt(phi^2 / min(dim(x) - 1))
  structure(
            list(table = x,
                 chisq_tests = tab,
                 phi = ifelse(all(dim(x) == 2L), phi, NA),
                 contingency = cont,
                 cramer = cramer),
            class = "assocstats"
            )
}

print.assocstats <- function(x,
                             digits = 3,
                             ...)
{
  print(x$chisq_tests, digits = 5, ...)
  cat("\n")
  cat("Phi-Coefficient   :", round(x$phi,         digits = digits), "\n")
  cat("Contingency Coeff.:", round(x$contingency, digits = digits), "\n")
  cat("Cramer's V        :", round(x$cramer,      digits = digits), "\n")
  invisible(x)
}

summary.assocstats <- function(object, percentage = FALSE, ...) {
  tab <- summary(object$table, percentage = percentage, ...)
  tab$chisq <- NULL
  structure(list(summary = tab,
                 object  = object),
            class   = "summary.assocstats"
            )
}

print.summary.assocstats <- function(x, ...) {
  cat("\n")
  print(x$summary, ...)
  print(x$object, ...)
  cat("\n")
  invisible(x)
}

tile <- function(x, ...)
  UseMethod("tile")

tile.formula <-
function(formula, data = NULL,
         ..., main = NULL, sub = NULL, subset = NULL, na.action = NULL)
{
  if (is.logical(main) && main)
    main <- deparse(substitute(data))
  else if (is.logical(sub) && sub)
    sub <- deparse(substitute(data))

  m <- match.call(expand.dots = FALSE)
  edata <- eval(m$data, parent.frame())

  fstr <- strsplit(paste(deparse(formula), collapse = ""), "~")
  vars <- strsplit(strsplit(gsub(" ", "", fstr[[1]][2]), "\\|")[[1]], "\\+")
  varnames <- vars[[1]]

  dep <- gsub(" ", "", fstr[[1]][1])
  if (!dep %in% c("","Freq")) {
     if (all(varnames == ".")) {
       varnames <- if (is.data.frame(data))
         colnames(data)
       else
         names(dimnames(as.table(data)))
       varnames <- varnames[-which(varnames %in% dep)]
     }

    varnames <- c(varnames, dep)
  }


  if (inherits(edata, "ftable") || inherits(edata, "table") || length(dim(edata)) > 2) {
    dat <- as.table(data)
    if(all(varnames != ".")) {
      ind <- match(varnames, names(dimnames(dat)))
      if (any(is.na(ind)))
        stop(paste("Can't find", paste(varnames[is.na(ind)], collapse=" / "), "in", deparse(substitute(data))))

      dat <- margin.table(dat, ind)
    }
    tile.default(dat, main = main, sub = sub, ...)
  } else {
      m <- m[c(1, match(c("formula", "data", "subset", "na.action"), names(m), 0))]
      m[[1]] <- as.name("xtabs")
      m$formula <-
          formula(paste(if("Freq" %in% colnames(data)) "Freq",
                        "~",
                        paste(varnames, collapse = "+")))
      tab <- eval(m, parent.frame())
      tile.default(tab, main = main, sub = sub, ...)
  }
}

tile.default <-
function(x,
         tile_type = c("area", "squaredarea", "height", "width"),
         halign = c("left", "center", "right"),
         valign = c("bottom", "center", "top"),
         split_vertical = NULL,
         shade = FALSE,
         spacing = spacing_equal(unit(1, "lines")),
         set_labels = NULL,
         margins = unit(3, "lines"),
         keep_aspect_ratio = FALSE,
         legend = NULL,
         legend_width = NULL,
         squared_tiles = TRUE,
         main = NULL, sub = NULL, ...)
{
    ## argument handling
    if (is.logical(main) && main)
        main <- deparse(substitute(x))
    else if (is.logical(sub) && sub)
        sub <- deparse(substitute(x))

    tile_type <- match.arg(tile_type)
    halign <- match.arg(halign)
    valign <- match.arg(valign)

    x <- as.table(x)
    dl <- length(d <- dim(x))

    ## determine starting positions
    xpos <- 1 - (halign == "left") - 0.5 * (halign == "center")
    ypos <- 1 - (valign == "bottom") - 0.5 * (valign == "center")

    ## heuristic to adjust right/bottom margin to obtain squared tiles
    ## FIXME: better push another viewport?
    if (squared_tiles) {
        ## splitting argument
        if (is.structable(x) && is.null(split_vertical))
            split_vertical <- attr(x, "split_vertical")
        if (is.null(split_vertical))
            split_vertical <- FALSE
        if (length(split_vertical) == 1)
            split_vertical <- rep(c(split_vertical, !split_vertical),
                                  length.out = dl)
        if (length(split_vertical) < dl)
            split_vertical <- rep(split_vertical, length.out = dl)

        ## compute resulting dimnension
        dflat <- dim(unclass(structable(x, split_vertical = split_vertical)))

        ## adjust margins
        spacing <- spacing(d)
        delta <- abs(dflat[1] - dflat[2])
        fac <- delta / max(dflat)
        un <-  unit(fac, "npc") -
               unit(fac * 5 / convertWidth(spacing[[1]], "lines",
                                    valueOnly=TRUE),
                    "lines")
        leg <- if (shade) {
            if (is.null(legend_width))
                unit(5, "lines")
            else legend_width
        } else unit(0, "npc")
        if (dflat[1] < dflat[2])
            margins <- margins + unit.c(unit(0, "npc"), unit(0, "npc"),
                                        un + leg, unit(0, "npc"))
        if (dflat[1] > dflat[2])
            margins <- margins + unit.c(unit(0, "npc"), un - leg,
                                        unit(0, "npc"), unit(0, "npc"))
        if (dflat[1] == dflat[2])
            margins <- margins + unit.c(unit(0, "npc"), unit(0, "npc"),
                                        leg, unit(0, "npc"))
    }

    ## create dummy labels if some are duplicated
    ## and set the labels via set_labels
    dn <- dimnames(x)
    if (any(unlist(lapply(dn, duplicated)))) {
        dimnames(x) <- lapply(dn, seq_along)
        if (is.null(set_labels))
            set_labels <- lapply(dn, function(i) structure(i, names = seq(i)))
    }

    ## workhorse function creating bars
    panelfun <- function(residuals, observed, expected, index, gp, name) {
        xprop <- expected / max(expected)
        if (tile_type == "height")
            grid.rect(x = xpos, y = ypos,
                      height = xprop[t(index)], width = 1,
                      gp = gp, just = c(halign, valign), name = name)
        else if (tile_type == "width")
            grid.rect(x = xpos, y = ypos,
                      width = xprop[t(index)], height = 1,
                      gp = gp, just = c(halign, valign), name = name)
        else if (tile_type == "area")
            grid.rect(x = xpos, y = ypos,
                      width = sqrt(xprop[t(index)]),
                      height = sqrt(xprop[t(index)]),
                      gp = gp, just = c(halign, valign), name = name)
        else
            grid.rect(x = xpos, y = ypos,
                      width = xprop[t(index)], height = xprop[t(index)],
                      gp = gp, just = c(halign, valign), name = name)

    }

    mycore <- function(residuals, observed, expected = NULL,
                       spacing, gp, split_vertical, prefix = "") {
        struc_mosaic(panel = panelfun)(residuals,
                     array(1, dim = d, dimnames = dimnames(observed)),
                     expected = observed,
                     spacing, gp, split_vertical, prefix)
    }

    strucplot(x,
              core = mycore,
              spacing = spacing,
              keep_aspect_ratio = keep_aspect_ratio,
              margins = margins,
              shade = shade,
              legend = legend,
              legend_width = legend_width,
              main = main,
              sub = sub,
              set_labels = set_labels,
              ...)
}

# This should be revised to allow graphical parameters to be more easily passed
# for points and lines
# For now, added lwd, lty and col args for lines, with more useful defaults

Ord_plot <- function(obj, legend = TRUE, estimate = TRUE, tol = 0.1,
                     type = NULL, xlim = NULL, ylim = NULL, xlab = "Number of occurrences",
		     ylab = "Frequency ratio", main = "Ord plot", gp = gpar(cex = 0.5),
		     lwd = c(2,2), lty=c(2,1), col=c("black", "red"),
		     name = "Ord_plot", newpage = TRUE, pop = TRUE,
                     return_grob = FALSE, ...)
{
  if(is.vector(obj)) {
    obj <- table(obj)
  }
  if(is.table(obj)) {
    if(length(dim(obj)) > 1) stop ("obj must be a 1-way table")
    x <- as.vector(obj)
    count <- as.numeric(names(obj))
  } else {
    if(!(!is.null(ncol(obj)) && ncol(obj) == 2))
      stop("obj must be a 2-column matrix or data.frame")
    x <- as.vector(obj[,1])
    count <- as.vector(obj[,2])
  }

  y <- count * x/c(NA, x[-length(x)])
  fm <- lm(y ~ count)
  fmw <- lm(y ~ count, weights = sqrt(pmax(x, 1) - 1))
  fit1 <- predict(fm, data.frame(count))
  fit2 <- predict(fmw, data.frame(count))
  if(is.null(xlim)) xlim <- range(count)
  if(is.null(ylim)) ylim <- range(c(y, fit1, fit2), na.rm = TRUE)
  xlim <- xlim + c(-1, 1) * diff(xlim) * 0.04
  ylim <- ylim + c(-1, 1) * diff(ylim) * 0.04

	lwd <- rep_len(lwd, 2)  # assure length=2
	lty <- rep_len(lty, 2)
	col <- rep_len(col, 2)

  if(newpage) grid.newpage()
  pushViewport(plotViewport(xscale = xlim, yscale = ylim, default.units = "native", name = name))
  grid.points(x = count, y = y, default.units = "native", gp = gp, ...)
  grid.lines(x = count, y = fit1, default.units = "native", gp = gpar(lwd=lwd[1], lty=lty[1], col=col[1]))
  grid.lines(x = count, y = fit2, default.units = "native", gp = gpar(lwd=lwd[2], lty=lty[2], col=col[2]))
  grid.rect(gp = gpar(fill = "transparent"))
  grid.xaxis()
  grid.yaxis()
  grid.text(xlab, y = unit(-3.5, "lines"))
  grid.text(ylab, x = unit(-3, "lines"), rot = 90)
  grid.text(main, y = unit(1, "npc") + unit(2, "lines"), gp = gpar(fontface = "bold"))

  RVAL <- coef(fmw)
  names(RVAL) <- c("Intercept", "Slope")
  if(legend)
  {
    legend.text <- c(paste("slope =", round(RVAL[2], digits = 3)),
                     paste("intercept =", round(RVAL[1], digits = 3)))
    if(estimate) {
      ordfit <- Ord_estimate(RVAL, type = type, tol = tol)
      legend.text <- c(legend.text, "", paste("type:", ordfit$type),
        paste("estimate:", names(ordfit$estimate),"=", round(ordfit$estimate, digits = 3)))
      legend.text <- paste(legend.text, collapse = "\n")
    }
    grid.text(legend.text, min(count), ylim[2] * 0.95, default.units = "native", just = c("left", "top"))
  }

  if(pop) popViewport() else upViewport()
  if(return_grob)
      invisible(structure(RVAL, grob = grid.grab()))
  else
      invisible(RVAL)
}

Ord_estimate <- function(x, type = NULL, tol = 0.1)
{
  a <- x[1]
  b <- x[2]
  if(!is.null(type))
    type <- match.arg(type, c("poisson", "binomial", "nbinomial", "log-series"))
  else {
    if(abs(b) < tol) type <- "poisson"
    else if(b < (-1 * tol)) type <- "binomial"
    else if(a > (-1 * tol)) type <- "nbinomial"
    else if(abs(a + b) < 4*tol) type <- "log-series"
    else type <- "none"
  }

  switch(type,

  "poisson" = {
    par <- a
    names(par) <- "lambda"
    if(par < 0) warning("lambda not > 0")
  },
  "binomial" = {
    par <- b/(b - 1)
    names(par) <- "prob"
    if(abs(par - 0.5) > 0.5) warning("prob not in (0,1)")
  },
  "nbinomial" = {
    par <- 1 - b
    names(par) <- "prob"
    if(abs(par - 0.5) > 0.5) warning("prob not in (0,1)")
  },

  "log-series" = {
    par <- b
    names(par) <- "theta"
    if(par < 0) warning("theta not > 0")
  },
  "none" = {
    par <- NA
  })
  list(estimate = par, type = type)
}

plot.loglm <- function(x,
                       panel = mosaic,
                       type = c("observed", "expected"),
                       residuals_type = c("pearson", "deviance"),
		       gp = shading_hcl,
		       gp_args = list(),
                       ...)
{
  residuals_type <- match.arg(tolower(residuals_type), c("pearson", "deviance"))
  if(is.null(x$fitted)) x <- update(x, fitted = TRUE)
  expected <- fitted(x)
  residuals <- residuals(x, type = "pearson")
  observed <- residuals * sqrt(expected) + expected
  if(residuals_type == "deviance") residuals <- residuals(x, type = "deviance")

  gp <- if(inherits(gp, "grapcon_generator"))
    do.call("gp", c(list(observed, residuals, expected, x$df), as.list(gp_args))) else gp

  panel(observed, residuals = residuals, expected = expected, type = type,
    residuals_type = residuals_type, gp = gp, ...)
}

mosaic.loglm <- function(x, ...)
{
  plot(x, panel = mosaic, ...)
}

assoc.loglm <- function(x, ...)
{
  plot(x, panel = assoc, ...)
}

sieve.loglm <- function(x, ...)
{
  plot(x, panel = sieve, ...)
}

spine <- function(x, ...)
  UseMethod("spine")

spine.formula <- function(formula, data = list(),
  breaks = NULL, ylab_tol = 0.05, off = NULL,
  main = "", xlab = NULL, ylab = NULL, ylim = c(0, 1), margins = c(5.1, 4.1, 4.1, 3.1),
  gp = gpar(), name = "spineplot", newpage = TRUE, pop = TRUE,
  ...)
{
  ## extract x, y from formula
  mf <- model.frame(formula, data = data)
  if(NCOL(mf) != 2) stop("`formula' should specify exactly two variables")
  y <- mf[,1]
  if(!is.factor(y)) stop("dependent variable should be a factor")
  x <- mf[,2]	
  if(is.null(xlab)) xlab <- names(mf)[2]
  if(is.null(ylab)) ylab <- names(mf)[1]
     
  spine(x, y,
    breaks = breaks, ylab_tol = ylab_tol, off = off,
    main = main, xlab = xlab, ylab = ylab, ylim = ylim, margins = margins,
    gp = gp, name = name, newpage = newpage, pop = pop, ...)
}

spine.default <- function(x, y = NULL,
  breaks = NULL, ylab_tol = 0.05, off = NULL,
  main = "", xlab = NULL, ylab = NULL, ylim = c(0, 1), margins = c(5.1, 4.1, 4.1, 3.1),
  gp = gpar(), name = "spineplot", newpage = TRUE, pop = TRUE,
  ...)
{
  ## either supply a 2-way table (i.e., both y and x are categorical)
  ## or two variables (y has to be categorical - x can be categorical or numerical)
  if(missing(y)) {
    if(length(dim(x)) != 2) stop("a 2-way table has to be specified")
    tab <- x
    x.categorical <- TRUE
    if(is.null(xlab)) xlab <- names(dimnames(tab))[1]
    if(is.null(ylab)) ylab <- names(dimnames(tab))[2]
    xnam <- dimnames(tab)[[1]]
    ynam <- dimnames(tab)[[2]]
    ny <- NCOL(tab)
    nx <- NROW(tab)
  } else {
    if(!is.factor(y)) stop("dependent variable should be a factor")
    x.categorical <- is.factor(x)
    if(!x.categorical) stopifnot(is.numeric(x), is.vector(x))
    if(is.null(xlab)) xlab <- deparse(substitute(x))
    if(is.null(ylab)) ylab <- deparse(substitute(y))
    if(x.categorical) {
      tab <- table(x, y)
      xnam <- levels(x)
      nx <- NROW(tab)
    }
    ynam <- levels(y)
    ny <- length(ynam)
  }

  ## graphical parameters
  if(is.null(gp$fill)) gp$fill <- gray.colors(ny)
  gp$fill <- rep(gp$fill, length.out = ny)
  off <- if(!x.categorical) 0 else if(is.null(off)) 0.02 else off/100

  if(x.categorical) {
    ## compute rectangle positions on x axis
    xat <- c(0, cumsum(prop.table(margin.table(tab, 1)) + off))
  } else {
    ## compute breaks for x
    if(is.null(breaks)) breaks <- list()
    if(!is.list(breaks)) breaks <- list(breaks = breaks)
    breaks <- c(list(x = x), breaks)
    breaks$plot <- FALSE
    breaks <- do.call("hist", breaks)$breaks
    ## categorize x
    x1 <- cut(x, breaks = breaks, include.lowest = TRUE)
    ## compute rectangle positions on x axis
    xat <- c(0, cumsum(prop.table(table(x1))))
    ## construct table
    tab <- table(x1, y)
    nx <- NROW(tab)
  }
  
  ## compute rectangle positions on y axis
  yat <- rbind(0, apply(prop.table(tab, 1), 1, cumsum))
  
  ## setup plot
  if(newpage) grid.newpage()
  pushViewport(plotViewport(xscale = c(0, 1 + off * (nx-1)), yscale = ylim,
    default.units = "native", name = name, margins = margins, ...))

  ## compute coordinates
  ybottom <- as.vector(yat[-(ny+1),])
  ybottom[ybottom < ylim[1]] <- ylim[1]
  ybottom[ybottom > ylim[2]] <- ylim[2]
  ytop <- as.vector(yat[-1,])
  ytop[ytop < ylim[1]] <- ylim[1]
  ytop[ytop > ylim[2]] <- ylim[2]
  xleft <- rep(xat[1:nx], rep(ny, nx))
  xright <- rep(xat[2:(nx+1)] - off, rep(ny, nx))
  gp$fill <- rep(gp$fill, nx)

  ## plot rectangles
  grid.rect(xleft, ybottom, width = (xright-xleft), height = (ytop-ybottom),
    just = c("left", "bottom"), default.units = "native", gp = gp)

  ## axes
  ## 1: either numeric or level names
  if(x.categorical)
    grid.text(x = unit((xat[1:nx] + xat[2:(nx+1)] - off)/2, "native"), y = unit(-1.5, "lines"),
      label = xnam, check.overlap = TRUE)
  else
    grid.xaxis(at = xat, label = breaks)
    
  ## 2: axis with level names of y
  yat <- yat[,1]
  equidist <- any(diff(yat) < ylab_tol)
  yat <- if(equidist) seq(1/(2*ny), 1-1/(2*ny), by = 1/ny)
           else (yat[-1] + yat[-length(yat)])/2
  grid.text(x = unit(-1.5, "lines"), y = unit(yat, "native"), label = ynam,
    rot = 90, check.overlap = TRUE)
  
  ## 3: none
  ## 4: simple numeric  
  grid.yaxis(main = FALSE)
  
  ## annotation
  grid.text(xlab, y = unit(-3.5, "lines"))
  grid.text(ylab, x = unit(-3, "lines"), rot = 90)
  grid.text(main, y = unit(1, "npc") + unit(2, "lines"), gp = gpar(fontface = "bold"))

  ## pop
  if(pop) popViewport()
  
  ## return table visualized
  names(dimnames(tab)) <- c(xlab, ylab)
  invisible(tab)  
}

binreg_plot <-
function(model, main = NULL, xlab = NULL, ylab = NULL,
         xlim = NULL, ylim = NULL, pred_var = NULL, pred_range = c("data", "xlim"),
         group_vars = NULL, base_level = NULL, subset,
         type = c("response", "link"), conf_level = 0.95, delta = FALSE,
         pch = NULL, cex = 0.6, jitter_factor = 0.1,
         lwd = 5, lty = 1, point_size = 0, col_lines = NULL, col_bands = NULL,
         legend = TRUE, legend_pos = NULL, legend_inset = c(0, 0.1),
         legend_vgap = unit(0.5, "lines"),
         labels = FALSE, labels_pos = c("right", "left"),
         labels_just = c("left","center"),
         labels_offset = c(0.01, 0),
         gp_main = gpar(fontface = "bold", fontsize = 14),
         gp_legend_frame = gpar(lwd = 1, col = "black"),
         gp_legend_title = gpar(fontface = "bold"),
         newpage = TRUE, pop = FALSE, return_grob = FALSE)
{
    if (!inherits(model, "glm"))
        stop("Method requires a model of class 'glm'.")
    type <- match.arg(type)
    labels_pos <- match.arg(labels_pos)
    if (is.character(pred_range))
        pred_range <- match.arg(pred_range)

    ## extract data from model
    mod <- model.frame(model)
    term <- terms(mod)
    data.classes <- attr(term, "dataClasses")
    nam <- names(data.classes)

    ## determine response
    r <- attr(term, "response")
    resp <- nam[r]
    data.classes <- data.classes[-r]
    nam <- nam[-r]

    ## determine numeric predictor (take first)
    if (is.null(pred_var)) {
        fac <- data.classes %in% c("factor","logical")
        pred_var_model <- names(data.classes[!fac][1])
        pred_var <-
            names(unlist(sapply(all.vars(term),
                                grep, pred_var_model)))[1]

    } else pred_var_model <- pred_var

    ## filter observed data using model (to account for models fitted with subset=...)
    dat <- model$data[row.names(mod),]
    ## sort observations using order of numeric predictor
    o <- order(dat[,pred_var,drop=TRUE])
    mod <- mod[o,]
    dat <- dat[o,]

    ## apply subset argument, if any
    if (!missing(subset)) {
        e <- substitute(subset)
        i <- eval(e, dat, parent.frame())
        i <- i & !is.na(i)
        dat <- dat[i,]
        mod <- mod[i,]
    }

    ## determine conditioning variables. Remove all those with only one level observed.
    if (is.null(group_vars)) {
        group_vars <- nam[data.classes %in% "factor"]
        sing <- na.omit(sapply(dat, function(i) all(i == i[1])))
        if (any(sing))
            group_vars <- setdiff(group_vars, names(sing)[sing])
        if(length(group_vars) < 1)
            group_vars <- NULL
    } else
        if (is.na(group_vars) || is.logical(group_vars) && !group_vars[1])
            group_vars <- NULL

    ## set y axis limits - either probability or logit scale
    if(is.null(ylim))
        ylim <- if (type == "response")
                    c(0,1)
                else
                    range(predict(model, dat, type = "link"))
    ## allow for some cosmetic extra space
    ylimaxis <- ylim + c(-1, 1) * diff(ylim) * 0.04

    if(is.null(xlim))
        xlim <- if (is.numeric(pred_range))
                    range(pred_range)
                else
                    range(dat[,pred_var])
    xlimaxis <- xlim + c(-1, 1) * diff(xlim) * 0.04

    ## set default base level ("no effect") of response to first level/0
    if (is.null(base_level))
        base_level <- if(is.matrix(mod[,resp]))
                          2
                      else if(is.factor(mod[,resp]))
                          levels(mod[,resp])[1]
                      else
                          0
    if (is.matrix(mod[,resp]) && is.character(base_level))
        base_level <- switch(base_level, success =, Success = 1, failure =, Failure = 2)

    ## determine labels of conditioning variables, if any
    if (is.null(group_vars)) {
        labels <- legend <- FALSE
    } else {
        ## compute cross-factors for more than two conditioning variables
        if (length(group_vars) > 1) {
            cross <- paste(group_vars, collapse = " x ")
            dat[,cross] <- factor(apply(dat[,group_vars], 1, paste, collapse = " : "))
            group_vars <- cross
        }
        lev <- levels(dat[,group_vars,drop=TRUE])
    }

    ## set x- and y-lab
    if (is.null(xlab))
        xlab <- pred_var
    if (is.null(ylab))
        ylab <- if (type == "response") {
                    if (is.matrix(mod[,resp]))
                        paste0("P(",
                               c("Failure","Success")[base_level],
                               ")")
                    else
                        paste0("P(", resp, ")")
                } else {
                    if (is.matrix(mod[,resp]))
                        paste0("logit(",
                               c("Failure","Success")[base_level],
                               ")")
                    else
                        paste0("logit(", resp, ")")
                }

    ## rearrange default plot symbol palette
    if (is.null(pch))
        pch <- c(19,15,17, 1:14, 16, 18, 20:25)

    ## determine normal quantile for confidence band
    quantile <- qnorm((1 + conf_level) / 2)

    ## determine default legend position, given the curve's slope
    ## (positive -> topleft, negative -> topright)
    if (is.null(legend_pos))
        legend_pos <-
            if (coef(model)[grep(pred_var, names(coef(model)))[1]] > 0)
                "topleft"
            else
                "topright"

    ## work horse for drawing points, fitted curve and confidence band
    draw <- function(ind, colband, colline, pch, label) {
        ## plot observed data as points on top or bottom
        ycoords <- if (is.matrix(mod[,resp])) {
            tmp <- prop.table(mod[ind,resp], 1)[,switch(base_level, 2, 1)]
            if (type == "link")
                family(model)$linkfun(tmp)
            else
                tmp
        } else
            jitter(ylim[1 + (mod[ind, resp] != base_level)], jitter_factor)
        if (cex > 0)
            grid.points(unit(dat[ind, pred_var, drop = TRUE], "native"),
                        unit(ycoords, "native"),
                        pch = pch, size = unit(cex, "char"), gp = gpar(col = colline),
                        default.units = "native"
                        )

        ## confidence band and fitted values
        typ <- if (type == "response" && !delta) "link" else type
        if (is.character(pred_range)) {
            if (pred_range == "data") {
                D <- dat[ind,]
                P <- D[,pred_var, drop = TRUE]
            } else {
                P <- seq(from = xlim[1L], to = xlim[2L], length.out = 100L)
                D <- dat[ind,][rep(1L, length(P)),]
                D[,pred_var] <- P
            }
        } else {
            P <- pred_range
            D <- dat[ind,][rep(1L, length(P)),]
            D[,pred_var] <- P
        }
        pr <- predict(model, D, type = typ, se.fit = TRUE)
        lower <- pr$fit - quantile * pr$se.fit
        upper <- pr$fit + quantile * pr$se.fit
        if (type == "response" && !delta) {
            lower <- family(model)$linkinv(lower)
            upper <- family(model)$linkinv(upper)
            pr$fit <- family(model)$linkinv(pr$fit)
        }
        if (type == "response") { ## cut probs at unit interval
            lower[lower < 0] <- 0
            upper[upper > 1] <- 1
        }
        grid.polygon(unit(c(P, rev(P)), "native"),
                     unit(c(lower, rev(upper)), "native"),
                     gp = gpar(fill = colband, col = NA))
        grid.lines(unit(P, "native"),
                   unit(pr$fit, "native"),
                   gp = gpar(col = colline, lwd = lwd, lty = lty))
        if (point_size > 0)
            grid.points(unit(P, "native"),
                        unit(pr$fit, "native"), pch = pch,
                        size = unit(point_size, "char"),
                        gp = gpar(col = colline))

        ## add labels, if any
        if (labels) {
            x = switch(labels_pos,
                       left = P[1],
                       right = P[length(P)])
            y = switch(labels_pos,
                       left = pr$fit[1],
                       right = pr$fit[length(pr$fit)])
            grid.text(x = unit(x, "native") + unit(labels_offset[1], "npc"),
                      y = unit(y, "native") + unit(labels_offset[2], "npc"),
                      label = label,
                      just = labels_just,
                      gp = gpar(col = colline))
        }
    }

    ## determine colors and plot symbols
    llev <- if (is.null(group_vars)) 1 else length(lev)
    pch <- rep(pch, length.out = llev)
    if (is.null(col_bands))
        col_bands <- colorspace::rainbow_hcl(llev, alpha = 0.2)
    if (is.null(col_lines))
        col_lines <- colorspace::rainbow_hcl(llev, l = 50)

    ## set up plot region, similar to plot.xy()
    if (newpage) grid.newpage()
    pushViewport(plotViewport(xscale = xlimaxis, yscale = ylimaxis, default.units = "native", name = "binreg_plot"))
    grid.rect(gp = gpar(fill = "transparent"))
    grid.xaxis()
    grid.yaxis()
    grid.text(xlab, y = unit(-3.5, "lines"))
    grid.text(ylab, x = unit(-3, "lines"), rot = 90)
    grid.text(main, y = unit(1, "npc") + unit(2, "lines"), gp = gp_main)
    pushViewport(viewport(xscale = xlimaxis, yscale = ylimaxis, default.units = "native", clip = "on"))

    ## draw fitted curve(s)
    if (is.null(group_vars)) {
        ## single curve
        draw(1:nrow(dat),
             col_bands,
             col_lines,
             pch[1])
    } else {
        ## multiple curves
        for (i in seq_along(lev)) {
            ind <- dat[,group_vars,drop=TRUE] == lev[i]
            draw(ind, col_bands[i], col_lines[i], pch[i], lev[i])
        }

        if (legend)
            grid_legend(legend_pos,
                        labels = lev,
                        col = col_lines,
                        lty = "solid",
                        lwd = lwd,
                        vgap = legend_vgap,
                        gp_frame = gp_legend_frame,
                        inset = legend_inset,
                        title = group_vars,
                        gp_title = gp_legend_title)
    }

    if (pop) popViewport(2)
    if (return_grob)
        invisible(grid.grab())
    else
        invisible(NULL)
}

###########
grid_abline <- function(a, b, ...)
{
    ## taken from graphics::abline()
    if (is.object(a) || is.list(a)) {
        p <- length(coefa <- as.vector(coef(a)))
        if (p > 2)
            warning(gettextf("only using the first two of %d regression coefficients",
                p), domain = NA)
        islm <- inherits(a, "lm")
        noInt <- if (islm)
            !as.logical(attr(stats::terms(a), "intercept"))
        else p == 1
        if (noInt) {
            a <- 0
            b <- coefa[1L]
        }
        else {
            a <- coefa[1L]
            b <- if (p >= 2)
                coefa[2L]
            else 0
        }
    }
    grid.abline(a, b, ...)
}

##################################################################
## spacings

spacing_equal <- function(sp = unit(0.3, "lines")) {
  if (!is.unit(sp)) sp <- unit(sp, "lines")
  function(d, condvars = NULL) lapply(d, function(x) if(x > 1) rep(sp, x - 1) else NA)
}
class(spacing_equal) <- "grapcon_generator"

spacing_dimequal <- function(sp) {
  if (!is.unit(sp)) sp <- unit(sp, "lines")
  function(d, condvars = NULL)
    lapply(seq_along(d), function(i) if(d[i] > 1) rep(sp[i], d[i] - 1) else NA)
}
class(spacing_dimequal) <- "grapcon_generator"

spacing_increase <- function(start = unit(0.3, "lines"), rate = 1.5) {
  if (!is.unit(start)) start <- unit(start, "lines")
  function(d, condvars = NULL) {
    sp <- start * rev(cumprod(c(1, rep.int(rate, length(d) - 1))))
    spacing_dimequal(sp)(d = d, condvars = condvars)
  }
}
class(spacing_increase) <- "grapcon_generator"

spacing_highlighting <- function(start = unit(0.2, "lines"), rate = 1.5) {
 if (!is.unit(start)) start <- unit(start, "lines")
  function(d, condvars = NULL)
    c(spacing_increase(start, rate)(d, condvars)[-length(d)],
      list(unit(rep(0, d[length(d)]), "lines")))
}
class(spacing_highlighting) <- "grapcon_generator"

spacing_conditional <- function(sp = unit(0.3, "lines"),
                                start = unit(2, "lines"), rate = 1.8) {
  condfun <- spacing_increase(start, rate)
  equalfun <- spacing_equal(sp)
  equalfun2 <- spacing_equal(start)
  function(d, condvars) {
    if (length(d) < 3)
      return(spacing_equal(sp)(d, condvars))
    condvars <- seq(condvars)
    ret <- vector("list", length(d))
    ret[condvars] <- if (length(condvars) < 3)
      equalfun2(d[condvars])
    else
      condfun(d[condvars])
    ret[-condvars] <- equalfun(d[-condvars])
    ret
  }
}
class(spacing_conditional) <- "grapcon_generator"

goodfit <- function(x, type = c("poisson", "binomial", "nbinomial"),
                    method = c("ML", "MinChisq"), par = NULL)
{
    if(is.vector(x)) {
        x <- table(x)
    }
    if(is.table(x)) {
        if(length(dim(x)) > 1) stop ("x must be a 1-way table")
        freq <- as.vector(x)
        count <- as.numeric(names(x))
    } else {
          if(!(!is.null(ncol(x)) && ncol(x) == 2))
              stop("x must be a 2-column matrix or data.frame")
          freq <- as.vector(x[,1])
          count <- as.vector(x[,2])
      }

    ## fill-in possibly missing cells
    nfreq <- rep(0, max(count) + 1)
    nfreq[count + 1] <- freq
    freq <- nfreq
    count <- 0:max(count)
    n <- length(count)

    ## starting value for degrees of freedom
    df <- -1

    type <- match.arg(type)
    method <- match.arg(method)

    switch(type,

           "poisson" = {

               if(!is.null(par)) {
                   if(!is.list(par))
                       stop("`par' must be a named list")
                   if(names(par) != "lambda")
                       stop("`par' must specify `lambda'")

                   par <- par$lambda
                   method <- "fixed"
               }
               else if(method == "ML") {
                   df <- df - 1
                   par <- weighted.mean(count,freq)
               }
               else if(method == "MinChisq") {
                   df <- df - 1

                   chi2 <- function(x)
                   {
                   p.hat <- diff(c(0, ppois(count[-n], lambda = x), 1))
                   expected <- sum(freq) * p.hat
                   sum((freq - expected)^2/expected)
                   }

                   par <- optimize(chi2, range(count))$minimum
               }
               par <- list(lambda = par)
               p.hat <- dpois(count, lambda = par$lambda)
           },

           "binomial" = {
               size <- par$size
               if(is.null(size)) {
                   size <- max(count)
                   warning("size was not given, taken as maximum count")
               }

               if(size > max(count)) {
                   nfreq <- rep(0, size + 1)
                   nfreq[count + 1] <- freq
                   freq <- nfreq
                   count <- 0:size
                   n <- length(count)
               }

               if(!is.null(par$prob)) {
                   if(!is.list(par))
                       stop("`par' must be a named list and specify `prob'")
                   par <- par$prob
                   method <- "fixed"
               }
               else if(method == "ML") {
                   df <- df - 1
                   par <- weighted.mean(count/size, freq)
               }
               else if(method == "MinChisq") {
                   df <- df - 1

                   chi2 <- function(x)
                   {
                   p.hat <- diff(c(0, pbinom(count[-n], prob = x, size = size), 1))
                   expected <- sum(freq) * p.hat
                   sum((freq - expected)^2/expected)
                   }

                   par <- optimize(chi2, c(0,1))$minimum
               }
               par <- list(prob = par, size = size)
               p.hat <- dbinom(count, prob = par$prob, size = par$size)
           },


           "nbinomial" = {

               if(!is.null(par)) {
                   if(!is.list(par)) stop("`par' must be a named list")
                   if(!(isTRUE(all.equal(names(par), "size")) | isTRUE(all.equal(sort(names(par)), c("prob", "size")))))
                       stop("`par' must specify `size' and possibly `prob'")
                   if(!is.null(par$prob)) method <- "fixed"
               }

               switch(method,

                      "ML" = {
                          if(is.null(par$size)) {
                              df <- df - 2
                              par <- fitdistr(rep(count, freq), "negative binomial")$estimate
                              par <- par[1]/c(1, sum(par))
                          } else {
                                df <- df - 1
                                method <- c("ML", "with size fixed")

                                size <- par$size
                                xbar <- weighted.mean(count,freq)
                                par <- c(size, size/(xbar+size))
                            }
                      },

                      "MinChisq" = {
                          if(is.null(par$size)) {
                              df <- df - 2

                              ## MM
                              xbar <- weighted.mean(count,freq)
                              s2 <- var(rep(count,freq))
                              p <- xbar / s2
                              size <- xbar^2/(s2 - xbar)
                              par1 <- c(size, p)

                              ## minChisq
                              chi2 <- function(x)
                              {
                              p.hat <- diff(c(0, pnbinom(count[-n], size = x[1], prob = x[2]), 1))
                              expected <- sum(freq) * p.hat
                              sum((freq - expected)^2/expected)
                              }

                              par <- optim(par1, chi2)$par
                          } else {
                                df <- df - 1
                                method <- c("MinChisq", "with size fixed")

                                chi2 <- function(x)
                                {
                                p.hat <- diff(c(0, pnbinom(count[-n], size = par$size, prob = x), 1))
                                expected <- sum(freq) * p.hat
                                sum((freq - expected)^2/expected)
}
                                par <- c(par$size, optimize(chi2, c(0, 1))$minimum)
                            }
                      },

                      "fixed" = {
                          par <- c(par$size, par$prob)
                      })

               par <- list(size = par[1], prob = par[2])
               p.hat <- dnbinom(count, size = par$size, prob = par$prob)
           })

    expected <- sum(freq) * p.hat

    df <- switch(method[1],
                 "MinChisq" = { length(freq) + df },
                 "ML" = { sum(freq > 0) + df },
                 "fixed" = { c(length(freq), sum(freq > 0)) + df }
                 )

    structure(list(observed = freq,
                   count = count,
                   fitted = expected,
                   type = type,
                   method = method,
                   df = df,
                   par = par),
              class = "goodfit")
}

# does this need a residuals_type arg?
print.goodfit <- function(x, residuals_type = c("pearson", "deviance", "raw"), ...)
{
    residuals_type <- match.arg(residuals_type)
    cat(paste("\nObserved and fitted values for", x$type, "distribution\n"))
    if(x$method[1] == "fixed")
      cat("with fixed parameters \n\n")
    else
      cat(paste("with parameters estimated by `", paste(x$method, collapse = " "), "' \n\n", sep = ""))

    resids <- residuals(x, type = residuals_type)
    RVAL <- cbind(x$count, x$observed, x$fitted, resids)
    colnames(RVAL) <- c("count", "observed", "fitted",
                        paste(residuals_type, "residual"))
    rownames(RVAL) <- rep("", nrow(RVAL))
    print(RVAL, ...)
    invisible(x)
}

summary.goodfit <- function(object, ...)
{
    df <- object$df
    obsrvd <- object$observed
    count <- object$count
    expctd <- fitted(object)

    G2 <- sum(ifelse(obsrvd == 0, 0, obsrvd * log(obsrvd/expctd))) * 2

    n <- length(obsrvd)
    pfun <- switch(object$type,
                   poisson = "ppois",
                   binomial = "pbinom",
                   nbinomial = "pnbinom")
    p.hat <- diff(c(0, do.call(pfun, c(list(q = count[-n]), object$par)), 1))
    expctd <- p.hat * sum(obsrvd)
    X2 <- sum((obsrvd - expctd)^2 / expctd)

    names(G2) <- "Likelihood Ratio"
    names(X2) <- "Pearson"
    if(any(expctd < 5) & object$method[1] != "ML")
        warning("Chi-squared approximation may be incorrect")

    RVAL <- switch(object$method[1],
                   ML = G2,
                   MinChisq = X2,
                   fixed = c(X2, G2)
                   )

    RVAL <- cbind(RVAL, df, pchisq(RVAL, df = df, lower.tail = FALSE))
    colnames(RVAL) <- c("X^2", "df", "P(> X^2)")

    cat(paste("\n\t Goodness-of-fit test for", object$type, "distribution\n\n"))
    print(RVAL, ...)
    invisible(RVAL)
}

plot.goodfit <- function(x, ...)
{
    rootogram(x, ...)
}

fitted.goodfit <- function(object, ...)
{
    object$fitted
}

residuals.goodfit <- function(object, type = c("pearson", "deviance", "raw"),  ...)
{
    obsrvd <- object$observed
    expctd <- fitted(object)
    count <- object$count
    n <- length(obsrvd)
    pfun <- switch(object$type,
                   poisson = "ppois",
                   binomial = "pbinom",
                   nbinomial = "pnbinom")
    p.hat <- diff(c(0, do.call(pfun, c(list(q = count[-n]), object$par)), 1))
    expctd <- p.hat * sum(obsrvd)

    res <- switch(match.arg(type),
                  pearson = (obsrvd - expctd) / sqrt(expctd),
                  deviance = ifelse(obsrvd == 0, 0,
                                    obsrvd * log(obsrvd / expctd)),
                  obsrvd - expctd)

    return(res)
}

predict.goodfit <- function(object, newcount = NULL, type = c("response", "prob"), ...)
{
    if(is.null(newcount)) newcount <- object$count
    type <- match.arg(type)

    densfun <- switch(object$type,
                      poisson = "dpois",
                      binomial = "dbinom",
                      nbinomial = "dnbinom")

    RVAL <- do.call(densfun, c(list(x = newcount), object$par))
    if (type == "response") RVAL <- RVAL * sum(object$observed)
    return(RVAL)
}

rootogram <- function(x, ...)
{
    UseMethod("rootogram")
}

rootogram.goodfit <- function(x, ...)
{
    rootogram.default(x$observed, x$fitted, names = x$count, df = x$df, ...)
}

rootogram.default <- function(x, fitted, names = NULL, scale = c("sqrt", "raw"),
                              type = c("hanging", "standing", "deviation"),
                              shade = FALSE, legend = TRUE,
                              legend_args = list(x = 0, y = 0.2, height = 0.6), df = NULL,
                              rect_gp = NULL, rect_gp_args = list(), lines_gp = gpar(col = "red", lwd = 2),
                              points_gp = gpar(col = "red"), pch = 19,
                              xlab = NULL, ylab = NULL, ylim = NULL,
                              main = NULL, sub = NULL,
                              margins = unit(0, "lines"), title_margins = NULL,
                              legend_width = NULL,

                              main_gp = gpar(fontsize = 20),
                              sub_gp = gpar(fontsize = 15),

                              name = "rootogram", prefix = "",
                              keep_aspect_ratio = FALSE,
                              newpage = TRUE, pop = TRUE,
                              return_grob = FALSE, ...)
{
    if(is.null(names)) names <- names(x)

    if(is.table(x)) {
        if(length(dim(x)) > 1) stop ("x must be a 1-way table")
        x <- as.vector(x)
    }
    obs <- x
    fit <- fitted
    res <- (obs - fit) / sqrt(fit)
    if(is.null(xlab)) {xlab <-  "Number of Occurrences"}

    if(match.arg(scale) == "sqrt") {
        obs <- sqrt(obs)
        fit <- sqrt(fit)
        if(is.null(ylab)) {ylab <- "sqrt(Frequency)"}
    } else {
        if(is.null(ylab)) {ylab <- "Frequency"}
    }

    ## rect_gp (color, fill, lty, etc.) argument
    if (shade) {
        if (is.null(rect_gp)) rect_gp <- shading_hcl
        if (is.function(rect_gp)) {
            if (is.null(legend) || (is.logical(legend) && legend))
                legend <- legend_resbased
            gpfun <- if (inherits(rect_gp, "grapcon_generator"))
                         do.call("rect_gp", c(list(obs, res, fit, df), rect_gp_args)) else rect_gp
            rect_gp <- gpfun(res)
        } else if (!is.null(legend) && !(is.logical(legend) && !legend))
              stop("rect_gp argument must be a shading function for drawing a legend")
    }

    if (is.null(rect_gp))
        rect_gp <- gpar(fill = "lightgray")

    ## set up page
    if (newpage)
        grid.newpage()
    if (keep_aspect_ratio)
        pushViewport(viewport(width = 1, height = 1, default.units = "snpc"))

    pushViewport(vcdViewport(mar = margins,
                             oma = title_margins,
                             legend = shade && !(is.null(legend) || is.logical(legend) && !legend),
                             main = !is.null(main), sub = !is.null(sub),
                             keep_aspect_ratio = keep_aspect_ratio,
                             legend_width = legend_width,
                             prefix = prefix))

    ## legend
    if (inherits(legend, "grapcon_generator"))
        legend <- do.call("legend", legend_args)
    if (shade && !is.null(legend) && !(is.logical(legend) && !legend)) {
        seekViewport(paste(prefix, "legend", sep = ""))
        legend(res, gpfun, "Pearson\nresiduals:")
    }

    ## titles
    if (!is.null(main)) {
        seekViewport(paste(prefix, "main", sep = ""))
        if (is.logical(main) && main)
            main <- deparse(substitute(x))
        grid.text(main, gp = main_gp)
    }

    if (!is.null(sub)) {
        seekViewport(paste(prefix, "sub", sep = ""))
        if (is.logical(sub) && sub && is.null(main))
            sub <- deparse(substitute(x))
        grid.text(sub, gp = sub_gp)
    }

    seekViewport(paste(prefix, "plot", sep = ""))
    switch(match.arg(type),

           "hanging" = {
               if(is.null(ylim)) {ylim <- range(-0.1 * c(fit-obs,fit),
                                                c(fit-obs,fit)) + c(0, 0.1)}
               dummy <- grid_barplot(obs, names = names, offset = fit - obs, gp = rect_gp,
                                     xlab = xlab, ylab = ylab, ylim = ylim,
                                     name = name, newpage = FALSE, pop = FALSE, return_grob = FALSE, ...)
               downViewport(name)
               grid.lines(x = dummy, y = fit, default.units = "native", gp = lines_gp)
               grid.points(x = dummy, y = fit, default.units = "native", gp = points_gp, pch = pch)
               grid.lines(x = unit(c(0, 1), "npc"), y = unit(0, "native"))
               if(pop) popViewport() else upViewport()
           },

           "standing" = {
               if(is.null(ylim)) {ylim <- range(-0.01 * c(obs,fit), c(obs,fit)) }
               dummy <- grid_barplot(obs, names = names, gp = rect_gp,
                                     xlab = xlab, ylab = ylab, ylim = ylim,
                                     name = name, newpage = FALSE, pop = FALSE, return_grob = FALSE, ...)
               downViewport(name)
               grid.lines(x = dummy, y = fit, default.units = "native", gp = lines_gp)
               grid.points(x = dummy, y = fit, default.units = "native", gp = points_gp, pch = pch)
               if(pop) popViewport() else upViewport()
           },

           "deviation" = {
               if(is.null(ylim)) {ylim <- range(-0.1 * c(fit-obs,fit),
                                                c(fit-obs,fit)) + c(0, 0.1)}
               dummy <- grid_barplot(fit - obs, names = names, gp = rect_gp,
                                     xlab = xlab, ylab = ylab, ylim = ylim,
                                     name = name, newpage = FALSE, pop = FALSE, return_grob = FALSE, ...)
               downViewport(name)
               grid.lines(x = dummy, y = fit, default.units = "native", gp = lines_gp)
               grid.points(x = dummy, y = fit, default.units = "native", gp = points_gp, pch = pch)
               if(pop) popViewport() else upViewport()
           }

           )

    if (return_grob)
        invisible(grid.grab())
    else
        invisible(NULL)
}

grid_barplot <- function(height, width = 0.8, offset = 0,
                         names = NULL, xlim = NULL, ylim = NULL, xlab = "", ylab = "", main = "",
                         gp = gpar(fill = "lightgray"), name = "grid_barplot", newpage = TRUE, pop = FALSE, return_grob = FALSE)
{
    if(is.null(names)) names <- names(height)
    height <- as.vector(height)
    n <- length(height)
    width <- rep(width, length.out = n)
    offset <- rep(offset, length.out = n)

    if(is.null(names)) names <- rep("", n)
    if(is.null(xlim)) xlim <- c(1 - mean(width[c(1, n)]), n + mean(width[c(1, n)]))
    if(is.null(ylim)) ylim <- c(min(offset), max(height + offset))

    if(newpage) grid.newpage()
    pushViewport(plotViewport(xscale = xlim, yscale = ylim, default.units = "native", name = name))
    grid.rect(x = 1:n, y = offset, width = width, height = height,
              just = c("centre", "bottom"), default.units = "native", gp = gp)
    grid.yaxis()
    grid.text(names, x = unit(1:n, "native"), y = unit(rep(-1.5, n), "lines"))
    grid.text(xlab, y = unit(-3.5, "lines"))
    grid.text(ylab, x = unit(-3, "lines"), rot = 90)
    grid.text(main, y = unit(1, "npc") + unit(2, "lines"), gp = gpar(fontface = "bold"))
    if(pop) popViewport() else upViewport()
    if (return_grob)
        invisible(structure(1:n, grob = grid.grab()))
    else
        invisible(1:n)
}

Kappa <- function (x, weights = c("Equal-Spacing", "Fleiss-Cohen"))
{
  if (is.character(weights))
      weights <- match.arg(weights)

  d  <- diag(x)
  n  <- sum(x)
  nc <- ncol(x)
  colFreqs <- colSums(x)/n
  rowFreqs <- rowSums(x)/n

  ## Kappa
  kappa <- function (po, pc)
    (po - pc) / (1 - pc)
  std  <- function (p, pc, kw, W = diag(1, ncol = nc, nrow = nc)) {
    sqrt((sum(p * sweep(sweep(W, 1, W %*% colSums(p) * (1 - kw)), 2, W %*% rowSums(p) * (1 - kw)) ^ 2) - (kw - pc * (1 - kw)) ^ 2) / crossprod(1 - pc) / n)
  }

  ## unweighted
  po <- sum(d) / n
  pc <- crossprod(colFreqs, rowFreqs)[1]
  k <- kappa(po, pc)
  s <- std(x / n, pc, k)

  ## weighted
  W <- if (is.matrix(weights))
    weights
  else if (weights == "Equal-Spacing")
    1 - abs(outer(1:nc, 1:nc, "-")) / (nc - 1)
  else
    1 - (abs(outer(1:nc, 1:nc, "-")) / (nc - 1))^2
  pow <- sum(W * x) / n
  pcw <- sum(W * colFreqs %o% rowFreqs)
  kw <- kappa(pow, pcw)
  sw <- std(x / n, pcw, kw, W)

  structure(
            list(Unweighted = c(
                   value = k,
                   ASE   = s
                   ),
                 Weighted = c(
                   value = kw,
                   ASE   = sw
                   ),
                 Weights = W
                 ),
            class = "Kappa"
       )
}

print.Kappa <-
		function (x, digits=max(getOption("digits") - 3, 3), CI=FALSE, level=0.95, ...)
{
	tab <- rbind(x$Unweighted, x$Weighted)
	z <- tab[,1] / tab[,2]
	tab <- cbind(tab, z, `Pr(>|z|)` = 2 * pnorm(-abs(z)))
	if (CI) {
		q <-  qnorm((1 + level)/2)
		lower <- tab[,1] - q * tab[,2]
		upper <- tab[,1] + q * tab[,2]
		tab <- cbind(tab, lower, upper)
	}

	rownames(tab) <- names(x)[1:2]
	print(tab, digits=digits, ...)
	invisible(x)
}

summary.Kappa <- function (object, ...)
  structure(object, class = "summary.Kappa")

print.summary.Kappa <- function (x, ...) {
  print.Kappa(x, ...)
  cat("\nWeights:\n")
  print(x$Weights, ...)
  invisible(x)
}

confint.Kappa <- function(object, parm, level = 0.95, ...) {
  q <- qnorm((1 + level) / 2)
  matrix(c(max(-1, object[[1]][1] - object[[1]][2] * q),
           min(1, object[[1]][1] + object[[1]][2] * q),
           max(-1, object[[2]][1] - object[[2]][2] * q),
           min(1, object[[2]][1] + object[[2]][2] * q)),
         ncol = 2, byrow = TRUE,
         dimnames = list(Kappa = c("Unweighted","Weighted"), c("lwr","upr"))
         )
}

## Modified 1/25/2012 11:43AM by M. friendly
# -- added fill_col argument, specifying a function to be used to fill the tiles
# -- added xscale, yscale arguments to show the marginal frequencies at top & right
# -- added line_col to change the color of the diagonal line
## Modified 3/24/2012 11:38AM by M. friendly
# -- fixed buglet with yscale=TRUE and reverse_y=FALSE

"agreementplot" <- function (x, ...)
	UseMethod ("agreementplot")

"agreementplot.formula" <-
		function (formula, data = NULL, ..., subset)
{
	m <- match.call(expand.dots = FALSE)
	edata <- eval(m$data, parent.frame())
	if (inherits(edata, "ftable") || inherits(edata, "table")) {
		data <- as.table(data)
		varnames <- attr(terms(formula), "term.labels")
		if (all(varnames != "."))
			data <- margin.table(data, match(varnames, names(dimnames(data))))
		agreementplot(data, ...)
	}
	else {
		if (is.matrix(edata))
			m$data <- as.data.frame(data)
		m$... <- NULL
		m[[1L]] <- quote(stats::model.frame)
		mf <- eval(m, parent.frame())
		if (length(formula) == 2) {
			by <- mf
			y <- NULL
		}
		else {
			i <- attr(attr(mf, "terms"), "response")
			by <- mf[-i]
			y <- mf[[i]]
		}
		by <- lapply(by, factor)
		x <- if (is.null(y))
					do.call("table", by)
				else if (NCOL(y) == 1)
					tapply(y, by, sum)
				else {
					z <- lapply(as.data.frame(y), tapply, by, sum)
					array(unlist(z), dim = c(dim(z[[1]]), length(z)),
							dimnames = c(dimnames(z[[1]]),
									list(names(z))))
				}
		x[is.na(x)] <- 0
		agreementplot(x, ...)
	}
}

"agreementplot.default" <- function(x,
				reverse_y = TRUE,
				main = NULL,
				weights = c(1, 1 - 1 / (ncol(x) - 1)^2),
				margins = par("mar"),
				newpage = TRUE,
				pop = TRUE,
				xlab = names(dimnames(x))[2],
				ylab = names(dimnames(x))[1],
				xlab_rot = 0, xlab_just = "center",
				ylab_rot = 90, ylab_just = "center",
				fill_col = function(j) gray((1 - (weights[j]) ^ 2) ^ 0.5),
				line_col = "red",
				xscale=TRUE, yscale = TRUE,
                                return_grob = FALSE,
                                prefix = "",
				...)
{
	if (length(dim(x)) > 2)
		stop("Function implemented for two-way tables only!")
	if (ncol(x) != nrow(x))
		stop("Dimensions must have equal length!")

	nc <- ncol(x)

	## compute relative frequencies
	n <- sum(x)
	colFreqs <- colSums(x) / n
	rowFreqs <- rowSums(x) / n

	## open viewport
	if (newpage) grid.newpage()
	pushViewport(plotViewport(margins,
                                  name = paste(prefix,"agreementplot")))
	pushViewport(viewport(width = unit(1, "snpc"),
                              height = unit(1, "snpc")))

	if(!is.null(main))
		grid.text(main, y = unit(1.1, "npc"),
				gp = gpar(fontsize = 25))

	## axis labels
	grid.text(xlab, y = -0.12, gp = gpar(fontsize = 20))
	grid.text(ylab, x = -0.1, gp = gpar(fontsize = 20), rot = 90)

	grid.rect(gp = gpar(fill = "transparent"))

	xc <- c(0, cumsum(colFreqs))
	yc <- c(0, cumsum(rowFreqs))

	my.text <- if(reverse_y)
				function(y, ...) grid.text(y = y, ...)
			else
				function(y, ...) grid.text(y = 1 - y, ...)

	my.rect <- if(reverse_y)
				function(xleft, ybottom, xright, ytop, ...)
					grid.rect(x = xleft, y = ybottom, width = xright - xleft,
							height = ytop - ybottom, just = c("left","bottom"), ...)
			else
				function(xleft, ybottom, xright, ytop, ...)
					grid.rect(x = xleft, y = 1 - ybottom, width = xright - xleft,
							height = ytop - ybottom, just = c("left","top"), ...)

	A <- matrix(0, length(weights), nc)
	for (i in 1:nc) {
		## x - axis
		grid.text(dimnames(x)[[2]][i],
				x = xc[i] + (xc[i + 1] - xc[i]) / 2,
				y = - 0.04, check.overlap = TRUE, rot = xlab_rot, just = xlab_just, ...)

		## y - axis
		my.text(dimnames(x)[[1]][i],
				y = yc[i] + (yc[i + 1] - yc[i]) / 2,
				x = - 0.03, check.overlap = TRUE, rot = ylab_rot, just = ylab_just, ...)

		## expected rectangle
		my.rect(xc[i], yc[i], xc[i + 1], yc[i + 1])

		## observed rectangle
		y0 <- c(0, cumsum(x[i,])) / sum(x[i,])
		x0 <- c(0, cumsum(x[,i])) / sum(x[,i])

		rec <- function (col, dens, lb, tr)
			my.rect(xc[i] + (xc[i + 1] - xc[i]) * x0[lb],
					yc[i] + (yc[i + 1] - yc[i]) * y0[lb],
					xc[i] + (xc[i + 1] - xc[i]) * x0[tr],
					yc[i] + (yc[i + 1] - yc[i]) * y0[tr],
					gp = gpar(fill = fill_col(j), col = col, rot = 135)
			)

		for (j in length(weights):1) {
			lb <- max(1, i - j + 1)
			tr <- 1 + min(nc, i + j - 1)
			A[j, i] <- sum(x[lb:(tr-1),i]) * sum(x[i, lb:(tr-1)])
			rec("white", NULL, lb, tr) ## erase background
			rec("black", if (weights[j] < 1) weights[j] * 20 else NULL, lb, tr)
		}

		## correct A[j,i] -> not done by Friendly==Bug?
		for (j in length(weights):1)
			if (j > 1) A[j, i] <- A[j, i] - A[j - 1, i]
	}
	if (reverse_y)
		grid.lines(c(0, 1), c(0, 1), gp = gpar(col = line_col, linetype = "longdash"))
	else
		grid.lines(c(0, 1), c(1, 0), gp = gpar(col = line_col, linetype = "longdash"))

	if (xscale) {
		cx <- xc[-(nc+1)] + diff(xc)/2
		grid.text(colSums(x),
				x = cx,
				y =  1.03, rot = xlab_rot, just = xlab_just, ...)
		grid.xaxis(at = xc, label = FALSE, main=FALSE,
				gp = gpar(fontsize=10), draw = TRUE, vp = NULL)
	}
	if (yscale) {
		cy <- yc[-(nc+1)] + diff(yc)/2
		my.text(rowSums(x),
				x = 1.04,
				y = cy, rot = 0, just = ylab_just, ...)
		grid.yaxis(at = if(reverse_y) yc else 1-yc,
				FALSE, main=FALSE,
				gp = gpar(fontsize=10), draw = TRUE, vp = NULL)
	}

	if (pop) popViewport(2) else upViewport(2)

	## Statistics - Returned invisibly
	ads <- crossprod(diag(x))
	ar  <- n * n * crossprod(colFreqs, rowFreqs)
        if (return_grob)
            invisible(structure(list(
                Bangdiwala = ads / ar,
                Bangdiwala_Weighted = (sum(weights * A)) /  ar,
                weights = weights),
                                grob = grid.grab()
                                )
                      )
        else
            invisible(list(
                Bangdiwala = ads / ar,
                Bangdiwala_Weighted = (sum(weights * A)) /  ar,
                weights = weights)
                      )
}

#################################################################
### pairsplot
## modified, 2-14-2014, MF:  fix expected values for type=

pairs.table <- function(x,
                        upper_panel = pairs_mosaic,
                        upper_panel_args = list(),

                        lower_panel = pairs_mosaic,
                        lower_panel_args = list(),

                        diag_panel = pairs_diagonal_mosaic,
                        diag_panel_args = list(),

                        main = NULL,
                        sub = NULL,
                        main_gp = gpar(fontsize = 20),
                        sub_gp = gpar(fontsize = 15),

                        space = 0.3,
                        newpage = TRUE,
                        pop = TRUE,
                        return_grob = FALSE,
                        margins = unit(1, "lines"),
                        ...)
{
  if (newpage) grid.newpage()

  if (inherits(upper_panel, "grapcon_generator"))
    upper_panel <- do.call("upper_panel", c(upper_panel_args, list(...)))
  if (inherits(lower_panel, "grapcon_generator"))
    lower_panel <- do.call("lower_panel", c(lower_panel_args, list(...)))
  if (inherits(diag_panel, "grapcon_generator"))
    diag_panel <- do.call("diag_panel", diag_panel_args)

  d <- length(dim(x))
  l <- grid.layout(d, d)
  pushViewport(viewport(width = unit(1, "snpc"), height = unit(1, "snpc")))
  pushViewport(vcdViewport(mar = margins, legend = FALSE, legend_width = NULL,
                           main = !is.null(main), sub = !is.null(sub)))
  ## titles
  if (!is.null(main)) {
    seekViewport("main")
    if (is.logical(main) && main)
      main <- deparse(substitute(x))
    grid.text(main, gp = main_gp)
  }

  if (!is.null(sub)) {
    seekViewport("sub")
    if (is.logical(sub) && sub && is.null(main))
      sub <- deparse(substitute(x))
    grid.text(sub, gp = sub_gp)
  }

  seekViewport("plot")
  pushViewport(viewport(layout = l, y = 0, just = "bottom"))

  for (i in 1:d)
    for(j in 1:d) {
      pushViewport(viewport(layout.pos.col = i, layout.pos.row = j))
      pushViewport(viewport(width = 1 - space, height = 1 - space))

      if (i > j) {
        if (!is.null(upper_panel)) upper_panel(x, j, i)
      } else if (i < j) {
        if (!is.null(lower_panel)) lower_panel(x, j, i)
      } else if (!is.null(diag_panel))
        diag_panel(x, i)

      if (pop) popViewport(2) else upViewport(2)
    }
  if (pop) popViewport(3) else upViewport(3)
  if (return_grob)
      invisible(structure(x, grob = grid.grab()))
  else
      invisible(x)
}

pairs.structable <- function(x, ...) pairs(as.table(x), ...)

## upper/lower panels

pairs_assoc <- function(...) pairs_strucplot(panel = assoc, ...)
class(pairs_assoc) <- "grapcon_generator"

pairs_mosaic <- function(...) pairs_strucplot(panel = mosaic, ...)
class(pairs_mosaic) <- "grapcon_generator"

pairs_sieve <- function(...) pairs_strucplot(panel = sieve, ...)
class(pairs_sieve) <- "grapcon_generator"

pairs_strucplot <- function(panel = mosaic,
                            type = c("pairwise", "total", "conditional", "joint"),
                            legend = FALSE, margins = c(0, 0, 0, 0),
                            labeling = NULL, ...) {
  type = match.arg(type)
  function(x, i, j) {
      index <- 1:length(dim(x))
      rest <- index[!index %in% c(i, j)]
      rest2 <- index[!index %in% 1:2]
      tl <- tail(index, 2)
      rest3 <- index[!index %in% tl]
      expected <- switch(type,
                         joint = list(1:2, rest2),
                         conditional = list(c(tl[1], rest3),
                                            c(tl[2], rest3)),
                         total = sapply(c(j, i, rest), list),
                         NULL)
      margin <- switch(type,
                       pairwise = c(j, i),
                       conditional = c(rest, j, i),
                       c(j, i, rest))
      panel(x = margin.table(x, margin),

            expected = expected,
            labeling = labeling,
            margins = margins,
            legend = legend,

            split_vertical = TRUE,

            newpage = FALSE,
            pop = FALSE,
            prefix = paste("panel:Y=",names(dimnames(x))[i],",X=",
            names(dimnames(x))[j],"|",sep = ""),
            ...)
  }
}
class(pairs_strucplot) <- "grapcon_generator"

## diagonal panels

pairs_text <- function(dimnames = TRUE,
                       gp_vartext = gpar(fontsize = 17),
                       gp_leveltext = gpar(),
                       gp_border = gpar(),
                       ...)

  function(x, i) {
    x <- margin.table(x, i)
    grid.rect(gp = gp_border)
    grid.text(names(dimnames(x)), gp = gp_vartext,  y = 0.5 + dimnames * 0.05, ...)
    if (dimnames)
      grid.text(paste("(",paste(names(x), collapse = ","), ")", sep = ""),
                y = 0.4, gp = gp_leveltext)
  }
class(pairs_text) <- "grapcon_generator"

pairs_diagonal_text <- function(varnames = TRUE,
                                gp_vartext = gpar(fontsize = 17, fontface = "bold"),
                                gp_leveltext = gpar(),
                                gp_border = gpar(),
                                pos = c("right","top"),
                                distribute = c("equal","margin"),
                                rot = 0,
                                ...) {

  xc <- unit(switch(pos[1], left = 0.1, center = 0.5, 0.9), "npc")
  yc <- unit(switch(pos[2], top = 0.9, center = 0.5, 0.1), "npc")
  distribute <- match.arg(distribute)

  function(x, i) {
    x <- margin.table(x, i)
    grid.rect(gp = gp_border)
    if (varnames)
      grid.text(names(dimnames(x)), gp = gp_vartext, x = xc, y = yc, just = pos, ...)

    l <- length(dimnames(x)[[1]])
    po <- if (distribute == "equal")
      unit(cumsum(rep(1 / (l + 1), l)), "npc")
    else {
      sizes = prop.table(x)
      unit(cumsum(c(0,sizes))[1:l] + sizes / 2, "npc")
    }
    grid.text(dimnames(x)[[1]], x = po, y = unit(1, "npc") - po,
              gp = gp_leveltext, rot = rot)
  }
}
class(pairs_diagonal_text) <- "grapcon_generator"

pairs_barplot <- function(gp_bars = NULL,
                          gp_vartext = gpar(fontsize = 17),
                          gp_leveltext = gpar(),
                          gp_axis = gpar(),
                          just_leveltext = c("center", "bottom"),
                          just_vartext = c("center", "top"),
                          rot = 0, abbreviate = FALSE,
                          check_overlap = TRUE,
                          fill = "grey",
                          var_offset = unit(1, "npc"),
                          ...)
  function(x, i) {
    if (!is.unit(var_offset))
      var_offset <- unit(var_offset, "npc")
    dn <- names(dimnames(x))
    x <- margin.table(x, i)
    if (is.function(fill)) fill <- rev(fill(dim(x)))
    if (is.null(gp_bars))
      gp_bars <- gpar(fill = fill)
    pushViewport(viewport(x = 0.3, y = 0.1, width = 0.7, height = 0.7,
                          yscale = c(0,max(x)), just = c("left", "bottom"))
                 )
    xpos <- seq(0, 1, length.out = length(x) + 1)[-1]
    halfstep <- (xpos[2] - xpos[1]) / 2
    grid.rect(xpos - halfstep, rep.int(0, length(x)), height = x,
              just = c("center", "bottom"), width = halfstep,
              gp = gp_bars, default.units = "native",
              name = paste("panel:diag=", dn[i], "|bars", sep = ""),
              ...)
    grid.yaxis(at = pretty(c(0,max(x))), gp = gp_axis)
    txt <- names(x)
    if (abbreviate)
      txt <- abbreviate(txt, abbreviate)
    grid.text(txt, y = unit(-0.15, "npc"), rot = rot,
              x = xpos - halfstep, just = just_leveltext, gp = gp_leveltext,
              check.overlap = check_overlap)
    popViewport(1)
    grid.text(names(dimnames(x)), y = var_offset, just = just_vartext, gp = gp_vartext)

  }
class(pairs_barplot) <- "grapcon_generator"

pairs_diagonal_mosaic <- function(split_vertical = TRUE,
                                  margins = unit(0, "lines"),
                                  offset_labels = -0.4,
                                  offset_varnames = 0,
                                  gp = NULL,
                                  fill = "grey",
                                  labeling = labeling_values,
                                  alternate_labels = TRUE,
                                  ...)
  function(x, i) {
    if (is.function(fill))
      fill <- rev(fill(dim(x)[i]))
    if (is.null(gp))
      gp <- gpar(fill = fill)
    mosaic(margin.table(x, i),
           newpage = FALSE,
           split_vertical = split_vertical,
           margins = margins,
           offset_labels = offset_labels,
           offset_varnames = offset_varnames,
           prefix = "diag",
           gp = gp,
           labeling = labeling_values,
           labeling_args = list(alternate_labels = TRUE),
           ...)
  }
class(pairs_diagonal_mosaic) <- "grapcon_generator"

#######################################
### doubledecker plot

doubledecker <- function(x, ...)
  UseMethod("doubledecker")

doubledecker.formula <-
function(formula, data = NULL, ..., main = NULL)
{
    if (is.logical(main) && main)
      main <- deparse(substitute(data))

    if (is.structable(data))
      data <- as.table(data)

    m <- match.call(expand.dots = FALSE)
    edata <- eval(m$data, parent.frame())

    fstr <- strsplit(paste(deparse(formula), collapse = ""), "~")
    vars <- strsplit(strsplit(gsub(" ", "", fstr[[1]][2]), "\\|")[[1]], "\\+")
    dep <- gsub(" ", "", fstr[[1]][1])
    varnames <- vars[[1]]
    if (dep == "")
      stop("Need a dependent variable!")
    varnames <- c(varnames, dep)

    if(inherits(edata, "ftable")
       || inherits(edata, "table")
       || length(dim(edata)) > 2) {
        dat <- as.table(data)
        if(all(varnames != ".")) {

          ind <- match(varnames, names(dimnames(dat)))
          if (any(is.na(ind)))
            stop(paste("Can't find", paste(varnames[is.na(ind)], collapse=" / "), "in", deparse(substitute(data))))

          dat <- margin.table(dat, ind)
        } else {
          ind <- match(dep, names(dimnames(dat)))
          if (is.na(ind))
            stop(paste("Can't find", dep, "in", deparse(substitute(data))))
          dat <- aperm(dat, c(seq_along(dim(dat))[-ind], ind))
        }
        doubledecker.default(dat, main = main, ...)
      } else {
        tab <- if ("Freq" %in% colnames(data))
          xtabs(formula(paste("Freq~", varnames, collapse = "+")),
                data = data)
        else
          xtabs(formula(paste("~", varnames, collapse = "+")),
                data = data)

        doubledecker.default(tab, main = main, ...)
      }
  }

doubledecker.default <- function(x,
                         depvar = length(dim(x)),
                         margins = c(1, 4, length(dim(x)) + 1, 1),
                         gp = gpar(fill = rev(gray.colors(tail(dim(x), 1)))),
                         labeling = labeling_doubledecker,
                         spacing = spacing_highlighting,
                         main = NULL,
                         keep_aspect_ratio = FALSE,
                         ...) {
  x <- as.table(x)
  d <- dim(x)
  l <- length(d)
  if (is.character(depvar))
    depvar <- match(depvar, names(dimnames(x)))
  condvars <- (1:l)[-depvar]
  ## order dependend var *last*
  x <- aperm(x, c(condvars, depvar))

  ## recycle gpar elements along *last* dimension, if needed
  size <- prod(d)
  FUN <- function(par) {
    if (is.structable(par))
      par <- as.table(par)
    if (length(par) < size || is.null(dim(par)))
        aperm(array(par, dim = rev(d)))
    else
        par
  }
  gp <- structure(lapply(gp, FUN), class = "gpar")


  strucplot(x,
            core = struc_mosaic(zero_split = FALSE, zero_shade = FALSE),
            condvars = l - 1,
            spacing = spacing,
            split_vertical = c(rep.int(TRUE, l - 1), FALSE),
            gp = gp,
            shade = TRUE,
            labeling = labeling,
            main = main,
            margins = margins,
            legend = NULL,
            keep_aspect_ratio = keep_aspect_ratio,
            ...
            )
}

co_table <- function(x, margin, collapse = ".")
{
  if (!is.array(x)) 
      stop("x is not an array")
  if("xtabs" %in% class(x)) attr(x, "call") <- NULL

  dx <- dim(x)
  idx <- lapply(dx, function(i) 1:i)
  dn <- dimnames(x)
  
  if(is.character(margin)) {
    if(is.null(dn)) stop("margin must be an index when no dimnames are given")
    margin <- which(names(dn) %in% margin)
  }

  idxm <- expand.grid(idx[margin])    
  cotab1 <- function(i) {
    idx[margin] <- lapply(1:length(margin), function(j) idxm[i,j])
    rval <- as.table(do.call("[", c(list(x), idx, list(drop = FALSE))))
    if(length(dim(rval)) > 1) {
      dim(rval) <- dim(x)[-margin]
      dimnames(rval) <- dimnames(x)[-margin]
    }
    return(rval)
  }    
  rval <- lapply(1:NROW(idxm), cotab1)
  if(!is.null(dn)) names(rval) <- apply(expand.grid(dn[margin]), 1, function(z) paste(z, collapse = collapse))    

  return(rval)
}

woolf_test <- function(x) {
  DNAME <- deparse(substitute(x))
  if (any(x == 0))
    x <- x + 1 / 2
  k <- dim(x)[3]
  or <- apply(x, 3, function(x) (x[1,1] * x[2,2]) / (x[1,2] * x[2,1]))
  w <-  apply(x, 3, function(x) 1 / sum(1 / x))
  o <- log(or)
  e <- weighted.mean(log(or), w)
  STATISTIC <- sum(w * (o - e)^2)
  PARAMETER <- k - 1
  PVAL <- 1 - pchisq(STATISTIC, PARAMETER)
  METHOD <- "Woolf-test on Homogeneity of Odds Ratios (no 3-Way assoc.)"
  names(STATISTIC) <- "X-squared"
  names(PARAMETER) <- "df"
  structure(list(statistic = STATISTIC, parameter = PARAMETER, 
                 p.value = PVAL, method = METHOD, data.name = DNAME, observed = o, 
                 expected = e), class = "htest")
}

#! /bin/bash

py_versions=( 3.7 3.8 3.9 3.10 3.11 )


for py in "${py_versions[@]}"
do
    echo "Building image with Python version: ${py}"
    docker build --build-arg PYTHON_VERSION=${py} -t registry.gitlab.com/vicomtech/v4/libraries/vcd/vcd-python/vcd_ci:py_${py} .
    docker push registry.gitlab.com/vicomtech/v4/libraries/vcd/vcd-python/vcd_ci:py_${py}
done

echo "Building image for pre-commit jobs"
docker build -f dockerfile_precommit -t registry.gitlab.com/vicomtech/v4/libraries/vcd/vcd-python/vcd_pre_commit .
docker pull registry.gitlab.com/vicomtech/v4/libraries/vcd/vcd-python/vcd_pre_commit

\documentclass[nojss]{jss}
%% need no \usepackage{Sweave}

%% omit thumbpdf at the moment due to problems on some systems
%% \usepackage{thumbpdf}

%% almost as usual
\author{David Meyer, Achim Zeileis, \textnormal{and} Kurt
Hornik\\Wirtschaftsuniversit\"at Wien, Austria}
\title{The Strucplot Framework:\\ Visualizing Multi-way Contingency
  Tables with \pkg{vcd}}

%% for pretty printing and a nice hypersummary also set:
\Plainauthor{David Meyer, Achim Zeileis, Kurt Hornik} %% comma-separated
\Shorttitle{The Strucplot Framework} %% a short title (if necessary)
\Plaintitle{The Strucplot Framework: Visualizing Multi-way Contingency
  Tables with vcd}

%% an abstract and keywords
\Abstract{
This paper has been published in the Journal of Statistical Software
\citep{vcd:Meyer+Zeileis+Hornik:2006b} and
describes the ``strucplot'' framework for the visualization of
multi-way contingency tables. Strucplot displays include hierarchical
conditional plots such as mosaic, association, and sieve plots, and
can be combined into more complex, specialized plots for visualizing
conditional independence, GLMs, and the results of
independence tests. The framework's modular design allows
flexible customization of the plots' graphical appearance, including
shading, labeling, spacing, and legend, by means of ``graphical
appearance control'' functions.
The framework is provided by the \proglang{R} package \pkg{vcd}.
}

\Keywords{contingency tables, mosaic plots, association plots, sieve plots,
categorical data, independence, conditional independence, HSV, HCL,
residual-based shading, \pkg{grid}, \proglang{R}}
\Plainkeywords{contingency tables, mosaic plots, association plots,
  sieve plots, categorical data, independence,
  conditional independence, HSV, HCL, residual-based shading, grid, R}

\Address{
David Meyer\\
E-mail: \email{David.Meyer@R-project.org}\\

Achim Zeileis\\
E-mail: \email{Achim.Zeileis@R-project.org}\\

Kurt Hornik\\
E-mail: \email{Kurt.Hornik@R-project.org}\\
}

\SweaveOpts{engine=R,eps=TRUE,height=6,width=7,results=hide,fig=FALSE,echo=TRUE,eps=FALSE}
\setkeys{Gin}{width=0.7\textwidth}
%\VignetteIndexEntry{The Strucplot Framework: Visualizing Multi-way Contingency Tables with vcd}
%\VignetteDepends{vcd,grid}
%\VignetteKeywords{contingency tables, mosaic plots, association plots, sieve plots, categorical data, independence, conditional independence, HSV, HCL, residual-based shading, grid, R}
%\VignettePackage{vcd}

<<preliminaries,echo=FALSE,results=hide>>=
set.seed(1071)
library(grid)
library(vcd)
data(Titanic)
data(HairEyeColor)
data(PreSex)
data(Arthritis)
art <- xtabs(~Treatment + Improved, data = Arthritis)
@

\newcommand{\var}[1]{\textit{\texttt{#1}}}
\newcommand{\data}[1]{\texttt{#1}}
\newcommand{\class}[1]{\textsf{#1}}
%% \code without `-' ligatures
\def\nohyphenation{\hyphenchar\font=-1 \aftergroup\restorehyphenation}
\def\restorehyphenation{\hyphenchar\font=`-}
{\catcode`\-=\active%
  \global\def\code{\bgroup%
    \catcode`\-=\active \let-\codedash%
    \Rd@code}}
\def\codedash{-\discretionary{}{}{}}
\def\Rd@code#1{\texttt{\nohyphenation#1}\egroup}

\newcommand{\codefun}[1]{\code{#1()}}


%% end of declarations %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


\begin{document}

%% include your article here, just as usual
%% Note that you should use the \pkg{}, \proglang{} and \code{} commands.

\section[Introduction]{Introduction}
%% Note: If there is markup in \(sub)section, then it has to be escape as above.


In order to explain multi-dimensional categorical data, statisticians
typically look for (conditional) independence structures. Whether the
task is purely exploratory or model-based, techniques such as mosaic and
association plots offer good support for visualization.  Both visualize
aspects of (possibly higher-dimensional) contingency tables, with
several extensions introduced over the last two decades, and
implementations available in many statistical environments.  A
\emph{mosaic plot} \citep{vcd:Hartigan+Kleiner:1984} is basically an
area-proportional visualization of (typically, observed) frequencies,
composed of tiles (corresponding to the cells) created by recursive
vertical and horizontal splits of a rectangle. Thus, the area of each tile
is proportional to the corresponding cell entry \emph{given} the
dimensions of previous splits. An \emph{association plot}
\citep{vcd:Cohen:1980} visualizes the standardized deviations of
observed frequencies from those expected under a certain independence
hypothesis.  Each cell is represented by a rectangle that has (signed)
height proportional to the residual and width proportional to the square
root of the expected counts, so that the area of the box is proportional
to the difference in observed and expected frequencies.

Extensions to these techniques have mainly focused on the following
aspects.

\begin{enumerate}
\item Varying the shape of bar plots and mosaic displays to yield, e.g.,
  double-decker plots \citep{vcd:hofmann:2001}, spine plots, or
  spinograms \citep{vcd:hofmann+theus}.
\item Using residual-based shadings to visualize log-linear models
  \citep{vcd:Friendly:1994,vcd:Friendly:2000} and significance of
  statistical tests \citep{vcd:Meyer+Zeileis+Hornik:2003,vcd:Zeileis+Meyer+Hornik:2007}.
\item Using pairs plots and trellis-like layouts for marginal, conditional and
  partial views \citep{vcd:Friendly:1999}.
\item Adding direct user interaction, allowing quick exploration and
  modification of the visualized models
  \citep{vcd:Unwin+Hawkins+Hofmann:1996,vcd:Theus:2003}.
\item Providing a modular and flexible implementation to easily allow
  user extensions \citep{vcd:Meyer+Zeileis+Hornik:2003,vcd:Meyer+Zeileis+Hornik:2006b}.
\end{enumerate}

\noindent Current implementations of mosaic displays can be found,
e.g., for \proglang{SAS} \citep{vcd:SAS:2005}, \pkg{ViSta} \citep{vcd:young:1996},
\pkg{MANET} \citep{vcd:Unwin+Hawkins+Hofmann:1996},
\pkg{Mondrian} \citep{vcd:Theus:2003},
\proglang{R} \citep{vcd:R:2006}, and \proglang{S-PLUS}
\citep{vcd:SPLUS:2005}. For \proglang{R}, currently three
implementations do exist in the packages \pkg{graphics} (in base \proglang{R}),
\pkg{vcd} \citep{vcd:Meyer+Zeileis+Hornik:2006b},
and \pkg{iplots} \citep{vcd:urbanek+wichtrey:2006}, respectively.
Table \ref{tab:compare} gives an overview of the available functionality in these systems.
Most environments are available on Windows, MacOS, and Linux/Unix
variants, except \pkg{MANET} which is only available for the
Macinthosh platforms.

\begin{table}[h]
  \centering
  \begin{tabular}{|l|c|c|c|c|c|c|c|c|c|}
    \hline
                             &                &
                             &\multicolumn{3}{c|}{}   &        & &\\
                             & \proglang{SAS} & \proglang{S-PLUS}
                             &\multicolumn{3}{c|}{\proglang{R}} & \pkg{ViSta} & \pkg{MANET}
                             & \pkg{Mondrian}\\
                             &                &
                             &\pkg{base}&\pkg{vcd}   &\pkg{iplots}&        & &\\\hline

    Basic functionality      &  $\times$  & $\times$  & $\times$ &$\times$ &$\times$ &   $\times$   & $\times$& $\times$\\
    Shape                    &            &           &          &$\times$ && $\times$  &  $\times$&\\
    Res.-based shadings  &  $\times$  &        & $\times$ & $\times$ & ($\times$) &     &($\times$)& ($\times$)\\
    Highlighting             &     &        &   &$\times$   &$\times$  & $\times$ & $\times$& $\times$\\
    Conditional views        &  $\times$  &        &    &$\times$ &          & $\times$ &  $\times$&\\
    Interaction              &     &        &   &  &$\times$  &   $\times$   & $\times$& $\times$\\
    Linking                  &     &        &   &  &$\times$  & $\times$ & $\times$& $\times$\\
    Extensible design        &     &        &  &$\times$ & &       &  &\\
    Language                 & \proglang{SAS} & \proglang{S} &
    \proglang{R} & \proglang{R} & \proglang{R}/\proglang{Java} & \proglang{XLisp} & \proglang{C++} & \proglang{Java}\\
    \hline
  \end{tabular}
  \caption{Comparison of current software environments.}
  \label{tab:compare}
\end{table}

Figures \ref{fig:arthritis} to \ref{fig:titanic} illustrate some of these
extensions. Figure~\ref{fig:arthritis} shows the results from a
double-blind clinical trial
investigating a new treatment for rheumatoid arthritis, using an
extended mosaic plot with residual-based shading based on the
maximum statistic: clearly, the new treatment is effective. The dark
blue cell indicates that the rate of treated patients showing marked improvement
is significant at the 1\% level. Figure
\ref{fig:ucbadmissions} visualizes the well-known UCB admissions data
by means of a conditional association plot. The panels show the
residuals from a conditional independence model (independence of gender
and admission, given department), stratified by department. Clearly,
the situation in department A (more women/less men accepted than
would be expected under the null hypothesis) causes the rejection of the
hypothesis of conditional independence. Figure~\ref{fig:presex}
illustrates the conditional independence of premarital and
extramarital sex, given gender and marital status. The $\chi^2$ test
of independence, based on the permutation distribution,
rejects the null hypothesis: possibly, because
the tendency of people to have extramarital
sex when they had premarital sex
is particularly marked among married people? The rate of such women and men
ist significant at the 0.01 and 0.1 level, respectively.
Finally, Figure~\ref{fig:titanic} visualizes the ``Survival on the Titanic''
data using a double-decker plot. Here, a binary response (survival of the disaster)
is to be explained by other factors (class, gender, and age). The gray
boxes represent the proportion of survived passengers in a particular
stratum. The proportions of saved women and children are indeed higher
than those of men, but they clearly decrease from the 1st to the 3rd class. In
addition, the proportion of saved men in the 1st class is higher than
in the others.

\setkeys{Gin}{width=0.7\textwidth}
\begin{figure}[p]
\begin{center}
<<Arthritis,fig=TRUE,echo=FALSE,height=6,width=7,results=hide>>=
mosaic(art, gp = shading_max, split_vertical = TRUE)
@
\caption{Mosaic plot for the \data{Arthritis} data.}
\label{fig:arthritis}
\end{center}
\end{figure}

\setkeys{Gin}{width=\textwidth}
\begin{figure}[p]
\begin{center}
<<UCBAdmissions,fig=TRUE,echo=FALSE,height=6,width=10,results=hide>>=
cotabplot(UCBAdmissions, panel = cotab_coindep, shade = TRUE, legend = FALSE,
          type = "assoc")
@
\caption{Conditional association plot for the \data{UCBAdmissions} data.}
\label{fig:ucbadmissions}
\end{center}
\end{figure}

\setkeys{Gin}{width=0.7\textwidth}
\begin{figure}[p]
\begin{center}
<<PreSex,fig=TRUE,echo=FALSE,height=6,width=7,results=hide>>=
presextest <- coindep_test(PreSex, margin = c(1,4),
                           indepfun = function(x) sum(x^2), n = 5000)
mosaic(PreSex, condvars = c(1, 4), shade = TRUE,
       gp_args = list(p.value = presextest$p.value))
@
\caption{Mosaic plot for the \data{PreSex} data.}
\label{fig:presex}
\end{center}
\end{figure}

\setkeys{Gin}{width=0.8\textwidth}
\begin{figure}[p]
\begin{center}
<<Titanic,fig=TRUE,echo=FALSE,height=6,width=10,results=hide>>=
doubledecker(Survived ~ ., data = Titanic, labeling_args =
             list(set_varnames = c(Sex = "Gender")))
@
\caption{Double-decker plot for the \data{Titanic} data.}
\label{fig:titanic}
\end{center}
\end{figure}

This paper describes the strucplot framework provided by the \pkg{vcd}
package for the \proglang{R} environment for statistical computing and
graphics, available from the Comprehensive \proglang{R} Archive Network
(\url{http://CRAN.R-project.org/}).  The framework integrates displays
such as mosaic, association, and sieve plots by their unifying property
of being flat representations of contingency tables.  These basic plots,
as well as specialized displays for conditional independence, can be
used both for exploratory visualization and model-based analysis.
Exploratory techniques include specialized displays for the bivariate
case, as well as pairs and trellis-type displays for
higher-dimensional tables. Model-based tools include methods suitable
for the visualization of conditional independence tests (including
permutation tests), as well as for the visualization of particular GLMs
(logistic regression, log-linear models). Additionally, two of the
framework's further strengths are its flexibility and extensibility:
graphical appearance aspects such as shading, labeling, and spacing are
modularized by means of ``\underline{\vphantom{g}gr}aphical
\underline{\vphantom{g}ap}pearance \underline{\vphantom{g}con}trol''
(\emph{grapcon}) functions, allowing
fine-granular customization and user-level extensions.

The remainder of the paper is organized as follows. In
Section \ref{sec:strucplot}, we give an overview of the strucplot
framework, describing the hierarchy of the main components and the
basic functionality.
In Section \ref{sec:shading}, we demonstrate how
(residual-based) shadings support the visualization of log-linear models and
the results of independence tests. Also, we explain step-by-step how
the concepts of generating and grapcon functions
can be combined to provide a flexible customization of
complex graphical displays as created by the strucplot framework.
Sections \ref{sec:labeling} and \ref{sec:spacing} discuss in detail the labeling
and spacing features, respectively. Section \ref{sec:example}
exemplifies the framework in the analysis of a four-way data set.
Section \ref{sec:conclusion} concludes the work.

\section[The strucplot framework]{The strucplot framework}
\label{sec:strucplot}

The strucplot framework in the \proglang{R} package \pkg{vcd}, used for visualizing
multi-way contingency tables, integrates techniques such as
mosaic displays, association plots, and sieve plots. The main idea is to visualize
the tables' cells arranged in rectangular form. For multi-way tables,
the variables are nested into rows and columns using recursive
conditional splits, given the margins. The result is a
``flat'' representation that can be visualized in
ways similar to a two-dimensional table.
This principle defines a class of conditional displays which allows
for granular control of graphical appearance aspects, including:

\begin{itemize}
\item the content of the tiles
\item the split direction for each dimension
\item the graphical parameters of the tiles' content
\item the spacing between the tiles
\item the labeling of the tiles
\end{itemize}

The strucplot framework is highly modularized: Figure~\ref{fig:struc}
shows the hierarchical relationship between the various components.
On the lowest level, there are several groups of workhorse and
parameter functions that directly or indirectly influence the final
appearance of the plot (see Table \ref{tab:grapcons} for an overview).
These are examples of grapcon functions. They are created by generating functions
(\emph{grapcon generators}), allowing
flexible parameterization and extensibility (Figure~\ref{fig:struc}
only shows the generators). The generator names
follow the naming convention \code{\textit{group\_foo}()},
where \code{\textit{group}} reflects the group the
generators belong to (strucplot core, labeling,
legend, shading, or spacing). The workhorse functions (created by
\code{struc\_\textit{foo}()},
\code{labeling\_\textit{foo}()}, and \code{legend\_\textit{foo}()})
directly produce graphical output (i.e., ``add ink to the canvas''), whereas
the parameter functions
(created by \code{spacing\_\textit{foo}()} and \code{shading\_\textit{foo}()}) compute
graphical parameters used by the others. The grapcon functions returned by
\code{struc\_\textit{foo}()} implement the core functionality,
creating the tiles and their
content.  On the second level of the framework, a suitable combination
of the low-level grapcon functions (or, alternatively, corresponding generating functions)
is passed as ``hyperparameters'' to \codefun{strucplot}.
This central function
sets up the graphical layout using grid viewports (see Figure~\ref{fig:layout}),
and coordinates the specified core, labeling, shading, and spacing functions to produce
the plot. On the third level, we provide
several convenience functions such as \codefun{mosaic},
\codefun{sieve}, \codefun{assoc}, and \codefun{doubledecker} which
interface \codefun{strucplot} through sensible parameter defaults
and support for model formulae. Finally, on the fourth
level, there are ``related'' \pkg{vcd} functions (such as \codefun{cotabplot}
and the \codefun{pairs} methods for table objects)
arranging collections of plots of the strucplot
framework into more complex displays (e.g., by means of panel functions).

\begin{table}
  \begin{tabular}{|l|l|l|}
    \hline
    \textbf{Group} & \textbf{Grapcon generator} & \textbf{Description}\\\hline
    strucplot & \codefun{struc\_assoc} & core function for association plots\\
    core      & \codefun{struc\_mosaic} & core function for mosaic plots\\
              & \codefun{struc\_sieve} & core function for sieve plots\\\hline\hline
     labeling & \codefun{labeling\_border} & border labels\\
              & \codefun{labeling\_cboxed} & centered labels with
              boxes, all labels clipped,\\
              && and on top and left border\\
              & \codefun{labeling\_cells} & cell labels\\
              & \codefun{labeling\_conditional} & border labels
                                                  for conditioning variables\\
              && and cell labels for conditioned variables\\
              & \codefun{labeling\_doubledecker} & draws labels for
              doubledecker plot\\
              & \codefun{labeling\_lboxed} & left-aligned labels with boxes\\
              & \codefun{labeling\_left} & left-aligned border labels\\
              & \codefun{labeling\_left2} & left-aligned border
              labels, all labels on top and left border\\
              & \codefun{labeling\_list} & draws a list of labels
              under the plot\\\hline\hline
     shading  & \codefun{shading\_binary} & visualizes the sign of the  residuals\\
              & \codefun{shading\_Friendly} & implements Friendly
              shading (based on HSV colors)\\
              & \codefun{shading\_hcl} & shading based on HCL colors\\
              & \codefun{shading\_hsv} & shading based on HSV colors\\
              & \codefun{shading\_max} & shading visualizing the
              maximum test statistic\\ && (based on HCL colors)\\
              & \codefun{shading\_sieve} & implements Friendly
              shading customized for sieve plots\\ && (based on HCL colors)\\\hline\hline
     spacing  & \codefun{spacing\_conditional} & increasing spacing
              for conditioning variables,\\&& equal spacing for
              conditioned variables\\
              & \codefun{spacing\_dimequal} & equal spacing for each dimension\\
              & \codefun{spacing\_equal} & equal spacing for all dimensions\\
              & \codefun{spacing\_highlighting} & increasing spacing,
              last dimension set to zero\\
              & \codefun{spacing\_increase} & increasing spacing\\\hline\hline
     legend   & \codefun{legend\_fixed} & creates a fixed number of
              bins (similar to \codefun{mosaicplot})\\
              & \codefun{legend\_resbased} & suitable for an
              arbitrary number of bins\\&& (also for continuous shadings)\\\hline

  \end{tabular}
  \caption{Available grapcon generators in the strucplot framework}
  \label{tab:grapcons}
\end{table}

\begin{figure}[h]
  \begin{center}
    \includegraphics[width=0.8\textwidth]{struc}
    \caption{Components of the strucplot framework.}
    \label{fig:struc}
  \end{center}
\end{figure}

\setkeys{Gin}{width=0.6\textwidth}
\begin{figure}[h]
\begin{center}
<<vcdlayout,fig=TRUE,echo=FALSE,width=6,height=6>>=
    pushViewport(vcd:::vcdViewport(legend = T, mar =4))
    seekViewport("main")
    grid.rect(gp = gpar(lwd = 3))
    grid.text("main", gp = gpar(fontsize = 20))
    seekViewport("sub")
    grid.rect(gp = gpar(lwd = 3))
    grid.text("sub", gp = gpar(fontsize = 20))
    seekViewport("plot")
    grid.rect(gp = gpar(lwd = 3))
    grid.text("plot", gp = gpar(fontsize = 20))
    seekViewport("legend")
    grid.text("legend", rot = 90, gp = gpar(fontsize = 20))
    grid.rect(gp = gpar(lwd = 3))
    seekViewport("legend_sub")
    grid.rect(gp = gpar(lwd = 3))
    grid.text("[F]", gp = gpar(fontsize = 20))
    seekViewport("legend_top")
    grid.rect(gp = gpar(lwd = 3))
    grid.text("[E]", gp = gpar(fontsize = 20))
    seekViewport("margin_top")
    grid.rect(gp = gpar(lwd = 3))
    grid.text("margin_top", gp = gpar(fontsize = 20))
    seekViewport("margin_bottom")
    grid.rect(gp = gpar(lwd = 3))
    grid.text("margin_bottom", gp = gpar(fontsize = 20))
    seekViewport("margin_right")
    grid.rect(gp = gpar(lwd = 3))
    grid.text("margin_right", rot = 90, gp = gpar(fontsize = 20))
    seekViewport("margin_left")
    grid.rect(gp = gpar(lwd = 3))
    grid.text("margin_left", rot = 90, gp = gpar(fontsize = 20))
    seekViewport("corner_top_left")
    grid.rect(gp = gpar(lwd = 3))
    grid.text("[A]", gp = gpar(fontsize = 20))
    seekViewport("corner_top_right")
    grid.rect(gp = gpar(lwd = 3))
    grid.text("[B]", gp = gpar(fontsize = 20))
    seekViewport("corner_bottom_left")
    grid.rect(gp = gpar(lwd = 3))
    grid.text("[C]", gp = gpar(fontsize = 20))
    seekViewport("corner_bottom_right")
    grid.rect(gp = gpar(lwd = 3))
    grid.text("[D]", gp = gpar(fontsize = 20))
@
\caption{Viewport layout for strucplot displays with their names. [A] =
  ``corner\_top\_left'', [B] = ``corner\_top\_right'',
  [C] = ``corner\_bottom\_left'', [D] = ``corner\_bottom\_right'', [E]
  = ``legend\_top'', [F] = ``legend\_sub''.}
\label{fig:layout}
\end{center}
\end{figure}
\setkeys{Gin}{width=0.7\textwidth}

\subsection{Mosaic, association, and sieve plots}

As an example, consider the \data{HairEyeColor}
data containing two polytomous variables (hair and eye color),
as well as one (artificial) dichotomous gender variable (\code{Sex}). The
``flattened'' contingency table can be obtained using the
\codefun{structable} function (quite similar to \codefun{ftable} in
base \proglang{R}, but allowing the specification of split directions):

<<structable,echo=TRUE,results=verbatim>>=
(HEC <- structable(Eye ~ Sex + Hair, data = HairEyeColor))
@

Let us first visualize the contingency table by means of a mosaic plot.
% \citep{vcd:Hartigan+Kleiner:1984} which is basically
% an area-proportional visualization of (typically, observed) frequencies, composed
% of tiles (corresponding to the cells) created by recursive
% vertical and horizontal splits of a square. Thus, the area of each tile
% is proportional to the corresponding cell entry \emph{given} the
% dimensions of previous splits.
The effect of

<<Observed>>=
mosaic(HEC)
@

\noindent equivalent to

<<Observed2>>=
mosaic(~ Sex + Eye + Hair, data = HairEyeColor)
@

%\setkeys{Gin}{width=0.75\textwidth}
\begin{figure}[p]
\begin{center}
<<Observedfig,fig=TRUE,echo=FALSE,height=6>>=
<<Observed>>
@
\caption{Mosaic plot for the \data{HairEyeColor} data.}
\label{fig:observed}
\end{center}
\end{figure}

\noindent depicts the observed frequencies of the \code{HairEyeColor}
data. If there are zero entries, tiles have zero area and are, additionally,
marked by small bullets (see, e.g, Figure~\ref{fig:titanic}). By
default, these cells are not split further. The bullets help
distinguishing very small cells from zero entries, and are
particularly useful when color shadings come into play
(see the example using the \data{Bundesliga} data in Section \ref{sec:overview}).
Note that in contrast to, e.g.,  \codefun{mosaicplot} in base
\proglang{R}, the default split direction and level ordering in all strucplot
displays correspond to the textual representation produced by the
print methods.
It is also possible to visualize the expected values instead of the
observed values (see Figure~\ref{fig:expected}):

<<Expected>>=
mosaic(HEC, type = "expected")
@

\begin{figure}[p]
\begin{center}
<<Expectedfig,fig=TRUE,echo=FALSE,height=6>>=
<<Expected>>
@
\caption{Mosaic plot for the \data{HairEyeColor} data (expected values).}
\label{fig:expected}
\end{center}
\end{figure}
%\setkeys{Gin}{width=0.7\textwidth}

\noindent In order to compare observed and expected values,
a sieve plot \citep{vcd:riedwyl+schuepbach:1994}
could be used (see Figure~\ref{fig:sieve}):

<<sieve>>=
sieve(~ Sex + Eye + Hair, data = HEC, spacing = spacing_dimequal(c(2,0,0)))
@

\begin{figure}[h]
\begin{center}
<<sievefig,fig=TRUE,echo=FALSE>>=
<<sieve>>
@
\caption{Sieve plot for the \data{HairEyeColor} data visualizing simultaneously
  observed and expected values.}
\label{fig:sieve}
\end{center}
\end{figure}

\noindent where \code{spacing\_dimequal} is used to set the spacing
of the second and third dimension to zero. Alternatively, we can directly inspect the residuals.
The Pearson residuals (standardized deviations of observed from expected values)
are conveniently visualized using association
plots \citep{vcd:Cohen:1980}. In contrast to \codefun{assocplot} in
base \proglang{R}, \pkg{vcd}'s \codefun{assoc}
function scales to more than two variables (see Figure~\ref{fig:residuals}):

<<Residuals>>=
assoc(HEC, compress = FALSE)
@

\begin{figure}[p]
\begin{center}
<<Residualsfig,fig=TRUE,echo=FALSE,width=6, height=10>>=
<<Residuals>>
@
\caption{Association plot for the \data{HairEyeColor} data.}
\label{fig:residuals}
\end{center}
\end{figure}

\noindent where the \code{compress} argument keeps distances between tiles
equal.

For both mosaic plots and association plots,
the splitting of the tiles can be controlled using the
\code{split\_vertical} argument. The default is to alternate splits starting
with a horizontal one (see Figure~\ref{fig:split}):

<<echo=FALSE>>=
options(width=60)
@

<<split1>>=
mosaic(HEC, split_vertical = c(TRUE, FALSE, TRUE),
labeling_args = list(abbreviate_labs = c(Eye = 3)))
@

<<echo=FALSE>>=
options(width=70)
@

\begin{figure}[p]
\begin{center}
<<splitfig,fig=TRUE,echo=FALSE,width=7,height=7>>=
<<split1>>
@
\caption{Mosaic plot for the \data{HairEyeColor} data---alternative splitting.}
\label{fig:split}
\end{center}
\end{figure}

\noindent (Note that \code{HEC}, a \class{structable} object, already
includes a splitting information which simply gets overloaded in this
example.)  For compatibility with \codefun{mosaicplot} in base
\proglang{R}, the \codefun{mosaic} function also allows the use of a
\code{direction} argument taking a vector of \code{"h"} and \code{"v"}
characters:

<<split2>>=
mosaic(HEC, direction = c("v","h","v"))
@

By a suitable combination of splitting, spacing, and
labeling settings, the functions provided by the strucplot framework
can be customized in a quite flexible way. For example,
the default method for \codefun{doubledecker} is simply a wrapper for
\codefun{strucplot}, setting the right defaults.
Most default settings such as colors, spacing, and labeling are
specified via the parameters and passed through to
\codefun{strucplot}. The additional code just handles the dependent
variable information, and in particular permutes the table to have the dependent
variable as the last dimension as required for the doubledecker plot.
Figure~\ref{fig:titanic} shows a doubledecker plot of
the \data{Titanic} data, explaining the probability of survival (``survived'') by age, given
sex, given class. It is created by:

<<doubledecker1>>=
doubledecker(Titanic)
@

\noindent equivalent to:

<<doubledecker2>>=
doubledecker(Survived ~ Class + Sex + Age, data = Titanic)
@

\subsection{Conditional and partial views}


So far, we have visualized either full or collapsed tables, as
suggested by the analysis task at hand. Subtables can be selected in a
similar way as for objects of class \class{table} using
indexing. Note, however, that subsetting of \class{structable} objects
is more restrictive because of their inherent conditional structure. Since the variables
on both the row and the columns side are nested,
subsetting is only possible ``outside-in'', that is, indexing operates
on blocks defined by the variable levels. In the following, we use the
Titanic data again, this time collapsed over \code{Survived} to investigate the
structure of crew and passengers (and having the \code{Child} and
\code{Age} labels of the \code{Age} variable swapped for optical clarity):

<<echo=FALSE>>=
options(width=75)
@

<<subsetting,results=verbatim>>=
(STD <- structable(~ Sex + Class + Age, data = Titanic[,,2:1,]))
STD["Male",]
STD["Male", c("1st","2nd","3rd")]
@

<<echo=FALSE>>=
options(width=70)
@

\noindent \emph{Conditioning} on levels (i.e., choosing a table subset for fixed levels
of the conditioning variable(s)) is done using the \code{[[} operator. %]]
Here again, the sequence of conditioning levels is restricted by the
hierarchical structure of the \class{structable} object. In the
following examples, note that compared to subsetting, the first
dimension(s) are dropped:

<<conditioning,results=verbatim>>=
STD[["Male",]]
STD[[c("Male", "Adult"),]]
STD[["Male","1st"]]
@

\noindent Now, there are several ways for visualizing
conditional independence structures. The ``brute force'' method is to
draw separate plots for the strata. The following example compares
the association between hair and eye color, given gender, by using
subsetting on the flat table and \pkg{grid}'s viewport
framework to visualize the two groups besides each other:

<<Variables1>>=
pushViewport(viewport(layout = grid.layout(ncol = 2)))
@

<<Variables2>>=
pushViewport(viewport(layout.pos.col = 1))
mosaic(STD[["Male"]], margins = c(left = 2.5, top = 2.5, 0), sub = "Male", newpage = FALSE)
popViewport()
@

<<Variables3>>=
pushViewport(viewport(layout.pos.col = 2))
mosaic(STD[["Female"]], margins = c(top = 2.5, 0), sub = "Female", newpage = FALSE)
popViewport(2)
@

\setkeys{Gin}{width=\textwidth}
\begin{figure}[p]
\begin{center}
<<Variablesfig,fig=TRUE,echo=FALSE,height=6,width=12>>=
<<Variables1>>
<<Variables2>>
<<Variables3>>
@
\caption{Two mosaic displays put side-by-side, visualizing the
distribution of class and age, given gender. The marginal
distribution of gender cannot be seen.}
\label{fig:parttable}
\end{center}
\end{figure}
\setkeys{Gin}{width=0.7\textwidth}

\noindent Note the use of the \code{margins} argument: it takes a
vector with up to four values whose unnamed components are recycled,
but ``overruled'' by the named arguments. Thus, in the second example, only
the top margin is set to 2.5 lines, and all other to 0.
This idea applies to almost all
vectorized arguments in the strucplot framework (with
\code{split\_vertical} as a prominent exception).

The \codefun{cotabplot} function does a much better job on this task:
it arranges stratified strucplot displays
in a lattice-like layout, conditioning on variable \emph{levels}.
The plot in Figure~\ref{fig:cotabplot} shows class and age group, given sex:

<<cotabplot>>=
cotabplot(~ Class + Age | Sex, data = STD, split_vertical = TRUE)
@

\setkeys{Gin}{width=\textwidth}
\begin{figure}[p]
\begin{center}
<<cotabplotfig,fig=TRUE,echo=FALSE,height=6,width=11>>=
<<cotabplot>>
@
\caption{Conditional table plot for the \data{Titanic} data,
  again visualizing the distribution of age and class, given
  gender, using separate mosaic displays like the ``manual'' plot
  in Figure~\ref{fig:parttable}.}
\label{fig:cotabplot}
\end{center}
\end{figure}
\setkeys{Gin}{width=0.7\textwidth}
%\noindent The \code{labeling\_args} argument modifies the labels'
%appearance: here, to be left-aligned and unclipped
%(see Section \ref{sec:labeling}).

\noindent Visualizing the strata separately ``hides'' the distribution of the
conditioning variable(s) which may or may not be appropriate or
sensible in a particular analysis step. If we wish to keep the
information on the marginal distribution(s), we can use one single
mosaic for the stratified plot since mosaic displays are
``conditional plots'' by definition. We just need to make sure that
conditioning variables are used first for splitting.
Both the default and the formula interface of \codefun{mosaic} allow the
specification of conditioning variables (see Figure~\ref{fig:conditioning}):

<<Conditioning1>>=
mosaic(STD, condvars = "Sex", split_vertical = c(TRUE, TRUE, FALSE))
@
<<Conditioning2>>=
mosaic(~ Class + Age | Sex, data = STD, split_vertical = c(TRUE, TRUE, FALSE))
@

\setkeys{Gin}{width=\textwidth}
\begin{figure}[h]
\begin{center}
<<Conditioningfig,fig=TRUE,echo=FALSE,width=12>>=
<<Conditioning2>>
@
\caption{Mosaic plot again visualizing the
distribution of class and age, given gender, this time using a
single mosaic plot. In contrast to Figures~\ref{fig:parttable} and \ref{fig:cotabplot},
this plot also visualizes the marginal distribution of gender.}
\label{fig:conditioning}
\end{center}
\end{figure}
\setkeys{Gin}{width=0.7}

\noindent The effect of using this is that
conditioning variables are permuted ahead of the the conditioned variables
in the table, and that \codefun{spacing\_conditional} is used as default to better
distinguish conditioning from conditioned dimensions.
This spacing uses equal space between tiles of conditioned variables,
and increasing space between tiles of conditioning variables (See
Section~\ref{sec:spacing}).

Another set of high-level functions for visualizing conditional independence
models are the \codefun{pairs} methods for \class{table} and \class{structable} objects.
In contrast to \codefun{cotabplot} which conditions on variables,
the \codefun{pairs} methods create pairwise views of the table.
They produce, by default, a plot matrix
having strucplot displays in the off-diagonal panels, and
the variable names (optionally, with univariate displays) in the diagonal cells.
Figure~\ref{fig:pairs} shows a pairs display for the
\data{Titanic} data with univariate mosaics in the diagonal,
and mosaic plots visualizing the corresponding bivariate mosaics
in the upper and lower triangles.
Due to the inherent asymmetry of mosaic displays, the corresponding plots in the
upper and lower triangle differ depending on which variable is used first
for splitting---inspecting both views might help detecting patterns in a data set.
Additionally,
we are using a special spacing and shading normally used to `highlight' %'
the second variable in the first (as will be discussed in Section \ref{sec:spacing}): here,
the intention of the spacing
is to emphasize the conditional distributions of the second variable,
given the first one, and the shading helps identifying the factor levels in
the second variable.

<<pairs>>=
pairs(STD, highlighting = 2, diag_panel = pairs_diagonal_mosaic,
      diag_panel_args = list(fill = grey.colors))
@


%\setkeys{Gin}{width=\textwidth}
\setkeys{Gin}{width=0.7\textwidth}
\begin{figure}[h]
\begin{center}
<<pairsfig,fig=TRUE,echo=FALSE,width=8,height=8>>=
<<pairs>>
@
\caption{Pairs plot for the \data{Titanic} data.}
\label{fig:pairs}
\end{center}
\end{figure}
\setkeys{Gin}{width=0.7\textwidth}

\noindent The labels of the variables are to be read from left to
right and from top to bottom. In addition, the levels can be matched
by position within the columns and by shading within the rows.
In plots produced by \codefun{pairs},
each panel's row and column define two variables $X$ and $Y$ used for the
specification of four different types of independence: pairwise,
total, conditional, and joint. The pairwise mosaic
matrix shows bivariate marginal relations between $X$ and $Y$, collapsed over all
other variables. The total independence mosaic matrix shows mosaic
plots for mutual independence, i.e., for marginal and conditional independence
among all pairs of variables. The conditional
independence mosaic matrix shows mosaic plots for marginal
independence of $X$ and $Y$, given all other variables. The joint independence
mosaic matrix shows mosaic plots for joint independence of all
pairs ($X$, $Y$) of variables from the others.

Upper and lower parts can
independently be used to display different
types of independence models, or different strucplot displays
(mosaic, association, or sieve plots).
The available panel functions (\codefun{pairs\_assoc},
\codefun{pairs\_mosaic}, and \codefun{pairs\_sieve})
are simple wrappers to \codefun{assoc},
\codefun{mosaic}, and \codefun{sieve}, respectively.
Obviously, seeing patterns in strucplot matrices
becomes increasingly difficult with
higher dimensionality. Therefore, this plot is typically used with a suitable
residual-based shading (see Section \ref{sec:shading}).

\subsection{Interactive plot modifications}

All strucplot core functions are supposed to produce conditional
hierarchical plots by the means of nested viewports, corresponding to
the provided splitting information. Thus, at the end of the plotting,
each tile is associated with a particular viewport. Each of those
viewports has to be conventionally named, enabling other strucplot
modules, in particular the labeling functions, to access specific tiles
after they have been plotted. The naming convention for the viewports
is:

\begin{center}
\code{\emph{[Optional prefix]}cell:\emph{Variable1}=\emph{Level1},\emph{Variable2}=\emph{Level2}} \dots
\end{center}

\noindent Clearly, these names depend on the splitting.
The following example shows how to access parts of the plot after it has
been drawn (see Figure~\ref{fig:afterplot}):

<<viewportnames>>=
mosaic(~ Hair + Eye, data = HEC, pop = FALSE)

seekViewport("cell:Hair=Blond")
grid.rect(gp = gpar(col = "red", lwd = 4))

seekViewport("cell:Hair=Blond,Eye=Blue")
grid.circle(r = 0.2, gp = gpar(fill = "cyan"))
@

\noindent Note that the viewport tree is removed by
default. Therefore, the \texttt{pop} argument has to be set to
\texttt{FALSE} when viewports shall be accessed.

\setkeys{Gin}{width=0.6\textwidth}
\begin{figure}[h]
\begin{center}
<<viewportnamesfig,fig=TRUE,echo=FALSE>>=
<<viewportnames>>
@
\caption{Adding elements to a mosaic plot after drawing.}
\label{fig:afterplot}
\end{center}
\end{figure}

In addition to the viewports, the main graphical elements get names
following a similar construction method. This allows to change
graphical parameters of plot elements \emph{after}
the plotting (see Figure~\ref{fig:changeplot}):

<<changeplot,results=verbatim>>=
assoc(Eye ~ Hair, data = HEC, pop = FALSE)
getNames()[1:6]
grid.edit("rect:Hair=Blond,Eye=Blue", gp = gpar(fill = "red"))
@

%% code-chunk reuse does not work with parameter changing
\begin{figure}[h]
\begin{center}
<<changeplotfig,fig=TRUE,echo=FALSE>>=
x <- tab <- margin.table(HairEyeColor, 1:2)
x[] <- "light gray"
x["Blond","Blue"] <- "Red"
assoc(tab, gp = gpar(fill = x))
@
\caption{Changing graphical parameters of elements after drawing.}
\label{fig:changeplot}
\end{center}
\end{figure}

\subsection{Performance issues}
\label{sec:performance}

As stated above, the implementation of strucplot displays is based on
creating and nesting \pkg{grid} viewports. The main time-consuming steps performed by
the core functions are the following:

\begin{enumerate}
\item recursively, split the table until the individual cells are reached
\item during the splits, add viewports to the plot
\item for the individual cells, add plot-specific content
      (rectangles for mosaics, bars for association plots, etc.)
\end{enumerate}

\noindent All these operations scale linearly with the amount of
created viewports. For a $d$-dimensional table with $k_i$
levels, $i=1 \dots d$, the total number of needed viewports
$T_d$ can roughly be estimated as

\begin{equation}
  \label{eq:numbervp}
  T_d \quad = \quad k_1 + k_1k_2 + \cdots + k_1 \cdots k_d \quad =\quad \sum_{i=1}^d \prod_{j \le i} k_j
\end{equation}

\noindent since we first push the $k_1$ viewports for the levels of the
first dimension, then, for \emph{each} of these, the $k_2$ levels of
the second dimension, etc. If the number of levels is equal ($k$) for
all dimensions, $T_d$ simplifies to

\begin{equation}
  \label{eq:equalvp}
  T_d \quad = \quad \sum_{i=1}^d k^i = \frac{k(k^d-1)}{k-1}
\end{equation}

\noindent and so the time complexity for drawing a strucplot display is of order $k^d$.

\section{Shadings}
\label{sec:shading}


Unlike other graphics functions in base \proglang{R}, the strucplot framework
allows almost full control over the graphical parameters of all plot elements. In
particular, in association plots, mosaic plots, and sieve plots,
the user can modify the graphical appearance of each tile individually.
Built on top of this functionality, the
framework supplies a set of shading functions choosing colors
appropriate for the visualization of log-linear models.
The tiles' graphical parameters are set using the \code{gp} argument
of the functions of the strucplot framework. This argument basically expects an object
of class \class{gpar} whose components are arrays of the same shape
(length and dimensionality) as the data table (see Section \ref{sec:gp}).
For convenience, however, the user can also
supply a grapcon function that computes such an object given a vector of
residuals, or, alternatively, a generating function that takes
certain arguments and returns such a grapcon function (see Section
\ref{sec:shadingcustom}). We provide several shading functions, including
support for both HSV and HCL colors, and the
visualization of significance tests (see Section \ref{sec:overview}).

\subsection{Specifying graphical parameters of strucplot displays}
\label{sec:gp}

As an example, consider the \data{UCBAdmissions} data.
In the table aggregated over departments, we
would like to highlight the (incidentally wrong) impression that there
were too many male students accepted compared to the presumably
discriminated female students (see Figure~\ref{fig:ucb}):

<<ucb,results=verbatim>>=
(ucb <- margin.table(UCBAdmissions, 1:2))

(fill_colors <- matrix(c("dark cyan","gray","gray","dark magenta"), ncol = 2))

mosaic(ucb, gp = gpar(fill = fill_colors, col = 0))
@

\begin{figure}[h]
\begin{center}
<<ucbfig,fig=TRUE,echo=FALSE>>=
<<ucb>>
@

\caption{Mosaic plot for the \data{UCBAdmissions} data with highlighted cells.}
\label{fig:ucb}
\end{center}
\end{figure}

\noindent As the example shows, we create a fourfold table
with appropriate colors (dark cyan for admitted male students and dark
magenta for rejected female students) and supply them to the \code{fill} component
of the \class{gpar} object passed to the \code{gp} argument of \codefun{mosaic}.
For visual clarity, we additionally hide the tiles' borders by setting the \code{col}
component to 0 (transparent).

If the parameters specified in the \class{gpar} object are ``incomplete'',
they will be recycled along the last splitting
dimension. In the following example based on the \data{Titanic} data,
we will highlight all cells corresponding to survived
passengers (see Figure~\ref{fig:recycling}):

<<recycling>>=
mosaic(Titanic, gp = gpar(fill = c("gray","dark magenta")),
                spacing = spacing_highlighting,
                labeling_args = list(abbreviate_labs = c(Age = 3), rep = c(Survived = FALSE))
      )
@

\noindent Note that \codefun{spacing\_highlighting} sets the spaces
between tiles in the last dimension to 0. The \code{labeling\_args}
argument ensures that labels do not overlap (see Section \ref{sec:labeling}).

\begin{figure}[h]
\begin{center}
<<recyclingfig,fig=TRUE,echo=FALSE>>=
<<recycling>>
@
\caption{Recycling of parameters, used for highlighting the survived
  passengers in the \data{Titanic} data.}
\label{fig:recycling}
\end{center}
\end{figure}

\subsection{Customizing residual-based shadings}
\label{sec:shadingcustom}

This flexible way of specifying graphical parameters is the basis for a suite of
shading functions that modify the tiles' appearance with respect to a vector of
residuals, resulting from deviations of observed from expected
frequencies under a given log-linear model.
The idea is to visualize at least sign and absolute size of the residuals, but some
shadings, additionally, indicate overall significance. One particular
shading, the maximum shading
\citep{vcd:Meyer+Zeileis+Hornik:2003,vcd:Zeileis+Meyer+Hornik:2007},
even allows to identify the cells that cause the rejection of the null hypothesis.

Conceptually, the strucplot framework offers three alternatives to
add residual-based shading to plots:

\begin{enumerate}
\item Precomputing the graphical parameters (e.g., fill colors),
  encapsulating them into an object of class
  \class{gpar} as demonstrated in the previous section, and
  passing this object to the \code{gp} argument.
\item Providing a grapcon function to the \code{gp} argument that
  takes residuals as input and returns an object as described in
  alternative 1.
 \item Providing a grapcon generator taking parameters and
   returning a function as described in alternative~2.
\end{enumerate}

\noindent For each of these approaches, we will demonstrate the
necessary steps to obtain a binary shading that visualizes the sign of
the residuals by a corresponding
fill color (for simplicity, we will treat 0 as positive).

\subsubsection*{Alternative 1: Precomputed \class{gpar} object}

The first method is precomputing the graphical parameters ``by
hand''. We will use \code{royalblue4} color for positive and \code{mediumorchid4}
color for negative residuals (see Figure~\ref{fig:binary}):

<<shading1,results=verbatim>>=
expected <- independence_table(ucb)
(x <- (ucb - expected)  / sqrt(expected))

(shading1_obj <- ifelse(x > 0, "royalblue4", "mediumorchid4"))

mosaic(ucb, gp = gpar(fill = shading1_obj))
@

\begin{figure}[h]
\begin{center}
<<shading1fig,fig=TRUE,echo=FALSE>>=
<<shading1>>
@
\caption{Binary shading visualizing the sign of the residuals.}
\label{fig:binary}
\end{center}
\end{figure}

\subsubsection*{Alternative 2: Grapcon function}

For implementing alternative 2, we need to create a ``shading function''
that computes \class{gpar} objects from
residuals. For that, we can just reuse the code from the previous step:

<<shading2>>=
shading2_fun <- function(x) gpar(fill = ifelse(x > 0, "royalblue4", "mediumorchid4"))
@

\noindent To create a mosaic display with binary shading,
it now suffices to specify the data table along with
\codefun{shading2\_fun}:

<<shading3>>=
mosaic(ucb, gp = shading2_fun)
@

\noindent \codefun{mosaic} internally calls
\codefun{strucplot} which computes the residuals from the specified
independence model (total independence by default), passes them to
\codefun{shading2\_fun}, and uses the \class{gpar} object returned
to finally create the plot.

Our \codefun{shading2\_fun} function might be useful,
but can still be improved: the hard-wired colors should be
customizable. We cannot
simply extend the argument list to include, e.g.,
a \code{fill = c("royalblue4", "mediumorchid4")} argument because
\codefun{strucplot} will neither know how to handle it, nor let us
change the defaults. In fact, the interface of shading functions is fixed,
they are expected to take exactly one argument: a table of residuals.
This is where generating functions (alternative 3) come
into play.

\subsubsection*{Alternative 3: Grapcon generator}

We simply wrap our grapcon shading function in another
function that takes all additional arguments it needs to use, possibly
preprocesses them, and returns the actual shading function. This returned
function will have access to the parameters since in \proglang{R}, nested
functions are lexically scoped. Thus, the grapcon generator
returns (``creates'') a
``parameterized'' shading function with the minimal standard interface
\codefun{strucplot} requires. The following example shows the necessary
extensions for our running example:

<<shading3>>=
shading3a_fun <- function(col = c("royalblue4", "mediumorchid4")) {
  col <- rep(col, length.out = 2)
  function(x) gpar(fill = ifelse(x > 0, col[1], col[2]))
}
@

\noindent The first statement just makes sure that exactly two colors
are specified. In the call to \codefun{mosaic}, using the new
\codefun{shading3a\_fun} function, we can now simply change the colors:

<<shading4>>=
mosaic(ucb, gp = shading3a_fun(c("royalblue4","mediumorchid4")))
@

\noindent (figure not shown).  The procedure described so far is a
rather general concept, applicable to a wide family of user-level
\pkg{grid} graphics. Indeed, the customization of other components of
the strucplot framework (labeling, spacing, legend, and core functions)
follows the same idea.  Now for the shading functions, more
customization is needed. Note that \codefun{shading3a\_fun} needs to be
evaluated by the user, even if the defaults are to be used. It is a
better idea to let \codefun{strucplot} call the generating function,
which, in particular, allows the passing of arguments that are computed
by \codefun{strucplot}. Since shading functions can be used for
visualizing significance (see Section \ref{sec:overview}), it makes
sense for generating functions to have access to the model, i.e.,
observed and expected values, residuals, and degrees of freedom. For
example, the \codefun{shading\_max} generating function computes a
permutation distribution of the maximum statistic and $p$ values for
specified significance levels based on the observed table to create
data-driven cut-off points. If this was done in the shading function
itself, the permutation statistic would be recomputed every time the
shading function is called, resulting in possibly severe performance
loss and numerical inconsistencies. Therefore, generating functions for
shadings are required to take at least the parameters \code{observed},
\code{expected}, \code{residuals}, and \code{df} (these are provided by
the strucplot framework), followed by other parameters controlling the
shading appearance (to be specified by the user):

<<shading4>>=
shading3b_fun <- function(observed = NULL, residuals = NULL, expected = NULL,
    df = NULL, col = c("royalblue4", "mediumorchid4")) {
  col <- rep(col, length.out = 2)
  function(x) gpar(fill = ifelse(x > 0, col[1], col[2]))
}
class(shading3b_fun) <- "grapcon_generator"
@

Note that in this simple binary shading example, the first four
parameters are not used. In some sense, generating
functions for shadings are parameterized both by the user and the
strucplot framework. For shading functions that require model
information, the user-specified parameters are
to be passed to the \code{gp\_args} argument instead, and for this to
work, the generating function needs a class attribute to be distinguishable from
the ``normal'' shading functions. For others
(like our simple \codefun{shading3b\_fun}) this is optional, but
recommended for consistency:

<<shading5>>=
mosaic(ucb, gp = shading3b_fun, gp_args = list(col = c("red","blue")))
@

\noindent The final \codefun{shading3b\_fun} pretty much resembles
\codefun{shading\_binary}, one of the standard shading
functions provided by the \pkg{vcd} package.

\subsection[An overview of the shading functions in vcd]{An overview of the shading functions in \pkg{vcd}}
\label{sec:overview}

\cite{vcd:Friendly:1994}
suggested a residual-based shading for the mosaic tiles that can also be applied
to the rectangles in association plots \citep{vcd:Meyer+Zeileis+Hornik:2003}.
Apart from \codefun{shading\_binary}, there are currently two basic
shadings available in \pkg{vcd}:
\codefun{shading\_hcl} and \codefun{shading\_hsv}, as well
as two derived functions: \codefun{shading\_Friendly} building upon
\codefun{shading\_hsv}, and \codefun{shading\_max} building upon \codefun{shading\_hcl}.
\codefun{shading\_hsv} and \codefun{shading\_hcl} provide
the same conceptual tools, but use different color spaces: the
Hue-Saturation-Value (HSV) and the Hue-Chroma-Luminance (HCL) scheme,
respectively. We will first expose the basic concept of these shading
functions using HSV space, and
then briefly explain the differences to HCL space \citep[a detailed
discussion can be found in][]{vcd:Zeileis+Meyer+Hornik:2007}.
Color palettes in HCL space are
preferable to palettes derived
from HSV space from a perceptual point of view. Functions for creating
palettes (see, e.g., \codefun{diverge\_hcl}) are provided with the
\pkg{vcd} package.

In HSV space, colors are specified in three dimensions: Hue,
Saturation (``colorfulness''), and Value (``lightness'', amount of gray).
These three dimensions are used by \codefun{shading\_hsv} to visualize information
about the residuals and the underlying independence model. The hue
indicates the residuals' sign: by default, blue for positive, and red
for negative residuals. The saturation of a residual
is set according to its size: high saturation for large, and low
saturation for small residuals.
Finally, the overall lightness is used to indicate the significance of
a test statistic: light colors for significant,
and dark colors for non-significant results.

As an example, we will visualize the association of hair and eye
color in the \data{HairEyeColor} data set (see
Figure~\ref{fig:haireye}, top):

<<haireye1>>=
haireye <- margin.table(HairEyeColor, 1:2)
mosaic(haireye, gp = shading_hsv)
@

\noindent As introduced before, the default shading scheme is not
\codefun{shading\_hsv} but \codefun{shading\_hcl} due to the better
perceptual characteristics of HCL color space.
The following example again illustrates the \data{HairEyeColor} data,
this time with HCL colors:

<<haireye2>>=
mosaic(haireye, gp = shading_hcl)
@
<<haireye3>>=
mosaic(haireye, gp = shading_hcl, gp_args = list(h = c(130, 43), c = 100, l = c(90, 70)))
@

\noindent In Figure~\ref{fig:haireye}, the plot
in the middle depicts the default palette,
and the bottom plot an alternative setting for Hue (\code{h}), Chroma (\code{c}), and
Luminance (\code{l}).

\setkeys{Gin}{width=0.5\textwidth}
\begin{figure}[htbp]
\begin{center}
<<haireyefig1,fig=TRUE,echo=FALSE,height=6,width=8>>=
mosaic(haireye, gp = shading_hsv, margin = c(bottom = 1), keep_aspect_ratio = FALSE)
@
<<haireyefig2,fig=TRUE,echo=FALSE,height=6,width=8>>=
mosaic(haireye, gp = shading_hcl, margin = c(bottom = 1), keep_aspect_ratio = FALSE)
@
<<haireyefig3,fig=TRUE,echo=FALSE,height=6,width=8>>=
mosaic(haireye, gp = shading_hcl, margin = c(bottom = 1), gp_args = list(h = c(130, 43), c = 100, l = c(90, 70)), keep_aspect_ratio = FALSE)
@
\caption{Three mosaic plots for the \data{HairEyeColor} data using
  different color palettes. Top: default HSV color palette. Middle: default HCL
 color palette. Bottom: a custom HCL color palette.}
\label{fig:haireye}
\end{center}
\end{figure}
\setkeys{Gin}{width=0.7\textwidth}

\noindent Large positive residuals (greater than $4$) can be found for
brown eyes/black hair and blue eyes/blond hair, and are colored in deep blue.
On the other hand, there is a large negative residual
(less than $-4$) for brown eyes/blond hair, colored deep red. There are also three
medium-sized positive (negative) residuals between 2 and 4 ($-2$ and
$-4$): the colors for them are less saturated. Residuals between $-2$ and $2$
are shaded in white (gray for HCL-shading).
The heuristic for choosing the cut-off points $2$ and $4$ is that the Pearson residuals
are approximately standard normal which implies that the highlighted cells are those with
residuals \emph{individually} significant at approximately the $\alpha
= 0.05$ and $\alpha = 0.0001$ levels, respectively. These default cut-off points can
be changed to alternative values using the \code{interpolate}
argument (see Figure~\ref{fig:interpolatecontinuous}):

<<interpolate>>=
mosaic(haireye, shade = TRUE, gp_args = list(interpolate = 1:4))
@

\noindent The elements of the numeric vector passed to
\code{interpolate} define the knots of an interpolating step function
used to map the absolute residuals to saturation levels.
The \code{interpolate} argument also accepts a user-defined function,
which then is called with the absolute residuals to get a vector of
cut-off points. Thus, it is possible to automatically choose the cut-off points
in a data-driven way. For example,
one might think that the extension from four cut-off points
to a continuous shading---visualizing the whole range of
residuals---could be useful. We simply need a one-to-one
mapping from the residuals to the saturation values:

<<continuous1>>=
ipol <- function(x) pmin(x/4, 1)
@

\noindent Note that this \codefun{ipol}
function maps residuals greater than 4 to a saturation
level of 1. However, the resulting plot
(Figure~\ref{fig:interpolatecontinuous}, right) is
deceiving:

<<continuous2>>=
mosaic(haireye, shade = TRUE, gp_args = list(interpolate = ipol),
labeling_args = list(abbreviate_labs = c(Sex = TRUE)))
@

\setkeys{Gin}{width=\textwidth}
\begin{figure}[htbp]
\begin{center}
<<interpolatefig,fig=TRUE,echo=FALSE,width=13,height=6>>=
pushViewport(viewport(layout = grid.layout(ncol = 2)))
pushViewport(viewport(layout.pos.col = 1))
mosaic(haireye, gp_args = list(interpolate = 1:4), margin = c(right = 1), keep_aspect_ratio= FALSE,newpage = FALSE,legend_width=5.5,shade = TRUE)
popViewport(1)
pushViewport(viewport(layout.pos.col = 2))
mosaic(haireye, gp_args = list(interpolate = ipol), margin = c(left=3,right = 1), keep_aspect_ratio = FALSE, newpage = FALSE, shade = TRUE)
popViewport(2)
@
\caption{\label{fig:interpolatecontinuous}The \data{HairEyeColor}
  data. Left: shading with 4 cut-off points. Right: continuous shading.}

\end{center}
\end{figure}
\setkeys{Gin}{width=0.7\textwidth}

\noindent Too much color makes it difficult to interpret the image, and the subtle color
differences are hard to catch. Therefore, we only included shadings
with discrete cut-off points.

The third remaining dimension, the value, is used for
visualizing the significance of a test statistic. The user can either
directly specify the $p$ value, or, alternatively, a function that
computes it, to the \code{p.value} argument. Such a function must take
observed and expected values, residuals, and degrees of freedom (used
by the independence model) as arguments.
If nothing is specified, the $p$ value is computed from
a $\chi^2$ distribution with \code{df} degrees of
freedom. The \code{level} argument is used to specify the confidence
level:  if \code{p.value} is smaller than \code{1 - level}, light colors are used,
otherwise dark colors are employed.
The following example using the \data{Bundesliga} data
shows the relationship of home goals and away
goals of Germany's premier soccer league in 1995: although there are two
``larger'' residuals (one greater than 2, one less then $-2$), the
$\chi^2$ test does not reject the null hypothesis of
independence. Consequently, the colors appear dark (see
Figure~\ref{fig:bundesliga}, left):

<<bundesliga>>=
BL <- xtabs(~ HomeGoals + AwayGoals, data = Bundesliga, subset = Year == 1995)
mosaic(BL, shade = TRUE)
@

\noindent Note that in extended mosaic plots, bullets drawn for zero
cells are shaded, too, bringing out non-zero residuals, if any.

A shading function building upon \codefun{shading\_hsv} is \codefun{shading\_Friendly},
implementing the shading introduced by \cite{vcd:Friendly:1994}. In addition
to the defaults of the HSV shading, it uses the border color and line type to
redundantly code the residuals' sign. The following example again uses
the \data{Bundesliga} data from above, this time using the Friendly
scheme and, in addition, an alternative legend (see
Figure~\ref{fig:bundesliga}, right):

<<friendly>>=
mosaic(BL, gp = shading_Friendly, legend = legend_fixed, zero_size = 0)
@

\setkeys{Gin}{width=\textwidth}
\begin{figure}[htbp]
\begin{center}
<<bundesligafig,fig=TRUE,echo=FALSE,height=5,width=12>>=
pushViewport(viewport(layout = grid.layout(ncol = 2)))
pushViewport(viewport(layout.pos.col = 1))
mosaic(BL, margin = c(right = 1), keep_aspect_ratio= FALSE, newpage = FALSE,
       legend_width=5.5, shade = TRUE)
popViewport(1)
pushViewport(viewport(layout.pos.col = 2))
mosaic(BL, gp = shading_Friendly, legend = legend_fixed, zero_size = 0,
       margin = c(right = 1), keep_aspect_ratio= FALSE, newpage = FALSE,
       legend_width=5.5)
popViewport(2)
@
\caption{The \data{Bundesliga} data for 1995. Left: Non-significant
  $\chi^2$ test. Right: using the Friendly
  shading and a legend with fixed bins.}
\label{fig:bundesliga}
\end{center}
\end{figure}
\setkeys{Gin}{width=0.7\textwidth}

\noindent (The \code{zero\_size = 0} argument removes the bullets indicating zero
observed values. This feature is not provided
in the original \proglang{SAS} implementation of the Friendly mosaic plots.)

% Figure~\ref{fig:shadingHSVHCL} depicts
% HSV space in the upper panel and HCL space in the lower panel.
% On the left (right) side, we see the color scales for red (blue)
% hue, respectively. The $x$-axis represents the colorfulness, and the
% $y$-axis the brightness.
% The boxes represent the diverging color palettes used for the shadings.
% For HSV space, we can see that the effect of changing the
% level of brightness (`value') is not the same for different levels of
% saturation, and again not the same for the two different hues.
% In fact, in HSV space all dimensions are confounded, which
% obviously is problematic for coding information. In contrast, HCL color
% space offers perceptually uniform colors: as can be seen from the lower panel,
% the chroma is homogeneous for different levels of luminance.
% Unfortunately, this comes at the
% price of the space being irregularly shaped, making it difficult to automatically select
% diverging color palettes.
% <<auxiliary_functions,echo=FALSE,results=hide>>=
% hue.slice <- function(hue, grid.n = 101, type = c("HCL", "HSV"), plot = TRUE, fixup = FALSE)
% {
%   type <- match.arg(type)
%   if(type == "HCL") {
%     chroma = seq(0, 100, length.out = grid.n)
%     luminance = seq(0, 100, length.out = grid.n)
%     nc <- length(chroma)
%     nl <- length(luminance)
%     color.slice <- outer(chroma, luminance, function(y, x) hcl(hue, x, y, fixup = fixup))
%     xlab <- "chroma"
%     ylab <- "luminance"
%     main <- paste("hue =", round(hue, digits = 0))
%   } else {
%     chroma = seq(0, 1, length.out = grid.n)
%     luminance = seq(0, 1, length.out = grid.n)
%     nc <- length(chroma)
%     nl <- length(luminance)
%     color.slice <- outer(chroma, luminance, function(y, x) hsv(hue, x, y))
%     xlab <- "saturation"
%     ylab <- "value"
%     main <- paste("hue =", round(hue, digits = 3))
%   }
%   if(plot) {
%     plot(0.5, 0.5, xlim = range(chroma), ylim = range(luminance), type = "n", axes = FALSE,
%          xlab = xlab, ylab = ylab, yaxs = "i", xaxs = "i", main = main)
%     for(i in 1:(nc-1)) {
%       rect(chroma[i], luminance[-nl], chroma[i] + 100/(nc-1), luminance[-1], border = color.slice[,i+1], col = color.slice[,i+1])
%     }
%     axis(1)
%     axis(2)
%     box()
%   }
%   colnames(color.slice) <- chroma
%   rownames(color.slice) <- luminance
%   attr(color.slice, "type") <- type
%   class(color.slice) <- "slice"
%   invisible(color.slice)
% }
% @

% \setkeys{Gin}{width=.8\textwidth}
% \begin{figure}[p]
% \begin{center}
% <<shading_HSV,fig=TRUE,echo=FALSE,height=7,width=8>>=
% ## generate colors
% hue23 <- hue.slice(2/3, grid.n = 101, plot = FALSE, type = "HSV")
% hue0 <- hue.slice(0, grid.n = 101, plot = FALSE, type = "HSV")
% saturation <- as.numeric(colnames(hue23))
% value <- as.numeric(rownames(hue23))

% ## select those with value >= 0.5
% hue23 <- hue23[value >= .5, ]
% hue0 <- hue0[value >= .5, ]
% value <- value[value >= .5]
% nl <- nrow(hue23)
% nc <- ncol(hue23)

% ## plot 2 slides from HSV space
% plot(0.5, 0.5, xlim = c(-1, 1), ylim = c(0, 1), type = "n", axes = FALSE,
%        xlab = "", ylab = "", yaxs = "i", xaxs = "i", main = "")
% for(i in 1:(nc-1)) {
%   rect(saturation[i], value[-nl], saturation[i] + 1/(nc-1), value[-1], border = hue23[,i+1], col = hue23[,i+1])
% }
% for(i in 1:(nc-1)) {
%   rect(-saturation[i], value[-nl], -(saturation[i] + 1/(nc-1)), value[-1], border = hue0[,i+1], col = hue0[,i+1])
% }
% axis(2, at = c(50, 75, 100)/100, labels = c(0.5, 0.75, 1))
% axis(4, at = c(50, 75, 100)/100, labels = c(0.5, 0.75, 1))
% axis(3, at = -4:4*.25, labels=c(4:0*.25, 1:4*.25))
% mtext(c("hue = 0", "hue = 2/3"), side = 3, at = c(-.5, .5), line = 3, cex = 1.2)
% mtext("saturation", side = 3, at = 0, line = 2)
% mtext("value", side = 2, at = .75, line = 2)
% mtext("value", side = 4, at = .75, line = 2)
% lines(c(-1, 1), c(.5, .5))

% ## significant colors
% rect(-1, 0.95, -.90, 1, col = hsv(0, 1, 1))
% rect(-0.45, 0.95, -.55, 1, col = hsv(0, 0.5, 1))
% rect(-.05, .95, .05, 1, col = hsv(2/3, 0, 1))
% rect(0.45, 0.95, .55, 1, col = hsv(2/3, 0.5, 1))
% rect(.90, .95, 1, 1, col = hsv(2/3, 1, 1))

% text(-1, .33, "significant", pos = 4, cex = 1.2)
% rect(-1, .20, -.80, .30, col = hsv(0, 1, 1))
% rect(-.40, .20, -0.6, .30, col = hsv(0, 0.5, 1))
% rect(-.20, .20, 0, .30, col = hsv(0, 0, 1))
% rect(0, .20, .20, .30, col = hsv(2/3, 0, 1))
% rect(0.4, .20, .60, .30, col = hsv(2/3, .5, 1))
% rect(.80, .20, 1, .30, col = hsv(2/3, 1, 1))

% lines(c(-.9, -.55), c(0.975, .975), lty = 2)
% lines(c(-.45, -.05), c(0.975, .975), lty = 2)
% lines(c(.45, .05), c(0.975, .975), lty = 2)
% lines(c(.9, .55), c(0.975, .975), lty = 2)

% ## non-significant colors
% rect(-1, 0.5, -.90, 0.55, col = hsv(0, 1, 0.5))
% rect(-0.4, 0.5, -.55, 0.55, col = hsv(0, 0.5, 0.5))
% rect(-.05, .5, .05, 0.55, col = hsv(2/3, 0, 0.5))
% rect(0.45, 0.5, .55, 0.55, col = hsv(2/3, 0.5, 0.5))
% rect(.90, .5, 1, 0.55, col = hsv(2/3, 1, 0.5))

% text(-1, .13, "non-significant", pos = 4, cex = 1.2)
% rect(-1, 0, -.80, .10, col = hsv(0, 1, 0.5))
% rect(-.60, 0, -.4, .10, col = hsv(0, 0.5, 0.5))
% rect(-.20, 0, 0, .10, col = hsv(0, 0, 0.5))
% rect(0, 0, .20, .10, col = hsv(2/3, 0, 0.5))
% rect(0.4, 0, .60, .1, col = hsv(2/3, .5, 0.5))
% rect(.80, 0, 1, .10, col = hsv(2/3, 1, 0.5))

% lines(c(-.9, -.55), c(0.525, .525), lty = 2)
% lines(c(-.45, -.05), c(0.525, .525), lty = 2)
% lines(c(.45, .05), c(0.525, .525), lty = 2)
% lines(c(.9, .55), c(0.525, .525), lty = 2)
% @

% <<shading_HCL,fig=TRUE,echo=FALSE,height=7,width=8>>=
% ## generate colors
% hue260 <- hue.slice(260, grid.n = 101, plot = FALSE)
% hue360 <- hue.slice(360, grid.n = 101, plot = FALSE)
% mychroma <- as.numeric(colnames(hue260))
% luminance <- as.numeric(rownames(hue260))

% ## select those with lumincance >= 50
% hue260 <- hue260[luminance >= 50, ]
% hue360 <- hue360[luminance >= 50, ]
% luminance <- luminance[luminance >= 50]
% nc <- ncol(hue260)
% nl <- nrow(hue260)

% ## plot 2 slides from HCL space
% plot(0.5, 0.5, xlim = c(-100, 100), ylim = c(0, 100), type = "n", axes = FALSE,
%        xlab = "", ylab = "", yaxs = "i", xaxs = "i", main = "")
% for(i in 1:(nc-1)) {
%   rect(mychroma[i], luminance[-nl], mychroma[i] + 100/(nc-1), luminance[-1], border = hue260[,i+1], col = hue260[,i+1])
% }
% for(i in 1:(nc-1)) {
%   rect(-mychroma[i], luminance[-nl], -(mychroma[i] + 100/(nc-1)), luminance[-1], border = hue360[,i+1], col = hue360[,i+1])
% }
% axis(2, at = c(50, 70, 90, 100), labels = c(50, 70, 90, 100))
% axis(4, at = c(50, 70, 90, 100), labels = c(50, 70, 90, 100))
% axis(3, at = -4:4*25, labels=c(4:0*25, 1:4*25))
% mtext(c("hue = 0", "hue = 260"), side = 3, at = c(-50, 50), line = 3, cex = 1.2)
% mtext("chroma", side = 3, at = 0, line = 2)
% mtext("luminance", side = 2, at = 75, line = 2)
% mtext("luminance", side = 4, at = 75, line = 2)
% lines(c(-100, 100), c(50, 50))

% ## significant colors
% rect(-100, 47.5, -90, 52.5, col = hcl(0, 100, 50))
% rect(-55, 67.5, -45, 72.5, col = hcl(0, 50, 70))
% rect(-5, 95, 5, 100, col = hcl(260, 0, 100))       ## grey vs. white
% rect(-5, 87.5, 5, 92.5, col = hcl(260, 0, 90))     ## grey vs. white
% rect(45, 67.5, 55, 72.5, col = hcl(260, 50, 70))
% rect(90, 47.5, 100, 52.5, col = hcl(260, 100, 50))

% text(-100, 33, "significant", pos = 4, cex = 1.2)
% rect(-100, 20, -80, 30, col = hcl(0, 100, 50))
% rect(-60, 20, -40, 30, col = hcl(0, 50, 70))
% rect(-20, 20, 0, 30, col = hcl(0, 0, 90))
% rect(0, 20, 20, 30, col = hcl(260, 0, 90))
% #white# rect(-20, 20, 0, 30, col = hcl(0, 0, 100))
% #white# rect(0, 20, 20, 30, col = hcl(260, 0, 100))
% rect(40, 20, 60, 30, col = hcl(260, 50, 70))
% rect(80, 20, 100, 30, col = hcl(260, 100, 50))

% lines(c(-45, -5), c(72.5, 87.5), lty = 2)
% lines(c(45, 5), c(72.5, 87.5), lty = 2)
% lines(c(-95, -55), c(52.5, 67.5), lty = 2)
% lines(c(95, 55), c(52.5, 67.5), lty = 2)

% ## non-significant colors
% rect(-25, 47.5, -15, 52.5, col = hcl(0, 20, 50))
% rect(-15, 67.5, -5, 72.5, col = hcl(0, 10, 70))
% rect(5, 67.5, 15, 72.5, col = hcl(260, 10, 70))
% rect(25, 47.5, 15, 52.5, col = hcl(260, 20, 50))


% text(-100, 13, "non-significant", pos = 4, cex = 1.2)
% rect(-60, 0, -40, 10, col = hcl(0, 20, 50))
% rect(-40, 0, -20, 10, col = hcl(0, 10, 70))
% rect(-20, 0, 0, 10, col = hcl(0, 0, 90))
% rect(0, 0, 20, 10, col = hcl(260, 0, 90))
% rect(20, 0, 40, 10, col = hcl(260, 10, 70))
% rect(40, 0, 60, 10, col = hcl(260, 20, 50))

% lines(c(-18.75, -11.25), c(52.5, 67.5), lty = 2)
% lines(c(-8.75, -1.25), c(72.5, 87.5), lty = 2)
% lines(c(18.75, 11.75), c(52.5, 67.5), lty = 2)
% lines(c(8.75, 1.25), c(72.5, 87.5), lty = 2)
% @
% \caption{Residual-based shadings in HSV (upper) and HCL space (lower).}
% \label{fig:shadingHSVHCL}
% \end{center}
% \end{figure}

A more ``advanced'' function building upon \codefun{shading\_hcl}
is \codefun{shading\_max}, using the
maximum statistic both to conduct the independence test and to
visualize significant \emph{cells} causing the
rejection of the independence hypothesis
\citep{vcd:Meyer+Zeileis+Hornik:2003,vcd:Zeileis+Meyer+Hornik:2007}. The
\code{level} argument of \codefun{shading\_max}
then can be used to specify several confidence
levels from which the corresponding cut-off points are computed.
By default, two cut-off points are computed corresponding
to confidence levels of $90\%$ and $99\%$, respectively.
In the following example, we investigate the effect of a new treatment
for rheumatoid arthritis on a group of female patients using the
maximum shading (see Figure~\ref{fig:maximum}):

<<arthritis>>=
set.seed(4711)
mosaic(~ Treatment + Improved, data = Arthritis, subset = Sex == "Female",
       gp = shading_max)
@

\begin{figure}[h]
\begin{center}
<<arthritisfig,fig=TRUE,echo=FALSE>>=
<<arthritis>>
@
\caption{The \data{Arthritis} data (female patients) with significant maximum test.}
\label{fig:maximum}
\end{center}
\end{figure}

\noindent The maximum test is significant although the residuals are
all in the $\left[-2,2\right]$ interval.
The \codefun{shading\_hcl} function with default cut-off points would
not have shown any color. In addition, since the test statistic is the
maximum of the absolute Pearson residuals, \emph{each} colored
residual violates the null hypotheses of independence, and thus, the
``culprits'' can immediately be identified.

\clearpage
\section[Labeling]{Labeling}
\label{sec:labeling}

One of the major enhancements in package \pkg{vcd} compared to
\codefun{mosaicplot} and \codefun{assocplot} in
base \proglang{R} is the labeling in the strucplot framework which
offers more features and greater flexibility.
Like shading, spacing, and drawing of legend and core plot,
labeling is now carried out by grapcon functions, rendering labeling completely
modular.
The user supplies either a labeling function, or, alternatively,
a generating function that parameterizes a labeling function,
to \codefun{strucplot} which then draws the labels.
Labeling is well-separated from the actual plotting that occurs in the
low-level core functions. It only relies on the viewport tree produced by them, and the
\code{dimnames} attribute of the visualized table. Labeling
functions are grapcons that ``add ink to the canvas'': the drawing of the
labels happens after the actual plot has been drawn by the core function.
Thus, it is possible to supply one's own labeling function, or
to combine some of the basic functions to produce a more complex labeling.
In the following, we describe the three basic modules
(\codefun{labeling\_text}, \codefun{labeling\_list}, and \codefun{labeling\_cells}) and
derived functions that build upon them.

\subsection[Labels in the borders]{Labels in the borders: \texttt{labeling\_text()}}

\codefun{labeling\_text} is the default for all strucplot displays. It
plots labels in the borders similar to the \codefun{mosaicplot}
function in base \proglang{R}, but is much more flexible: it is not limited
to 4 dimensions, and the positioning and graphical parameters
of levels and variable names are customizable. In addition,
the problem of overlapping labels can be handled in several ways.

As an example, again consider the \data{Titanic} data:
by default, the variable names and levels are plotted ``around'' the
plot in a counter-clockwise way (see Figure~\ref{fig:labels1}, top left):

<<default>>=
mosaic(Titanic)
@

% \begin{figure}[p]
% \begin{center}
% <<defaultfig,fig=TRUE,echo=FALSE>>=
% <<default>>
% @
% \caption{Mosaic plot for the \data{Titanic} data with default settings
%   for labeling.}
% \label{fig:defaults}
% \end{center}
% \end{figure}

\noindent Note that the last two levels of the \code{survived} variable do
overlap, as well as some adult and child labels of the \code{age} Variable.
This issue can be addressed in several ways. The ``brute force''
method is to enable clipping for these dimensions (see
Figure~\ref{fig:labels1}, top right):

<<clipping>>=
mosaic(Titanic, labeling_args = list(clip = c(Survived = TRUE, Age = TRUE)))
@

% \begin{figure}[p]
% \begin{center}
% <<clippingfig,fig=TRUE,echo=FALSE>>=
% <<clipping>>
% @
% \caption{The effect of clipping.}
%  \label{fig:clipping}
% \end{center}
% \end{figure}

\noindent The \code{clip} parameter is passed to the labeling function
via the \code{labeling\_args} argument which takes a list of
parameters. \code{clip} itself takes a vector of logicals (one for
each dimension).
% as mentioned before
Almost all vectorized arguments in the strucplot
framework can be abbreviated in the following way: unnamed components
(or the defaults, if there are none) are recycled as needed, but
overridden by the named components. Here, the default is \code{FALSE},
and therefore clipping is enabled only for the \code{survived} and \code{age} variables.
A more sensible solution to the overlap problem is to abbreviate the
levels (see Figure~\ref{fig:labels1}, middle left):

<<abbreviating>>=
mosaic(Titanic, labeling_args = list(abbreviate_labs = c(Survived = TRUE, Age = 3)))
@

% \begin{figure}[h]
% \begin{center}
% <<abbreviatingfig,fig=TRUE,echo=FALSE>>=
% <<abbreviating>>
% @
% \caption{Abbreviating.}
% \label{fig:abbreviating}
% \end{center}
% \end{figure}

\noindent The \code{abbreviate} argument takes a vector of
integers indicating the number of significant characters the levels should be
abbreviated to (\code{TRUE} is interpreted as 1, obviously). Abbreviation
is performed using the \codefun{abbreviate} function in base \proglang{R}. Another
possibility is to rotate the levels (see Figure~\ref{fig:labels1}, bottom):

<<rotate>>=
mosaic(Titanic, labeling_args = list(rot_labels = c(bottom = 90, right = 0),
offset_varnames = c(right = 1), offset_labels = c(right = 0.3)),
margins = c(right = 4, bottom = 3))
@

% \begin{figure}[p]
% \begin{center}
% <<rotatefig,fig=TRUE,echo=FALSE,height=6>>=
% <<rotate>>
% @
% \caption{Rotating labels.}
% \label{fig:rotating}
% \end{center}
% \end{figure}

\noindent Finally, we could also inhibit the output of repeated levels
(see Figure~\ref{fig:labels1}, middle right):

<<repeat>>=
mosaic(Titanic, labeling_args = list(rep = c(Survived = FALSE, Age = FALSE)))
@

\setkeys{Gin}{width=0.9\textwidth}
\begin{figure}[p]
\begin{center}
<<label1fig,fig=TRUE,echo=FALSE,height=12,width=8>>=
pushViewport(viewport(layout = grid.layout(ncol = 2,nrow=3)))
pushViewport(viewport(layout.pos.col = 1, layout.pos.row = 1))
mosaic(Titanic, newpage = FALSE, keep = TRUE, margin = c(right = 3),
       gp_labels = gpar(fontsize = 10))
popViewport()
pushViewport(viewport(layout.pos.col = 2, layout.pos.row = 1))
mosaic(Titanic, labeling_args = list(clip = c(Survived = TRUE, Age = TRUE)),
       newpage = FALSE, keep = TRUE, margin = c(left = 3), gp_labels = gpar(fontsize = 10))
popViewport()
pushViewport(viewport(layout.pos.col = 1, layout.pos.row = 2))
mosaic(Titanic, labeling_args = list(abbreviate_labs = c(Survived = TRUE, Age = 2)),
       newpage = FALSE, keep = TRUE, margin = c(right = 3), gp_labels = gpar(fontsize = 10))
popViewport()
pushViewport(viewport(layout.pos.col = 2, layout.pos.row = 2))
mosaic(Titanic, labeling_args = list(rep = c(Survived = FALSE, Age = FALSE)),
       newpage = FALSE, keep = TRUE, margin = c(left = 3), gp_labels = gpar(fontsize = 10))
popViewport()
pushViewport(viewport(layout.pos.col = 1:2, layout.pos.row = 3))
pushViewport(viewport(width = 0.55))
mosaic(Titanic, labeling_args = list(rot_labels = c(bottom = 90, right = 0),
                  offset_varnames = c(right = 1), offset_labels = c(right = 0.3)),
       margins = c(right = 4, bottom = 3), newpage = FALSE, keep = FALSE, gp_labels = gpar(fontsize = 10))
popViewport(3)
@
\caption{Examples for possible labeling strategies for the Titanic data mosaic.
  Top left: default labeling (many labels overlap).
  Top right: with clipping turned on. Middle left: \texttt{Age} and
  \texttt{Survived} labels abbreviated. Middle right: \texttt{Age} labels
  not repeated. Bottom: \texttt{Age} and \texttt{Survived} labels rotated.}
\label{fig:labels1}
\end{center}
\end{figure}
\setkeys{Gin}{width=0.7\textwidth}

We now proceed with a few more ``cosmetic'' features (which do not all
produce satisfactory results for our sample data).
A first simple, but effectful modification is to position
all labels and variables left-aligned (see Figure~\ref{fig:labels2},
top left):

<<left>>=
mosaic(Titanic, labeling_args = list(pos_varnames = "left", pos_labels = "left",
       just_labels = "left", rep = FALSE))
@

% \begin{figure}[h]
% \begin{center}
% <<leftfig,fig=TRUE,echo=FALSE>>=
% <<left>>
% @
% \caption{Left-aligning.}
% \label{fig:left}
% \end{center}
% \end{figure}

\noindent Note that obviously we need to change the justification
to \code{"left"} as well. We can
achieve the same effect by using the convenience function \codefun{labeling\_left}:

<<left2>>=
mosaic(Titanic, labeling = labeling_left)
@

\noindent Next, we show how to put all levels to the
bottom and right margins, and all variable names to the top and left
margins (see Figure~\ref{fig:labels2}, top right):

<<margins>>=
mosaic(Titanic, labeling_args = list(tl_labels = FALSE, tl_varnames = TRUE, abbreviate_labs = c(Survived = 1, Age = 3)))
@

% \begin{figure}[p]
% \begin{center}
% <<marginsfig,fig=TRUE,echo=FALSE,height=6>>=
% <<margins>>
% @
% \caption{Changes in the margins.}
% \label{fig:margins}
% \end{center}
% \end{figure}

\noindent The tl\_\var{foo} (``top left'') arguments are \code{TRUE} by default.
Now, we will add boxes to the labels and additionally
enable clipping (see Figure~\ref{fig:labels2}, bottom left):

<<boxes>>=
mosaic(Titanic, labeling_args = list(tl_labels = FALSE, tl_varnames = TRUE,
       boxes = TRUE, clip = TRUE))
@

% \begin{figure}[p]
% \begin{center}
% <<boxesfig,fig=TRUE,echo=FALSE,height=6>>=
% <<boxes>>
% @
% \caption{Boxes and Clipping.}
% \label{fig:boxes}
% \end{center}
% \end{figure}

\noindent The values to \code{boxes} and \code{clip} are recycled for
all dimensions. The result is pretty close to what calling
\codefun{mosaic} with the \codefun{labeling\_cboxed}
wrapper does, except that variables and levels, by default, are put to the top and
to the left of the plot:

<<boxes2>>=
mosaic(Titanic, labeling = labeling_cboxed)
@

\noindent Another variant is to put the variable names into the
same line as the levels (see Figure~\ref{fig:labels2}, bottom
right---clipping for \code{Survived} and \code{Age} is, additionally,
disabled, and \code{Age} abbreviated):

<<labbl>>=
mosaic(Titanic, labeling_args = list(tl_labels = TRUE, boxes = TRUE,
       clip = c(Survived = FALSE, Age = FALSE, TRUE), abbreviate_labs = c(Age = 4),
       labbl_varnames = TRUE), margins = c(left = 4, right = 1, 3))
@

% \begin{figure}[h]
% \begin{center}
% <<labblfig,fig=TRUE,echo=FALSE>>=
% <<labbl>>
% @
% \caption{Variable names beneath levels, and clipping disabled for the
%   survival variable.}
% \label{fig:labbl}
% \end{center}
% \end{figure}

\noindent \code{labbl\_varnames} (``variable names to the bottom/left of the
labels'') is a vector of logicals indicating the side for the
variable names. The resulting layout is close to what
\codefun{labeling\_lboxed} produces, except that variables and levels,
by default, are left-aligned and put to the bottom and to the right of the plot:

<<labbl2>>=
mosaic(Titanic, labeling = labeling_lboxed, margins = c(right = 4, left = 1, 3))
@

\noindent A similar design is used by the \codefun{doubledecker} function.

\setkeys{Gin}{width=\textwidth}
\begin{figure}[p]
\begin{center}
<<label2fig,fig=TRUE,echo=FALSE,height=12,width=12>>=
pushViewport(viewport(layout = grid.layout(ncol = 2, nrow = 2)))
pushViewport(viewport(layout.pos.col = 1, layout.pos.row = 1))
mosaic(Titanic, labeling_args = list(pos_varnames = "left", pos_labels = "left",
       just_labels = "left", rep = FALSE), newpage = FALSE, keep = TRUE,
       gp_labels = gpar(fontsize = 12))
popViewport()
pushViewport(viewport(layout.pos.col = 2, layout.pos.row = 1))
mosaic(Titanic, labeling_args = list(tl_labels = FALSE, tl_varnames = TRUE,
                  abbreviate_labs = c(Survived = 1, Age = 3)), newpage = FALSE, keep = TRUE,
       margins = c(left = 4, right = 1, 3), gp_labels = gpar(fontsize = 12))
popViewport()
pushViewport(viewport(layout.pos.col = 1, layout.pos.row = 2))
mosaic(Titanic, labeling_args = list(tl_labels = FALSE, tl_varnames = TRUE,
       boxes = TRUE, clip = TRUE), newpage = FALSE, keep = TRUE,
       gp_labels = gpar(fontsize = 12))
popViewport()
pushViewport(viewport(layout.pos.col = 2, layout.pos.row = 2))
mosaic(Titanic, labeling_args = list(tl_labels = TRUE, boxes = TRUE,
       clip = c(Survived = FALSE, Age = FALSE, TRUE),
       labbl_varnames = TRUE, abbreviate_labs = c(Age = 4)),
       margins = c(left = 4, right = 1, 3),
       newpage = FALSE, keep = TRUE, gp_labels = gpar(fontsize = 12))
popViewport(2)
@
\caption{Advanced strategies for labeling of the Titanic data. Top
  left: left aligning of both variable names and labels.
  Top right: changes in the margins (all variable names are in the top
  and left margins, and all labels in the bottom and right margins).
  Bottom left: clipping turned on, and boxes used. Bottom right:
  variable names beneath levels, clipping disabled for the
  survival and age variables, and \texttt{Age} abbreviated.}
\label{fig:labels2}
\end{center}
\end{figure}
\setkeys{Gin}{width=0.7\textwidth}

\subsection[Labels in the cells]{Labels in the cells: \texttt{labeling\_cells()}}

This labeling draws both variable names and levels in the
cells. As an example, we use the \data{PreSex} data on pre- and
extramarital sex and divorce (see Figure~\ref{fig:labels3}, top left):

<<cell>>=
mosaic(~ MaritalStatus + Gender, data = PreSex, labeling = labeling_cells)
@

% \begin{figure}[h]
% \begin{center}
% <<cellfig,fig=TRUE,echo=FALSE,width=7,height=7>>=
% <<cell>>
% @
% \caption{Cell labeling for the \data{PreSex} data.}
% \label{fig:cell}
% \end{center}
% \end{figure}

\noindent In the case of narrow cells, it might be useful to
abbreviate labels and/or variable names and turn off clipping (see
Figure~\ref{fig:labels3}, top right):

<<cell2>>=
mosaic(~ PremaritalSex + ExtramaritalSex, data = PreSex,
labeling = labeling_cells(abbreviate_labels = TRUE, abbreviate_varnames = TRUE, clip = FALSE))
@

% \begin{figure}[h]
% \begin{center}
% <<cell2fig,fig=TRUE,echo=FALSE>>=
% <<cell2>>
% @
% \caption{Cell labeling for the \data{PreSex} data, labels abbreviated.}
% \label{fig:cell2}
% \end{center}
% \end{figure}

\noindent For some data, it might be convenient to combine cell
labeling with border labeling as done by \codefun{labels\_conditional}
(see Figure~\ref{fig:labels3}, bottom left):

<<conditional>>=
mosaic(~ PremaritalSex + ExtramaritalSex | MaritalStatus + Gender, data = PreSex, labeling = labeling_conditional(abbreviate_varnames = TRUE, abbreviate_labels = TRUE, clip = FALSE, gp_text = gpar(col = "red")))
@

% \begin{figure}[h]
% \begin{center}
% <<conditionalfig,fig=TRUE,echo=FALSE>>=
% <<conditional>>
% @
% \caption{Conditional labeling for the \data{PreSex} data, labels (in
%   red for clarity) abbreviated.}
% \label{fig:conditional}
% \end{center}
% \end{figure}

\noindent Additionally, the cell labeling allows the user to add
arbitrary text to the cells by supplying a character array in the same
shape as the data array to the \code{text} argument (cells with missing values
are ignored). In the following example using the \code{Titanic} data,
this is used to add all observed values greater
than 5 to the cells after the mosaic has been plotted (see
Figure~\ref{fig:labels3}, bottom right):

<<text>>=
mosaic(Titanic, labeling_args = list(abbreviate_labs = c(Survived = 1, Age = 4)), pop = FALSE)

tab <- ifelse(Titanic < 6, NA, Titanic)
labeling_cells(text = tab, clip = FALSE)(Titanic)
@

% \begin{figure}[h]
% \begin{center}
% <<textfig,fig=TRUE,echo=FALSE>>=
% <<text>>
% @
% \caption{User-supplied text (observed frequencies exceeding 5)
% added to a mosaic display of the \data{Titanic} data.}
% \label{fig:text}
% \end{center}
% \end{figure}


\setkeys{Gin}{width=\textwidth}
\begin{figure}[p]
\begin{center}
<<label3fig,fig=TRUE,echo=FALSE,height=8,width=8>>=
grid.newpage()
pushViewport(viewport(layout = grid.layout(ncol = 2, nrow = 2)))
pushViewport(viewport(layout.pos.col = 1, layout.pos.row = 1))
mosaic(~ MaritalStatus + Gender, data = PreSex, labeling = labeling_cells, newpage = FALSE, keep = TRUE, gp_labels = gpar(fontsize = 10))
popViewport()
pushViewport(viewport(layout.pos.col = 2, layout.pos.row = 1))
mosaic(~ PremaritalSex + ExtramaritalSex, data = PreSex,
       labeling = labeling_cells(abbreviate_labels = TRUE, abbreviate_varnames = TRUE,
         clip = FALSE), newpage = FALSE, keep = TRUE, gp_labels = gpar(fontsize = 10))
popViewport()
pushViewport(viewport(layout.pos.col = 1, layout.pos.row = 2))
mosaic(~ PremaritalSex + ExtramaritalSex | MaritalStatus + Gender, data = PreSex,
       labeling = labeling_conditional(abbreviate_varnames = TRUE,
         abbreviate_labels = TRUE, clip = FALSE, gp_text = gpar(col = "red")),
       newpage = FALSE, keep = TRUE, gp_labels = gpar(fontsize = 10))
popViewport()
pushViewport(viewport(layout.pos.col = 2, layout.pos.row = 2))
mosaic(Titanic, labeling_args = list(abbreviate_labs = c(Survived = 1, Age = 3)),
       pop = FALSE, newpage = FALSE, keep = TRUE, gp_labels = gpar(fontsize = 10))
tab <- ifelse(Titanic < 6, NA, Titanic)
labeling_cells(text = tab, clip = FALSE)(Titanic)
@
\caption{Cell labeling. Top left: default labeling using the
  \data{PreSex} data. Top right: abbreviated labels. Bottom left:
  conditional labeling (labels abbreviated and in red for clarity).
  Bottom right: user-supplied text (observed frequencies exceeding 5)
  added to a mosaic display of the \data{Titanic} data. Note that
  clipping is on by default (top left), and has explicitly been turned
  off for the three other plots.}
\label{fig:labels3}
\end{center}
\end{figure}
\setkeys{Gin}{width=0.7\textwidth}

\subsection[A simple list of labels]{A simple list of labels: \texttt{labeling\_list()}}

If problems with overlapping labels cannot satisfactorily resolved,
the last remedy could be to simply list the levels below the plot
(see Figure~\ref{fig:list}):

<<list>>=
mosaic(Titanic, labeling = labeling_list, margins = c(bottom = 5))
@

\setkeys{Gin}{width=0.7\textwidth}
\begin{figure}[p]
\begin{center}
<<listfig,fig=TRUE,echo=FALSE>>=
mosaic(Titanic, labeling = labeling_list, margins = c(bottom = 5), keep = TRUE)
@
\caption{Labels indicated below the plot.}
\label{fig:list}
\end{center}
\end{figure}

\noindent The number of columns can be specified.

\section{Spacing}
\label{sec:spacing}

Spacing of strucplot displays is customizable in a similar way than shading. The
\code{spacing} argument of the \codefun{strucplot} function takes a
list of \class{unit} vectors, one for each dimension, specifying the
space between the tiles corresponding to the levels.
Consider again the introductory example of the \data{Arthritis} data
(Figure~\ref{fig:arthritis}). Since we are interested in the effect of
the medicament in the placebo and treatment groups, a mosaic plot is
certainly appropriate to visualize the three levels of \code{Improved} in
the two \code{Treatment} strata. Another conceptual approach is to use
spine plots with highlighting \citep{vcd:hummel:1996}. A spine plot is
a variation of a bar plot where the heights of the bars are held
constant, whereas the widths are used to represent the number of cases
in each category. This is equivalent to a mosaic plot for a one-way table.
If a second (indicator) variable is highlighted in a spine plot, we
obtain a display equivalent to a simple mosaic display for a two-way
table, except that no space between the levels of the highlighted
variable is used.
In the \data{Arthritis} example, we will highlight patients with \code{Marked}
improvement in both groups. To obtain such a display within
the strucplot framework, it suffices to set the space between the \code{Improved}
tiles to 0 (see Figure~\ref{fig:artspine}):

<<artspine,echo=TRUE,results=verbatim>>=
(art <- structable(~Treatment + Improved, data = Arthritis, split_vertical = TRUE))
(my_spacing <- list(unit(0.5, "lines"), unit(c(0, 0), "lines")))
my_colors <- c("lightgray", "lightgray", "black")
mosaic(art, spacing = my_spacing, gp = gpar(fill = my_colors, col = my_colors))
@

\begin{figure}[p]
\begin{center}
<<artspinefig,echo=FALSE,fig=TRUE>>=
<<artspine>>
@
\caption{Spine plot for the \data{Arthritis} data using the strucplot framework.}
\label{fig:artspine}
\end{center}
\end{figure}

\noindent Note that the default and formula methods for
\codefun{mosaic} provide a
convenience interface for highlighting. A similar plot (with slightly
different shading) than the previous one can be obtained using:

<<artspine>>=
mosaic(Improved ~ Treatment, data = Arthritis, split_vertical = TRUE)
@

\noindent The strucplot framework also provides a set of spacing grapcon
generators which compute suitable spacing objects for
typical applications. The simplest spacing is \codefun{spacing\_equal}
that uses the same space between all tiles (see
Figure~\ref{fig:spacing}, top left):

<<space1>>=
mosaic(art, spacing = spacing_equal(unit(2, "lines")))
@

\noindent \codefun{spacing\_equal} is the default grapcon generator for
two-dimensional tables. Slightly more flexible is \codefun{spacing\_dimequal} that
allows an individual setting for each dimension (see
Figure~\ref{fig:spacing}, top right):

<<space2>>=
mosaic(art, spacing = spacing_dimequal(unit(1:2, "lines")))
@

\noindent The default for multi-way contingency tables is
\codefun{spacing\_increase} which uses increasing spaces for the
dimensions. The user can specify a start value and the increase factor
(see Figure~\ref{fig:spacing}, bottom left):

<<space3>>=
mosaic(art, spacing = spacing_increase(start = unit(0.5, "lines"), rate = 1.5))
@

\noindent For the arthritis example above, we could as well have used
\codefun{spacing\_highlighting} which is similar to \codefun{spacing\_increase} but
sets the spacing in the last splitting dimension to 0 (see
Figure~\ref{fig:spacing}, bottom right):

<<spine4>>=
mosaic(art, spacing = spacing_highlighting, gp = my_colors)
@

\noindent Finally, \codefun{spacing\_conditional} can be used for
visualizing conditional independence: it combines
\codefun{spacing\_equal} (for the conditioned dimensions) and
\codefun{spacing\_increase} (for the conditioning dimensions). As an example,
consider Figure~\ref{fig:presex}: the spacing clearly allows to better
distinguish the conditioning variables
(\code{Gender} and \code{MaritalStatus}) from the conditioned
variables (\code{PremaritalSex} and \code{ExtramaritalSex}). This
spacing is the default when conditional variables are specified for a
strucplot display (see Section \ref{sec:strucplot}).


\setkeys{Gin}{width=\textwidth}
\begin{figure}[p]
\begin{center}
<<spacingfig,fig=TRUE,echo=FALSE,height=12,width=12>>=
pushViewport(viewport(layout = grid.layout(ncol = 2, nrow = 2)))
pushViewport(viewport(layout.pos.col = 1, layout.pos.row = 1))
mosaic(art, spacing = spacing_equal(unit(2, "lines")), keep = TRUE, newpage = FALSE)
popViewport()
pushViewport(viewport(layout.pos.col = 2, layout.pos.row = 1))
mosaic(art, spacing = spacing_dimequal(unit(c(0.5, 2), "lines")), keep = TRUE, newpage = FALSE)
popViewport()
pushViewport(viewport(layout.pos.col = 1, layout.pos.row = 2))
mosaic(art, spacing = spacing_increase(start = unit(0.3, "lines"), rate = 2.5), keep = TRUE, newpage = FALSE)
popViewport()
pushViewport(viewport(layout.pos.col = 2, layout.pos.row = 2))
mosaic(art, spacing = spacing_highlighting, keep = TRUE, newpage = FALSE)
popViewport(2)
@
\caption{Varying spacing for the Arthritis  data. Top left: equal
  spacing for all dimensions. Top right: different spacings for
  individial dimensions. Bottom left: increasing spacing. Bottom right:
  spacing used for highlighting.}
\label{fig:spacing}
\end{center}
\end{figure}
\setkeys{Gin}{width=0.7\textwidth}


\section{Example: Ovarian cancer survival}
\label{sec:example}

In the following, we demonstrate some of the described techniques in
analyzing a data set originating from \citep{vcd:obel:1975}
\cite[taken from][]{vcd:andersen:1991} about a retrospective study of ovary cancer
carried out in 1973.  Information was obtained from 299 women, who
were operated for ovary cancer 10 years before. The data consists of
four binary variables: the \code{stage} of the cancer at the time of
operation (levels: \code{early}, \code{advanced}), the type of \code{operation} performed
(\code{radical}, \code{limited}), the \code{survival} status after 10 years (\code{yes},
\code{no}), and \code{xray} indicating whether X-ray treatment was received (\code{yes},
\code{no}).

The dataset in \pkg{vcd} comes pretabulated in a data frame, so we
first create the four-way table:

<<oc1>>=
tab <- xtabs(Freq ~ stage + operation + xray + survival, data = OvaryCancer)
@

\noindent A ``flattened'' textual representation can be obtained using \codefun{structable}:

<<oc2,echo=TRUE,results=verbatim>>=
structable(survival ~ ., data = tab)
@

\noindent A first overview can be obtained using a pairs plot (Figure~\ref{fig:ocpairs}):

<<oc3>>=
dpa <- list(var_offset = 1.2, rot = -30, just_leveltext= "left")
pairs(tab, diag_panel = pairs_barplot, diag_panel_args = dpa)
@

\begin{figure}[h]
\begin{center}
<<ocpairs,echo=FALSE,fig=TRUE,width=8,height=8>>=
<<oc3>>
@
\caption{Pairs plot for the \data{OvaryCancer} data showing mosaic displays for all pairwise distributions and bar plots for all marginal distributions.}
\label{fig:ocpairs}
\end{center}
\end{figure}

\noindent The pairs plot, by default, creates mosaic displays for all pairwise
variable combinations, and bar plots in the diagonal to visualize the absolute
frequencies of the variables. The \texttt{var\_offset} argument modifies the offset of the
(centered) variable names to avoid overlap with the bars.
Additionally, we use the \texttt{rot} and the
\texttt{just\_leveltext} arguments to rotate the level names, again to avoid their overlap.
First, we consider the marginal distributions. The study design involved
(nearly) the same number of survived (150) and deceased (149) patients. Similarly balanced,
158 cases were in an advanced and 141 in an early stage. Most patients (251, 84\%)
were treated with a radical operation, and 186 (62\%) were submitted to
X-ray treatment. Next, we inspect
the two-way interaction of the influencing factors (\code{stage}, \code{operation},
and \code{xray}): the
corresponding mosaics exhibit symmetric, regular shapes with aligned tiles,
which indicate no marginal interaction between these variables.
The same is true for the interactions of \code{survival}
with \code{operation} and \code{xray}, respectively. Only the stage seems to influence
survival: here, the tiles are ``shifted''.

A different view on the data, focused on the influence of the
explanatory variables on \code{Survival},
can be obtained using a doubledecker plot (Figure~\ref{fig:ocdoubledecker}):

<<oc4>>=
doubledecker(survival ~ stage + operation + xray, data = tab)
@

\begin{figure}[h]
\begin{center}
<<ocdoubledecker,echo=FALSE,fig=TRUE,width=8>>=
<<oc4>>
@
\caption{Doubledecker plot for the \data{OvaryCancer} data showing the
 conditional distribution of X-ray, given operation, given stage, and with
 survival highlighted.}
\label{fig:ocdoubledecker}
\end{center}
\end{figure}

\noindent From a technical point of view, the display is constructed as a mosaic plot
showing the conditional distribution of \code{survival}, given \code{xray},
given \code{operation}, given \code{stage},
with vertical splits for the conditioning variables and horizontal ones for
\code{survival}.
Additionally, there is zero space between the tiles of the last dimension and
a binary shading is used for survived and deceased patients.
Conceptually, this plot is interpreted as a mosaic plot of just the influencing
variables, with \code{survival} highlighted in the tiles. Thus, the plot really shows
the influence of the explanatory variables on \code{survival}. Clearly, the survival
rate is higher among patients in an early stage, but neither radical operation
nor X-ray treatment seem to improve the situation. From this exploratory
phase, the survival rate seems to be slightly higher for patients who
received a limited operation only, whereas the effect for X-ray
treatment is less marked.

To visualize inference results, we can make use of residual-based
shadings, investigating log-linear models for the four-way table.
Figure~\ref{fig:ocmosaicnull} visualizes the null model, where
survival is independent from the combined effect of operation, X-ray
treatment, and stage:

<<oc6>>=
split <- c(TRUE, TRUE, TRUE, FALSE)
mosaic(tab, expected = ~ survival + operation * xray * stage, split_vertical = split)
@

\begin{figure}[p]
\begin{center}
<<ocmosaicnull,echo=FALSE,fig=TRUE,width=10>>=
<<oc6>>
@
\caption{Mosaic plot for the \data{OvaryCancer} data, with residual-based shading
for the (clearly rejected) null model (survival)(operation, X-ray, stage).}
\label{fig:ocmosaicnull}
\end{center}
\end{figure}

\noindent The model is clearly rejected ($p$-value: 0.000). From the exploratory phase of
our analysis, we (only) suspect \code{stage} to be influential on the survival rate. A
corresponding hypothesis is that \code{survival} be independent of \code{xray}
and \code{operation}, given
\code{stage}. The model is specified using the \texttt{expected} argument,
either using the \codefun{loglin} interface or
the \codefun{loglm} formula interface (the resulting mosaic plot is shown in Figure
\ref{fig:ocmosaicstage}):

<<oc7>>=
mosaic(tab, expected = ~ (survival + operation * xray) * stage, split_vertical = split)
@

\begin{figure}[p]
\begin{center}
<<ocmosaicstage,echo=FALSE,fig=TRUE,width=10>>=
<<oc7>>
@
\caption{Mosaic plot for the \data{OvaryCancer} data, with residual-based shading
for the hypothesis of survival being independent of X-ray and operation, given
stage. The hypothesis is not rejected.}
\label{fig:ocmosaicstage}
\end{center}
\end{figure}

\noindent Thus, based on this data, only pre-diagnosis seems to matter in ovarian
cancer therapy.

\section{Conclusion}
\label{sec:conclusion}

In this paper, we describe the ``strucplot'' framework for the visualization of
multi-way contingency tables. Strucplot displays include popular
basic plots such as mosaic, association, and sieve plots, integrated
in a unified framework: all can be seen as visualizations of
hierarchical conditional flat tables.
Additionally, these core strucplot displays can be combined into more complex,
specialized plots, such as pairs and trellis-like displays for visualizing
conditional independence. Residual-based shadings permit the
visualization of log-linear models and the results of independence tests.
The framework's modular design allows
flexible customization of the plots' graphical appearance, including
shading, labeling, spacing, and legend, by means of graphical
appearance control (``grapcon'') functions. These ``graphical
hyperparameters'' are customized and created by
generating functions. Our work includes a set of predefined grapcon generators for
typical analysis tasks, and user-level extensions can easily be added.

\bibliography{vcd}

\begin{appendix}

\section{Data sets}
\label{sex:data}

The data set names in the paper are those from the \proglang{R}
system. In the following, we give a short description of each data set.

\begin{description}
\item[\texttt{Arthritis}] Data  from a
  double-blind clinical trial
  investigating a new treatment for rheumatoid arthritis.
  Source: \cite{vcd:Koch+Edwards:1988}. Taken from: \cite{vcd:Friendly:2000}.
  Package: \pkg{vcd}.
\item[\texttt{Bundesliga}] Results from the first German soccer league in the
  years 1995/6 \citep{vcd:Knorr-Held:1999} and 2001/2 (Collected by: Achim
  Zeileis). Package: \pkg{vcd}.
\item[\texttt{HairEyeColor}] Distribution of hair and eye color and
  gender in 592 statistics
  students. The gender information is artificial. Source: \cite{vcd:Snee:1974}.
  Taken from: \cite{vcd:Friendly:2000}.
  Package: \pkg{datasets} (included in base \proglang{R}).
\item[\texttt{OvaryCancer}]  Data about a retrospective study of ovary cancer
     carried out in 1973.  Information was obtained from 299 women, who
     were operated for ovary cancer 10 years before. Source:
     \cite{vcd:obel:1975}. Taken fromn: \cite{vcd:andersen:1991}. Package: \pkg{vcd}.
\item[\texttt{PreSex}]  Data on pre- and extra-marital sex and divorce.
  Source: \cite{vcd:thornes+collard:1979}. Taken from \cite{vcd:gilbert:1981}.
  Package: \pkg{vcd}.
\item[\texttt{Titanic}] Information on the fate of passengers on
  the fatal maiden voyage of the ocean liner ``Titanic'', summarized
  according to economic status (class), gender (\code{Sex}), age and
  survival. Data originally collected by the British Board of Trade
  in their investigation of the sinking. Taken from: \cite{vcd:dawson:1995}.
  Package: \pkg{datasets} (included in base \proglang{R}).
\item[\texttt{UCBAdmissions}]
  Aggregate data on applicants to graduate school at Berkeley for
  the six largest departments in 1973 classified by admission and
  gender. Source: \cite{vcd:Bickel+Hammel+O'Connell:1975}.
  Taken from: \cite{vcd:Friendly:2000}.
  Package: \pkg{datasets} (included in base \proglang{R}).
\end{description}

\end{appendix}

\end{document}

\documentclass[nojss]{jss}
%% need no \usepackage{Sweave}
\usepackage{rotating}
\newcommand{\given}{\, | \,}


\title{Residual-Based Shadings in \pkg{vcd}}
\Plaintitle{Residual-Based Shadings in vcd}
\author{Achim Zeileis, David Meyer, \textnormal{and} Kurt Hornik\\Wirtschaftsuniversit\"at Wien, Austria}
\Plainauthor{Achim Zeileis, David Meyer, Kurt Hornik}

\Abstract{
  This vignette is a companion paper to \cite{vcd:Zeileis+Meyer+Hornik:2007}
  which introduces several extensions to residual-based shadings for enhancing
  mosaic and association plots. The paper introduces (a)~perceptually uniform
  Hue-Chroma-Luminance (HCL) palettes and (b)~incorporates the result of an
  associated significance test into the shading. Here, we show how the
  examples can be easily reproduced using the \pkg{vcd} package.
}

\Keywords{association plots, conditional inference, contingency tables, HCL colors, HSV colors, mosaic plots}

\Address{
Achim Zeileis\\
E-mail: \email{Achim.Zeileis@R-project.org}\\

David Meyer\\
E-mail: \email{David.Meyer@R-project.org}\\

Kurt Hornik\\
E-mail: \email{Kurt.Hornik@R-project.org}\\
}

\begin{document}

%\VignetteIndexEntry{Residual-Based Shadings in vcd}
%\VignetteDepends{vcd,colorspace,MASS,grid,HSAUR3,grid}
%\VignetteKeywords{association plots, conditional inference, contingency tables, HCL colors, HSV colors, mosaic plots}
%\VignettePackage{vcd}

\SweaveOpts{engine=R,eps=FALSE}

\section{Introduction} \label{sec:intro}

In this vignette, we show how all empirical examples from \cite{vcd:Zeileis+Meyer+Hornik:2007}
can be reproduced in \proglang{R}\citep[\mbox{\url{http://www.R-project.org/}}]{vcd:R:2006},
in particular using the package \pkg{vcd} \citep{vcd:Meyer+Zeileis+Hornik:2006}. Additionally,
the pakcages \pkg{MASS} \citep[see][]{vcd:Venables+Ripley:2002},
\pkg{grid} \citep[see][]{vcd:Murrell:2002} and \pkg{colorspace} \citep{vcd:Ihaka:2004}
are employed. All are automatically loaded together with \pkg{vcd}:

<<preliminaries, results=hide>>=
library("grid")
library("vcd")
rseed <- 1071
@

Furthermore, we define a \code{rseed} which will be used as the random seed for making
the results of the permutation tests (conditional inference) below exactly reproducible.
In the following, we focus on the \proglang{R} code and output---for background information
on the methods and the data sets, please consult \cite{vcd:Zeileis+Meyer+Hornik:2007}.

\section{Arthritis data} \label{sec:arthritis}

First, we take a look at the association of treatment type and improvement in the
\code{Arthritis} data. The data set can be loaded and brought into tabular form
via:

<<Arthritis-data>>=
data("Arthritis", package = "vcd")
(art <- xtabs(~ Treatment + Improved, data = Arthritis, subset = Sex == "Female"))
@

Two basic explorative views of such a 2-way table are mosaic plots and association plots.
They can be generated via \code{mosaic()} and \code{assoc()} from \pkg{vcd}, respectively.
For technical documentation of these functions, please see \cite{vcd:Meyer+Zeileis+Hornik:2006b}.
When no further arguments are supplied as in

<<Arthritis-classic, eval=FALSE>>=
mosaic(art)
assoc(art)
@

this yields the plain plots without any color shading, see Figure~\ref{fig:classic}.
Both indicate that there are more patients in the treatment group with marked improvement
and less without improvement than would be expected under independence---and vice versa
in the placebo group.

\setkeys{Gin}{width=\textwidth}
\begin{figure}[b!]
\begin{center}
<<Arthritis-classic1, fig=TRUE, echo=FALSE, height=6, width=11, results=hide>>=
grid.newpage()
pushViewport(viewport(layout = grid.layout(1, 2)))
pushViewport(viewport(layout.pos.col=1, layout.pos.row=1))
mosaic(art, newpage = FALSE, margins = c(2.5, 4, 2.5, 3))
popViewport()
pushViewport(viewport(layout.pos.col=2, layout.pos.row=1))
assoc(art, newpage = FALSE, margins = c(5, 2, 5, 4))
popViewport(2)
@
\caption{Classic mosaic and association plot for the arthritis data.}
\label{fig:classic}
\end{center}
\end{figure}

For 2-way tables, \cite{vcd:Zeileis+Meyer+Hornik:2007} suggest to extend the
shading of \cite{vcd:Friendly:1994} to also visualize the outcome of an independence
test---either using the sum of squares of the Pearson residuals as the test statistic
or their absolute maximum. Both statistics and their corresponding (approximate) permutation
distribution can easily be computed using the function \code{coindep_test()}. Its arguments
are a contingency table, a specification of margins used for conditioning (only for conditional
independence models), a functional for aggregating the Pearson residuals (or alternatively the
raw counts) and the number of permutations that should be drawn. The conditional table needs
to be a 2-way table and the default is to compute the maximum statistic (absolute maximum of
Pearson residuals). For the Arthritis data, both, the maximum test

<<Arthritis-max>>=
set.seed(rseed)
(art_max <- coindep_test(art, n = 5000))
@

and the sum-of-squares test, indicate a significant departure from independence. 

<<Arthritis-sumsq>>=
ss <- function(x) sum(x^2)
set.seed(rseed)
coindep_test(art, n = 5000, indepfun = ss)
@

Thus, it can be concluded that the treatment is effective and leads to significantly more
improvement than the placebo. The classic views from Figure~\ref{fig:classic} and the inference
above can also be combined, e.g., using the maximum shading that highlights the cells in
an association or mosaic plot when the associated residuals exceed critical values of the
maximum test (by default at levels 90\% and 99\%). To compare this shading (using either HSV
or HCL colors) with the Friendly shading (using HSV colors), we generate all three versions
of the mosaic plot:

<<Arthritis-extended, eval=FALSE>>=
mosaic(art, gp = shading_Friendly(lty = 1, eps = NULL))
mosaic(art, gp = shading_hsv, gp_args = list(
  interpolate = art_max$qdist(c(0.9, 0.99)), p.value = art_max$p.value))
set.seed(rseed)
mosaic(art, gp = shading_max, gp_args = list(n = 5000))
@

the  results are shown in the upper row of Figure~\ref{fig:shadings}. The last plot could
hae also been generated analogously to the second plot using \code{shading_hcl()} instead of
\code{shading_hsv()}---\code{shading_max()} is simply a wrapper function which performs the
inference and then visualizes it based on HCL colors.


\section{Piston rings data} \label{sec:arthritis}

Instead of bringing out the result of the maximum test in the shading, we could also use
a sum-of-squares shading that visualizes the result of the sum-of-squares test. As an illustration,
we use the \code{pistonrings} data from the \code{HSAUR3} \citep{vcd:Everitt+Hothorn:2006}
package giving the number of piston ring failurs in different legs of different compressors
at an industry plant:

<<pistonrings-data>>=
data("pistonrings", package = "HSAUR3")
pistonrings
@

\begin{sidewaysfigure}[p]
\begin{center}
<<shadings,fig=TRUE,echo=FALSE,height=12,width=18,results=hide,include=FALSE>>=
mymar <- c(1.5, 0.5, 0.5, 2.5)
grid.newpage()
pushViewport(viewport(layout = grid.layout(2, 3)))
pushViewport(viewport(layout.pos.row = 1, layout.pos.col = 1))
mosaic(art, margins = mymar, newpage = FALSE,
  gp = shading_Friendly(lty = 1, eps = NULL))
popViewport()
pushViewport(viewport(layout.pos.row = 1, layout.pos.col = 2))
mosaic(art, gp = shading_hsv, margins = mymar, newpage = FALSE,
  gp_args = list(interpolate = art_max$qdist(c(0.9, 0.99)), p.value = art_max$p.value))
popViewport()
pushViewport(viewport(layout.pos.row = 1, layout.pos.col = 3))
set.seed(rseed)
mosaic(art, gp = shading_max, margins = mymar, newpage = FALSE,
  gp_args = list(n = 5000))
popViewport()

pushViewport(viewport(layout.pos.row = 2, layout.pos.col = 1))
mosaic(pistonrings, margins = mymar, newpage = FALSE,
  gp = shading_Friendly(lty = 1, eps = NULL, interpolate = c(1, 1.5)))
popViewport()
pushViewport(viewport(layout.pos.row = 2, layout.pos.col = 2))
mosaic(pistonrings, gp = shading_hsv, margins = mymar, newpage = FALSE,
  gp_args = list(p.value = 0.069, interpolate = c(1, 1.5)))
popViewport()
pushViewport(viewport(layout.pos.row = 2, layout.pos.col = 3))
mosaic(pistonrings, gp = shading_hcl, margins = mymar, newpage = FALSE,
  gp_args = list(p.value = 0.069, interpolate = c(1, 1.5)))
popViewport(2)
@
\includegraphics[width=.9\textwidth,keepaspectratio]{residual-shadings-shadings}
\caption{Upper row: Mosaic plot for the arthritis data with Friendly shading (left),
HSV maximum shading (middle), HCL maximum shading (right). Lower row: Mosaic
plot for the piston rings data with fixed user-defined cut offs 1 and 1.5 and 
Friendly shading (left), HSV sum-of-squares shading (middle), HCL sum-of-squares shading
(right).}
\label{fig:shadings}
\end{center}
\end{sidewaysfigure}

Although there seems to be some slight association between the leg (especially center and South)
and the compressor (especially numbers 1 and 4), there is no significant deviation from
independence:

<<pistonrings-inference>>=
set.seed(rseed)
coindep_test(pistonrings, n = 5000)
set.seed(rseed)
(pring_ss <- coindep_test(pistonrings, n = 5000, indepfun = ss))
@

This can also be brought out graphically in a shaded mosaicplot by enhancing the Friendly
shading (based on the user-defined cut-offs 1 and 1.5, here) to use a less colorful palette,
either based on HSV or HCL colors:

<<pistonrings-plot, eval=FALSE>>=
mosaic(pistonrings, gp = shading_Friendly(lty = 1, eps = NULL, interpolate = c(1, 1.5)))
mosaic(pistonrings, gp = shading_hsv, gp_args = list(p.value = pring_ss$p.value, interpolate = c(1, 1.5)))
mosaic(pistonrings, gp = shading_hcl, gp_args = list(p.value = pring_ss$p.value, interpolate = c(1, 1.5)))
@

The resulting plots can be found in the lower row of Figure~\ref{fig:shadings}. The default
in \code{shading_hcl()} and \code{shading_hsv()} is to use the asymptotical $p$~value, hence
we set it explicitely to the permtuation-based $p$~value computed above.



\section{Alzheimer and smoking} \label{sec:alzheimer}

For illustrating that the same ideas can be employed for visualizing (conditional)
independence in multi-way tables, \cite{vcd:Zeileis+Meyer+Hornik:2007} use a
3-way and a 4-way table. The former is taken from a case-control study of smoking
and {A}lzheimer's disease (stratified by gender). The data set is available in \proglang{R}
in the package \pkg{coin} \cite{vcd:Hothorn+Hornik+VanDeWiel:2006}.

<<alzheimer-data>>=
data("alzheimer", package = "coin")
alz <- xtabs(~ smoking + disease + gender, data = alzheimer)
alz
@

\begin{figure}[b!]
\begin{center}
<<alzheimer-plot1, fig=TRUE, echo=FALSE, height=5, width=9>>=
set.seed(rseed)
cotabplot(~ smoking + disease | gender, data = alz, panel = cotab_coindep, panel_args = list(n = 5000))
@
\caption{Conditional mosaic plot with double maximum shading for conditional
independence of smoking and disease given gender.}
\label{fig:alz}
\end{center}
\end{figure}

To assess whether smoking behaviour and disease status are conditionally independent
given gender, \cite{vcd:Zeileis+Meyer+Hornik:2007} use three different types of test
statistics: double maximum (maximum of maximum statistics in the two strata), 
maximum sum of squares (maximum of sum-of-squares statistics), and sum of squares
(sum of sum-of-squares statistics). All three can be computed and assessed via
permutation methods using the function \code{coindep_test()}:

<<alzheimer-inference>>=
set.seed(rseed)
coindep_test(alz, 3, n = 5000)
set.seed(rseed)
coindep_test(alz, 3, n = 5000, indepfun = ss)
set.seed(rseed)
coindep_test(alz, 3, n = 5000, indepfun = ss, aggfun = sum)
@

The conditional mosaic plot in Figure~\ref{fig:alz}
shows clearly that the association of smoking and disease is present only in
the group of male patients. The double maximum shading employed allows for
identification of the male heavy smokers as the cells `responsible' for the dependence:
other dementias are more frequent and Alzheimer's disease less frequent in this group than expected
under independence. Interestingly, there seems to be another large residual for
the light smoker group ($<$10 cigarettes) and Alzheimer's disease---however, this
is only significant at 10\% and not at the 1\% level as the other two cells.

<<alzheimer-plot, eval=FALSE>>=
<<alzheimer-plot1>>
@


\section{Corporal punishment of children}

As a 4-way example, data from a study of the Gallup Institute in Denmark in 1979 about
the attitude of a random sample of 1,456 persons towards corporal
punishment of children is used. The contingency table comprises four margins:
memory of punishments as a child (yes/no), attitude as a binary variable
(approval of ``moderate'' punishment or ``no'' approval), highest level
of education (elementary/secondary/high), and age group (15--24, 25--39, $\ge$40 years).

<<Punishment-data>>=
data("Punishment", package = "vcd")
pun <- xtabs(Freq ~ memory + attitude + age + education, data = Punishment)
ftable(pun, row.vars = c("age", "education", "memory"))
@

It is of interest whether there is an association between memories of corporal punishments
as a child and attitude towards punishment of children as an adult, controlling
for age and education. All three test statistics already used above confirm that 
memories and attitude are conditionally associated:

\setkeys{Gin}{width=\textwidth}
\begin{figure}[t!]
\begin{center}
<<Punishment-assoc1,fig=TRUE,echo=FALSE,height=11,width=11>>=
set.seed(rseed)
cotabplot(~ memory + attitude | age + education, data = pun, panel = cotab_coindep,
  n = 5000, type = "assoc", test = "maxchisq", interpolate = 1:2)
@
\caption{Conditional association plot with maximum sum-of-squares shading for conditional
independence of memory and attitude given age and education.}
\label{fig:pun}
\end{center}
\end{figure}

\setkeys{Gin}{width=\textwidth}
\begin{figure}[t!]
\begin{center}
<<Punishment-mosaic1,fig=TRUE,echo=FALSE,height=11,width=11>>=
set.seed(rseed)
cotabplot(~ memory + attitude | age + education, data = pun, panel = cotab_coindep,
  n = 5000, type = "mosaic", test = "maxchisq", interpolate = 1:2)
@
\caption{Conditional mosaic plot with maximum sum-of-squares shading for conditional
independence of memory and attitude given age and education.}
\label{fig:pun2}
\end{center}
\end{figure}

<<Punishment-inference>>=
set.seed(rseed)
coindep_test(pun, 3:4, n = 5000)
set.seed(rseed)
coindep_test(pun, 3:4, n = 5000, indepfun = ss)
set.seed(rseed)
coindep_test(pun, 3:4, n = 5000, indepfun = ss, aggfun = sum)
@

Graphically, this dependence can be brought out using conditional association
or mosaic plots as shown in Figure~\ref{fig:pun} and \ref{fig:pun2}, respectively.
Both reveal an association between memories and attitude
for the lowest education group (first column) and highest age group (last row):
experienced violence seems to engender violence again as there are less adults
that disapprove punishment in the group with memories of punishments than expected
under independence. For the remaining four age-education groups, there seems to be
no association: all residuals of the conditional independence model are very close to
zero in these cells. The figures employ the maximum sum-of-squares shading
with user-defined cut offs 1 and 2, chosen to be within the range of the residuals.
The full-color palette is used only for those strata associated with a
sum-of-squares statistic significant at (overall) 5\% level, the reduced-color
palette is used otherwise. This highlights that the dependence pattern is significant
only for the middle and high age group in the low education column. The other panels
in the first column and last row also show a similar dependence pattern, however, it is not
significant at 5\% level and hence graphically down-weighted by using reduced color.

<<Punishment-assoc, eval=FALSE>>=
<<Punishment-assoc1>>
@
<<Punishment-mosaic, eval=FALSE>>=
<<Punishment-mosaic1>>
@


\bibliography{vcd}

\end{document}

Basic principles of SCL (Scene Configuration Library)
-----------------------------

SCL provides some routines and functions, but it is also a guide to produce scenes coherently.
Read carefully these principles, and see them in practice on the samples.

SCL is based on common conventions. In some cases, SCL allows using different conventions,
but eventually it enforces using specific conventions, such as 'right-hand' coordinate systems,
homogeneous coordinates, right-multiplication of matrices, alias (vs alibi) rotation matrices, etc.

Numpy Arrays
-----------------------------
All geometric data is expressed as numpy n-dimensional arrays.

Homogeneous Data
-----------------------------
All geometric data (points, lines) are expressed in homogeneous coordinates (columns)
All matrices are also expressed in homogeneous coordinates.
E.g. 3D points are column vectors \(4x1\)
        camera calibration matrix (K) is \(3x4\) matrix, where last row is all zeros and 1.0
        all poses are expressed as \(4x4\) matrices
        rotation matrices are \(3x3\)
        If not specified otherwise, all one-dimensional vectors are column vectors
NOTE: Since OpenLABEL 1.0.0, quaternion + translation is also added as an option to express poses.

Internally, SCL converts this form into \(4x4\) matrices

Poses and Coordinates systems
-----------------------------
Each coordinate system may have a pose with respect to a parent coordinate system.
Examples of this are sensors which are installed in vehicles. The Sensor Coordinate System (SCS)
then has a pose with respect to the Local Coordinate System (LCS) of the vehicle.
In SCL this LCS is a point at the ego-vehicle located at the middle of the rear axle, projected
to the ground, being X-to-front, Y-to-left, Z-up as defined in the ISO8855.

There are some other usual acronyms used in the code:

    - LCS : Local Coordinate System (e.g. the vehicle coordinate system at the rear axis,
            as in ISO8855)
    - SCS : Sensor Coordinate System (e.g. the camera coordinate system, or the lidar
            coordinate system)
    - WCS : World Coordinate System (static coordinate system for the entire scene, typically
            equal to the first LCS)
    - GCS : Geographic Coordinate System (UTM Universal Transverse Mercator)
    - ICS : Image coordinate system (this is the 2D image plane)

NOTE: Nevertheless, in VCD and SCL, an entirely customizable graph of dependencies can be
built, declaring coordinate systems and parent-child relations. See vcd.core functions
add_coordinate_system() where static poses can be defined, and add_transform()which is a
function to add frame-specific poses between coordinate systems.

For readability, let's use the following letter conventions:
    - P : Pose \(4x4\)
    - T : Transform \(4x4\)
    - R : Rotation matrix \(3x3\)
    - C : Position of coordinate system wrt to another \(3x1\)
    - K : Camera calibration matrix \(3x4\)
    - d : Distortion coefficients \(1x5\) (or \(1x9\), or \(1x14\))

All Poses are defined right-handed. A Pose encodes the passive rotation and position of a
coordinate system wrt (with respect to) a reference system.

Usually, P_scs_wrt_lcs = [[R_3x3, C_3x1], [0, 0, 0, 1]], where R_3x3 is a 3x3 rotation matrix,
and C_3x1 is the position of the SCS expressed in the LCS.

To actually convert a point from the reference system (e.g. LCS) into another system (e.g. SCS),
the transformation matrix is built as the inverse of the pose
https://en.wikipedia.org/wiki/Active_and_passive_transformation

Cameras coordinate systems are defined with X-to-right, Y-to-bottom, and Z-to-front, following
usual OpenCV camera model convention.
This is the common practice in computer vision, so that image coordinates are defined
x-to-right, y-to-bottom.

Transforming a 3D point expressed in a given coordinate system into another coordinate system
is carried out using right-to-left matrix multiplication:
e.g. X_scs = T_lcs_to_scs @ X_lcs (e.g. X_lcs is 4x1, Transform_lcs_to_scs is 4x4, X_scs is 4x1)
        X_scs = T_lcs_to_scs @ X_lcs
NOTE: @ operator in Python (>3.5) is matrix multiplication, equivalent to Numpy's dot operator

In addition, transformations can be understood as inverse Poses.
e.g. if T_lcs_to_scs converts a point from LCS to SCS, then
        T_lcs_to_scs = np.linalg.inv(P_scs_wrt_lcs)  # or using utils.inv(P_scs_wrt_lcs)

Note that to build a Pose and Transform by knowing the passive rotation R and position C of
a coordinate system wrt to another, it is possible to do:

(pseudo-code)
P_scs_wrt_lcs = (R_scs_wrt_lcs C_scs_wrt_lcs; 0 0 0 1)
or
T_lcs_to_scs = (transpose(R_scs_wrt_lcs) transpose(-R_scs_wrt_lcs)C_scs_wrt_lcs; 0 0 0 1)
Note P = T^-1 and T = P^-1

Since conversion from one system to another is useful, the following equations hold true:
P_scs_wrt_lcs = (T_lcs_to_scs)^-1
P_scs_wrt_lcs = T_scs_to_lcs


Odometry
-----------------------------
The sensors of the setup can move through time (e.g. if onboarded into a moving vehicle or
drone), the motion of the set-up is defined by the odometry information.

As each sensor has its own capturing timestamp, not necessarily coincident with the timestamp of
an odometry entry, odometry information may be provided associated to a specific sensor frame
(this way, it is possible to locate globally sensing information from that sensor).
The library provides tools to interpolate odometry and associate odometry entries to specific
timestamps.

The SCL library is defined in a way it is possible to add odometry values from multiple set-ups
e.g. V2V, a second vehicle sends location information about itself and its sensors, along
     with detections.

Odometry is treated just like a normal frame-specific transform. Therefore, all previous
discussion applies.
See vcd's add_transform() function at core.py


Geo-coordinates
-----------------------------
Just like any other coordinate system, geo-coordinates can be specified in VCD and then are
supported in SCL.

# Contributing to VCD for Python

The VCD for Python source code is managed using Git and is hosted on GitLab.

Clone using SSH (provided you have configured a SSH key in Gitlab):
```bash
git clone git@gitlab.com:vicomtech/v4/libraries/vcd/vcd-python.git
```
Clone using HTTPS (you will need to type your GitLab username and password):

```bash
git clone https://gitlab.com/vicomtech/v4/libraries/vcd/vcd-python.git
```

## Reporting Bugs and Asking Questions

If you think you have encountered a bug in VCD for Python or have an idea for a new feature? Great! We like to hear from you!

There are several options to participate:

- Send us an email to the project's official Service Desk email address: contact-project+vicomtech-v4-libraries-vcd-vcd-python-35661666-issue-@incoming.gitlab.com. Tell us your ideas or ask your questions.
- Look into our GitLab [issues](https://gitlab.com/vicomtech/v4/libraries/vcd/vcd-python/-/issues) tracker or open a new issue.

## Prerequisites
Before you make changes to the code, we would highly appreciate if you consider the following general requirements:

- Make sure your code adheres to the [Semantic Versioning](https://semver.org/) specification.
- To warranty the best code quality, the VCD for Python project has been configured to use different formatting, linting and code checks to try to reduce as much many common mistakes. To enable this tools please install the development requirements:

    ```bash
    pip install -r requirements-dev.txt
    ```
This will install [pre-commit](https://pre-commit.com/) package (among others) which needs to be activated in the VCD root folder:

```bash
pre-commit install
```

## Modifying the Code

### Repository Structure:
The repository uses two permanent branches:

- _master_: This branch allocates the released stable versions.
- _develop_: This branch contains the development code that will be integrated in future releases.

### Adding features:
We recommend the following workflow:

1. Clone the repository locally.
2. Checkout the _develop_ branch.
3. (Optional): Create a new temporal branch that must start by: _feature/_ or _hotfix/_. e.g. _feature/my-feature_.
4. Add all changes to the code and corresponding tests.
5. Check all [tests run](#running-the-test-suite) correctly.
5. Add change notes using the CHANGELOG [procedure](#adding-a-changelog-entry)
6. When finished, commit the changes to git.
7. Create a merge request (MR) to develop.
8. When the MR is accepted and integrated, if a branch was created, remove it.

## Running the Test Suite
We use [pytest](https://docs.pytest.org/) and [tox](https://tox.wiki/) to run tests against all supported Python versions. All test dependencies are resolved automatically.

We **HIGHLY** encourage you to add tests for all your new implemented features and run the tests if you have changed any functionality before submitting your work.

You can decide to run the complete test suite or only part of it:

- To run all tests, use:

    ```bash
    tox
    ```

    If you have not all Python interpreters installed on your system it will probably give you some errors (```InterpreterNotFound```). To avoid such errors, use:

    ```bash
    tox --skip-missing-interpreters
    ```

    It is possible to use one or more specific Python versions. Use the -e option and one or more abbreviations (py37 for Python 3.7, py38 for Python 3.8 etc.):

    ```bash
    tox -e py37
    tox -e py37,py38
    ```

    To get a complete list and a short description, run:

    ```bash
    tox -av
    ```

## Run the Code Analysis
This code is checked against formatting, style, type, and docstring issues ([black](), [flake8](), [pylint](), [mypy](), and [docformatter]()). It is recommended to run your tests in combination with ```pre-commit``` checks, for example:

```bash
pre-commit run --all
```

## Documenting VCD for Python
Documenting the features of VCD is very important. It gives our developers an overview what is possible with VCD, how it "feels", and how it is used efficiently.

### Building the documentation

To build the documentation locally use the following command:

$ tox -e docs

The built documentation is available in [docs/pdoc/vcd](.docs/pdoc/vcd/index.html).

A new feature is not complete if it isn't properly documented. A good documentation includes:

- A **docstring**

    Each docstring contains a summary line in imperative mode ending in a period, a linebreak, an optional directive, the description of its arguments, returns and raises in [Google style](https://google.github.io/styleguide/pyguide.html#38-comments-and-docstrings). The docstring is extracted and reused by [pdoc](https://pdoc3.github.io/pdoc/) to extract the API documentation.

    An appropriate docstring should look like this:

    ```python
    def sum(a: int, b: int) -> int:
        """
        Sum two int values:

        This function takes the two int values and sum its values.

        e.g.
        ```
        val_1 = 30
        val_2 = 45
        result = sum(val_1, val_2)
        ```

        Args:
            a (int): First attribute.
            b (int): Second attribute.

        Returns:
            (int): The result of the sum
        """
    ```

- **Type Annotations**

    To help the developers to know the use of the classes and functions. It is recommended to type annotate all the functions.

    The VCD library is designed to follow [PEP 604](https://peps.python.org/pep-0604/). Therefore the type annotations can be written as ```X | Y```.

## Adding a Changelog Entry

If you make any change to the VCD library it is mandatory to add an entry in the library CHANGELOG. To do this we have enabled a mechanism to add distributed changes using the tool [towncrier](https://github.com/twisted/towncrier).

This tools allows to create small files in the folder [.changelog.d](./.changelog.d) than can be committed and pushed to git to be shared by other developers. Then when a new release is prepared the maintainer of the Library can use towncrier to automatically collect and build the updated CHANGELOG.md file

For detailed instructions on how to add entries to the Changelog check [here](changelog.d/README.md).

\name{grid_barplot}
\alias{grid_barplot}
\title{Barplot}
\description{
  Bar plots of 1-way tables in grid.
}
\usage{
grid_barplot(height, width = 0.8, offset = 0,
  names = NULL, xlim = NULL, ylim = NULL, xlab = "", ylab = "", main = "",
  gp = gpar(fill = "lightgray"), name = "grid_barplot",
  newpage = TRUE, pop = FALSE, return_grob = FALSE)

}
\arguments{
  \item{height}{either a vector or a 1-way table of frequencies.}
  \item{width}{width of the bars (recycled if needed to the number of bars).}
  \item{offset}{offset of the bars (recycled if needed to the number of bars).}
  \item{names}{a vector of names for the bars, if set
    to \code{NULL} the names of \code{height} are used.}
  \item{xlim}{limits for the x axis.}
  \item{ylim}{limits for the y axis.}
  \item{xlab}{a label for the x axis.}
  \item{ylab}{a label for the y axis.}
  \item{main}{a title for the plot.}
  \item{gp}{a \code{"gpar"} object controlling the grid graphical
    parameters of the rectangles.}
  \item{name}{name of the plotting viewport.}
  \item{newpage}{logical. Should \code{\link{grid.newpage}} be called 
    before plotting?}
  \item{pop}{logical. Should the viewport created be popped?}
  \item{return_grob}{logical. Shall the plot be returned as a grob object?}
}
\details{
  \code{grid_barplot} mimics (some of) the features of \code{\link{barplot}},
  but currently it only supports 1-way tables.
}
\author{
  Achim Zeileis \email{Achim.Zeileis@R-project.org}
}
\examples{
grid_barplot(sample(1:6), names = letters[1:6])
}
\keyword{hplot}

\name{Bundesliga}
\alias{Bundesliga}
\title{Ergebnisse der Fussball-Bundesliga}
\description{
  Results from the first German soccer league (1963-2008).
}
\usage{
data("Bundesliga")
}
\format{A data frame with 14018 observations and 7 variables.
  \describe{
    \item{HomeTeam}{factor. Name of the home team.}
    \item{AwayTeam}{factor. Name of the away team.}
    \item{HomeGoals}{number of goals scored by the home team.}
    \item{AwayGoals}{number of goals scored by the away team.}
    \item{Round}{round of the game.}
    \item{Year}{year in which the season started.}
    \item{Date}{starting time of the game (in \code{"POSIXct"} format).}
  }
}
\details{
  The data comprises all games in the first German soccer league since
  its foundation in 1963. The data have been queried online from the 
  official Web page of the DFB and prepared as a data frame in R by
  Daniel Dekic, Torsten Hothorn, and Achim Zeileis (replacing earlier versions of the
  data in the package containing only subsets of years).
  
  Each year/season comprises 34 rounds (except 1963, 1964, 1991) so that
  all 18 teams play twice against each other (switching home court advantage).
  In 1963/64, there were only 16 teams, hence only 30 rounds.
  In 1991, after the German unification, there was one season with 20 teams
  and 38 rounds.
}
\source{
  Homepage of the Deutscher Fussball-Bund (DFB, German Football Association):
  \url{https://www.dfb.de/index/}
}
\references{
Leonhard Knorr-Held (1999), Dynamic rating of sports teams.
SFB 386 \dQuote{Statistical Analysis of Discrete Structures},
Discussion paper \bold{98}.
}
\seealso{
  \code{\link{UKSoccer}}
}
\examples{
data("Bundesliga")

## number of goals per game poisson distributed?
ngoals1 <- xtabs(~ HomeGoals, data = Bundesliga, subset = Year == 1995)
ngoals2 <- xtabs(~ AwayGoals, data = Bundesliga, subset = Year == 1995)
ngoals3 <- table(apply(subset(Bundesliga, Year == 1995)[,3:4], 1, sum))

gf1 <- goodfit(ngoals1)
gf2 <- goodfit(ngoals2)
gf3 <- goodfit(ngoals3)

summary(gf1)
summary(gf2)
summary(gf3)
plot(gf1)
plot(gf2)
plot(gf3)

Ord_plot(ngoals1)
distplot(ngoals1)
}

\keyword{datasets}

\name{BrokenMarriage}
\alias{BrokenMarriage}
\docType{data}
\title{Broken Marriage Data}
\description{
  Data from the Danish Welfare Study about broken marriages or permanent
  relationships depending on gender and social rank.
}
\usage{
data("BrokenMarriage")
}
\format{
  A data frame with 20 observations and 4 variables.
  \describe{
    \item{Freq}{frequency.}
    \item{gender}{factor indicating gender (male, female).}
    \item{rank}{factor indicating social rank (I, II, III, IV, V).}
    \item{broken}{factor indicating whether the marriage or permanent
      relationship was broken (yes, no).}
  }
}
\references{
  E. B. Andersen (1991),
  \emph{The Statistical Analysis of Categorical Data}.
  2nd edition.
  Springer-Verlag, Berlin.
}
\source{
  E. B. Andersen (1991),
  The Statistical Analysis of Categorical Data, page 177.
}
\examples{
data("BrokenMarriage")
structable(~ ., data = BrokenMarriage)
}

\keyword{datasets}

\name{loddsratio}
\alias{loddsratio}
\alias{oddsratio}
\alias{loddsratio.default}
\alias{loddsratio.formula}

\alias{coef.loddsratio}
\alias{confint.loddsratio}
\alias{dim.loddsratio}
\alias{dimnames.loddsratio}
\alias{print.loddsratio}
\alias{vcov.loddsratio}
\alias{as.matrix.loddsratio}
\alias{as.array.loddsratio}
\alias{aperm.loddsratio}
\alias{t.loddsratio}
\alias{as.data.frame.loddsratio}

\title{
Calculate Generalized Log Odds Ratios for Frequency Tables
}
\description{
Computes (log) odds ratios and their asymptotic variance covariance
matrix for R x C (x strata) tables.
Odds ratios are calculated for two array dimensions, separately for each level of
all stratifying dimensions. See Friendly et al. (2011) for a sketch of a
general theory.
}
\usage{

loddsratio(x, \dots)
\method{loddsratio}{default}(x, strata = NULL, log = TRUE, 
  ref = NULL, correct = any(x == 0L), \dots)

\method{loddsratio}{formula}(formula, data = NULL, \dots,
subset = NULL, na.action = NULL)

oddsratio(x, stratum = NULL, log = TRUE)

\method{coef}{loddsratio}(object, log = object$log, \dots)
\method{vcov}{loddsratio}(object, log = object$log, \dots)
\method{print}{loddsratio}(x, log = x$log, \dots)
\method{confint}{loddsratio}(object, parm, level = 0.95, log = object$log, \dots)

%as.array(x, \dots)
\method{as.array}{loddsratio}(x, log=x$log, \dots)
\method{t}{loddsratio}(x)
\method{aperm}{loddsratio}(a, perm, \dots)
}

%- maybe also 'usage' for other objects documented here.
\arguments{

  \item{x}{an object. For the default method a k-way matrix/table/array
    of frequencies. The number of margins has to be at least 2.}
  \item{strata, stratum}{Numeric or character indicating the margins of a
    $k$-way table \code{x} (with $k$ greater than 2) that should be employed
    as strata. By default all dimensions except the first two are used.}
  \item{ref}{numeric or character. Reference categories for the (non-stratum)
    row and column dimensions that should be employed for computing the
    odds ratios. By default, odds ratios for profile contrasts (or sequential
    contrasts, i.e., successive differences of adjacent categories) are
    used. See details below.}
  \item{formula}{a formula specifying the variables used to create a 
    contingency table from \code{data}. A conditioning
    formula can be specified; the conditioning variables will then be
    used as strata variables.}
  \item{data}{either a data frame, or an object of class \code{"table"}
    or \code{"ftable"}.}
  \item{subset}{an optional vector specifying a subset of observations to be used.}
  \item{na.action}{a function which indicates what should happen when
    the data contain \code{NA}s. Ignored if \code{data} is a contingency table.}
  \item{log}{logical. Should the results be displayed on a log scale
    or not? All internal computations are always on the log-scale but the
    results are transformed by default if \code{log = TRUE}.}
  \item{correct}{logical or numeric. Should a continuity correction
    be applied before computing odds ratios?
    If \code{TRUE}, 0.5 is  added to  all cells; if numeric (or an
    array conforming to the data) that value is added to all cells.  By default,
    this not  employed unless there are any zero cells in  the table, but this
    correction is often recommended to reduce bias when some frequencies are small
    (Fleiss, 1981).}
  \item{a, object}{an object of class \code{loddsratio} as computed by
    \code{loddsratio}.}
  \item{perm}{numeric or character vector specifying a permutation of strata.}
  \item{\dots}{arguments passed to methods.}
  
  \item{parm}{a specification of which parameters are to be given confidence intervals, 
    either a vector of numbers or a vector of names. If missing, all parameters are considered.}
  
  \item{level}{the confidence level required for the \code{confint} method.}

}
\details{
For an R x C table, (log) odds ratios are formed for the set of (R-1) x (C-1)
2 x 2 tables, corresponding to some set of contrasts among the row and column
variables.  The \code{ref} argument allows these to be specified in a general
way.

\code{ref = NULL} (default) corresponds to \dQuote{profile contrasts} 
(or sequential contrasts or successive differences) for ordered categories,
i.e., R1--R2, R2--R3, R3--R4, etc., and similarly for the column categories.
These are sometimes called \dQuote{local odds ratios}.

\code{ref = 1} gives contrasts with the first category; \code{ref = dim(x)}
gives contrasts with the last category; \code{ref = c(2, 4)} or \code{ref = list(2, 4)}
corresponds to the reference being the second category in rows and
the fourth in columns.

Combinations like \code{ref = list(NULL, 3)} are also possible, as are character
vectors, e.g., \code{ref = c("foo", "bar")} also works ("foo" pertaining again to the
    row reference and "bar" to column reference).

Note that all such parameterizations are equivalent, in that one can derive all
other possible odds ratios from any non-redundant set, but the interpretation
of these values depends on the parameterization.

Note also that these reference level parameterizations only have meaning when the
primary (non-strata) table dimensions are larger than 2x2.  In the 2x2 case,
the odds ratios are defined by the order of levels of those variables in the table,
so you can achieve a desired interpretation by manipulating the table.

See the help page of \code{\link{plot.loddsratio}} for visualization methods.
}
\value{
An object of class \code{loddsratio}, with the following components: 
\item{coefficients}{A named vector, of length (R-1) x (C-1) x \code{prod(dim(x)[strata])}
	containing the log odds ratios. Use the \code{coef} method to 
	extract these from the object, and the \code{confint} method for confidence intervals.
	For a two-way table, the names for the log odds ratios are constructed in the form
	Ri:Rj/Ci:Cj using the table names for rows and columns. For a stratified table,
	the names are constructed in the form Ri:Rj/Ci:Cj|Lk.
	}
\item{vcov}{Variance covariance matrix of the log odds ratios.}
\item{dimnames}{Dimension names for the log odds ratios, considered as a table of
  size (R-1, C-1, \code{dim(x)[strata]}). Use the \code{dim} and \code{dimnames} methods
  to extract these and manipulate the log odds ratios in relation to the original table.}
\item{dim}{Corresponding dimension vector.}
\item{contrasts}{A matrix C, such that \code{C \%*\% as.vector(log(x))} gives the log odds
  ratios. Each row corresponds to one log odds ratio, and is all zero, except for 4 elements
  of \code{c(1, -1, -1, 1)} for a given 2 x 2 subtable.}
\item{log}{A logical, indicating the value of \code{log} in the original call.}
}

\references{
A. Agresti (2013), \emph{Categorical Data Analysis}, 3rd Ed. New York: Wiley.

Fleiss, J. L. (1981). \emph{Statistical Methods for Rates and Proportions}.
2nd Edition. New York: Wiley.

M. Friendly (2000), \emph{Visualizing Categorical Data}. SAS Institute, Cary, NC.

Friendly, M., Turner, H,, Firth, D., Zeileis, A. (2011).
\emph{Advances in Visualizing Categorical Data Using the vcd, gnm and vcdExtra Packages in R}.
Correspondence Analysis and Related Methods (CARME 2011).
\url{http://www.datavis.ca/papers/adv-vcd-4up.pdf}

}
\author{
Achim Zeileis, Michael Friendly and David Meyer.
}
\note{
The method of calculation is an example of the use of the delta method
described by Agresti (2013), Section 16.1.6, giving estimates of
log odds ratios and their asymptotic covariance matrix.

The \code{coef} method returns the \code{coefficients} component as a vector
of length (R-1) x (C-1) x \code{prod(dim(x)[strata])}.
The \code{dim} and \code{dimnames} methods provide the proper attributes for
treating the \code{coefficients} vector as an (R-1) x (C-1) x strata array.
\code{as.matrix} and \code{as.array} methods are also provided for this purpose.

The \code{confint} method computes confidence intervals for the log odds ratios
(or for odds ratios, with \code{log = FALSE}).
The \code{\link[lmtest]{coeftest}} method (\code{summary} is an alias)
prints the asymptotic standard errors, z tests (standardized log odds
ratios), and the corresponding p values.

\emph{Structural zeros}: In addition to the options for zero cells provided by \code{correct},
the function allows for structural zeros to be represented as \code{NA} in the data argument.
\code{NA} in the data yields \code{NA} as the \code{LOR} estimate, but does not affect other
cells.

\code{oddsratio} is just an alias to \code{loddsratio} for backward
compatibility.

}

%% ~Make other sections like Warning with \section{Warning }{....} ~

\seealso{
%%\code{\link[vcd]{oddsratio}},
\code{\link{plot.loddsratio}} for some plotting methods;
\code{\link[stats]{confint}} for confidence intervals;
\code{\link[lmtest]{coeftest}} for z-tests of significance

}
\examples{
## artificial example
set.seed(1)
x <- matrix(rpois(5 * 3, 7), ncol = 5, nrow = 3)
dimnames(x) <- list(Row = head(letters, 3), Col = tail(letters, 5))

x_lor <- loddsratio(x)
coef(x_lor)
x_lor
confint(x_lor)
summary(x_lor)

## 2 x 2 x k cases
#data(CoalMiners, package = "vcd")
lor_CM <- loddsratio(CoalMiners)
lor_CM
coef(lor_CM)
confint(lor_CM)
confint(lor_CM, log = FALSE)

## 2 x k x 2
lor_Emp <-loddsratio(Employment)
lor_Emp
confint(lor_Emp)

## 4 way tables 
data(Punishment, package = "vcd")
lor_pun <- loddsratio(Freq ~ memory + attitude | age + education, data = Punishment)
lor_pun
confint(lor_pun)
summary(lor_pun)

# fit linear model using WLS
lor_pun_df <- as.data.frame(lor_pun)
pun_mod1 <- lm(LOR ~ as.numeric(age) * as.numeric(education),
               data = lor_pun_df, weights = 1 / ASE^2)
anova(pun_mod1)

## illustrate ref levels
VA.fem <- xtabs(Freq ~ left + right, subset=gender=="female", data=VisualAcuity)
VA.fem
loddsratio(VA.fem)                  # profile contrasts
loddsratio(VA.fem, ref=1)           # contrasts against level 1
loddsratio(VA.fem, ref=dim(VA.fem)) # contrasts against level 4


}
% Add one or more standard keywords, see file 'KEYWORDS' in the
% R documentation directory.
\keyword{category}
%\keyword{ ~kwd2 }% __ONLY ONE__ keyword per line

\name{agreementplot}
\alias{agreementplot}
\alias{agreementplot.default}
\alias{agreementplot.formula}
\title{Bangdiwala's Observer Agreement Chart}
\description{
  Representation of a \eqn{k \times k}{k by k} confusion matrix,
  where the observed and expected diagonal elements are represented by
  superposed black and white rectangles, respectively.  The function
  also computes a statistic measuring the strength of agreement
  (relation of respective area sums).
}
\usage{
\method{agreementplot}{default}(x, reverse_y = TRUE, main = NULL,
              weights = c(1, 1 - 1/(ncol(x) - 1)^2), margins = par("mar"),
              newpage = TRUE, pop = TRUE, 
              xlab = names(dimnames(x))[2],
              ylab = names(dimnames(x))[1],
              xlab_rot = 0, xlab_just = "center",
              ylab_rot = 90, ylab_just = "center",
              fill_col = function(j) gray((1 - (weights[j]) ^ 2) ^ 0.5),
              line_col = "red", xscale = TRUE, yscale = TRUE,
              return_grob = FALSE,
              prefix = "", \dots)
\method{agreementplot}{formula}(formula, data = NULL, ..., subset)
}
\arguments{
  \item{x}{a confusion matrix, i.e., a table with equal-sized dimensions.}
  \item{reverse_y}{if \code{TRUE}, the y axis is reversed (i.e., the
    rectangles' positions correspond to the contingency table).}
  \item{main}{user-specified main title.}
  \item{weights}{vector of weights for successive larger observed areas,
    used in the agreement strength statistic, and also for the
    shading.  The first element should be 1.}
  \item{margins}{vector of margins (see \code{\link[graphics]{par}}).}
  \item{newpage}{logical; if \code{TRUE}, the plot is drawn on a new page.}
  \item{pop}{logical; if \code{TRUE}, all newly generated viewports are popped after plotting.}
  \item{return_grob}{logical. Should a snapshot of the display be
    returned as a grid grob?}
  \item{xlab, ylab}{labels of x- and y-axis.}
  \item{xlab_rot, ylab_rot}{rotation angle for the category labels.}
  \item{xlab_just, ylab_just}{justification for the category labels.}
  \item{fill_col}{a function, giving the fill colors used for exact and partial agreement}
  \item{line_col}{color used for the diagonal reference line}
  \item{formula}{a formula, such as \code{y ~ x}.
    For details, see \code{\link{xtabs}}.}
  \item{data}{a data frame (or list), or a contingency table from which
    the variables in \code{formula} should be taken.}
  \item{subset}{an optional vector specifying a subset of the rows in
    the data frame to be used for plotting.}
  \item{xscale, yscale}{logicals indicating whether the marginals should
  be added on the x-axis/y-axis, respectively.}
  \item{prefix}{character string used as prefix for the viewport name}
  \item{\dots}{further graphics parameters (see \code{\link{par}}).}
}
\details{
  Weights can be specified to allow for partial agreement, taking into
  account contributions from off-diagonal cells.  Partial agreement
  is typically represented in the display by lighter shading, as given by
  \code{fill_col(j)}, corresponding to \code{weights[j]}.
  
  A weight vector of
  length 1 means strict agreement only, each additional element
  increases the maximum number of disagreement steps.

  \code{\link{cotabplot}} can be used for stratified analyses (see examples).
}
\value{
  Invisibly returned, a list with components
  \item{Bangdiwala}{the unweighted agreement strength statistic.}
  \item{Bangdiwala_Weighted}{the weighted statistic.}
  \item{weights}{the weight vector used.}
}
\references{
Bangdiwala, S. I. (1988). The Agreement Chart.  Department of Biostatistics,
University of North Carolina at Chapel Hill,
Institute of Statistics Mimeo Series No. 1859,
\url{https://repository.lib.ncsu.edu/bitstream/handle/1840.4/3827/ISMS_1988_1859.pdf}

Bangdiwala, S. I.,  Ana S. Haedo, Marcela L. Natal, and Andres
Villaveces. The agreement chart as an alternative to the
receiver-operating characteristic curve for diagnostic tests. 
\emph{Journal of Clinical Epidemiology}, 61 (9), 866-874.

Michael Friendly (2000),
  \emph{Visualizing Categorical Data}.
  SAS Institute, Cary, NC.
}
\author{
  David Meyer \email{David.Meyer@R-project.org}
}

\examples{
data("SexualFun")
agreementplot(t(SexualFun))

data("MSPatients")
\dontrun{
## best visualized using a resized device, e.g. using:
## get(getOption("device"))(width = 12)
pushViewport(viewport(layout = grid.layout(ncol = 2)))
pushViewport(viewport(layout.pos.col = 1))
agreementplot(t(MSPatients[,,1]), main = "Winnipeg Patients",
              newpage = FALSE)
popViewport()
pushViewport(viewport(layout.pos.col = 2))
agreementplot(t(MSPatients[,,2]), main = "New Orleans Patients",
              newpage = FALSE)
popViewport(2)
dev.off()
}

## alternatively, use cotabplot:
cotabplot(MSPatients, panel = cotab_agreementplot)
}
\keyword{category}
\keyword{hplot}

\name{rootogram}
\alias{rootogram}
\alias{rootogram.default}
\alias{rootogram.goodfit}
\title{Rootograms}
\description{
  Rootograms of observed and fitted values.
}
\usage{
\method{rootogram}{default}(x, fitted, names = NULL, scale = c("sqrt", "raw"),
  type = c("hanging", "standing", "deviation"),
  shade = FALSE, legend = TRUE,
  legend_args = list(x = 0, y = 0.2, height = 0.6), df = NULL,
  rect_gp = NULL, rect_gp_args = list(),
  lines_gp = gpar(col = "red", lwd = 2),
  points_gp = gpar(col = "red"), pch = 19,
  xlab = NULL, ylab = NULL, ylim = NULL,
  main = NULL, sub = NULL,
  margins = unit(0, "lines"),
  title_margins = NULL, legend_width = NULL,
  main_gp = gpar(fontsize = 20),
  sub_gp = gpar(fontsize = 15),
  name = "rootogram", prefix = "",
  keep_aspect_ratio = FALSE,
  newpage = TRUE, pop = TRUE,
  return_grob = FALSE, \dots)
}
\arguments{
  \item{x}{either a vector or a 1-way table of frequencies.}
  \item{fitted}{a vector of fitted frequencies.}
  \item{names}{a vector of names passed to \code{\link{grid_barplot}}, if set
    to \code{NULL} the names of \code{x} are used.}
  \item{scale}{a character string indicating whether the values should be
    plotted on the raw or square root scale.}
  \item{type}{a character string indicating if the bars for the observed
    frequencies should be \code{hanging} or \code{standing} or indicate the \code{deviation}
    between observed and fitted frequencies.}

  \item{shade}{logical specifying whether \code{rect_gp} should be set
    to colors corresponding to the pearson residuals, i.e., if a
    residual-based shading should be applied to the bars.}
  
  \item{legend}{either a legend-generating function, or a legend
    function (see details and \code{\link{legends}}), or a logical.
    If \code{legend} is \code{NULL} or \code{TRUE} and \code{gp} is a
    function, legend defaults to \code{\link{legend_resbased}}.}
  \item{legend_args}{list of arguments for the legend-generating
    function, if specified.}
  \item{df}{degrees of freedom passed to the shading functions
    used for inference.}
  
  \item{rect_gp}{a \code{"gpar"} object controlling the grid graphical
    parameters of the rectangles, shading function or a
    corresponding generating function (see \code{\link{shadings}}). If
    unspecified and no shading is applied, defaults to light grey fill color for the bars.}
  \item{rect_gp_args}{list of arguments for the shading-generating
    function, if specified for \code{rect_gp}.}
  \item{lines_gp}{a \code{"gpar"} object controlling the grid graphical
    parameters of the lines.}
  \item{points_gp}{a \code{"gpar"} object controlling the grid graphical
    parameters of the points.}
  \item{pch}{plotting character for the points.}
  \item{xlab}{a label for the x axis.}
  \item{ylab}{a label for the y axis.}
  \item{ylim}{limits for the y axis.}
  \item{main}{either a logical, or a character string used for plotting
    the main title.  If \code{main} is a logical and \code{TRUE}, the
    name of the object supplied as \code{x} is used.}
  \item{sub}{a character string used for plotting the subtitle.
    If \code{sub} is a logical and \code{TRUE} and \code{main} is unspecified, the
    name of the object supplied as \code{x} is used.}
  \item{margins}{either an object of class \code{"unit"} of length 4, or
    a numeric vector of length 4. The elements are recycled as needed.
    The four components specify the top, right,
    bottom, and left margin of the plot, respectively.
    When a numeric vector is supplied, the numbers are interpreted as
    \code{"lines"} units.  In addition, the unit or numeric vector
    may have named arguments
    (\samp{top}, \samp{right}, \samp{bottom}, and \samp{left}), in which
    case the non-named arguments specify the default values (recycled as
    needed), overloaded by the named arguments.}
  \item{title_margins}{either an object of class \code{"unit"} of length 2, or
    a numeric vector of length 2. The elements are recycled as needed.
    The two components specify the top and bottom \emph{title} margin
    of the plot, respectively. The default for each
    \emph{specified} title are 2 lines (and 0 else), except when a
    legend is plotted and \code{keep_aspect_ratio} is \code{TRUE}: in
    this case, the default values of both margins are set as to align
    the heights of legend and actual plot.
    When a numeric vector is supplied, the numbers are interpreted as
    \code{"lines"} units.  In addition, the unit or numeric vector
    may have named arguments (\samp{top} and \samp{bottom}), in which
    case the non-named argument specify the default value (recycled as
    needed), overloaded by the named arguments.}
  \item{legend_width}{An object of class \code{"unit"} of length
    1 specifying the width of the legend (if any). Default: 5 lines.}
  \item{main_gp, sub_gp}{object of class \code{"gpar"} containing the graphical parameters used for the main (sub) title, if specified.}

  \item{name}{name of the plotting viewport.}
  \item{keep_aspect_ratio}{logical indicating whether the aspect ratio should be fixed or not.}
  \item{prefix}{optional character string used as a prefix for the
    generated viewport and grob names.}
  \item{newpage}{logical. Should \code{\link{grid.newpage}} be called 
    before plotting?}
  \item{pop}{logical. Should the viewport created be popped?}
  \item{return_grob}{logical. Should a snapshot of the display be
    returned as a grid grob?}
  \item{\dots}{further arguments passed to \code{\link{grid_barplot}}.}
}
\details{
  The observed frequencies are displayed as bars and the fitted
  frequencies as a line.  By default a sqrt scale is used to make the
  smaller frequencies more visible.
}
\author{
  Achim Zeileis \email{Achim.Zeileis@R-project.org},
  David Meyer \email{David.Meyer@R-project.org}
}
\references{
  J. W. Tukey (1977),
  \emph{Exploratory Data Analysis}.
  Addison Wesley, Reading, MA.

  M. Friendly (2000),
  \emph{Visualizing Categorical Data}.
  SAS Institute, Cary, NC.
}
\seealso{\code{\link{grid_barplot}}}
\examples{
## Simulated data examples:
dummy <- rnbinom(200, size = 1.5, prob = 0.8)
observed <- table(dummy)
fitted1 <- dnbinom(as.numeric(names(observed)),
                   size = 1.5, prob = 0.8) * sum(observed)
fitted2 <- dnbinom(as.numeric(names(observed)),
                   size = 2, prob = 0.6) * sum(observed)
rootogram(observed, fitted1)
rootogram(observed, fitted2)

## Real data examples:
data("HorseKicks")
HK.fit <- goodfit(HorseKicks)
summary(HK.fit)
plot(HK.fit)
## or equivalently
rootogram(HK.fit)

data("Federalist")
F.fit <- goodfit(Federalist, type = "nbinomial")
summary(F.fit)
plot(F.fit)

## (Pearson) residual-based shading
data("Federalist")
Fed_fit0 <- goodfit(Federalist, type = "poisson")
plot(Fed_fit0, shade = TRUE)
}
\keyword{hplot}

\name{Ord_plot}
\alias{Ord_plot}
\alias{Ord_estimate}
\title{Ord Plots}
\description{
  Ord plots for diagnosing discrete distributions.
}
\usage{
Ord_plot(obj, legend = TRUE, estimate = TRUE, tol = 0.1, type = NULL,
  xlim = NULL, ylim = NULL, xlab = "Number of occurrences",
  ylab = "Frequency ratio", main = "Ord plot", gp = gpar(cex = 0.5),
  lwd = c(2,2), lty=c(2,1), col=c("black", "red"),
  name = "Ord_plot", newpage = TRUE, pop = TRUE,
  return_grob = FALSE, \dots)
Ord_estimate(x, type = NULL, tol = 0.1)
}
\arguments{
  \item{obj}{either a vector of counts, a 1-way table of frequencies
    of counts or a data frame or matrix with frequencies
    in the first column and the corresponding counts in the
    second column.}
  \item{legend}{logical. Should a legend be plotted?.}
  \item{estimate}{logical. Should the distribution and its parameters
    be estimated from the data? See details.}
  \item{tol}{tolerance for estimating the distribution. See details.}
  \item{type}{a character string indicating the distribution, must be
    one of \code{"poisson"}, \code{"binomial"}, \code{"nbinomial"}
    or \code{"log-series"} or \code{NULL}. In the latter case the
    distribution is estimated from the data. See details.}
  \item{xlim}{limits for the x axis.}
  \item{ylim}{limits for the y axis.}
  \item{xlab}{a label for the x axis.}
  \item{ylab}{a label for the y axis.}
  \item{main}{a title for the plot.}
  \item{gp}{a \code{"gpar"} object controlling the grid graphical
    parameters of the points.}
  \item{lwd, lty}{vectors of length 2, giving the line width and line type
  used for drawing the OLS line and the WLS lines.} 
  \item{col}{vector of length 2 giving the colors used for drawing the OLS
  and WLS lines.}
  \item{name}{name of the plotting viewport.}
  \item{newpage}{logical. Should \code{\link{grid.newpage}} be called 
    before plotting?}
  \item{pop}{logical. Should the viewport created be popped?}
  \item{return_grob}{logical. Should a snapshot of the display be
    returned as a grid grob?}
  \item{\dots}{further arguments passed to \code{\link{grid.points}}.}
  \item{x}{a vector giving intercept and slope for the (fitted)
    line in the Ord plot.}

}
\details{
  The Ord plot plots the number of occurrences against a certain
  frequency ratio (see Friendly (2000) for details) and should give a
  straight line if the data comes from a poisson, binomial, negative
  binomial or log-series distribution.  The intercept and slope of this
  straight line conveys information about the underlying distribution.

  \code{Ord_plot} fits a usual OLS line (black) and a weighted OLS line
  (red).  From the coefficients of the latter the distribution is
  estimated by \code{Ord_estimate} as described in Table 2.10 in
  Friendly (2000).  To judge whether a coefficient is positive or
  negative a tolerance given by \code{tol} is used.  If none of the
  distributions fits well, no parameters are estimated.  Be careful with
  the conclusions from \code{Ord_estimate} as it implements just some
  simple heuristics!
}
\value{
  A vector giving the intercept and slope of the weighted OLS line.
}
\author{
  Achim Zeileis \email{Achim.Zeileis@R-project.org}
}
\references{
  J. K. Ord (1967),
  Graphical methods for a class of discrete distributions,
  \emph{Journal of the Royal Statistical Society}, \bold{A 130},
  232--238.

  Michael Friendly (2000),
  \emph{Visualizing Categorical Data}.
  SAS Institute, Cary, NC.
}
\examples{
## Simulated data examples:
dummy <- rnbinom(1000, size = 1.5, prob = 0.8)
Ord_plot(dummy)

## Real data examples:
data("HorseKicks")
data("Federalist")
data("Butterfly")
data("WomenQueue")

\dontrun{
grid.newpage()
pushViewport(viewport(layout = grid.layout(2, 2)))

pushViewport(viewport(layout.pos.col=1, layout.pos.row=1))
Ord_plot(HorseKicks, main = "Death by horse kicks", newpage = FALSE)
popViewport()

pushViewport(viewport(layout.pos.col=1, layout.pos.row=2))
Ord_plot(Federalist, main = "Instances of 'may' in Federalist papers", newpage = FALSE)
popViewport()

pushViewport(viewport(layout.pos.col=2, layout.pos.row=1))
Ord_plot(Butterfly, main = "Butterfly species collected in Malaya", newpage = FALSE)
popViewport()

pushViewport(viewport(layout.pos.col=2, layout.pos.row=2))
Ord_plot(WomenQueue, main = "Women in queues of length 10", newpage = FALSE)
popViewport(2)
}

## same
mplot(
  Ord_plot(HorseKicks, return_grob = TRUE, main = "Death by horse kicks"),
  Ord_plot(Federalist, return_grob = TRUE, main = "Instances of 'may' in Federalist papers"),
  Ord_plot(Butterfly, return_grob = TRUE, main = "Butterfly species collected in Malaya"),
  Ord_plot(WomenQueue, return_grob = TRUE, main = "Women in queues of length 10")
)
}
\keyword{category}

\name{Baseball}
\alias{Baseball}
\docType{data}
\title{Baseball Data}
\description{
  Baseball data.
}
\usage{
data("Baseball")
}
\format{
  A data frame with 322 observations and 25 variables.
  \describe{
    \item{name1}{player's first name.}
    \item{name2}{player's last name.}
    \item{atbat86}{times at Bat: number of official plate appearances by
      a hitter.  It counts as an official at-bat as long as the batter
      does not walk, sacrifice, get hit by a pitch or reach base due to
      catcher's interference.}
    \item{hits86}{hits.}
    \item{homer86}{home runs.}
    \item{runs86}{the number of runs scored by a player.  A run is
      scored by an offensive player who advances from batter to runner
      and touches first, second, third and home base in that order
      without being put out.} 
    \item{rbi86}{Runs Batted In: A hitter earns a run batted in when he
      drives in a run via a hit, walk, sacrifice (bunt or fly) fielder's
      choice, hit-batsman or on an error (when the official scorer rules
      that the run would have scored anyway).}
    \item{walks86}{A \dQuote{walk} (or \dQuote{base on balls}) is an
      award of first base granted to a batter who receives four pitches
      outside the strike zone.}
    \item{years}{Years in the Major Leagues.  Seems to count all years a
      player has actually played in the Major Leagues, not necessarily
      consecutive.}
    \item{atbat}{career times at bat.}
    \item{hits}{career hits.}
    \item{homeruns}{career home runs.}
    \item{runs}{career runs.}
    \item{rbi}{career runs batted in.}
    \item{walks}{career walks.}
    \item{league86}{player's league.}
    \item{div86}{player's division.}
    \item{team86}{player's team.}
    \item{posit86}{player's position (see \code{\link{Hitters}}).}
    \item{outs86}{number of putouts (see \code{\link{Hitters}})}
    \item{assist86}{number of assists (see \code{\link{Hitters}})}
    \item{error86}{number of assists (see \code{\link{Hitters}})}
    \item{sal87}{annual salary on opening day (in USD 1000).}
    \item{league87}{league in 1987.}
    \item{team87}{team in 1987.}
  }
}

\references{
  M. Friendly (2000),
  \emph{Visualizing Categorical Data}.
  SAS Institute, Cary, NC.
}

\source{
  SAS System for Statistical Graphics, First Edition, page A2.3
}

\seealso{\code{\link{Hitters}}}

\examples{
data("Baseball")
}
\keyword{datasets}

\name{struc_sieve}
\alias{struc_sieve}
\title{Core-generating Function for Sieve Plots}
\encoding{UTF-8}
\description{
  Core-generating function for \code{strucplot} returning a function
  producing sieve plots.
}
\usage{
struc_sieve(sievetype = c("observed","expected"), gp_tile = gpar(),
scale = 1)
}
\arguments{
  \item{sievetype}{logical indicating whether rectangles should be filled
    according to \code{observed} or \code{expected} frequencies.}
  \item{gp_tile}{object of class \code{"gpar"}, controlling the
    appearance of all \emph{static}
    elements of the cells (e.g., border and fill color).}
  \item{scale}{Scaling factor for the sieve.}
}
\details{
  This function is usually called by \code{\link{strucplot}} (typically
  when called by \code{\link{sieve}}) and returns a function used by
  \code{\link{strucplot}} to produce sieve plots. 
}
\value{
  A function with arguments:
  \item{residuals}{table of residuals.}
  \item{observed}{table of observed values.}
  \item{expected}{not used by \code{struc_sieve}.}
  \item{spacing}{object of class \code{"unit"} specifying the space
    between the tiles.}
  \item{gp}{list of \code{gpar} objects used for the drawing the tiles.}
  \item{split_vertical}{vector of logicals indicating, for each
    dimension of the table, the split direction.}
}
\author{
  David Meyer \email{David.Meyer@R-project.org}
}
\seealso{
  \code{\link{sieve}},
  \code{\link{strucplot}},
  \code{\link{structable}}
}
\references{
  Riedwyl, H., and Schüpbach, M. (1994),
  Parquet diagram to plot contingency tables.
  In F. Faulbaum (ed.),
  \emph{Softstat '93: Advances in Statistical Software}, 293--299.
  Gustav Fischer, New York.

  Friendly, M. (2000),
  Visualizing Categorical Data,
  SAS Institute, Cary, NC.

  Meyer, D., Zeileis, A., and Hornik, K. (2006),
  The strucplot framework: Visualizing multi-way contingency tables with
  \pkg{vcd}.
  \emph{Journal of Statistical Software}, \bold{17(3)}, 1-48.
  \doi{10.18637/jss.v017.i03} and available as 
  \code{vignette("strucplot")}.

}
\examples{
## Titanic data
data("Titanic")
strucplot(Titanic, core = struc_sieve)
}
\keyword{hplot}

\name{Pairs plot panel functions for diagonal cells}
\alias{pairs_barplot}
\alias{pairs_text}
\alias{pairs_diagonal_text}
\alias{pairs_diagonal_mosaic}
\title{Diagonal Panel Functions for Table Pairs Plot}
\description{
  Diagonal panel functions for \code{\link{pairs.table}}.
}
\usage{
pairs_barplot(gp_bars = NULL,
  gp_vartext = gpar(fontsize = 17),
  gp_leveltext = gpar(),
  gp_axis = gpar(),
  just_leveltext = c("center", "bottom"),
  just_vartext = c("center", "top"),
  rot = 0, abbreviate = FALSE, check_overlap = TRUE, fill = "grey",
  var_offset = unit(1, "npc"), \dots)
pairs_text(dimnames = TRUE, gp_vartext = gpar(fontsize = 17), 
  gp_leveltext = gpar(), gp_border = gpar(), \dots) 
pairs_diagonal_text(varnames = TRUE, gp_vartext = gpar(fontsize = 17, fontface = "bold"),
  gp_leveltext = gpar(), gp_border = gpar(), pos = c("right","top"),
  distribute = c("equal","margin"), rot = 0, \dots)
pairs_diagonal_mosaic(split_vertical = TRUE, margins = unit(0, "lines"),
  offset_labels = -0.4, offset_varnames = 0,
  gp = NULL, fill = "grey", labeling = labeling_values, alternate_labels = TRUE, ...)
}
\arguments{
  \item{dimnames}{vector of logicals indicating whether the factor
    levels should be displayed (only used for \code{pairs_text}).}
  \item{varnames}{vector of logicals indicating whether the variable
    names should be displayed (only used for \code{pairs_text_diagonal}).}
  \item{gp_bars}{object of class \code{"gpar"} used for bars (only used
    for \code{pairs_barplot}). If unspecified, the default is to set the \code{fill}
    component of this object to the \code{fill} argument.}
  \item{gp_vartext}{object of class \code{"gpar"} used for the factor
    names.}
  \item{gp_leveltext}{object of class \code{"gpar"} used for the factor
    levels.}
    \item{gp_axis}{object of class \code{"gpar"} used for the y axis.}
  \item{gp_border}{object of class \code{"gpar"} used for the border
    (only used for \code{pairs_text}).}
  \item{gp}{object of class \code{"gpar"} used for the tiles (only used
    for \code{pairs_diagonal_mosaic}).
    If unspecified, the default is to set the \code{fill}
    component of this object to the \code{fill} argument.}
  \item{fill}{color vector or palette function used for the fill colors
    of bars (for \code{pairs_barplot}) or tiles (for
    \code{pairs_diagonal_mosaic}).}
  \item{labeling}{labeling function, passed to \code{mosaic()}}
  \item{alternate_labels}{should labels alternate top/bottom?}
  \item{just_leveltext, just_vartext}{character string indicating the
    justification for variable names and levels.}
  \item{pos}{character string of length 2 controlling the
    horizontal and vertical position of the variable names
    (only used for \code{pairs_text_diagonal}).}
  \item{rot}{rotation angle for the variable levels.}
  \item{distribute}{character string indicating whether levels should be
     distributed equally or according to the margins
     (only used for \code{pairs_text_diagonal}).}
  \item{abbreviate}{integer or logical indicating
    the number of characters the labels should be abbreviated
    to. \code{TRUE} means 1 character, \code{FALSE} causes no
    abbreviation.}
  \item{check_overlap}{If \code{TRUE}, some levels will suppressed to
    avoid overlapping, if any.}
  \item{split_vertical}{vector of logicals of length \eqn{k}, where
    \eqn{k} is the number of margins of \code{x} (values are recycled as needed).
    A \code{TRUE} component indicates that the tile(s) of the
    corresponding dimension should be split vertically, \code{FALSE}
    means horizontal splits. Default is \code{FALSE.}}
  \item{margins}{either an object of class \code{"unit"} of length 4, or
    a numeric vector of length 4. The elements are recycled as needed.
    The four components specify the top, right,
    bottom, and left margin of the plot, respectively.
    When a numeric vector is supplied, the numbers are interpreted as
    \code{"lines"} units.  In addition, the unit or numeric vector
    may have named arguments
    (\samp{top}, \samp{right}, \samp{bottom}, and \samp{left}), in which
    case the non-named arguments specify the default values (recycled as
    needed), overloaded by the named arguments.}
  \item{offset_labels, offset_varnames}{numeric vector of length 4
    indicating the offset of the
    labels (variable names) for each of the four sides of the plot.}
  \item{var_offset}{object of class \code{"unit"} specifying the
        offset of variable names from the bottom of the bar plots
        created by \code{pairs_barplot}. If numeric, the unit defaults to "npc".}
  \item{\dots}{other parameters passed to the underlying graphics
    functions.}
}
\details{
  In the diagonal cells, the pairsplot visualizes statistics or
  information for each dimension (that is: the single factors) alone.
  \code{\link{pairs_text}} displays the factor's name, and optionally
  also the factor levels. \code{\link{pairs_barplot}} produces a bar plot
  of the corresponding factor, along with the factor's name.
}
\value{
  A function with one argument: the marginal table for the corresponding
  dimension.
}
\seealso{
  \code{\link{pairs.table}},
  \code{\link{pairs_assoc}},
  \code{\link{pairs_mosaic}}
}
\author{
  David Meyer \email{David.Meyer@R-project.org}
}
\examples{
data("UCBAdmissions")

pairs(UCBAdmissions) # pairs_barplot is default
pairs(UCBAdmissions, diag_panel = pairs_text)

pairs(UCBAdmissions, diag_panel = pairs_diagonal_text)
pairs(Titanic, diag_panel = pairs_diagonal_text)
pairs(Titanic, diag_panel = pairs_diagonal_text(distribute = "margin"))
pairs(Titanic,
      diag_panel = pairs_diagonal_text(distribute = "margin", rot = 45))
}
\keyword{hplot}

\name{JointSports}
\alias{JointSports}
\docType{data}
\title{Opinions About Joint Sports}
\description{
  Data from a Danish study in 1983 and 1985 about sports activities and
  the opinion about joint sports with the other gender among 16--19 year
  old high school students.
}
\usage{
data("JointSports")
}
\format{
  A data frame with 40 observations and 5 variables.
  \describe{
    \item{Freq}{frequency.}
    \item{opinion}{factor indicating opinion about sports joint with the
      other gender (very good, good, indifferent, bad, very bad).}
    \item{year}{factor indicating year of study (1983, 1985).}
    \item{grade}{factor indicating school grade (1st, 3rd).}
    \item{gender}{factor indicating gender (Boy, Girl).}
  }
}

\references{
  E. B. Andersen (1991),
  \emph{The Statistical Analysis of Categorical Data}.
  2nd edition.
  Springer-Verlag, Berlin.
}

\source{
  E. B. Andersen (1991),
  The Statistical Analysis of Categorical Data, page 210.
}

\examples{
library(MASS)
data("JointSports")
tab <- xtabs(Freq ~ gender + opinion + grade + year, data = JointSports)
doubledecker(opinion ~ gender + year + grade, data = tab)
loglm(~ opinion* (gender + grade+ year) + gender*year*grade, data = tab)
}

\keyword{datasets}

\name{co_table}
\alias{co_table}
\title{Compute Conditional Tables}
\description{
  For a contingency table in array form, compute a list of conditional
  tables given some margins.
}
\usage{
co_table(x, margin, collapse = ".")
}
\arguments{
  \item{x}{a contingency table in array form.}
  \item{margin}{margin index(es) or corresponding name(s) of the
    conditioning variables.}
  \item{collapse}{character used when collapsing level names
    (if more than 1 \code{margin} is specified).}
}
\details{
  This is essentially an interface to \code{\link[base]{[}}
  which is more convenient for arrays of arbitrary dimension.   
}
\value{
  A list of the resulting conditional tables.
}
\author{
  Achim Zeileis \email{Achim.Zeileis@R-project.org}
}
\examples{
data("HairEyeColor")
co_table(HairEyeColor, 1)
co_table(HairEyeColor, c("Hair", "Eye"))
co_table(HairEyeColor, 1:2, collapse = "")
}
\keyword{array}

\name{strucplot}
\alias{strucplot}
\title{Structured Displays of Contingency Tables}
\description{
  This modular function visualizes certain aspects of high-dimensional
  contingency tables in a hierarchical way.
}
\usage{
strucplot(x, residuals = NULL, expected = NULL,
  condvars = NULL, shade = NULL, type = c("observed", "expected"),
  residuals_type = NULL, df = NULL, split_vertical = NULL, 
  spacing = spacing_equal, spacing_args = list(),
  gp = NULL, gp_args = list(),
  labeling = labeling_border, labeling_args = list(),
  core = struc_mosaic, core_args = list(),
  legend = NULL, legend_args = list(),
  main = NULL, sub = NULL, margins = unit(3, "lines"),
  title_margins = NULL, legend_width = NULL,
  main_gp = gpar(fontsize = 20), sub_gp = gpar(fontsize = 15),
  newpage = TRUE, pop = TRUE, return_grob = FALSE,
  keep_aspect_ratio = NULL, prefix = "", \dots)
}
\arguments{
  \item{x}{a contingency table in array form, with optional category
    labels specified in the \code{dimnames} attribute.}
  \item{residuals}{optionally, an array of residuals of the same dimension
    as \code{x} (see details).}
  \item{expected}{optionally, an array of expected values of the same dimension
    as \code{x}, or alternatively the corresponding independence model specification
    as used by \code{\link[stats]{loglin}} or \code{\link[MASS]{loglm}}
    (see details).}
  \item{df}{degrees of freedom passed to the shading functions
    used for inference. Will be calculated (and overwritten if
    specified) if both \code{expected} and
    \code{residuals} are \code{NULL}, or if \code{expected} is given a formula.}
  \item{condvars}{number of conditioning variables, if any; those are
    expected to be ordered first in the table.
    This information is used for computing the expected values, and is
    also passed to the spacing functions (see \code{\link{spacings}}).}
  \item{shade}{logical specifying whether \code{gp} should be used or not
    (see \code{gp}). If \code{TRUE} and \code{expected} is unspecified,
    a default model is fitted: if \code{condvars} is specified, a
    corresponding conditional independence model, and else the total
    independence model.}
  \item{residuals_type}{a character string indicating the type of
    residuals to be computed when none are supplied.
    If \code{residuals} is \code{NULL}, \code{residuals_type} must
    be one of \code{"pearson"} (default; giving components of Pearson's
    chi-squared), \code{"deviance"} (giving components of the likelihood
    ratio chi-squared), or \code{"FT"} for the Freeman-Tukey residuals.
    The value of this argument can be abbreviated. If \code{residuals}
    are specified, the value of \code{residuals_type} is just passed
    \dQuote{as is} to the legend function.}
  \item{type}{a character string indicating whether the observed or the
    expected values of the table should be visualized.}
  \item{split_vertical}{vector of logicals of length \eqn{k}, where
    \eqn{k} is the number of margins of \code{x} (values are recycled as needed).
    A \code{TRUE} component indicates that the tile(s) of the
    corresponding dimension should be split vertically, \code{FALSE}
    means horizontal splits. Default is \code{FALSE.}}
  \item{spacing}{spacing object, spacing function, or a corresponding
    generating function (see details and \code{\link{spacings}}).}
  \item{spacing_args}{list of arguments for the spacing-generating
    function, if specified.}
  \item{gp}{object of class \code{"gpar"}, shading function or a
    corresponding generating function (see details and
    \code{\link{shadings}}). Components of \code{"gpar"}
    objects are recycled as needed along the last splitting dimension.
    Ignored if \code{shade = FALSE}.} 
  \item{gp_args}{list of arguments for the shading-generating function,
    if specified.}
  \item{labeling}{either a logical, or a labeling function, or a corresponding
    generating function (see details and \code{\link{labelings}}. If
    \code{FALSE} or \code{NULL}, no labeling is produced.}
  \item{labeling_args}{list of arguments for the labeling-generating
    function, if specified.}
  \item{core}{either a core function, or a corresponding generating
    function (see details).  Currently, generating functions for
    mosaic plots (\code{\link{struc_mosaic}}), association plots
    (\code{\link{struc_assoc}}), and sieve plots
    (\code{\link{struc_sieve}}) are provided.}
  \item{core_args}{list of arguments for the core-generating function,
    if specified.}
  \item{legend}{either a legend-generating function, or a legend
    function (see details and \code{\link{legends}}), or a logical.
    If \code{legend} is \code{NULL} or \code{TRUE} and \code{gp} is a
    function, legend defaults to \code{\link{legend_resbased}}.}
  \item{legend_args}{list of arguments for the legend-generating
    function, if specified.}
  \item{main}{either a logical, or a character string used for plotting
    the main title.  If \code{main} is a logical and \code{TRUE}, the
    name of the object supplied as \code{x} is used.}
  \item{sub}{a character string used for plotting the subtitle.
    If \code{sub} is a logical and \code{TRUE} and \code{main} is unspecified, the
    name of the object supplied as \code{x} is used.}
  \item{margins}{either an object of class \code{"unit"} of length 4, or
    a numeric vector of length 4. The elements are recycled as needed.
    The four components specify the top, right,
    bottom, and left margin of the plot, respectively.
    When a numeric vector is supplied, the numbers are interpreted as
    \code{"lines"} units.  In addition, the unit or numeric vector
    may have named arguments
    (\samp{top}, \samp{right}, \samp{bottom}, and \samp{left}), in which
    case the non-named arguments specify the default values (recycled as
    needed), overloaded by the named arguments.}
  \item{title_margins}{either an object of class \code{"unit"} of length 2, or
    a numeric vector of length 2. The elements are recycled as needed.
    The two components specify the top and bottom \emph{title} margin
    of the plot, respectively. The default for each
    \emph{specified} title are 2 lines (and 0 else), except when a
    legend is plotted and \code{keep_aspect_ratio} is \code{TRUE}: in
    this case, the default values of both margins are set as to align
    the heights of legend and actual plot.
    When a numeric vector is supplied, the numbers are interpreted as
    \code{"lines"} units.  In addition, the unit or numeric vector
    may have named arguments (\samp{top} and \samp{bottom}), in which
    case the non-named argument specify the default value (recycled as
    needed), overloaded by the named arguments.}
  \item{legend_width}{An object of class \code{"unit"} of length
    1 specifying the width of the legend (if any). Default: 5 lines.}
  \item{pop}{logical indicating whether the generated viewport tree
    should be removed at the end of the drawing or not.}
  \item{main_gp, sub_gp}{object of class \code{"gpar"} containing the graphical
    parameters used for the main (sub) title, if specified.}
  \item{newpage}{logical indicating whether a new page should be created
    for the plot or not.}
  \item{return_grob}{logical. Should a snapshot of the display be
    returned as a grid grob?}
  \item{keep_aspect_ratio}{logical indicating whether the aspect ratio should be
    fixed or not. If unspecified, the default is \code{TRUE} for
    two-dimensional tables and \code{FALSE} otherwise.}
  \item{prefix}{optional character string used as a prefix for the
    generated viewport and grob names.}
  \item{\dots}{For convenience, list of arguments passed to the
    labeling-generating function used.}
}
\details{
  This function---usually called by higher-level functions such as
  \code{\link{assoc}} and \code{\link{mosaic}}---generates conditioning
  plots of contingency tables. First, it sets up a set of viewports for
  main- and subtitles, legend, and the actual plot region.  Then,
  residuals are computed as needed from observed and expected
  frequencies, where the expected frequencies are optionally computed
  for a specified independence model.  Finally, the specified functions
  for spacing, gp, main plot, legend, and labeling are called to produce
  the plot.  The function invisibly returns the \code{"structable"} object
  visualized.

  Most elements of the plot, such as the core function, the spacing
  between the tiles, the shading of the tiles, the labeling, and the
  legend, are modularized in graphical appearance control (``grapcon'')
  functions and specified as parameters.  For
  each element \emph{foo} (= \code{spacing}, \code{labeling}, \code{core},
  or \code{legend}), \code{strucplot} takes two arguments:
  \var{foo} and \var{foo_args}, which can be used to specify the
  parameters in the following alternative ways:

  \enumerate{
    \item Passing a suitable function to \var{foo} which subsequently
    will be called from \code{strucplot} to compute shadings, labelings,
    etc.
    \item Passing a corresponding \emph{generating} function to \var{foo},
    along with parameters passed to \var{foo_args}, that generates such a
    function. Generating functions must inherit from classes
    \code{"grapcon_generator"} and \code{"\var{foo}"}.
    
    \item Except for the shading functions (\var{shading_bar}),
    passing \var{foo(foo_args)} to the \var{foo} argument.
    \item For shadings and spacings, passing the final parameter object
    itself; see the corresponding help pages for more details on the
    data structures.
  }
  
  If legends are drawn, a \sQuote{cinemascope}-like layout is used for
  the plot to preserve the 1:1 aspect ratio.

  If \code{type = "expected"}, the expected values are passed to the
  \code{observed} argument of the core function, and the observed
  values to the \code{expected} argument.

  Although the \code{gp} argument is typically used for shading, it can
  be used for arbitrary modifications of the tiles' graphics parameters
  (e.g., for highlighting particular cells, etc.).

}
\note{
  The created viewports, as well as the tiles and bullets, are named and
  thus can conveniently be modified after a plot has been drawn (and
  \code{pop = FALSE}).
}

\value{
  Invisibly, an object of class \code{"structable"} corresponding to the
  plot. If \code{return_grob} is \code{TRUE}, additionally, the plot as
  a grob object is returned in a \code{grob} attribute.
}
\author{
  David Meyer \email{David.Meyer@R-project.org}
}

\references{
  Meyer, D., Zeileis, A., and Hornik, K. (2006),
  The strucplot framework: Visualizing multi-way contingency tables with
  \pkg{vcd}.
  \emph{Journal of Statistical Software}, \bold{17(3)}, 1-48.
  \doi{10.18637/jss.v017.i03} and available as 
  \code{vignette("strucplot")}.
}

\seealso{
  \code{\link{assoc}},
  \code{\link{mosaic}},
  \code{\link{sieve}},
  \code{\link{struc_assoc}},
  \code{\link{struc_sieve}},
  \code{\link{struc_mosaic}},
  \code{\link{structable}},
  \code{\link{doubledecker}},
  \code{\link{labelings}},
  \code{\link{shadings}},
  \code{\link{legends}},
  \code{\link{spacings}}
}
\examples{
data("Titanic")

strucplot(Titanic)
strucplot(Titanic, core = struc_assoc)
strucplot(Titanic, spacing = spacing_increase,
                   spacing_args = list(start = 0.5, rate = 1.5))
strucplot(Titanic, spacing = spacing_increase(start = 0.5, rate = 1.5))

## modify a tile's color
strucplot(Titanic, pop = FALSE)
grid.edit("rect:Class=1st,Sex=Male,Age=Adult,Survived=Yes",
          gp = gpar(fill = "red"))
}
\keyword{hplot}

\name{labeling_border}
\alias{labeling_border}
\alias{labeling_conditional}
\alias{labeling_left}
\alias{labeling_left2}
\alias{labeling_cboxed}
\alias{labeling_lboxed}
\alias{labeling_doubledecker}
\alias{labeling_values}
\alias{labeling_residuals}
\alias{labelings}
\title{Labeling Functions for Strucplots}
\description{
  These functions generate labeling functions used for strucplots.
}
\usage{
labeling_border(labels = TRUE, varnames = labels,
  set_labels = NULL, set_varnames = NULL, 
  tl_labels = NULL, alternate_labels = FALSE, tl_varnames = NULL, 
  gp_labels = gpar(fontsize = 12),
  gp_varnames = gpar(fontsize = 12, fontface = 2),
  rot_labels = c(0, 90, 0, 90), rot_varnames = c(0, 90, 0, 90),
  pos_labels = "center", pos_varnames = "center",
  just_labels = "center", just_varnames = pos_varnames,
  boxes = FALSE, fill_boxes = FALSE,
  offset_labels = c(0, 0, 0, 0), offset_varnames = offset_labels,
  labbl_varnames = NULL, labels_varnames = FALSE, sep = ": ",
  abbreviate_labs = FALSE, rep = TRUE, clip = FALSE, \dots)
labeling_values(value_type = c("observed", "expected", "residuals"),
                suppress = NULL, digits = 1, clip_cells = FALSE, \dots)
labeling_residuals(suppress = NULL, digits = 1, clip_cells = FALSE, \dots)
labeling_conditional(\dots)
labeling_left(rep = FALSE, pos_varnames = "left",
  pos_labels = "left", just_labels = "left", \dots)
labeling_left2(tl_labels = TRUE, clip = TRUE, pos_varnames = "left",
  pos_labels = "left", just_labels = "left", \dots)
labeling_cboxed(tl_labels = TRUE, boxes = TRUE, clip = TRUE,
  pos_labels = "center", \dots)
labeling_lboxed(tl_labels = FALSE, boxes = TRUE, clip = TRUE,
  pos_labels = "left", just_labels = "left",
  labbl_varnames = FALSE, \dots)
labeling_doubledecker(lab_pos = c("bottom", "top"), dep_varname = TRUE,
  boxes = NULL, clip = NULL, labbl_varnames = FALSE,
  rot_labels = rep.int(0, 4),
  pos_labels = c("left", "center", "left", "center"),
  just_labels = c("left", "left", "left", "center"),
  varnames = NULL, gp_varnames = gpar(fontsize = 12, fontface = 2),
  offset_varnames = c(0, -0.6, 0, 0), tl_labels = NULL, \dots)
}
\arguments{
  \item{labels}{vector of logicals indicating whether labels should be
    drawn for a particular dimension.}
  \item{varnames}{vector of logicals indicating whether variable names
    should be drawn for a particular dimension.}
  \item{set_labels}{An optional character vector with named components
    replacing the so-specified variable names. The component names must
    exactly match the variable names to be replaced.}
  \item{set_varnames}{An optional list with named components of
    character vectors replacing the labels of the so-specified
    variables. The component names must
    exactly match the variable names whose labels should be replaced.}
  \item{tl_labels}{vector of logicals indicating whether labels should
    be positioned on top (column labels) / left (row labels) for a
    particular dimension.}
  \item{alternate_labels}{vector of logicals indicating whether labels should
    be alternated on the top/bottom (left/right) side of the plot for a
    particular dimension.}
  \item{tl_varnames}{vector of logicals indicating whether variable
    names should be positioned on top (column labels) / on left (row
    labels) for a particular dimension.}
  \item{gp_labels}{list of objects of class \code{"gpar"} used for
    drawing the labels.}
  \item{gp_varnames}{list of objects of class \code{"gpar"} used for
    drawing the variable names.}
  \item{rot_labels}{vector of rotation angles for the labels for each of
    the four sides of the plot.}
  \item{rot_varnames}{vector of rotation angles for the variable names
    for each of the four sides of the plot.}
  \item{pos_labels}{character string of label positions (\code{"left"},
    \code{"center"}, \code{"right"}) for each of the variables.}
  \item{pos_varnames}{character string of variable names positions
    (\code{"left"}, \code{"center"}, \code{"right"}) for each of the
    four sides of the plot.}
  \item{just_labels}{character string of label justifications
    (\code{"left"}, \code{"center"}, \code{"right"}) for each of the
    variables.}
  \item{just_varnames}{character string of variable names justifications
    (\code{"left"}, \code{"center"}, \code{"right"}) for each of the
    four sides of the plot.}
  \item{boxes}{vector of logicals indicating whether boxes should be
    drawn around the labels for a particular dimension.}
  \item{fill_boxes}{Either a vector of logicals, or a vector of characters,
    or a list of such vectors, specifying the fill colors for the
    boxes. \code{"TRUE"} and \code{"FALSE"} values are
    transformed into \code{"grey"} and \code{"white"}, respectively. If
    \code{fill_boxes} is atomic, each component
    specifies a basic color for the corresponding dimension. This color
    is transformed into its HSV representation, and the value is varied
    from 50\% to 100\% to give a sequential color palette for the
    levels. For \code{NA} components, no palette is produced (no fill
    color). If \code{fill_boxes} is a list of vectors,
    each vector specifies the level colors of the corresponding
    dimension.}
  \item{offset_labels, offset_varnames}{numeric vector of length 4
    indicating the offset of the
    labels (variable names) for each of the four sides of the plot.}
  \item{labbl_varnames}{vector of logicals indicating whether variable
    names should be drawn on the left (column variables) / on top (row
    variables) of the corresponding labels.}
  \item{labels_varnames}{vector of logicals indicating, for each
    dimension, whether the variable name should be added to the
    corresponding labels or not.}
  \item{sep}{separator used if any component of \code{"labels_varnames"}
    is \code{TRUE}.}
  \item{abbreviate_labs}{vector of integers or logicals indicating, for each
    dimension, the number of characters the labels should be abbreviated
    to.
    \code{TRUE} means 1 character, \code{FALSE} causes no abbreviation.
    Values are recycled as needed.}
  \item{rep}{vector of logicals indicating, for each dimension, whether
    labels should be repeated for all conditioning strata, or appear
    only once.}
  \item{clip}{vector of integers indicating, for each dimension, whether
    labels should be clipped to not overlap.}
  \item{lab_pos}{character string switching between \code{"top"} or
    \code{"bottom"} position of the labels (only used for
    \code{labeling_doubledecker}).}
  \item{dep_varname}{logical or character string. If logical, this is
    indicating whether the name of the dependent variable should be
    printed or not. A character string will be printed instead of the
    variable name taken from the dimnames.}
  \item{value_type}{character string specifying which values should be
    displayed in the cells.}
  \item{suppress}{numeric vector of length 2 specifying an interval of
    values that are not displayed. 0 values are never displayed.
    A single number, \var{k}, is treated as \code{c(-\var{k},
      \var{k})}. The default for labeling residuals is
    \code{c(-2,2)}. Use \code{suppress = 0} to show all non-zero values.}
  \item{digits}{integer specifying the number of digits used for rounding.}
  \item{clip_cells}{logical indicating whether the values should be clipped at
    the cell borders.}
  \item{\dots}{only used for \code{labeling_conditional} and
    \code{labeling_doubledecker}: parameters
    passed to \code{labeling_cells} and \code{labeling_border}.}
}
\details{
  These functions generate labeling functions called by
  \code{\link{strucplot}} for their side-effect of adding labels to the
  plot. They suppose that a strucplot has been drawn and the
  corresponding viewport structure is pushed, since the positions of the
  viewports are used for the label positioning.
  Note that the functions can also be used \sQuote{stand-alone} as
  shown in the examples.
  
  All values supplied to vectorized arguments can be
  \sQuote{abbreviated} by using named components which override the
  default component values.  In addition, these defaults can be
  overloaded by the sequence of non-named components which are recycled
  as needed (see examples).

  This help page only documents \code{labeling_border} and
  derived functions, more functions are described on the help page
  for \code{\link{labeling_cells}} and \code{\link{labeling_list}}.

  \code{labeling_left}, \code{labeling_left2}, \code{labeling_cboxed},
  and \code{labeling_lboxed}
  are really just wrappers to \code{labeling_border}, and good examples for
  the parameter usage.

  \code{labeling_residuals} is a trivial wrapper for
  \code{labeling_values}, which in turn calls \code{labeling_border} by
  additionally adding the observed or expected frequencies or residuals
  to the cells.
}
\value{
  A function with arguments:
  \item{d}{\code{"dimnames"} attribute from the visualized contingency
    table, or the visualized table itself from which the
    \code{"dimnames"} attributes will then be extracted.}
  \item{split_vertical}{vector of logicals indicating the split
    directions.}
  \item{condvars}{integer vector of conditioning dimensions.}
}
\author{
  David Meyer \email{David.Meyer@R-project.org}
}
\references{
  Meyer, D., Zeileis, A., and Hornik, K. (2006),
  The strucplot framework: Visualizing multi-way contingency tables with
  \pkg{vcd}.
  \emph{Journal of Statistical Software}, \bold{17(3)}, 1-48.
  \doi{10.18637/jss.v017.i03} and available as 
  \code{vignette("strucplot")}.
}
  
\seealso{
  \code{\link{labeling_cells}}, 
  \code{\link{labeling_list}}, 
  \code{\link{structable}}, 
  \code{\link[grid]{grid.text}}
}
\examples{
data("Titanic")

mosaic(Titanic)

mosaic(Titanic, labeling = labeling_left)
labeling_left

mosaic(Titanic, labeling = labeling_cboxed)
labeling_cboxed

mosaic(Titanic, labeling = labeling_lboxed)
labeling_lboxed

data("PreSex")
mosaic(~ PremaritalSex + ExtramaritalSex | Gender + MaritalStatus,
  data = PreSex, labeling = labeling_conditional)

## specification of vectorized arguments 
mosaic(Titanic, labeling_args = list(abbreviate_labs = c(Survived = TRUE)))

mosaic(Titanic, labeling_args = list(clip = TRUE, boxes = TRUE,
  fill_boxes = c(Survived = "green", "red")))

mosaic(Titanic, labeling_args = list(clip = TRUE, boxes = TRUE,
  fill_boxes = list(Sex = "red", "green")))

mosaic(Titanic, labeling_args = list(clip = TRUE, boxes = TRUE,
  fill_boxes = list(Sex = c(Male = "red", "blue"), "green")))

## change variable names
mosaic(Titanic, labeling_args = list(set_varnames = c(Sex = "Gender")))

## change labels
mosaic(Titanic, labeling_args = list(set_varnames = c(Survived = "Status"),
  set_labels = list(Survived = c("Survived", "Not Survived")), rep = FALSE))

## show frequencies
mosaic(Titanic, labeling = labeling_values)
}
\keyword{hplot}

\name{fourfold}
\alias{fourfold}
\title{Fourfold Plots}
\description{
  Creates an (extended) fourfold display of a \eqn{2 \times 2 \times k}{2 x 2 x k}
  contingency table, allowing for the visual inspection of the association
  between two dichotomous variables in one or several populations (strata).
}
\usage{
fourfold(x,
  color = c("#99CCFF", "#6699CC", "#FFA0A0", "#A0A0FF", "#FF0000", "#000080"),
  conf_level = 0.95, std = c("margins", "ind.max", "all.max"),
  margin = c(1, 2), space = 0.2, main = NULL, sub = NULL,
  mfrow = NULL, mfcol = NULL, extended = TRUE, ticks = 0.15,
  p_adjust_method = p.adjust.methods, newpage = TRUE,
  fontsize = 12, default_prefix = c("Row", "Col", "Strata"),
  sep = ": ", varnames = TRUE, return_grob = FALSE)
}
\arguments{
  \item{x}{a \eqn{2 \times 2 \times k}{2 x 2 x k} contingency table in array form, or a
    \eqn{2 \times 2}{2 x 2} matrix if \eqn{k} is 1.  If \code{length(dim(x)>3}, 
    dimensions \code{3:length(dim(x)} are silently raveled into a combined strata dimension
    with \code{k=prod(dim(x)[-(1:2)]))}.} 
  \item{color}{a vector of length 6 specifying the colors to use for the
    smaller and larger diagonals of each \eqn{2 \times 2}{2 x 2} table. The first pair
    is used for the standard (non-extended) plots, the other two
    for the extended version: the second/third pair is used for tables with
    non-significant/significant log-odds ratios, respectively, the latter
    being visualized in brighter colors.}
  \item{conf_level}{confidence level used for the confidence rings on
    the odds ratios.  Must be a single non-negative number less than 1;
    if set to 0, confidence rings are suppressed.}
  \item{std}{a character string specifying how to standardize the table.
    Must be one of \code{"margins"}, \code{"ind.max"}, or
    \code{"all.max"}, and can be abbreviated by the initial letter.
    If set to \code{"margins"}, each \eqn{2 \times 2}{2 x 2} table is standardized to
    equate the margins specified by \code{margin} while preserving the
    odds ratio.  If \code{"ind.max"} or \code{"all.max"}, the tables are
    either individually or simultaneously standardized to a maximal cell
    frequency of 1.}
  \item{margin}{a numeric vector with the margins to equate.  Must be
    one of \code{1}, \code{2}, or \code{c(1, 2)} (the default), which
    corresponds to standardizing only the row, only column, or both row and column in
    each \eqn{2 \times 2}{2 x 2} table.
    Only used if \code{std} equals \code{"margins"}.}
  \item{space}{the amount of space (as a fraction of the maximal radius
    of the quarter circles) used for the row and column labels.}
  \item{main, sub}{character string for the fourfold plot title/subtitle.}
  \item{mfrow, mfcol}{a numeric vector with two components:
    \var{nr} and \var{nc}, indicating that the displays for the \eqn{2 \times 2}{2 x 2}
    tables should be arranged in an \var{nr} by \var{nc} layout, filled
    by rows/columns. The defaults are calculated to give a collection of plots in
    landscape orientation when \var{k} is not a perfect square.}
  \item{extended}{logical; if \code{TRUE}, extended plots are plotted,
    i.e., colors are brighter for significant log-odds ratios, and ticks
    are plotted showing the direction of association for positive log-odds.}
  \item{ticks}{the length of the ticks. If set to 0, no ticks are
    plotted.}
  \item{p_adjust_method}{method to be used for p-value adjustments for
    multi-stratum plots, as provided by \code{link[stats]{p.adjust}}.
    Use \code{p_adjust_method="none"} to disable this adjustment.
    The p-values are used for the \sQuote{visual}
    significance tests of the odds ratios.}
  \item{newpage}{logical; if \code{TRUE}, \code{grid.newpage()} is
    called before plotting.}
  \item{fontsize}{fontsize of main title.  Other labels are scaled
    relative to this.}
  \item{default_prefix}{character vector of length 3 with default labels
    for possibly missing row/column/strata variable names.}
  \item{sep}{default separator between variable names and levels for
    labels.}
  \item{varnames}{Logical; should the variable names
    be printed in the labeling of stratifed plots?}
  \item{return_grob}{Logical; shall a snapshot of the display be
    returned as a grob object?}
}
\details{
  The fourfold display is designed for the display of \eqn{2 \times 2 \times k}{2 x 2 x k}
  tables.

  Following suitable standardization, the cell frequencies
  \eqn{f_{ij}}{f[i,j]} of each \eqn{2 \times 2}{2 x 2} table are shown as a quarter
  circle whose radius is proportional to
  \eqn{\sqrt{f_{ij}}}{sqrt(f[i,j])} so that its area is proportional to
  the cell frequency.  An association (odds ratio different from 1)
  between the binary row and column variables is indicated by the
  tendency of diagonally opposite cells in one direction to differ in
  size from those in the other direction; color is used to show this
  direction.  Confidence rings for the odds ratio allow a visual test of
  the null of no association; the rings for adjacent quadrants overlap
  iff the observed counts are consistent with the null hypothesis.

  Typically, the number \eqn{k} corresponds to the number of levels of a
  stratifying variable, and it is of interest to see whether the
  association is homogeneous across strata.  The fourfold display
  visualizes the pattern of association.  Note that the confidence rings
  for the individual odds ratios are not adjusted for multiple testing.
}
\references{
  Friendly, M. (1994),
  \emph{A fourfold display for 2 by 2 by \eqn{k} tables}.
  Technical Report 217, York University, Psychology Department,
  \url{http://datavis.ca/papers/4fold/4fold.pdf}.

  Friendly, M. (2000),
  \emph{Visualizing Categorical Data}.
  SAS Institute, Cary, NC.
}
\seealso{
  \code{\link{mosaic}},
  \code{\link{assoc}}
  
  \code{link[stats]{p.adjust}} for methods of p value adjustment
}
\examples{
data("UCBAdmissions")
## Use the Berkeley admission data as in Friendly (1995).
x <- aperm(UCBAdmissions, c(2, 1, 3))
dimnames(x)[[2]] <- c("Yes", "No")
names(dimnames(x)) <- c("Sex", "Admit?", "Department")
ftable(x)

## Fourfold display of data aggregated over departments, with
## frequencies standardized to equate the margins for admission
## and sex.
## Figure 1 in Friendly (1994).
fourfold(margin.table(x, c(1, 2)))

## Fourfold display of x, with frequencies in each table
## standardized to equate the margins for admission and sex.
## Figure 2 in Friendly (1994).
fourfold(x)
cotabplot(x, panel = cotab_fourfold)

## Fourfold display of x, with frequencies in each table
## standardized to equate the margins for admission. but not
## for sex.
## Figure 3 in Friendly (1994).
fourfold(x, margin = 2)
}
\keyword{hplot}

\name{spacings}
\alias{spacings}
\alias{spacing_highlighting}
\alias{spacing_equal}
\alias{spacing_dimequal}
\alias{spacing_increase}
\alias{spacing_conditional}
\title{Spacing-generating Functions}
\description{
  These functions generate spacing functions to be used with
  \code{\link{strucplot}} to obtain customized spaces between the
  elements of a strucplot.
}
\usage{
spacing_equal(sp = unit(0.3, "lines"))
spacing_dimequal(sp)
spacing_increase(start = unit(0.3, "lines"), rate = 1.5)
spacing_conditional(sp = unit(0.3, "lines"), start = unit(2, "lines"), rate = 1.8)
spacing_highlighting(start = unit(0.2, "lines"), rate = 1.5)
}
\arguments{
  \item{start}{object of class \code{"unit"} indicating the start value
    for increasing spacings.}
  \item{rate}{increase rate for spacings.}
  \item{sp}{object of class \code{"unit"} specifying a fixed spacing.}
}
\details{
  These generating functions return a function used by
  \code{\link{strucplot}} to generate appropriate spaces between tiles of
  a strucplot, using the \code{dimnames} information of the visualized
  table.

  \code{spacing_equal} allows to specify one fixed space for \emph{all}
  dimensions.

  \code{spacing_dimequal} allows to specify a fixed space for
  \emph{each} dimension.

  \code{spacing_increase} creates increasing spaces for all dimensions,
  based on a starting value and an increase rate.

  \code{spacing_conditional} combines \code{spacing_equal} and
  \code{spacing_increase} to create fixed spaces for conditioned
  dimensions, and increasing spaces for conditioning dimensions.

  \code{spacing_highlighting} is essentially \code{spacing_conditional} but with
  the space of the last dimension set to 0. With a corresponding color
  scheme, this gives the impression of the last class being
  \sQuote{highlighted} in the penultimate class (as, e.g., in
  \code{\link{doubledecker}} plots).
}
\value{
  A spacing function with arguments:
  \item{d}{\code{"dim"} attribute of a contingency table.}
  \item{condvars}{index vector of conditioning dimensions (currently only used by
    \code{spacing_conditional}).}
  This function computes a list of objects of class \code{"unit"}.
  Each list element contains the spacing information for the
  corresponding dimension of the table.  The length of the
  \code{"unit"} objects is \eqn{k-1}, \eqn{k} number of levels of the
  corresponding factor.
}
\author{
  David Meyer \email{David.Meyer@R-project.org}
}

\references{
  Meyer, D., Zeileis, A., and Hornik, K. (2006),
  The strucplot framework: Visualizing multi-way contingency tables with
  \pkg{vcd}.
  \emph{Journal of Statistical Software}, \bold{17(3)}, 1-48.
  \doi{10.18637/jss.v017.i03} and available as 
  \code{vignette("strucplot")}.
}

\seealso{
  \code{\link{strucplot}},
  \code{\link{doubledecker}}
}
\examples{
data("Titanic")
strucplot(Titanic, spacing = spacing_increase(start = 0.5, rate = 1.5))
strucplot(Titanic, spacing = spacing_equal(1))
strucplot(Titanic, spacing = spacing_dimequal(1:4 / 4))
strucplot(Titanic, spacing = spacing_highlighting,
                   gp = gpar(fill = c("light gray","dark gray")))
data("PreSex")
strucplot(aperm(PreSex, c(1,4,2,3)), spacing = spacing_conditional,
          condvars = 2)  
}
\keyword{hplot}

\name{independence_table}
\alias{independence_table}
\title{Independence Table}
\description{
  Computes table of expected frequencies (under the null hypotheses of
  independence) from an \eqn{n}-way table.
}
\usage{
independence_table(x, frequency = c("absolute", "relative"))
}
\arguments{
  \item{x}{a table.}
  \item{frequency}{indicates whether absolute or relative frequencies
    should be computed.}
}
\value{
  A table with either absolute or relative frequencies.
}
\author{
  David Meyer \email{David.Meyer@R-project.org}
}
\examples{
data("MSPatients")
independence_table(MSPatients)
independence_table(MSPatients, frequency = "relative")
}
\keyword{category}
\keyword{array}

\name{Hitters}
\alias{Hitters}
\docType{data}
\title{Hitters Data}
\description{
  This data set is deduced from the \code{\link{Baseball}} fielding data
  set: fielding performance basically includes the numbers of Errors,
  Putouts and Assists made by each player.  In order to reduce the
  number of observations, the was compressed by calculating the mean
  number of errors, putouts and assists for each team and for only 6
  positions (1B, 2B, 3B, C, OF, SS and UT).  In addition, each of these
  three variables was scaled to a common range by dividing each variable
  by the maximum of the variable.
}
\usage{data("Hitters")}
\format{
  A data frame with 154 observations and 4 variables.
  \describe{
    \item{Positions}{factor indicating the field position (1B=first
      baseman, 2B=second baseman, 3B=third baseman, C=catcher,
      OF=outfielder, SS=Short Stop, UT=Utility Players).}
    \item{Putouts}{occur when a fielder causes an opposing player to be
      tagged or forced out.}
    \item{Assists}{are credited to other fielders involved in making
      that putout.}
    \item{Errors}{count the errors made by a player.}
  }
}

\references{
  M. Friendly (2000),
  \emph{Visualizing Categorical Data}.
  SAS Institute, Cary, NC.
}

\source{
  SAS System for Statistical Graphics, First Edition, Page A2.3
}
\examples{
data("Hitters")
attach(Hitters)

colors <- c("black","red","green","blue","red","black","blue")
pch <- substr(levels(Positions), 1, 1)
ternaryplot(Hitters[,2:4],
  pch = as.character(Positions),
  col = colors[as.numeric(Positions)],
  main = "Baseball Hitters Data")
grid_legend(0.8, 0.9, pch, colors, levels(Positions),
  title = "POSITION(S)")

detach(Hitters)
}

\keyword{datasets}

\name{coindep_test}
\alias{coindep_test}
\alias{fitted.coindep_test}
\title{Test for (Conditional) Independence}
\description{
  Performs a test of (conditional) independence of 2 margins
  in a contingency table by simulation from the marginal
  distribution of the input table under (conditional)
  independence.
}
\usage{
coindep_test(x, margin = NULL, n = 1000, 
  indepfun = function(x) max(abs(x)), aggfun = max,
  alternative = c("greater", "less"),
  pearson = TRUE)
}
\arguments{
  \item{x}{a contingency table.}
  \item{margin}{margin index(es) or corresponding name(s) of the
    conditioning variables. Each resulting conditional table has to
    be a 2-way table.}
  \item{n}{number of (conditional) independence tables to be drawn.}
  \item{indepfun}{aggregation function capturing independence in
    (each conditional) 2-way table.}
  \item{aggfun}{aggregation function aggregating the test statistics
    computed by \code{indepfun}.}
  \item{alternative}{a character string specifying the alternative
    hypothesis; must be either \code{"greater"} (default) or
    \code{"less"} (and may be abbreviated.)}
  \item{pearson}{logical. Should the table of Pearson residuals under
    independence be computed and passed to \code{indepfun} (default)
    or the raw table of observed frequencies?}
}
\details{
  If \code{margin} is \code{NULL} this computes a simple independence
  statistic in a 2-way table. Alternatively, \code{margin} can give 
  several conditioning variables and then conditional independence in
  the resulting conditional table is tested.
  
  By default, this uses a (double) maximum statistic of Pearson residuals.
  By changing \code{indepfun} or \code{aggfun} a (maximum of) Pearson Chi-squared
  statistic(s) can be computed or just the usual Pearson Chi-squared statistics
  and so on. Other statistics can be computed by changing \code{pearson}
  to \code{FALSE}.
  
  The function uses \code{\link{r2dtable}} to simulate the distribution
  of the test statistic under the null.
}
\value{
  A list of class \code{"coindep_test"} inheriting from \code{"htest"}
  with following components: 
  \item{statistic}{the value of the test statistic.}
  \item{p.value}{the \eqn{p} value for the test.}
  \item{method}{a character string indicating the type of the test.}
  \item{data.name}{a character string giving the name(s) of the data.}
  \item{observed}{observed table of frequencies}
  \item{expctd}{expected table of frequencies}
  \item{residuals}{corresponding Pearson residuals}
  \item{margin}{the \code{margin} used}
  \item{dist}{a vector of size \code{n} with simulated values of the
    distribution of the statistic under the null.}
  \item{qdist}{the corresponding quantile function (for computing 
    critical values).}
  \item{pdist}{the corresponding distribution function (for computing
    \eqn{p} values).}
}
\author{
  Achim Zeileis \email{Achim.Zeileis@R-project.org}
}
\seealso{
  \code{\link{chisq.test}},
  \code{\link{fisher.test}},
  \code{\link{r2dtable}}
}

\examples{
library(MASS)
TeaTasting <- matrix(c(3, 1, 1, 3), nrow = 2,
                     dimnames = list(Guess = c("Milk", "Tea"),
                                     Truth = c("Milk", "Tea"))
)
## compute maximum statistic
coindep_test(TeaTasting)
## compute Chi-squared statistic
coindep_test(TeaTasting, indepfun = function(x) sum(x^2))
## use unconditional asymptotic distribution
chisq.test(TeaTasting, correct = FALSE)
chisq.test(TeaTasting)


data("UCBAdmissions")
## double maximum statistic
coindep_test(UCBAdmissions, margin = "Dept")
## maximum of Chi-squared statistics
coindep_test(UCBAdmissions, margin = "Dept", indepfun = function(x) sum(x^2))
## Pearson Chi-squared statistic
coindep_test(UCBAdmissions, margin = "Dept", indepfun = function(x) sum(x^2), aggfun = sum)
## use unconditional asymptotic distribution
loglm(~ Dept * (Gender + Admit), data = UCBAdmissions)
}
\keyword{htest}

\name{ternaryplot}
\alias{ternaryplot}
\title{Ternary Diagram}
\description{
  Visualizes compositional, 3-dimensional data in an equilateral
  triangle.
}
\usage{
ternaryplot(x, scale = 1, dimnames = NULL,
  dimnames_position = c("corner","edge","none"),
  dimnames_color = "black", dimnames_rot = c(-60, 60, 0),
  id = NULL, id_color = "black",
  id_just = c("center", "center"),
  coordinates = FALSE, grid = TRUE, grid_color = "gray",
  labels = c("inside", "outside", "none"),
  labels_color = "darkgray", labels_rot = c(120, -120, 0),
  border = "black", bg = "white",
  pch = 19, cex = 1, prop_size = FALSE, col = "red",
  main = "ternary plot", newpage = TRUE, pop = TRUE,
  return_grob = FALSE, \dots)
}
\arguments{
  \item{x}{a matrix with three columns.}
  \item{scale}{row sums scale to be used.}
  \item{dimnames}{dimension labels (defaults to the column names of
    \code{x}).}
  \item{dimnames_position, dimnames_color}{position and color of
    dimension labels.}
  \item{dimnames_rot}{Numeric vector of length 3, specifying the angle
    of the dimension labels.}
  \item{id}{optional labels to be plotted below the plot
    symbols. \code{coordinates} and \code{id} are mutual exclusive.}
  \item{id_color}{color of these labels.}
  \item{id_just}{character vector of length 1 or 2 indicating the
    justification of these labels.}
  \item{coordinates}{if \code{TRUE}, the coordinates of the points are
    plotted below them. \code{coordinates} and \code{id} are mutual
    exclusive.}
  \item{grid}{if \code{TRUE}, a grid is plotted. May optionally
    be a string indicating the line type (default: \code{"dotted"}).}
  \item{grid_color}{grid color.}
  \item{labels, labels_color}{position and color of the grid labels.}
  \item{labels_rot}{Numeric vector of length 3, specifying the angle
    of the grid labels.}
  \item{border}{color of the triangle border.}
  \item{bg}{triangle background.}
  \item{pch}{plotting character. Defaults to filled dots.}
  \item{cex}{a numerical value giving the amount by which plotting text
    and symbols should be scaled relative to the default.  Ignored for
    the symbol size if \code{prop_size} is not \code{FALSE}.}
  \item{prop_size}{if \code{TRUE}, the symbol size is plotted
    proportional to the row sum of the three variables, i.e., represents
    the weight of the observation.}
  \item{col}{plotting color.}
  \item{main}{main title.}
  \item{newpage}{if \code{TRUE}, the plot will appear on a new graphics
    page.} 
  \item{pop}{logical; if \code{TRUE}, all newly generated viewports are popped after plotting.}
  \item{return_grob}{logical. Should a snapshot of the display be
    returned as a grid grob?}
  \item{\dots}{additional graphics parameters (see \code{par})}
}
\details{
  A points' coordinates are found by computing the gravity center of
  mass points using the data entries as weights.  Thus, the coordinates
  of a point \eqn{P(a,b,c)}, \eqn{a + b + c = 1}, are: \eqn{P(b + c/2, c
    \sqrt{3}/2)}{P(b + c/2, c * sqrt(3)/2)}.
}
\references{
  M. Friendly (2000),
  \emph{Visualizing Categorical Data}.
  SAS Institute, Cary, NC.
}
\author{
  David Meyer \email{David.Meyer@R-project.org}
}

\examples{
data("Arthritis")

## Build table by crossing Treatment and Sex
tab <- as.table(xtabs(~ I(Sex:Treatment) + Improved, data = Arthritis))

## Mark groups
col <- c("red", "red", "blue", "blue")
pch <- c(1, 19, 1, 19)

## plot
ternaryplot(
            tab,
            col = col,
            pch = pch,
            prop_size = TRUE,
            bg = "lightgray",
            grid_color = "white",
            labels_color = "white",
            main = "Arthritis Treatment Data"
           )
## legend
grid_legend(0.8, 0.7, pch, col, rownames(tab), title = "GROUP")

## Titanic
data("Lifeboats")
attach(Lifeboats)
ternaryplot(
  Lifeboats[,4:6],
  pch = ifelse(side == "Port", 1, 19),
  col = ifelse(side == "Port", "red", "blue"),
  id  = ifelse(men / total > 0.1, as.character(boat), NA),
  prop_size = 2,
  dimnames_position = "edge",
  main = "Lifeboats on Titanic"
)
grid_legend(0.8, 0.9, c(1, 19),
  c("red", "blue"), c("Port", "Starboard"),
  title = "SIDE")


## Hitters
data("Hitters")
attach(Hitters)
colors <- c("black","red","green","blue","red","black","blue")
pch <- substr(levels(Positions), 1, 1)
ternaryplot(
  Hitters[,2:4],
  pch = as.character(Positions),
  col = colors[as.numeric(Positions)],
  main = "Baseball Hitters Data"
)
grid_legend(0.8, 0.9, pch, colors, levels(Positions),
  title = "POSITION(S)")

}
\keyword{hplot}

\name{sieve}
\alias{sieve}
\alias{sieve.default}
\alias{sieve.formula}
\title{Extended Sieve Plots}
\encoding{UTF-8}
\description{
  (Extended) sieve displays for n-way contingency tables: plots rectangles with
  areas proportional to the expected cell frequencies and filled with a
  number of squares equal to the observed frequencies.  Thus, the
  densities visualize the deviations of the observed from the expected
  values.
}
\usage{
\method{sieve}{default}(x, condvars = NULL, gp = NULL, shade = NULL,
  legend = FALSE, split_vertical = NULL, direction = NULL, spacing = NULL,
  spacing_args = list(), sievetype = c("observed","expected"),
  gp_tile = gpar(), scale = 1, main = NULL, sub = NULL, \dots)
\method{sieve}{formula}(formula, data, \dots, main = NULL, sub = NULL, subset = NULL)
}
\arguments{
  \item{x}{a contingency table in array form, with optional category
    labels specified in the \code{dimnames(x)} attribute.}
  \item{condvars}{vector of integers or character strings indicating
    conditioning variables, if any. The table will be permuted to order
    them first.}
  \item{formula}{a formula specifying the variables used to create a 
    contingency table from \code{data}.  For convenience, conditioning
    formulas can be specified; the conditioning variables will then be
    used first for splitting.  Formulas for sieve displays (unlike
    those for doubledecker plots) have no response variable.}
  \item{data}{either a data frame, or an object of class \code{"table"}
    or \code{"ftable"}.}
  \item{subset}{an optional vector specifying a subset of observations to be used.}
  \item{shade}{logical specifying whether \code{gp} should be used or not
    (see \code{gp}). If \code{TRUE} and \code{expected} is unspecified,
    a default model is fitted: if \code{condvars} is specified, a
    corresponding conditional independence model, and else the total
    independence model. If \code{shade} is \code{NULL} (default),
    \code{gp} is used if specified.}
  \item{sievetype}{logical indicating whether rectangles should be filled
    according to \code{observed} or \code{expected} frequencies.}
  \item{gp}{object of class \code{"gpar"}, shading function or a
    corresponding generating function (see details of \code{\link{strucplot}} and
    \code{\link{shadings}}). Components of \code{"gpar"}
    objects are recycled as needed along the last splitting
    dimension. The default is a modified version of
    \code{\link{shading_Friendly}}:
    if \code{sievetype} is \code{"observed"}, cells with positive
    residuals are painted with a red sieve, and cells with negative residuals
    with a blue one.
    If \code{sievetype} is \code{"expected"}, the sieves' color is
    gray. Ignored if \code{shade = FALSE}.}
  \item{gp_tile}{object of class \code{"gpar"}, controlling the
    appearance of all \emph{static}
    elements of the cells (e.g., border and fill color).}
  \item{scale}{scaling factor for the sieve.}
  \item{legend}{either a legend-generating function, a legend
    function (see details of \code{\link{strucplot}} and
    \code{\link{legends}}), or a logical value.
    If \code{legend} is \code{NULL} or \code{TRUE} and \code{gp} is a
    function, legend defaults to \code{\link{legend_resbased}}.}
  \item{split_vertical}{vector of logicals of length \eqn{k}, where \eqn{k}
    is the number of margins of \code{x} (default: \code{FALSE}).
    Values are recycled as needed. 
    A \code{TRUE} component indicates that the tile(s) of the
    corresponding dimension should be split vertically, \code{FALSE}
    means horizontal splits.
    Ignored if \code{direction} is not \code{NULL}.}
  \item{direction}{character vector of length \eqn{k}, where \eqn{k} is the
    number of margins of \code{x} (values are recycled as needed).
    For each component, a value of \code{"h"} indicates that the tile(s)
    of the corresponding dimension should be split horizontally, whereas
    \code{"v"} indicates vertical split(s).}
  \item{spacing}{spacing object, spacing function, or corresponding
    generating function (see \code{\link{strucplot}} for more
    information).
    The default is no spacing at all if \code{x} has two dimensions,
    and  \code{spacing_increase} for more dimensions.}
  \item{spacing_args}{list of arguments for the generating function, if
    specified (see \code{\link{strucplot}} for more information).}
  \item{main, sub}{either a logical, or a character string used for plotting
    the main (sub) title.  If logical and \code{TRUE}, the
    name of the \code{data} object is used.}
  \item{\dots}{Other arguments passed to \code{\link{strucplot}}}
}
\details{
  \code{sieve} is a generic function which currently has a default method and a
  formula interface.  Both are high-level interfaces to the
  \code{\link{strucplot}} function, and produce (extended) sieve
  displays.  Most of the functionality is described there, such as
  specification of the independence model, labeling, legend, spacing,
  shading, and other graphical parameters.

  The layout is very flexible: the specification of shading, labeling,
  spacing, and legend is modularized (see \code{\link{strucplot}} for
  details).
}
\value{
  The \code{"structable"} visualized is returned invisibly.
}
\note{To be faithful to the original definition by Riedwyl & Schüpbach,
   the default is to have no spacing between the tiles for two-way tables.}
\references{
  H. Riedwyl & M. Schüpbach (1994),
  Parquet diagram to plot contingency tables.
  In F. Faulbaum (ed.),
  \emph{Softstat '93: Advances in Statistical Software}, 293--299.
  Gustav Fischer, New York.

  M. Friendly (2000),
  Visualizing Categorical Data,
  SAS Institute, Cary, NC.

  Meyer, D., Zeileis, A., and Hornik, K. (2006),
  The strucplot framework: Visualizing multi-way contingency tables with
  \pkg{vcd}.
  \emph{Journal of Statistical Software}, \bold{17(3)}, 1-48.
  \doi{10.18637/jss.v017.i03} and available as 
  \code{vignette("strucplot")}.
  
}
\author{
  David Meyer \email{David.Meyer@R-project.org}
}
\seealso{
  \code{\link{assoc}},
  \code{\link{strucplot}},
  \code{\link{mosaic}},
  \code{\link{structable}},
  \code{\link{doubledecker}}
}
\examples{
data("HairEyeColor")

## aggregate over 'sex':
(haireye <- margin.table(HairEyeColor, c(2,1)))

## plot expected values:
sieve(haireye, sievetype = "expected", shade = TRUE)

## plot observed table:
sieve(haireye, shade = TRUE)

## plot complete diagram:
sieve(HairEyeColor, shade = TRUE)

## example with observed values in the cells:
sieve(haireye, shade = TRUE, labeling = labeling_values,
               gp_text = gpar(fontface = 2))

## example with expected values in the cells:
sieve(haireye, shade = TRUE, labeling = labeling_values,
               value_type = "expected", gp_text = gpar(fontface = 2))

## an example for the formula interface:
data("VisualAcuity")
sieve(Freq ~ right + left,  data = VisualAcuity)

}
\keyword{hplot}

\name{WeldonDice}
\alias{WeldonDice}
\docType{data}
\title{Weldon's Dice Data}
\description{
  Data from Pearson (1900) about the frequency of 5s and 6s in throws of
  12 dice.  Weldon tossed the dice 26,306 times and reported his results
  in a letter to Francis Galton on 1894-02-02.
}
\usage{
data("WeldonDice")
}
\format{
  A 1-way table giving the frequency of a 5 or a 6 in 26,306 throws of
  12 dice where 10 indicates \sQuote{10 or more} 5s or 6s.  The variable
  and its levels are
  \tabular{rll}{
    No \tab Name \tab Levels \cr
    1 \tab n56 \tab 0, 1, \dots, 10 \cr
  }
}

\references{
  K. Pearson (1900),
  On the criterion that a given system of deviations from the probable
  in the case of a correlated system of variables is such that it can be
  reasonably supposed to have arisen by random sampling,
  \emph{Philosophical Magazine}, \bold{50} (5th series), 157--175.

  M. Friendly (2000),
  \emph{Visualizing Categorical Data}.
  SAS Institute, Cary, NC.
}

\source{
  M. Friendly (2000),
  Visualizing Categorical Data,
  pages 20--21.
}

\examples{
data("WeldonDice")
gf <- goodfit(WeldonDice, type = "binomial")
summary(gf)
plot(gf)
}

\keyword{datasets}

\name{plot.loddsratio}
\alias{plot.loddsratio}
\alias{lines.loddsratio}
\title{Plotting (Log) Odds Ratios}
\description{
  Produces a (conditional) line plot of extended (log) odds ratios.
}
\usage{
\method{plot}{loddsratio}(x, baseline = TRUE, gp_baseline = gpar(lty = 2),
     lines = TRUE, lwd_lines = 3,
     confidence = TRUE, conf_level = 0.95, lwd_confidence = 2,
     whiskers = 0, transpose = FALSE,
     col = NULL, cex = 0.8, pch = NULL,
     bars = NULL, gp_bars = gpar(fill = "lightgray", alpha = 0.5),
     bar_width = unit(0.05, "npc"),
             
     legend = TRUE, legend_pos = "topright", legend_inset = c(0, 0),
     legend_vgap = unit(0.5, "lines"),
     gp_legend_frame = gpar(lwd = 1, col = "black"),
     gp_legend_title = gpar(fontface = "bold"),
     gp_legend = gpar(), legend_lwd = 1, legend_size = 1,
             
     xlab = NULL, ylab = NULL, xlim = NULL, ylim = NULL,
     main = NULL, gp_main = gpar(fontsize = 12, fontface = "bold"),
     newpage = TRUE, pop = FALSE, return_grob = FALSE,
     add = FALSE, prefix = "", \dots)
\method{lines}{loddsratio}(x, legend = FALSE, confidence = FALSE, cex = 0, \dots)
}
\arguments{
  \item{x}{an object of class \code{loddsratio}.}
  \item{baseline}{if \code{TRUE}, a dashed line is plotted at a
    value of 1 (in case of odds) or 0 (in case of log-odds).}
  \item{gp_baseline}{object of class \code{"gpar"} used for the baseline.}
  \item{lines}{if \code{TRUE}, the points are connected by lines (only
    sensible if the variable represented by the x-axis is ordinal).}
  \item{lwd_lines}{Width of the connecting lines (in \code{char} units).}
  \item{confidence}{logical; shall confindence intervals be plotted?}
  \item{conf_level}{confidence level used for confidence intervals.}
  \item{lwd_confidence}{Line width of the confidence interval bars (in \code{char} units).}
  \item{whiskers}{width of the confidence interval whiskers.}
  \item{transpose}{if \code{TRUE}, the plot is transposed.}

  \item{col}{character vector specifying the colors of the fitted
    lines, by default chosen with \code{\link[colorspace]{rainbow_hcl}}.}
  \item{cex}{size of the plot symbols (in lines).}
  \item{pch}{character or numeric vector of symbols used for plotting
    the (possibly conditioned) observed values, recycled as needed.}
  \item{bars}{logical; shall bars be plotted additionally to the points?
    Defaults to \code{TRUE} in case of only one conditioning variable.}
  \item{gp_bars}{object of class \code{"gpar"} used for the bars.}
  \item{bar_width}{Width of the bars, if drawn.}
  
  \item{legend}{logical; if \code{TRUE} (default), a legend is drawn.}
  \item{legend_pos}{numeric vector of length 2, specifying x and y
    coordinates of the legend, or a character string (e.g., \code{"topleft"},
    \code{"center"} etc.). Defaults to \code{"topleft"}
    if the fitted curve's slope is
    positive, and \code{"topright"} else.}
  \item{legend_inset}{numeric vector or length 2 specifying the inset
    from the legend's x and y coordinates in npc units.}
  \item{legend_vgap}{vertical space between the legend's line entries.}
  \item{gp_legend_frame}{object of class \code{"gpar"} used for the
    legend frame.}
  \item{gp_legend_title}{object of class \code{"gpar"} used for the
    legend title.}
  \item{gp_legend}{object of class \code{"gpar"} used for the
    legend defaults.}
  \item{legend_lwd}{line width used in the legend for the different
    groups.}
  \item{legend_size}{size used for the group symbols (in char units).}

  \item{xlab}{label for the x-axis. Defaults to \code{"Strata"} if
    \code{transpose} is \code{FALSE}.}
  \item{ylab}{label for the y-axis. Defaults to \code{"Strata"} if
    \code{transpose} is \code{TRUE}.}
  \item{xlim}{x-axis limits. Ignored if \code{transpose} is \code{FALSE}.}
  \item{ylim}{y-axis limits. Ignored if \code{transpose} is \code{TRUE}.}
  \item{main}{user-specified main title.}
  \item{gp_main}{object of class \code{"gpar"} used for the main title.}
  \item{newpage}{logical; if \code{TRUE}, the plot is drawn on a new page.}
  \item{pop}{logical; if \code{TRUE}, all newly generated viewports are
    popped after plotting.}
  \item{return_grob}{logical. Should a snapshot of the display be
    returned as a grid grob?}
  \item{add}{logical; should the plot added to an existing log odds
    ratio plot?}
  \item{prefix}{character string used as prefix for the viewport name.}
  \item{\dots}{other graphics parameters (see \code{\link{par}}).}
}
\value{
  if \code{return_grob} is \code{TRUE}, a grob object corresponding to
  the plot. \code{NULL} (invisibly) else.
}
\details{
  The function basically produces conditioned line plots of the (log)
  odds ratios structure provided in \code{x}.

  The \code{lines} method can be used to overlay different plots (for
  example, observed and expected values).

  \code{\link{cotabplot}} can be used for stratified analyses (see examples).

  }
\references{
  M. Friendly (2000),
  \emph{Visualizing Categorical Data}.
  SAS Institute, Cary, NC.
}
\author{
  David Meyer \email{David.Meyer@R-project.org}
}

\seealso{
  \code{\link{loddsratio}}
}
\examples{
## 2 x 2 x k cases
data(CoalMiners, package = "vcd")
lor_CM <- loddsratio(CoalMiners)
plot(lor_CM)
lor_CM_df <- as.data.frame(lor_CM)

# fit linear models using WLS
age <- seq(20, 60, by = 5)
lmod <- lm(LOR ~ age, weights = 1 / ASE^2, data = lor_CM_df)
grid.lines(seq_along(age), fitted(lmod), gp = gpar(col = "blue", lwd = 2), default.units = "native")
qmod <- lm(LOR ~ poly(age, 2), weights = 1 / ASE^2, data = lor_CM_df)
grid.lines(seq_along(age), fitted(qmod), gp = gpar(col = "red", lwd = 2), default.units = "native")

## 2 x k x 2
lor_Emp <-loddsratio(Employment)
plot(lor_Emp)

## 4 way tables 
data(Punishment, package = "vcd")
mosaic(attitude ~ age + education + memory, data = Punishment,
highlighting_direction="left", rep = c(attitude = FALSE))

# visualize the log odds ratios, by education
plot(loddsratio(~ attitude + memory | education, data = Punishment))

# visualize the log odds ratios, by age
plot(loddsratio(~ attitude + memory | age, data = Punishment))

# visualize the log odds ratios, by age and education
plot(loddsratio(~ attitude + memory | age + education, data = Punishment))

# same, transposed
plot(loddsratio(~ attitude + memory | age + education, data = Punishment), transpose = TRUE)

# alternative visualization methods
image(loddsratio(Freq ~ ., data = Punishment))
tile(loddsratio(Freq ~ ., data = Punishment))

## cotabplots for more complex tables
cotabplot(Titanic, cond = c("Age","Sex"), panel = cotab_loddsratio)

cotabplot(Freq ~ opinion + grade + year | gender, data = JointSports, panel = cotab_loddsratio)
cotabplot(Freq ~ opinion + grade | year + gender, data = JointSports, panel = cotab_loddsratio)
}
\keyword{category}

\name{legends}
\alias{legends}
\alias{legend_resbased}
\alias{legend_fixed}
\title{Legend Functions for Strucplots}
\description{
  These functions generate legend functions for residual-based shadings.
}
\usage{
legend_resbased(fontsize = 12, fontfamily = "",
  x = unit(1, "lines"), y = unit(0.1,"npc"),
  height = unit(0.8, "npc"),
  width = unit(0.7, "lines"),
  digits = 2, pdigits = max(1, getOption("digits") - 2),
  check_overlap = TRUE, text = NULL,
  steps = 200, ticks = 10, pvalue = TRUE,  range = NULL)
legend_fixed(fontsize = 12, fontfamily = "", x = unit(1, "lines"), y = NULL,
  height = NULL, width = unit(1.5, "lines"), steps = 200,
  digits = 1, space = 0.05, text = NULL, range = NULL)
}
\arguments{
  \item{fontsize}{fontsize of title and p-value text.}
  \item{fontfamily}{fontfamily of all text.}
  \item{x, y}{objects of class \code{"unit"} indicating the coordinates
    of the title. For \code{legend_fixed}, the default for \code{y} is
    computed as to leave enough space for the specified \code{text}.}
  \item{height, width}{object of class \code{"unit"} indicating the
    height/width of the legend. For \code{legend_fixed}, the default for
    \code{y} is computed as to align upper margins of legend and actual plot.} 
  \item{digits}{number of digits for the scale labels.}
  \item{pdigits}{number of digits for the p-value.}
  \item{check_overlap}{logical indicating whether overlap of scale
    labels should be inhibited or not.}
  \item{space}{For \code{legend_fixed} only: proportion of space
    between the tiles.}
  \item{text}{character string indicating the title of the legend.}
  \item{steps}{granularity of the color gradient.}
  \item{ticks}{number of scale ticks.}
  \item{pvalue}{logical indicating whether the \eqn{p}-value should be
    visualized under the scale or not.}
  \item{range}{Numeric vector of length 2 for setting the legend
    range. Computed from the residuals if omitted. \code{NA} values are
    replaced by the corresponding minimum / maximum of the residuals.}
}
\value{
  A function with arguments:
  \item{residuals}{residuals from the fitted independence model to be
    visualized.}
  \item{shading}{shading function computing colors from residuals (see
    details).}
  \item{autotext}{character vector indicating the title to be used when
    no \code{text} argument is specified. Allows strucplot to generate
    sensible defaults depending on the residuals type.}
}
\details{
  These functions generate legend functions for residual-based shadings,
  visualizing deviations from expected values of an hypothesized
  independence model.  Therefore, the legend uses a supplied shading
  function to visualize the color gradient for the residuals range.
  \code{legend_fixed} is inspired by the legend used in
  \code{\link[graphics]{mosaicplot}}. For
  more details on the shading functions and their return values, see
  \code{\link{shadings}}.
}
\references{
  Meyer, D., Zeileis, A., and Hornik, K. (2006),
  The strucplot framework: Visualizing multi-way contingency tables with
  \pkg{vcd}.
  \emph{Journal of Statistical Software}, \bold{17(3)}, 1-48.
  \doi{10.18637/jss.v017.i03} and available as 
  \code{vignette("strucplot")}.

  Meyer, D., Zeileis, A., Hornik, K. (2003), Visualizing independence using
  extended association plots. \emph{Proceedings of the 3rd International
  Workshop on Distributed Statistical Computing}, K. Hornik, F. Leisch,
  A. Zeileis (eds.), ISSN 1609-395X.
  \url{https://www.R-project.org/conferences/DSC-2003/Proceedings/}
}

\author{
  David Meyer \email{David.Meyer@R-project.org}
}
\seealso{
  \code{\link{structable}},
  \code{\link{shadings}}
}

\examples{
data("Titanic")

mosaic(Titanic, shade = TRUE, legend = legend_resbased)
mosaic(Titanic, shade = TRUE, legend = legend_fixed, gp = shading_Friendly)
}
\keyword{hplot}

\name{plot.loglm}
\alias{plot.loglm}
\alias{assoc.loglm}
\alias{mosaic.loglm}
\title{Visualize Fitted Log-linear Models}
\description{
  Visualize fitted \code{"loglm"} objects by mosaic or 
  association plots.
}
\usage{
\method{plot}{loglm}(x, panel = mosaic, type = c("observed", "expected"),
  residuals_type = c("pearson", "deviance"), gp = shading_hcl, gp_args = list(),
  \dots)
}
\arguments{
  \item{x}{a fitted \code{"loglm"} object, see \code{\link[MASS]{loglm}}.}
  \item{panel}{a panel function for visualizing the observed values,
    residuals and expected values. Currently, \code{\link{mosaic}} and
    \code{\link{assoc}} in \pkg{vcd}.}
  \item{type}{a character string indicating whether the observed or the
    expected values of the table should be visualized.}
  \item{residuals_type}{a character string indicating the type of
    residuals to be computed.}
  \item{gp}{object of class \code{"gpar"}, shading function or a
    corresponding generating function (see details and
    \code{\link{shadings}}).
    Ignored if \code{shade = FALSE}.} 
  \item{gp_args}{list of arguments for the shading-generating function,
    if specified.}
  \item{\dots}{Other arguments passed to the \code{panel} function.}
}
\details{
  The \code{plot} method for \code{"loglm"} objects by default visualizes 
  the model using a mosaic plot (can be changed to an association plot
  by setting \code{panel = assoc}) with a shading based on the residuals
  of this model. The legend also reports the corresponding p value of the
  associated goodness-of-fit test. The \code{mosaic} and \code{assoc} methods
  are simple convenience interfaces to this \code{plot} method, setting 
  the \code{panel} argument accordingly.
}
\value{
  The \code{"structable"} visualized is returned invisibly.
}
\author{
  Achim Zeileis \email{Achim.Zeileis@R-project.org}
}
\seealso{
  \code{\link[MASS]{loglm}},
  \code{\link{assoc}},
  \code{\link{mosaic}},
  \code{\link{strucplot}}
}
\examples{
library(MASS)
## mosaic display for PreSex model
data("PreSex")
fm <- loglm(~ PremaritalSex * ExtramaritalSex * (Gender + MaritalStatus),
  data = aperm(PreSex, c(3, 2, 4, 1)))
fm
## visualize Pearson statistic
plot(fm, split_vertical = TRUE)
## visualize LR statistic
plot(fm, split_vertical = TRUE, residuals_type = "deviance")

## conditional independence in UCB admissions data
data("UCBAdmissions")
fm <- loglm(~ Dept * (Gender + Admit), data = aperm(UCBAdmissions))

## use mosaic display
plot(fm, labeling_args = list(abbreviate_labs = c(Admit = 3)))

## and association plot
plot(fm, panel = assoc)
assoc(fm)
}
\keyword{hplot}

\name{Suicide}
\alias{Suicide}
\docType{data}
\title{Suicide Rates in Germany}
\description{
  Data from Heuer (1979) on suicide rates in West Germany classified by
  age, sex, and method of suicide.
}
\usage{
data("Suicide")
}
\format{
  A data frame with 306 observations and 6 variables.
  \describe{
    \item{Freq}{frequency of suicides.}
    \item{sex}{factor indicating sex (male, female).}
    \item{method}{factor indicating method used.}
    \item{age}{age (rounded).}
    \item{age.group}{factor.  Age classified into 5 groups.}
    \item{method2}{factor indicating method used (same as \code{method}
      but some levels are merged).}
  }
}
\references{
  J. Heuer (1979),
  \emph{Selbstmord bei Kindern und Jugendlichen}.
  Ernst Klett Verlag, Stuttgart.

  M. Friendly (2000),
  \emph{Visualizing Categorical Data}.
  SAS Institute, Cary, NC.
}
\source{
  Michael Friendly (2000),
  Visualizing Categorical Data:
  \url{http://euclid.psych.yorku.ca/ftp/sas/vcd/catdata/suicide.sas}
}
\examples{
data("Suicide")
structable(~ sex + method2 + age.group, data = Suicide)
}
\keyword{datasets}

\name{cotabplot}
\alias{cotabplot}
\alias{cotabplot.default}
\alias{cotabplot.formula}
\title{Coplot for Contingency Tables}
\description{
  \code{cotabplot} is a generic function for creating trellis-like
  coplots (conditional plots) for contingency tables.
}
\usage{
cotabplot(x, \dots)
\method{cotabplot}{default}(x, cond = NULL,
  panel = cotab_mosaic, panel_args = list(),
  margins = rep(1, 4), layout = NULL,
  text_gp = gpar(fontsize = 12), rect_gp = gpar(fill = grey(0.9)),
  pop = TRUE, newpage = TRUE, return_grob = FALSE,
  \dots)
\method{cotabplot}{formula}(formula, data = NULL, \dots)
}
\arguments{
  \item{x}{an object. The default method can deal with contingency tables
    in array form.}
  \item{cond}{margin index(es) or corresponding name(s) of the
    conditioning variables.}
  \item{panel}{panel function applied for each conditioned plot,
    see details.}
  \item{panel_args}{list of arguments passed to \code{panel} if this is
    a panel-generating function inheriting from class \code{"grapcon_generator"}.}
  \item{margins}{either an object of class \code{"unit"} of length 4, or
    a numeric vector of length 4. The elements are recycled as needed.
    giving the margins around the whole plot.}
  \item{layout}{integer vector (of length two), giving the number of
    rows and columns for the panel.}
  \item{text_gp}{object of class \code{"gpar"} used for the text in the panel titles.}
  \item{rect_gp}{object of class \code{"gpar"} used for the rectangles with the panel titles.}
  \item{pop}{logical indicating whether the generated viewport tree
    should be removed at the end of the drawing or not.}
  \item{newpage}{logical controlling whether a new grid page should be
    created.}
  \item{return_grob}{logical. Should a snapshot of the display be
    returned as a grid grob?}
  \item{\dots}{further arguments passed to the panel-generating function.}
  \item{formula}{a formula specifying the variables used to create a 
    contingency table from \code{data}. It has to be of type \code{~ x + y | z}
    where \code{z} is/are the conditioning variable(s) used.}
  \item{data}{either a data frame, or an object of class \code{"table"}
    or \code{"ftable"}.}
}
\details{
  \code{cotabplot} is a generic function designed to create coplots or
  conditional plots (see Cleveland, 1993, and Becker, Cleveland, Shyu, 1996)
  similar to \code{\link{coplot}} but for contingency tables.
  
  \code{cotabplot} takes on computing the conditioning information 
  and setting up the trellis display, and then relies on a panel function
  to create plots from the full table and the conditioning information.
  A simple example would be a contingency table \code{tab} with margin
  names \code{"x"}, \code{"y"} and \code{"z"}. To produce this plot
  either the default interface can be used or the formula interface via
  
    \code{cotabplot(tab, "z")}
    \code{cotabplot(~ x + y | z, data = tab)}
    
  The panel function needs to be of the form
  
    \code{panel(x, condlevels)}
    
  where \code{x} is the \emph{full} table (\code{tab} in the example above)
  and \code{condlevels} is a named vector with the levels (e.g., 
  \code{c(z = "z1")} in the example above). 
  
  Alternatively, \code{panel} can also be a panel-generating function
  of class \code{"grapcon_generator"} which creates a function with the
  interface described above. The panel-generating function is called
  with the interface
  
    \code{panel(x, condvars, \dots)}
    
  where again \code{x} is the full table, \code{condvars} is now only
  a vector with the names of the conditioning variables (and not their
  levels, e.g., \code{"z"} in the example above). Further arguments
  can be passed to the panel-generating function via \code{\dots} which
  also includes the arguments set in \code{panel_args}.
  
  Suitable panel-generating functions for mosaic, association and sieve
  plots can be found at \code{\link{cotab_mosaic}}.
  
  A description of the underlying ideas is given in Zeileis, Meyer, Hornik
  (2005).
}
\seealso{
  \code{\link{cotab_mosaic}},
  \code{\link{cotab_coindep}},
  \code{\link{co_table}},
  \code{\link{coindep_test}}
}
\references{
  Becker, R.A., Cleveland, W.S., Shyu, M.-J. (1996),
  The visual design and control of trellis display.
  \emph{Journal of Computational and Graphical Statistics}, \bold{5},
  123--155.

  Cleveland, W.S. (1993), \emph{Visualizing Data}, Summit, New Jersey: Hobart Press.

  Meyer, D., Zeileis, A., and Hornik, K. (2006),
  The strucplot framework: Visualizing multi-way contingency tables with
  \pkg{vcd}.
  \emph{Journal of Statistical Software}, \bold{17(3)}, 1-48.
  \doi{10.18637/jss.v017.i03} and available as 
  \code{vignette("strucplot")}.


  Zeileis, A., Meyer, D., Hornik K. (2007), \emph{Residual-based shadings for visualizing
  (conditional) independence}, \emph{Journal of Computational and Graphical Statistics}, \bold{16},
  507--525.  
}
\author{
  Achim Zeileis \email{Achim.Zeileis@R-project.org}
}
\examples{
data("UCBAdmissions")

cotabplot(~ Admit + Gender | Dept, data = UCBAdmissions)
cotabplot(~ Admit + Gender | Dept, data = UCBAdmissions, panel = cotab_assoc)

ucb <- cotab_coindep(UCBAdmissions, condvars = "Dept", type = "assoc",
                     n = 5000, margins = c(3, 1, 1, 3))
cotabplot(~ Admit + Gender | Dept, data = UCBAdmissions, panel = ucb)
}
\keyword{hplot}

\name{OvaryCancer}
\alias{OvaryCancer}
\docType{data}
\title{Ovary Cancer Data}
\description{
  Data from Obel (1975) about a retrospective study of ovary cancer
  carried out in 1973.  Information was obtained from 299 women, who
  were operated for ovary cancer 10 years before.
}
\usage{
data("OvaryCancer")
}
\format{
  A data frame with 16 observations and 5 variables.
  \describe{
    \item{Freq}{frequency.}
    \item{stage}{factor indicating the stage of the cancer at the
      time of operation (early, advanced).}
    \item{operation}{factor indicating type of operation (radical,
      limited).}
    \item{survival}{factor indicating survival status after 10 years
      (yes, no).}
    \item{xray}{factor indicating whether X-ray treatment was received
      (yes, no).}
  }
}

\references{
  E. B. Obel (1975),
  A Comparative Study of Patients with Cancer of the Ovary Who Have
  Survived More or Less Than 10 Years.
  \emph{Acta Obstetricia et Gynecologica Scandinavica}, 
  \bold{55}, 429-439.

  E. B. Andersen (1991),
  \emph{The Statistical Analysis of Categorical Data}.
  2nd edition.
  Springer-Verlag, Berlin.
}

\source{
  E. B. Andersen (1991),
  The Statistical Analysis of Categorical Data, Table 6.4.
}

\examples{
data("OvaryCancer")
tab <- xtabs(Freq ~ xray + survival + stage + operation, data = OvaryCancer)
ftable(tab, col.vars = "survival", row.vars = c("stage", "operation", "xray"))

## model: ~ xray * operation * stage + survival * stage
## interpretation: treat xray, operation, stage as fixed margins,
##   the survival depends on stage, but not xray and operation.
doubledecker(survival ~ stage + operation + xray, data = tab)
mosaic(~ stage + operation + xray + survival,
  split_vertical = c(FALSE, TRUE, TRUE, FALSE), data = tab, keep_aspect_ratio = FALSE,
  gp = gpar(fill = rev(grey.colors(2))))
mosaic(~ stage + operation + xray + survival,
  split_vertical = c(FALSE, TRUE, TRUE, FALSE), data = tab, keep_aspect_ratio = FALSE,
  expected = ~ xray * operation * stage + survival*stage)
}

\keyword{datasets}

\name{grid_legend}
\alias{grid_legend}
\title{Legend Function for grid Graphics}
\description{
  This function can be used to add legends to \emph{grid-based} plots.
}
\usage{
grid_legend(x, y, pch = NA, col = par('col'), labels, frame = TRUE,
            hgap = unit(0.8, "lines"), vgap = unit(0.8, "lines"),
            default_units = "lines", gp = gpar(), draw = TRUE,
            title = NULL, just = 'center', lwd = NA, lty = NA,
            size = 1,
            gp_title = NULL, gp_labels = NULL,
            gp_frame = gpar(fill = "transparent"),
            inset = c(0, 0))
}
\arguments{
  \item{x}{character string \code{"topright"}, \code{"topleft"}, \code{"bottomright"},
    \code{"bottomleft"}, \code{"top"}, \code{"bottom"}, \code{"left"}, \code{"right"}, \code{"center"} or x coordinate of the legend.}
  \item{y}{y coordinates of the legend.}
  \item{pch}{integer vector of plotting symbols, if any.}
  \item{col}{character vector of colors for the symbols.}
  \item{labels}{character vector of labels corresponding to the symbols.}
  \item{frame}{logical indicating whether the legend should have a
    border or not.}
  \item{hgap}{object of class \code{"unit"} specifying the space between
  symbols and labels.}
  \item{vgap}{object of class \code{"unit"} specifying the space between
  the lines.}
  \item{default_units}{character string indicating the default unit.}
  \item{gp}{object of class \code{"gpar"} used for the legend.}
  \item{draw}{logical indicating whether the legend be drawn or not.}
  \item{title}{character string indicating the plot's title.}
  \item{just}{justification of the legend relative to its (x, y)
    location. see ?viewport for more details.}
  \item{lwd}{positive number to set the line width. if specified lines are drawn.}
  \item{lty}{line type. if specified lines are drawn.}
  \item{size}{size of the group symbols (in char units).}
  \item{gp_title}{object of class \code{"gpar"} used for the title.}
  \item{gp_labels}{object of class \code{"gpar"} used for the labels.}
  \item{gp_frame}{object of class \code{"gpar"} used for the frame.}
  \item{inset}{numeric vector of length 2 specifying the inset of the
    legend in npc units, relative to the specified x and y coordinates.}
}
\value{
  Invisibly, the legend as a \code{"grob"} object.
}
\author{
  David Meyer \email{David.Meyer@R-project.org}
  Florian Gerber \email{florian.gerber@math.uzh.ch}
}
\seealso{
  \code{\link[graphics]{legend}}
}
\examples{
data("Lifeboats")
attach(Lifeboats)
ternaryplot(Lifeboats[,4:6],
  pch = ifelse(side == "Port", 1, 19),
  col = ifelse(side == "Port", "red", "blue"),
  id  = ifelse(men / total > 0.1, as.character(boat), NA),
  prop_size = 2,
  dimnames_position = "edge",
  main = "Lifeboats on Titanic")
grid_legend(0.8, 0.9, c(1, 19), c("red", "blue"),
  c("Port", "Starboard"), title = "SIDE")


grid.newpage()
pushViewport(viewport(height = .9, width = .9 ))
grid.rect(gp = gpar(lwd = 2, lty = 2))

grid_legend(x = unit(.05,'npc'),
            y = unit(.05,'npc'),
            just = c(0,0),
            pch = c(1,2,3),
            col = c(1,2,3),
            lwd=NA, 
            lty=NA,
            labels = c("b",'r','g'),
            title = NULL,
            gp=gpar(lwd=2, cex=1),
            hgap = unit(.8, "lines"),
            vgap = unit(.9, "lines"))

grid_legend(x = unit(1,'npc'),
            y = unit(1,'npc'),
            just = c(1,1),
            pch = NA,
            col = c(1,2,3,4),
            lwd=c(1,1,1,3), 
            lty=c(1,2,1,3),
            labels = c("black",'red','green','blue'),
            gp_labels = list(gpar(col = 1), gpar(col = 2), gpar(col = 3), gpar(col = 4)),
            title = NULL,
            gp=gpar(lwd=2, cex=1),
            hgap = unit(.8, "lines"),
            vgap = unit(.9, "lines"))

grid_legend(x = 'topleft',
            pch = c(1,NA,2,NA),
            col = c(1,2,3,4),
            lwd=NA, 
            lty=c(NA,2,NA,3),
            labels = c("black",'red','green','blue'),
            title = 'Some LONG Title',
            gp_title = gpar(col = 3),
            gp_frame = gpar(col = 4, lty = 2, fill = "transparent"),
            gp_labels = gpar(col = 6),
            gp=gpar(lwd=2, cex=2, col = 1),
            hgap = unit(.8, "lines"),
            vgap = unit(.9, "lines"))

grid_legend(x = .7,
            y = .7,
            pch = c(1,NA,2,NA),
            col = c(1,2,3,4),
            lwd=1, 
            lty=c(NA,2,NA,3),
            labels = c("black",'red','green','blue'),
            title = 'short T',
            gp=gpar(lwd=1, cex=.7,col = 1),
            hgap = unit(.8, "lines"),
            vgap = unit(.9, "lines"))

grid_legend(x = 'bottomright',
            pch = c(1,NA,2,NA),
            col = c(2),
            lwd=NA, 
            lty=c(NA,2,NA,3),
            labels = c("black",'red','green','blue'),
            title = NULL,
            gp=gpar(lwd=2, cex=1,col = 1),
            hgap = unit(.8, "lines"),
            vgap = unit(.9, "lines"))
}


\keyword{hplot}

\name{Arthritis}
\alias{Arthritis}
\docType{data}
\title{Arthritis Treatment Data}
\description{
  Data from Koch & Edwards (1988) from a double-blind clinical trial
  investigating a new treatment for rheumatoid arthritis.
}
\usage{data("Arthritis")}
\format{
  A data frame with 84 observations and 5 variables.
  \describe{
    \item{ID}{patient ID.}
    \item{Treatment}{factor indicating treatment (Placebo, Treated).}
    \item{Sex}{factor indicating sex (Female, Male).}
    \item{Age}{age of patient.}
    \item{Improved}{ordered factor indicating treatment outcome (None,
      Some, Marked).}
  }
}

\references{
  G. Koch & S. Edwards (1988),
  Clinical efficiency trials with categorical data.
  In K. E. Peace (ed.), \emph{Biopharmaceutical Statistics for Drug
    Development}, 403--451.
  Marcel Dekker, New York.

  M. Friendly (2000),
  \emph{Visualizing Categorical Data}.
  SAS Institute, Cary, NC.
}
\source{
  Michael Friendly (2000),
  Visualizing Categorical Data:
  \url{http://euclid.psych.yorku.ca/ftp/sas/vcd/catdata/arthrit.sas}
}
\examples{
data("Arthritis")
art <- xtabs(~ Treatment + Improved, data = Arthritis, subset = Sex == "Female")
art

mosaic(art, gp = shading_Friendly)
mosaic(art, gp = shading_max)
}

\keyword{datasets}

\name{Employment}
\alias{Employment}
\docType{data}
\title{Employment Status}
\description{
  Data from a 1974 Danish study given by Andersen (1991) on the
  employees who had been laid off.  The workers are classified by 
  their employment status on 1975-01-01, the cause of their layoff
  and the length of employment before they were laid off.
}
\usage{
data("Employment")
}
\format{
  A 3-dimensional array resulting from cross-tabulating variables for
  1314 employees.  The variables and their levels are as follows:

  \tabular{rll}{
    No \tab Name \tab Levels \cr
    1 \tab EmploymentStatus \tab NewJob, Unemployed \cr
    2 \tab EmploymentLength \tab <1Mo, 1-3Mo, 3-12Mo, 1-2Yr, 2-5Yr, >5Yr \cr
    3 \tab LayoffCause \tab Closure, Replaced
  }
}

\references{
  E. B. Andersen (1991),
  \emph{The Statistical Analysis of Categorical Data}.
  Springer-Verlag, Berlin.

  M. Friendly (2000),
  \emph{Visualizing Categorical Data}.
  SAS Institute, Cary, NC.
}

\source{
  Michael Friendly (2000),
  Visualizing Categorical Data, pages 126--129.
}

\examples{
data("Employment")

## Employment Status
mosaic(Employment,
       expected = ~ LayoffCause * EmploymentLength + EmploymentStatus,
       main = "Layoff*EmployLength + EmployStatus")

mosaic(Employment,
       expected = ~ LayoffCause * EmploymentLength + LayoffCause * EmploymentStatus,
       main = "Layoff*EmployLength + Layoff*EmployStatus")

## Stratified view

grid.newpage()
pushViewport(viewport(layout = grid.layout(ncol = 2)))
pushViewport(viewport(layout.pos.col = 1))

## Closure
mosaic(Employment[,,1], main = "Layoff: Closure", newpage = FALSE)

popViewport(1)
pushViewport(viewport(layout.pos.col = 2))

## Replaced
mosaic(Employment[,,2], main = "Layoff: Replaced", newpage = FALSE)
popViewport(2)
}

\keyword{datasets}

\name{Saxony}
\alias{Saxony}
\docType{data}
\title{Families in Saxony}
\description{
  Data from Geissler, cited in Sokal & Rohlf (1969) and Lindsey (1995)
  on gender distributions in families in Saxony in the 19th century.
}
\usage{
data("Saxony")
}
\format{
  A 1-way table giving the number of male children in 6115 families of
  size 12.  The variable and its levels are
  \tabular{rll}{
    No \tab Name \tab Levels \cr
    1 \tab nMales \tab 0, 1, \dots, 12 \cr
  }
}

\references{
  J. K. Lindsey (1995),
  \emph{Analysis of Frequency and Count Data}.
  Oxford University Press, Oxford, UK.

  R. R. Sokal & F. J. Rohlf (1969),
  \emph{Biometry. The Principles and Practice of Statistics}.
  W. H. Freeman, San Francisco, CA.

  M. Friendly (2000),
  \emph{Visualizing Categorical Data}.
  SAS Institute, Cary, NC.
}
\source{
  M. Friendly (2000),
  Visualizing Categorical Data, pages 40--42.
}
\examples{
data("Saxony")
gf <- goodfit(Saxony, type = "binomial")
summary(gf)
plot(gf)
}
\keyword{datasets}

\name{binreg_plot}
\alias{binreg_plot}
\alias{grid_abline}
\title{Binary Regression Plot}
\description{
  Creates a display of observed and fitted values for a binary regression
  model with one numeric predictor, conditioned by zero or many co-factors.
}
\usage{
binreg_plot(model, main = NULL, xlab = NULL, ylab = NULL,
            xlim = NULL, ylim = NULL,
            pred_var = NULL, pred_range = c("data", "xlim"),
            group_vars = NULL, base_level = NULL, subset,
            type = c("response", "link"), conf_level = 0.95, delta = FALSE,
            pch = NULL, cex = 0.6, jitter_factor = 0.1,
            lwd = 5, lty = 1, point_size = 0, col_lines = NULL, col_bands = NULL, 
            legend = TRUE, legend_pos = NULL, legend_inset = c(0, 0.1),
            legend_vgap = unit(0.5, "lines"),
            labels = FALSE, labels_pos = c("right", "left"),
            labels_just = c("left","center"), labels_offset = c(0.01, 0),
            gp_main = gpar(fontface = "bold", fontsize = 14),
            gp_legend_frame = gpar(lwd = 1, col = "black"),
            gp_legend_title = gpar(fontface = "bold"),
            newpage = TRUE, pop = FALSE, return_grob = FALSE)
grid_abline(a, b, \dots)
}
\arguments{
  \item{model}{a binary regression model fitted with \code{\link[stats]{glm}}.}
  \item{main}{user-specified main title.}
  \item{xlab}{x-axis label. Defaults to the name of the (first) numeric predictor.}
  \item{ylab}{y-axis label. Defaults to the name of the response -
    within either 'P(...)' or 'logit(...)', depending on the response
    type.}
  \item{xlim}{Range of the x-axis. Defaults to the range of the numeric
    predictor.}
  \item{ylim}{Range of the y-axis. Defaults to the unit interval on
    probability scale or the fitted values range on the link scale,
    depending on \code{type}.}
  \item{pred_var}{character string of length 1 giving the name of
    the numeric predictor. Defaults to the first one found in the data
    set.}
  \item{pred_range}{\code{"data"}, \code{"xlim"}, or a numeric vector.
    If \code{"data"}, the numeric predictor corresponds to the observed values. If
    \code{"xlim"}, 100 values are taken from the \code{"xlim"}
    range. A numeric vector will be interpreted as the values to be predicted.}
  \item{group_vars}{optional character string of conditioning
    variables. Defaults to all factors found in the data set, response
    excluded. If \code{FALSE}, no variables are used for conditioning.}
  \item{base_level}{vector of length one. If the response is a vector,
    this specifies the base ('no effect') value of the
    response variable
    (e.g., "Placebo", 0, FALSE, etc.) and defaults
    to the first level for
    factor responses, or 0 for numeric/binary variables. This controls
    which observations will be plotted on the top or the bottom of the
    display. If the response is a matrix with success and failure
    column, this specifies the one to be interpreted as failure
    (default: 2), either as an integer, or as a
    string (\code{"success"}  or \code{"failure"}). The proportions of
    \emph{successes} will be plotted as observed values.}
  \item{subset}{an optional vector specifying a subset of the data
    rows. The value is evaluated in the data environment, so expressions
    can be used to select the data (see examples).}
  \item{type}{either "response" or "link" to select the scale of the
    fitted values. The y-axis will be adapted accordingly.}
  \item{conf_level}{confidence level used for calculating 
    confidence bands.}
  \item{delta}{logical; indicates whether the delta method should be
    employed for calculating the limits of the confidence band or not
    (see details).}
  \item{pch}{character or numeric vector of symbols used for plotting
    the (possibly conditioned) observed values, recycled as needed.}
  \item{cex}{size of the plot symbols (in lines).}
  \item{jitter_factor}{argument passed to \code{\link[base]{jitter}}
    used for the points representing the observed values.}
  \item{lwd}{Line width for the fitted values.}
  \item{lty}{Line type for the fitted values.}
  \item{point_size}{size of points for the fitted values in char units (default: 0, so
    no points are plotted).}
  \item{col_lines, col_bands}{character vector specifying the colors of the fitted
    lines and confidence bands,
    by default chosen with \code{\link[colorspace]{rainbow_hcl}}. The
    confidence bands are using alpha blending with alpha = 0.2.}
  \item{legend}{logical; if \code{TRUE} (default), a legend is drawn.}
  \item{legend_pos}{numeric vector of length 2, specifying x and y
    coordinates of the legend, or a character string (e.g., \code{"topleft"},
    \code{"center"} etc.). Defaults to \code{"topleft"} if the fitted curve's slope is
    positive, and \code{"topright"} else.}
  \item{legend_inset}{numeric vector or length 2 specifying the inset
    from the legend's x and y coordinates in npc units.}
  \item{legend_vgap}{vertical space between the legend's line entries.}
  \item{labels}{logical; if \code{TRUE}, labels corresponding to the
    factor levels are plotted next to the fitted lines.}
  \item{labels_pos}{either \code{"right"} or \code{"left"}, determining on which side
    of the fitted lines (start or end) the labels should be placed.}
  \item{labels_just}{character vector of length 2, specifying the
    relative justification of the labels to their coordinates. See the
    documentation of the \code{just} parameter of
    \code{\link[grid]{grid.text}} for more details.}
  \item{labels_offset}{numeric vector of length 2, specifying the offset    
    of the labels' coordinates in npc units.}
  \item{gp_main}{object of class \code{"gpar"} used for the main title.}
  \item{gp_legend_frame}{object of class \code{"gpar"} used for the
    legend frame.}
  \item{gp_legend_title}{object of class \code{"gpar"} used for the
    legend title.}
  \item{newpage}{logical; if \code{TRUE}, the plot is drawn on a new page.}
  \item{pop}{logical; if \code{TRUE}, all newly generated viewports are
    popped after plotting.}
  \item{return_grob}{logical. Should a snapshot of the display be
    returned as a grid grob?}
  \item{a}{intercept; alternatively, a regression model from which
    coefficients can be extracted via \code{\link[stats]{coef}}.}
  \item{b}{slope.}
  \item{\dots}{Further arguments passed to \code{\link[grid]{grid.abline}}.}
}
\details{
  The primary purpose of \code{binreg_plot()} is to visualize observed and
  fitted values for binary regression models (like the logistic or probit
  regression model) with one numeric predictor. If one or more
  categorical predictors are used in the model, the \emph{fitted} values are
  conditioned on them, i.e. separate curves are drawn corresponding to
  the factor level combinations. Thus, it shows a \emph{full-model plot}, not a
  conditional plot where several models would be fit to data subsets.
  
  The implementation relies on objects returned by
  \code{\link[stats]{glm}}, as it uses its \code{"terms"} and
  \code{"model"} components. 

  The function tries to determine suitable values for the legend and/or
  labels, but depending on the data, this might require some tweaking.

  By default, the limits of the confidence band are determined for the
  linear predictor (i.e., on the link scale) and transformed to response
  scale (if this is the chosen plot type) using the inverse link
  function. If \code{delta} is \code{TRUE}, the limits
  are determined on the response scale. Note that the resulting band using the
  delta method is symmetric around the fitted mean,
  but may exceed the unit interval (on the response scale) and 
  will be cut off.

  \code{grid_abline()} is a simple convenience wrapper for
  \code{\link[grid]{grid.abline}} with similar behavior than
  \code{\link[graphics]{abline}} in that it extracts coefficients from
  a regression model, if given instead of the intercept \code{a}.
}
\value{
  if \code{return_grob} is \code{TRUE}, a grob object corresponding to
  the plot. \code{NULL} (invisibly) else.
}
\references{
Michael Friendly (2000),
  \emph{Visualizing Categorical Data}.
  SAS Institute, Cary, NC.
}
\author{
  David Meyer \email{David.Meyer@R-project.org}
}

\examples{
## Simple model with no conditioning variables
art.mod0 <- glm(Improved > "None" ~ Age, data = Arthritis, family = binomial)

binreg_plot(art.mod0, "Arthritis Data")
binreg_plot(art.mod0, type = "link") ## logit scale

## one conditioning factor
art.mod1 <- update(art.mod0, . ~ . + Sex)
binreg_plot(art.mod1)
binreg_plot(art.mod1, legend = FALSE, labels = TRUE, xlim = c(20, 80))

## two conditioning factors
art.mod2 <- update(art.mod1, . ~ . + Treatment)
binreg_plot(art.mod2)
binreg_plot(art.mod2, subset = Sex == "Male") ## subsetting

## some tweaking
binreg_plot(art.mod2, gp_legend_frame = gpar(col = NA, fill = "white"), col_bands = NA)
binreg_plot(art.mod2, legend = FALSE, labels = TRUE,
            labels_pos = "left", labels_just = c("left", "top"))

## model with grouped response data
shuttle.mod <- glm(cbind(nFailures, 6 - nFailures) ~ Temperature,
                   data = SpaceShuttle, na.action = na.exclude, family = binomial)
binreg_plot(shuttle.mod, xlim = c(30, 81), pred_range = "xlim",
            ylab = "O-Ring Failure Probability", xlab = "Temperature (F)")
}
\keyword{category}
\keyword{hplot}

\name{structable}
\alias{structable.default}
\alias{structable.formula}
\alias{structable}
\alias{Extract.structable}
\alias{aperm.structable}
\alias{t.structable}
\alias{is.structable}
\alias{cbind.structable}
\alias{rbind.structable}
\alias{length.structable}
\alias{is.na.structable}
\alias{as.matrix.structable}
\alias{as.vector.structable}
\alias{dim.structable}
\alias{dimnames.structable}
\alias{as.table.structable}
\title{Structured Contingency Tables}
\description{
  This function produces a \sQuote{flat} representation of a
  high-dimensional contingency table constructed by recursive
  splits (similar to the construction of mosaic displays).
}
\usage{
\method{structable}{formula}(formula, data,
direction = NULL, split_vertical = NULL, \dots, subset, na.action)
\method{structable}{default}(\dots, direction = NULL, split_vertical = FALSE)
}
\arguments{
  \item{formula}{a formula object with possibly both left and right hand
    sides specifying the column and row variables of the flat table.}
  \item{data}{a data frame, list or environment containing the variables
    to be cross-tabulated, or an object inheriting from class \code{table}.}
  \item{subset}{an optional vector specifying a subset of observations
    to be used.
    Ignored if \code{data} is a contingency table.}
  \item{na.action}{a function which indicates what should happen when
    the data contain \code{NA}s.
    Ignored if \code{data} is a contingency table}
  \item{\dots}{\R objects which can be interpreted as factors (including
    character strings), or a list (or data frame) whose components can
    be so interpreted, or a contingency table object of class
    \code{"table"} or \code{"ftable"}.}
  \item{split_vertical}{logical vector indicating, for each dimension,
    whether it should be split vertically or not (default:
    \code{FALSE}). Values are recycled as needed.
    If the argument is of length 1, the value is alternated
    for all dimensions. Ignored if \code{direction} is provided.}
  \item{direction}{character vector alternatively specifying the
    splitting direction (\code{"h"} for horizontal and \code{"v"} for
    vertical splits).  Values are recycled as needed.  If the argument
    is of length 1, the value is alternated for all dimensions.}
}
\details{
  This function produces textual representations of mosaic displays, and
  thus \sQuote{flat} contingency tables. The formula interface is quite
  similar to the one of \code{\link{ftable}}, but also accepts the
  \code{\link{mosaic}}-like formula interface (empty left-hand
  side).  Note that even if the \code{\link{ftable}} interface is used,
  the \code{split_vertical} or \code{direction} argument is needed to
  specify the \emph{order} of the horizontal and vertical splits.
  If pretabulated data with a \code{Freq} column is used, than the
  left-hand side should be left empty---the \code{Freq} column will be
  handled correctly.

  \code{"structable"} objects can be subset using the \code{[} and \code{[[}
  operators, using either level indices or names (see
  examples). The corresponding replacement functions are available as well. In
  addition, appropriate \code{\link{aperm}}, \code{\link{cbind}},
  \code{\link{rbind}}, \code{\link{length}}, \code{\link{dim}}, and
  \code{\link{is.na}} methods do exist. 
}
\value{
  An object of class \code{"structable"},
  inheriting from class \code{"ftable"}, with the splitting
  information (\code{"split_vertical"}) as additional attribute.
}
\author{
  David Meyer \email{David.Meyer@R-project.org}
}

\references{
  Meyer, D., Zeileis, A., and Hornik, K. (2006),
  The strucplot framework: Visualizing multi-way contingency tables with
  \pkg{vcd}.
  \emph{Journal of Statistical Software}, \bold{17(3)}, 1-48.
  \doi{10.18637/jss.v017.i03} and available as 
  \code{vignette("strucplot")}.
}

\seealso{
  \code{\link{strucplot}},
  \code{\link{mosaic}},
  \code{\link[stats]{ftable}}
}

\examples{
structable(Titanic)
structable(Titanic, split_vertical = c(TRUE, TRUE, FALSE, FALSE))
structable(Titanic, direction = c("h","h","v","v"))
structable(Sex + Class ~ Survived + Age, data = Titanic)

## subsetting of structable objects
(hec <- structable(aperm(HairEyeColor)))

## The "[" operator treats structables as a block-matrix and selects parts of the matrix:
hec[1]
hec[2]
hec[1,c(2,4)]
hec["Male",c("Blue","Green")]

## replacement funcion:
tmp <- hec
(tmp[1,2:3] <- tmp[2,c(1,4)])

## In contrast, the "[[" operator treats structables as two-dimensional
## lists. Indexing conditions on specified levels and thus reduces the dimensionality:

## seek subtables conditioning on levels of the first dimension:
hec[[1]]
hec[[2]]

## Seek subtable from the first two dimensions, given the level "Male"
## of the first variable, and "Brown" from the second
## (the following two commands are equivalent):
hec[["Male"]][["Brown"]]
hec[[c("Male","Brown")]]

## Seeking subtables by conditioning on row and/or column variables:
hec[["Male","Hazel"]]
hec[[c("Male","Brown"),]]
hec[[c("Male","Brown"),"Hazel"]]

## a few other operations
t(hec)
dim(hec)
dimnames(hec)
as.matrix(hec)
length(hec)
cbind(hec[,1],hec[,3])

as.vector(hec) ## computed on the _multiway_ table
as.vector(unclass(hec))
}
\keyword{hplot}

\name{Punishment}
\alias{Punishment}
\docType{data}
\title{Corporal Punishment Data}
\description{
  Data from a study of the Gallup Institute in Denmark in 1979 about the
  attitude of a random sample of 1,456 persons towards corporal punishment
  of children.
}
\usage{
data("Punishment")
}
\format{
  A data frame with 36 observations and 5 variables.
  \describe{
    \item{Freq}{frequency.}
    \item{attitude}{factor indicating attitude: (no, moderate)
      punishment of children.}
    \item{memory}{factor indicating whether the person had memories of
      corporal punishment as a child (yes, no).}
    \item{education}{factor indicating highest level of education
      (elementary, secondary, high).}
    \item{age}{factor indicating age group in years (15-24, 25-39, 40-).}
  }
}

\note{Anderson (1991) erroneously indicates the total sum of respondents to be 783.}

\references{
  E. B. Andersen (1991),
  \emph{The Statistical Analysis of Categorical Data}.
  2nd edition.
  Springer-Verlag, Berlin.
}
\source{
  E. B. Andersen (1991),
  The Statistical Analysis of Categorical Data, pages 207--208.
}
\examples{
data("Punishment", package = "vcd")
pun <- xtabs(Freq ~ memory + attitude + age + education, data = Punishment)

## model: ~ (memory + attitude) * age * education
## use maximum sum-of-squares test/shading

cotabplot(~ memory + attitude | age + education, data = pun,
          panel = cotab_coindep, panel_args = list( 
  n = 5000, type = "assoc", test = "maxchisq", interpolate = 1:2))
}

\keyword{datasets}

\name{mosaic}
\alias{mosaic}
\alias{mosaic.default}
\alias{mosaic.formula}
\title{Extended Mosaic Plots}
\description{
  Plots (extended) mosaic displays.
}
\usage{
\method{mosaic}{default}(x, condvars = NULL,
  split_vertical = NULL, direction = NULL, spacing = NULL,
  spacing_args = list(), gp = NULL, expected = NULL, shade = NULL,
  highlighting = NULL, highlighting_fill = rev(gray.colors(tail(dim(x), 1))),
  highlighting_direction = NULL,
  zero_size = 0.5, zero_split = FALSE, zero_shade = NULL,
  zero_gp = gpar(col = 0), panel = NULL, main = NULL, sub = NULL, \dots)
\method{mosaic}{formula}(formula, data, highlighting = NULL,
  \dots, main = NULL, sub = NULL, subset = NULL, na.action = NULL)
}
\arguments{
  \item{x}{a contingency table in array form, with optional category
    labels specified in the \code{dimnames(x)} attribute, or an object
    of class \code{"structable"}.}
  \item{condvars}{vector of integers or character strings indicating
    conditioning variables, if any. The table will be permuted to order
    them first.}
  \item{formula}{a formula specifying the variables used to create a 
    contingency table from \code{data}.  For convenience, conditioning
    formulas can be specified; the conditioning variables will then be
    used first for splitting. If any, a specified response variable will be
    highlighted in the cells.}
  \item{data}{either a data frame, or an object of class \code{"table"}
    or \code{"ftable"}.}
  \item{subset}{an optional vector specifying a subset of observations to be used.}
  \item{na.action}{a function which indicates what should happen when
    the data contain \code{NA}s. Ignored if \code{data} is a contingency table.}
  \item{zero_size}{size of the bullets used for zero entries (if 0, no
    bullets are drawn).}
  \item{zero_split}{logical controlling whether zero cells should be
    further split. If \code{FALSE} and \code{zero_shade} is
    \code{FALSE}, only one bullet is drawn (centered) for unsplit
    zero cells. If \code{FALSE} and \code{zero_shade} is
    \code{TRUE}, a bullet for each zero cell is drawn to allow, e.g.,
    residual-based shadings to be effective also for zero cells.}
  \item{zero_shade}{logical controlling whether zero bullets should be
    shaded. The default is \code{TRUE} if \code{shade} is \code{TRUE}
    or \code{expected} is not null or \code{gp} is not null, and
   \code{FALSE} otherwise.}
  \item{zero_gp}{object of class \code{"gpar"} used
    for zero bullets in case they are \emph{not} shaded.}
  \item{split_vertical}{vector of logicals of length \eqn{k}, where \eqn{k}
    is the number of margins of \code{x} (default: \code{FALSE}).
    Values are recycled as needed. 
    A \code{TRUE} component indicates that the tile(s) of the
    corresponding dimension should be split vertically, \code{FALSE}
    means horizontal splits. Ignored if \code{direction} is not \code{NULL}.}
  \item{direction}{character vector of length \eqn{k}, where \eqn{k} is the
    number of margins of \code{x} (values are recycled as needed).
    For each component, a value of \code{"h"} indicates that the tile(s)
    of the corresponding dimension should be split horizontally, whereas
    \code{"v"} indicates vertical split(s).}
  \item{spacing}{spacing object, spacing function, or corresponding
    generating function (see \code{\link{strucplot}} for more
    information).
    The default is \code{spacing_equal} if \code{x} has two dimensions,
    \code{spacing_increase} for more dimensions, and
    \code{spacing_conditional} if conditioning variables are specified
    using \code{condvars} or the formula interface.}
  \item{spacing_args}{list of arguments for the generating function, if
    specified (see \code{\link{strucplot}} for more information).}
  \item{gp}{object of class \code{"gpar"}, shading function or a
    corresponding generating function (see details and
    \code{\link{shadings}}). Components of \code{"gpar"}
    objects are recycled as needed along the last splitting dimension.
    Ignored if \code{shade = FALSE}.}
  \item{shade}{logical specifying whether \code{gp} should be used or not
    (see \code{gp}). If \code{TRUE} and \code{expected} is unspecified,
    a default model is fitted: if \code{condvars} (see
    \code{\link{strucplot}}) is specified, a
    corresponding conditional independence model, and else the total
    independence model.}
  \item{expected}{optionally, an array of expected values of the same dimension
    as \code{x}, or alternatively the corresponding independence model specification
    as used by \code{\link[stats]{loglin}} or \code{\link[MASS]{loglm}}
    (see \code{\link{strucplot}}).}
  \item{highlighting}{character vector or integer specifying a variable
    to be highlighted in the cells.}
  \item{highlighting_fill}{color vector or palette function
    used for a highlighted variable, if any.}
  \item{highlighting_direction}{Either \code{"left"}, \code{"right"},
    \code{"top"}, or \code{"bottom"} specifying the direction of
    highlighting in the cells.}
  \item{panel}{Optional function with arguments: \code{residuals},
    \code{observed}, \code{expected}, \code{index}, \code{gp}, and
    \code{name} called by the
    \code{struc_mosaic} workhorse for each tile that is drawn in the
    mosaic. \code{index} is an integer vector with the tile's
    coordinates in the contingency table, \code{gp} a \code{gpar} object
    for the tile, and \code{name} a label to be assigned to the drawn
    grid object.}
  \item{main, sub}{either a logical, or a character string used for plotting
    the main (sub) title.  If logical and \code{TRUE}, the
    name of the \code{data} object is used.}
  \item{\dots}{Other arguments passed to \code{\link{strucplot}}}
}
\details{
  Mosaic displays have been suggested in the statistical literature
  by Hartigan and Kleiner (1984) and have been extended by Friendly
  (1994). \code{\link[graphics]{mosaicplot}} is a base graphics
  implementation and \code{mosaic} is a much more flexible and extensible
  grid implementation.

  \code{mosaic} is a generic function which currently has a default method and a
  formula interface.  Both are high-level interfaces to the
  \code{\link{strucplot}} function, and produce (extended) mosaic
  displays.  Most of the functionality is described there, such as
  specification of the independence model, labeling, legend, spacing,
  shading, and other graphical parameters.

  A mosaic plot is an area proportional visualization of a (possibly
  higher-dimensional) table of expected frequencies.  It is composed of
  tiles (corresponding to the cells) created by recursive vertical and
  horizontal splits of a square.  The area of each tile is proportional
  to the corresponding cell entry, \emph{given} the dimensions of
  previous splits.

  An \emph{extended} mosaic plot, in addition, visualizes the fit of a
  particular log-linear model. Typically, this is done by residual-based
  shadings where color and/or outline of the tiles visualize sign, size
  and possibly significance of the corresponding residual.
  
  The layout is very flexible: the specification of shading, labeling,
  spacing, and legend is modularized (see \code{\link{strucplot}} for
  details).

  In contrast to the \code{\link[graphics]{mosaicplot}} function in
  \pkg{graphics}, the splits start with the \emph{horizontal} direction
  by default to match the printed output of \code{\link{structable}}.
}
\value{
  The \code{"structable"} visualized is returned invisibly.
}
\references{
  Hartigan, J.A., and Kleiner, B. (1984),
  A mosaic of television ratings.
  \emph{The American Statistician},
  \bold{38}, 32--35.

  Emerson, J. W. (1998),
  Mosaic displays in S-PLUS: A general implementation and a case study.
  \emph{Statistical Computing and Graphics Newsletter (ASA)},
  \bold{9}, 1, 17--23.

  Friendly, M. (1994),
  Mosaic displays for multi-way contingency tables.
  \emph{Journal of the American Statistical Association}, \bold{89},
  190--200.

  Meyer, D., Zeileis, A., and Hornik, K. (2006),
  The strucplot framework: Visualizing multi-way contingency tables with
  \pkg{vcd}.
  \emph{Journal of Statistical Software}, \bold{17(3)}, 1-48.
  \doi{10.18637/jss.v017.i03} and available as 
  \code{vignette("strucplot", package = "vcd")}.

  The home page of Michael Friendly
  (\url{http://datavis.ca}) provides
  information on various aspects of graphical methods for analyzing
  categorical data, including mosaic plots. In particular, there are
  many materials for his course \dQuote{Visualizing Categorical Data with SAS and R}
  at \url{http://datavis.ca/courses/VCD/}.
}
\author{
  David Meyer \email{David.Meyer@R-project.org}
}

\seealso{
  \code{\link{assoc}},
  \code{\link{strucplot}},
  \code{\link{mosaicplot}},
  \code{\link{structable}},
  \code{\link{doubledecker}}
}
\examples{
library(MASS)
data("Titanic")
mosaic(Titanic)

## Formula interface for tabulated data plus shading and legend:
mosaic(~ Sex + Age + Survived, data = Titanic,
  main = "Survival on the Titanic", shade = TRUE, legend = TRUE)

data("HairEyeColor")
mosaic(HairEyeColor, shade = TRUE)
## Independence model of hair and eye color and sex.  Indicates that
## there are significantly more blue eyed blond females than expected
## in the case of independence (and too few brown eyed blond females).

mosaic(HairEyeColor, shade = TRUE, expected = list(c(1,2), 3))
## Model of joint independence of sex from hair and eye color.  Males
## are underrepresented among people with brown hair and eyes, and are
## overrepresented among people with brown hair and blue eyes, but not
## "significantly".

## Formula interface for raw data: visualize crosstabulation of numbers
## of gears and carburettors in Motor Trend car data.
data("mtcars")
mosaic(~ gear + carb, data = mtcars, shade = TRUE)

data("PreSex")
mosaic(PreSex, condvars = c(1,4))
mosaic(~ ExtramaritalSex + PremaritalSex | MaritalStatus + Gender,
       data = PreSex)

## Highlighting:
mosaic(Survived ~ ., data = Titanic)

data("Arthritis")
mosaic(Improved ~ Treatment | Sex, data = Arthritis, zero_size = 0)
mosaic(Improved ~ Treatment | Sex, data = Arthritis, zero_size = 0,
       highlighting_direction = "right")
}
\keyword{hplot}

\name{cd_plot}
\alias{cd_plot}
\alias{cd_plot.default}
\alias{cd_plot.formula}
\title{Conditional Density Plots}
\description{
  Computes and plots conditional densities describing how the
  distribution of a categorical variable \code{y} changes over a
  numerical variable \code{x}.
}
\usage{
cd_plot(x, \dots)
\method{cd_plot}{default}(x, y,
  plot = TRUE, ylab_tol = 0.05,
  bw = "nrd0", n = 512, from = NULL, to = NULL,
  main = "", xlab = NULL, ylab = NULL, margins = c(5.1, 4.1, 4.1, 3.1),
  gp = gpar(), name = "cd_plot", newpage = TRUE, pop = TRUE, return_grob = FALSE, \dots)
\method{cd_plot}{formula}(formula, data = list(),
  plot = TRUE, ylab_tol = 0.05,
  bw = "nrd0", n = 512, from = NULL, to = NULL,
  main = "", xlab = NULL, ylab = NULL, margins = c(5.1, 4.1, 4.1, 3.1),
  gp = gpar(), name = "cd_plot", newpage = TRUE, pop = TRUE, return_grob = FALSE, \dots)
}
\arguments{
  \item{x}{an object, the default method expects either a single numerical variable.}
  \item{y}{a \code{"factor"} interpreted to be the dependent variable}
  \item{formula}{a \code{"formula"} of type \code{y ~ x} with a single dependent \code{"factor"}
    and a single numerical explanatory variable.}    
  \item{data}{an optional data frame.}
  \item{plot}{logical. Should the computed conditional densities be plotted?}
  \item{ylab_tol}{convenience tolerance parameter for y-axis annotation.
    If the distance between two labels drops under this threshold, they are
    plotted equidistantly.}
  \item{bw, n, from, to, \dots}{arguments passed to \code{\link{density}}}
  \item{main, xlab, ylab}{character strings for annotation}
  \item{margins}{margins when calling \code{\link{plotViewport}}}
  \item{gp}{a \code{"gpar"} object controlling the grid graphical
    parameters of the rectangles. It should specify in particular a vector of
    \code{fill} colors of the same length as \code{levels(y)}. The default is
    to call \code{\link{gray.colors}}.}  
  \item{name}{name of the plotting viewport.}
  \item{newpage}{logical. Should \code{\link{grid.newpage}} be called 
    before plotting?}
  \item{return_grob}{logical. Should a snapshot of the display be
    returned as a grid grob?}
  \item{pop}{logical. Should the viewport created be popped?}
}
\details{
  \code{cd_plot} computes the conditional densities of \code{x} given
  the levels of \code{y} weighted by the marginal distribution of \code{y}.
  The densities are derived cumulatively over the levels of \code{y}.

  This visualization technique is similar to spinograms (see \code{\link{spine}})
  but they do not discretize the explanatory variable, but rather use a smoothing
  approach. Furthermore, the original x axis and not a distorted x axis (as for
  spinograms) is used. This typically results in conditional densities that
  are based on very few observations in the margins: hence, the estimates are less
  reliable there.
}
\value{
  The conditional density functions (cumulative over the levels of \code{y})
  are returned invisibly.
}
\seealso{
  \code{\link{spine}}, \code{\link{density}}
}
\references{
  Hofmann, H., Theus, M. (2005), \emph{Interactive graphics for visualizing
  conditional distributions}, Unpublished Manuscript.
}
\author{
  Achim Zeileis \email{Achim.Zeileis@R-project.org}
}
\examples{
## Arthritis data
data("Arthritis")
cd_plot(Improved ~ Age, data = Arthritis)
cd_plot(Improved ~ Age, data = Arthritis, bw = 3)
cd_plot(Improved ~ Age, data = Arthritis, bw = "SJ")
## compare with spinogram
spine(Improved ~ Age, data = Arthritis, breaks = 3)

## Space shuttle data
data("SpaceShuttle")
cd_plot(Fail ~ Temperature, data = SpaceShuttle, bw = 2)

## scatter plot with conditional density
cdens <- cd_plot(Fail ~ Temperature, data = SpaceShuttle, bw = 2, plot = FALSE)
plot(I(-1 * (as.numeric(Fail) - 2)) ~ jitter(Temperature, factor = 2), data = SpaceShuttle,
  xlab = "Temperature", ylab = "Failure")
lines(53:81, cdens[[1]](53:81), col = 2)
}
\keyword{hplot}

\name{shadings}
\alias{shadings}
\alias{shading_hsv}
\alias{shading_hcl}
\alias{shading_max}
\alias{shading_Friendly}
\alias{shading_Friendly2}
\alias{shading_Marimekko}
\alias{shading_diagonal}
\alias{shading_sieve}
\alias{shading_binary}
\alias{hcl2hex}
\encoding{UTF-8}
\title{Shading-generating Functions for Residual-based Shadings}
\description{
  Shading-generating functions for computing residual-based shadings 
  for mosaic and association plots.
}
\usage{
shading_hcl(observed, residuals = NULL, expected = NULL, df = NULL,
  h = NULL, c = NULL, l = NULL, interpolate = c(2, 4), lty = 1,
  eps = NULL, line_col = "black", p.value = NULL, level = 0.95, \dots)

shading_hsv(observed, residuals = NULL, expected = NULL, df = NULL,
  h = c(2/3, 0), s = c(1, 0), v = c(1, 0.5),
  interpolate = c(2, 4), lty = 1, eps = NULL, line_col = "black",
  p.value = NULL, level = 0.95, \dots)

shading_max(observed = NULL, residuals = NULL, expected = NULL, df = NULL,
  h = NULL, c = NULL, l = NULL, lty = 1, eps = NULL, line_col = "black",
  level = c(0.9, 0.99), n = 1000, \dots)

shading_Friendly(observed = NULL, residuals = NULL, expected = NULL, df = NULL,
  h = c(2/3, 0), lty = 1:2, interpolate = c(2, 4),
  eps = 0.01, line_col = "black", \dots)

shading_Friendly2(observed = NULL, residuals = NULL,
  expected = NULL, df = NULL, lty = 1:2, interpolate = c(2, 4), eps =
  0.01, line_col = "black", \dots)

shading_sieve(observed = NULL, residuals = NULL, expected = NULL, df = NULL,
  h = c(260, 0), lty = 1:2, interpolate = c(2, 4),
  eps = 0.01, line_col = "black", \dots)

shading_binary(observed = NULL, residuals = NULL, expected = NULL, df = NULL,
  col = NULL)

shading_Marimekko(x, fill = NULL, byrow = FALSE)

shading_diagonal(x, fill = NULL)

hcl2hex(h = 0, c = 35, l = 85, fixup = TRUE)
}
\arguments{
  \item{observed}{contingency table of observed values}
  \item{residuals}{contingency table of residuals}
  \item{expected}{contingency table of expected values}
  \item{df}{degrees of freedom of the associated independence model.}  
  \item{h}{hue value in the HCL or HSV color description, has to be
    in [0, 360] for HCL and in [0, 1] for HSV colors. The default is
    to use blue and red for positive and negative residuals respectively.
    In the HCL specification it is \code{c(260, 0)} by default and for HSV
    \code{c(2/3, 0)}.}
  \item{c}{chroma value in the HCL color description. This controls the maximum
    chroma for significant and non-significant results respectively and defaults 
    to \code{c(100, 20)}.}
  \item{l}{luminance value in the HCL color description. Defaults to \code{c(90, 50)}
    for small and large residuals respectively.}
  \item{s}{saturation value in the HSV color description. Defaults to \code{c(1, 0)}
    for large and small residuals respectively.}
  \item{v}{saturation value in the HSV color description. Defaults to \code{c(1, 0.5)}
    for significant and non-significant results respectively.}
  \item{interpolate}{a specification for mapping the absolute size of the residuals to
    a value in [0, 1]. This can be either a function or a numeric vector. In the latter
    case, a step function with steps of equal size going from 0 to 1 is used.}
  \item{lty}{a vector of two line types for positive and negative residuals respectively.
    Recycled if necessary.}
  \item{eps}{numeric tolerance value below which absolute residuals are considered to be
    zero, which is used for coding the border color and line type. If set to \code{NULL} (default),
    all borders have the default color specified by \code{line\_col}.
    If set to a numeric value, all border colors corresponding
    to residuals with a larger absolute value are set to the full positive or negative
    color, respectively;  borders corresponding to smaller residuals are are drawn with
    \code{line\_col} and \code{lty[1]}}.  This is used principally in \code{shading\_Friendly}.
  \item{line_col}{default border color (for \code{shading_sieve}: default sieve color).}
  \item{p.value}{the \eqn{p} value associated with the independence model. By default,
    this is computed from a Chi-squared distribution with \code{df} degrees of freedom.
    \code{p.value} can be either a scalar or a \code{function(observed, residuals, expected, df)}
    that computes the \eqn{p} value from the data. If set to \code{NA} no inference
    is performed.}
  \item{level}{confidence level of the test used. If \code{p.value} is smaller than
    \code{1 - level}, bright colors are used, otherwise dark colors are employed. For
    \code{shading_max} a vector of levels can be supplied. The corresponding critical
    values are then used as \code{interpolate} cut-offs.}
  \item{n}{number of permutations used in the call to \code{coindep_test}.}
  \item{col}{a vector of two colors for positive and negative residuals respectively.}
  \item{fixup}{logical. Should the color be corrected to a valid RGB
    value before correction?}
  \item{x}{object of class \code{table} used to determine the
    dimension.}
  \item{fill}{Either a character vector of color codes, or a palette
    function that generates such a vector. Defaults to
    \code{\link[colorspace]{rainbow_hcl}}}
  \item{byrow}{logical; shall tiles be filled by row or by column?}
  \item{\dots}{Other arguments passed to \code{\link{hcl2hex}}
    or \code{\link{hsv}}, respectively.}
}
\details{
  These shading-generating functions can be passed to \code{strucplot} to generate
  residual-based shadings for contingency tables. \code{strucplot} calls these
  functions with the arguments \code{observed}, \code{residuals}, \code{expected},
  \code{df} which give the observed values, residuals, expected values and associated
  degrees of freedom for a particular contingency table and associated independence 
  model.
  
  The shadings \code{shading_hcl} and \code{shading_hsv} do the same thing conceptually,
  but use HCL or HSV colors respectively. The former is usually preferred because they
  are perceptually based. Both shadings visualize the \emph{sign} of the residuals of
  an independence model using two hues (by default: blue and red). The \emph{absolute size} of 
  the residuals is visualized by the colorfulness and the amount of grey, by default in three categories:
  very colorful for large residuals (> 4), less colorful for medium sized residuals (< 4 and > 2),
  grey/white for small residuals (< 2). More categories or a continuous scale can
  be specified by setting \code{interpolate}. Furthermore, the result of a significance
  test can be visualized by the amount of grey in the colors. If significant, a colorful
  palette is used, if not, the amount of color is reduced.
  See Zeileis, Meyer, and Hornik (2007) and \code{\link[colorspace]{diverge_hcl}} for more details.
  
  The shading \code{shading_max} is applicable in 2-way contingency tables and uses
  a similar strategy as \code{shading_hcl}. But instead of using the cut-offs 2 and 4,
  it employs the critical values for the maximum statistic (by default at 90\% and 99\%).
  Consequently, color in the plot signals a significant result at 90\% or 99\% significance
  level, respectively. The test is carried out by calling \code{\link{coindep_test}}.
  
  The shading \code{shading_Friendly} is very similar to \code{shading_hsv}, but additionally
  codes the sign of the residuals by different line types. See Friendly
  (1994) for more details. \code{shading_Friendly2} and
  \code{shading_sieve} are similar, but use HCL colors.
  
  The shading \code{shading_binary} just visualizes the sign of the residuals by using
  two different colors (default: blue HCL(260, 50, 70) and red HCL(0, 50, 70)).

  \code{shading_Marimekko} is a simple generating function for
  producing, in conjunction with \code{\link{mosaic}}, so-called
  \emph{Marimekko-charts}, which paint the tiles of each columns of a
  mosaic display in the same color to better display departures from
  independence.

  \code{shading_diagonal} generates a color shading for basically square
  matrices (or arrays having the first two dimensons of same length)
  visualizing the diagonal cells, and the off-diagonal cells 1, 2, \dots
  steps removed.
  
  
  
  
  The color implementations employed are \code{\link{hsv}}
  from base R and \code{\link[colorspace]{polarLUV}} from the \pkg{colorspace}
  package, respectively. To transform the HCL coordinates to
  a hexadecimal color string (as returned by \code{hsv}), the function
  \code{\link[colorspace]{hex}} is employed. A convenience wrapper \code{hcl2hex}
  is provided.
}

\references{
  Friendly M. (1994),
  Mosaic Displays for Multi-Way Contingency Tables.
  \emph{Journal of the American Statistical Association},
  \bold{89}, 190--200.

  Meyer D., Zeileis A., and Hornik K. (2006),
  The Strucplot Framework: Visualizing Multi-Way Contingency Tables with \pkg{vcd}.
  \emph{Journal of Statistical Software}, \bold{17}(3), 1--48.
  \doi{10.18637/jss.v017.i03}. See also \code{vignette("strucplot", package = "vcd")}.

  Zeileis A., Meyer D., Hornik K. (2007), Residual-Based Shadings for Visualizing
  (Conditional) Independence. \emph{Journal of Computational and Graphical Statistics},
  \bold{16}, 507--525.

  Zeileis A., Hornik K. and Murrell P. (2008),
  Escaping RGBland: Selecting Colors for Statistical Graphics.
  \emph{Computational Statistics & Data Analysis}, \bold{53}, 3259--3270.
  Preprint available from \url{https://www.zeileis.org/papers/Zeileis+Hornik+Murrell-2009.pdf}.
}

\value{A shading function which takes only a single argument, interpreted as a
  vector/table of residuals, and returns a \code{"gpar"} object with the
  corresponding vector(s)/table(s) of graphical parameter(s).

}

\author{
  Achim Zeileis \email{Achim.Zeileis@R-project.org}
}
\seealso{
  \code{\link[colorspace]{hex}},
  \code{\link[colorspace]{polarLUV}},
  \code{\link{hsv}},
  \code{\link{mosaic}},
  \code{\link{assoc}},
  \code{\link{strucplot}},
  \code{\link[colorspace]{diverge_hcl}}
}
\examples{
## load Arthritis data
data("Arthritis")
art <- xtabs(~Treatment + Improved, data = Arthritis)

## plain mosaic display without shading
mosaic(art)

## with shading for independence model
mosaic(art, shade = TRUE)
## which uses the HCL shading
mosaic(art, gp = shading_hcl)
## the residuals are too small to have color,
## hence the cut-offs can be modified
mosaic(art, gp = shading_hcl, gp_args = list(interpolate = c(1, 1.8)))
## the same with the Friendly palette 
## (without significance testing)
mosaic(art, gp = shading_Friendly, gp_args = list(interpolate = c(1, 1.8)))

## assess independence using the maximum statistic
## cut-offs are now critical values for the test statistic
mosaic(art, gp = shading_max)

## association plot with shading as in base R
assoc(art, gp = shading_binary(col = c(1, 2)))

## Marimekko Chart
hec <- margin.table(HairEyeColor, 1:2)
mosaic(hec, gp = shading_Marimekko(hec))
mosaic(HairEyeColor, gp = shading_Marimekko(HairEyeColor))

## Diagonal cells shading
ac <- xtabs(VisualAcuity)
mosaic(ac, gp = shading_diagonal(ac))

}
\keyword{hplot}

\name{SpaceShuttle}
\alias{SpaceShuttle}
\docType{data}
\title{Space Shuttle O-ring Failures}
\description{
  Data from Dalal et al. (1989) about O-ring failures in the NASA space
  shuttle program.  The damage index comes from a discussion of the data
  by Tufte (1997).
}
\usage{
data("SpaceShuttle")
}
\format{
  A data frame with 24 observations and 6 variables.
  \describe{
    \item{FlightNumber}{Number of space shuttle flight.}
    \item{Temperature}{temperature during start (in degrees F).}
    \item{Pressure}{pressure.}
    \item{Fail}{did any O-ring failures occur? (no, yes).}
    \item{nFailures}{how many (of six) 0-rings failed?.}
    \item{Damage}{damage index.}
  }
}
\references{
  S. Dalal, E. B. Fowlkes, B. Hoadly (1989),
  Risk analysis of the space shuttle: Pre-Challenger prediction of
  failure,
  \emph{Journal of the American Statistical Association},
  \bold{84}, 945--957.

  E. R. Tufte (1997),
  \emph{Visual Explanations: Images and Quantities, Evidence and
    Narrative}.
  Graphics Press, Cheshire, CT. 

  M. Friendly (2000),
  \emph{Visualizing Categorical Data}.
  SAS Institute, Cary, NC.
}

\source{
  Michael Friendly (2000),
  Visualizing Categorical Data:
  \url{http://euclid.psych.yorku.ca/ftp/sas/vcd/catdata/orings.sas}
}
\examples{
data("SpaceShuttle")
plot(nFailures/6 ~ Temperature, data = SpaceShuttle,
     xlim = c(30, 81), ylim = c(0,1),
     main = "NASA Space Shuttle O-Ring Failures",
     ylab = "Estimated failure probability",
     pch = 19, col = 4)
fm <- glm(cbind(nFailures, 6 - nFailures) ~ Temperature,
          data = SpaceShuttle,
          family = binomial)
lines(30 : 81,
      predict(fm, data.frame(Temperature = 30 : 81), type = "re"),
      lwd = 2)
abline(v = 31, lty = 3)
}

\keyword{datasets}

\name{Hospital}
\alias{Hospital}
\docType{data}
\title{Hospital data}
\description{
The table relates the length of stay (in years) of 132
long-term schizophrenic patients in two London mental hospitals with
the frequency of visits.
}
\usage{
data("Hospital")
}
\format{
  A 2-dimensional array resulting from cross-tabulating 
  132 patients.  The variables and their levels are as follows:

  \tabular{rll}{
    No \tab Name \tab Levels \cr
    1 \tab Visit Frequency \tab Regular, Less than monthly, Never \cr
    2 \tab Length of Stay \tab 2--9 years, 10--19 years, 20+ years
  }
}

\references{
  J.K. Wing (1962): Institutionalism in mental hospitals.
  British Journal of Social Clinical Psychology, 1:38--51.
}

\source{
S.J Haberman (1974): Log-linear models for frequency tables with ordered
classifications. Biometrics, 30:689--700.
}

\details{
Wing (1962) who collected this data concludes 
that the longer the length of stay in hospital, the less frequent the
visits.

Haberman (1974) notes that this pattern does not increase from the "Less
than monthly" to the "Never" group, which are homogeneous.
}

\examples{
data("Hospital")

mosaic(t(Hospital), shade = TRUE)
mosaic(Hospital, shade = TRUE)
sieve(Hospital, shade = TRUE)
assoc(Hospital, shade = TRUE)
}

\keyword{datasets}

\name{Federalist}
\alias{Federalist}
\docType{data}
\title{`May' in Federalist Papers}
\description{
  Data from Mosteller & Wallace (1984) investigating the use of certain
  keywords (\sQuote{may} in this data set) to identify the author of 12
  disputed \sQuote{Federalist Papers} by Alexander Hamilton, John Jay
  and James Madison.
}
\usage{
data("Federalist")
}
\format{
  A 1-way table giving the number of occurrences of \sQuote{may} in 262
  blocks of text.  The variable and its levels are
  \tabular{rll}{
    No \tab Name \tab Levels \cr
    1 \tab nMay \tab 0, 1, \dots, 6 \cr
  }
}
\references{
  F. Mosteller & D. L. Wallace (1984),
  \emph{Applied Bayesian and Classical Inference: The Case of the
    Federalist Papers}.
  Springer-Verlag, New York, NY. 

  M. Friendly (2000),
  \emph{Visualizing Categorical Data}.
  SAS Institute, Cary, NC.
}
\source{
  Michael Friendly (2000),
  Visualizing Categorical Data, page 19.
}
\examples{
data("Federalist")
gf <- goodfit(Federalist, type = "nbinomial")
summary(gf)
plot(gf)
}
\keyword{datasets}

\name{WomenQueue}
\alias{WomenQueue}
\docType{data}
\title{Women in Queues}
\description{
  Data from Jinkinson & Slater (1981) and Hoaglin & Tukey (1985)
  reporting the frequency distribution of females in 100 queues of
  length 10 in a London Underground station.
}
\usage{
data("WomenQueue")
}
\format{
  A 1-way table giving the number of women in 100 queues of length
  10.  The variable and its levels are 
  \tabular{rll}{
    No \tab Name \tab Levels \cr
    1 \tab nWomen \tab 0, 1, \dots, 10 \cr
  }
}
\references{
  D. C. Hoaglin & J. W. Tukey (1985),
  Checking the shape of discrete distributions.
  In D. C. Hoaglin, F. Mosteller, J. W. Tukey (eds.),
  \emph{Exploring Data Tables, Trends and Shapes}, chapter 9.
  John Wiley & Sons, New York.

  R. A. Jinkinson & M. Slater (1981),
  Critical discussion of a graphical method for identifying discrete
  distributions,
  \emph{The Statistician}, \bold{30}, 239--248.

  M. Friendly (2000),
  \emph{Visualizing Categorical Data}.
  SAS Institute, Cary, NC.
}

\source{
  M. Friendly (2000),
  Visualizing Categorical Data,
  pages 19--20.
}

\examples{
data("WomenQueue")
gf <- goodfit(WomenQueue, type = "binomial")
summary(gf)
plot(gf)
}

\keyword{datasets}

\name{Trucks}
\alias{Trucks}
\docType{data}
\title{Truck Accidents Data}
\description{
  Data from a study in England in two periods from November 1969 to
  October 1971 and November 1971 to October 1973.  A new compulsory
  safety measure for trucks was introduced in October 1971.  Therefore,
  the question is whether the safety measure had an effect on the number
  of accidents and on the point of collision on the truck.
}
\usage{
data("Trucks")
}
\format{
  A data frame with 24 observations on 5 variables.
  \describe{
    \item{Freq}{frequency of accidents involving trucks.}
    \item{period}{factor indicating time period (before, after)
      1971-11-01.} 
    \item{collision}{factor indicating whether the collision was in the
      back or forward (including the front and the sides) of the truck
      (back, forward).}
    \item{parked}{factor indicating whether the truck was parked (yes,
      no).}
    \item{light}{factor indicating light conditions: day light (daylight),
      night on an illuminated road (night, illuminate), night on a dark
      road (night, dark).}
  }
}
\references{
  E. B. Andersen (1991),
  \emph{The Statistical Analysis of Categorical Data}.
  2nd edition.
  Springer-Verlag, Berlin.
}
\source{
  E. B. Andersen (1991),
  The Statistical Analysis of Categorical Data, Table 6.8.
}
\examples{
library(MASS)
data("Trucks")
tab <- xtabs(Freq ~ period + collision + light + parked, data = Trucks)
loglm(~ (collision + period) * parked * light, data = tab)
doubledecker(collision ~ parked + light + period, data = tab)
cotabplot(tab, panel = cotab_coindep)
}
\keyword{datasets}

\name{struc_mosaic}
\alias{struc_mosaic}
\title{Core-generating Function for Mosaic Plots}
\description{
  Core-generating function for \code{strucplot} returning a function
  producing mosaic plots.
}
\usage{
struc_mosaic(zero_size = 0.5, zero_split = FALSE, zero_shade = TRUE,
             zero_gp = gpar(col = 0), panel = NULL)
}
\arguments{
  \item{zero_size}{size of the bullets used for zero-entries in the
    contingency table (if 0, no bullets are drawn).} 
  \item{zero_split}{logical controlling whether zero cells should be
    further split. If \code{FALSE} and \code{zero_shade} is
    \code{FALSE}, only one bullet is drawn (centered) for unsplit
    zero cells. If \code{FALSE} and \code{zero_shade} is
    \code{TRUE}, a bullet for each zero cell is drawn to allow, e.g.,
    residual-based shadings to be effective also for zero cells.}
  \item{zero_shade}{logical controlling whether zero bullets should be
    shaded.}
  \item{zero_gp}{object of class \code{"gpar"} used
    for zero bullets in case they are \emph{not} shaded.}
  \item{panel}{Optional function with arguments: \code{residuals},
    \code{observed}, \code{expected}, \code{index}, \code{gp}, and
    \code{name} called by the
    \code{struc_mosaic} workhorse for each tile that is drawn in the
    mosaic. \code{index} is an integer vector with the tile's
    coordinates in the contingency table, \code{gp} a \code{gpar} object
    for the tile, and \code{name} a label to be assigned to the drawn
    grid object.}
}
\details{
  This function is usually called by \code{\link{strucplot}} (typically
  when called by \code{\link{mosaic}}) and returns a function used by
  \code{\link{strucplot}} to produce mosaic plots. 
}
\value{
  A function with arguments:
  \item{residuals}{table of residuals.}
  \item{observed}{table of observed values.}
  \item{expected}{not used by \code{struc_mosaic}.}
  \item{spacing}{object of class \code{"unit"} specifying the space
    between the tiles.}
  \item{gp}{list of \code{gpar} objects used for the drawing the tiles.}
  \item{split_vertical}{vector of logicals indicating, for each
    dimension of the table, the split direction.}
}
\author{
  David Meyer \email{David.Meyer@R-project.org}
}
\seealso{
  \code{\link{mosaic}},
  \code{\link{strucplot}},
  \code{\link{structable}}
}
\references{
  Cohen, A. (1980),
  On the graphical display of the significant components in a two-way
  contingency table.
  \emph{Communications in Statistics---Theory and Methods}, \bold{A9},
  1025--1041.

  Friendly, M. (1992),
  Graphical methods for categorical data.
  \emph{SAS User Group International Conference Proceedings}, \bold{17},
  190--200.
  \url{http://datavis.ca/papers/sugi/sugi17.pdf}

  Meyer, D., Zeileis, A., and Hornik, K. (2006),
  The strucplot framework: Visualizing multi-way contingency tables with
  \pkg{vcd}.
  \emph{Journal of Statistical Software}, \bold{17(3)}, 1-48.
  \doi{10.18637/jss.v017.i03} and available as 
  \code{vignette("strucplot")}.
}
\examples{
## Titanic data
data("Titanic")
## mosaic plot with large zeros
strucplot(Titanic, core = struc_mosaic(zero_size = 1))
}
\keyword{hplot}

\name{DanishWelfare}
\alias{DanishWelfare}
\docType{data}
\title{Danish Welfare Study Data}
\description{
  Data from the Danish Welfare Study.
}
\usage{data("DanishWelfare")}
\format{
  A data frame with 180 observations and 5 variables.
  \describe{
    \item{Freq}{frequency.}
    \item{Alcohol}{factor indicating daily alcohol consumption: less
      than 1 unit (<1), 1-2 units (1-2) or more than 2 units (>2).
      1 unit is approximately 1 bottle of beer or 4cl 40\% alcohol.}
    \item{Income}{factor indicating income group in 1000 DKK (0-50,
      50-100, 100-150, >150).} 
    \item{Status}{factor indicating marriage status (Widow, Married,
      Unmarried).}
    \item{Urban}{factor indicating urbanization: Copenhagen
      (Copenhagen), Suburbian Copenhagen (SubCopenhagen), three largest
      cities (LargeCity), other cities (City), countryside (Country).}
  }
}

\references{
  E. B. Andersen (1991),
  \emph{The Statistical Analysis of Categorical Data}.
  2nd edition.
  Springer-Verlag, Berlin.
}

\source{
  E. B. Andersen (1991),
  The Statistical Analysis of Categorical Data, page 205.
}
\examples{
data("DanishWelfare")
ftable(xtabs(Freq ~ ., data = DanishWelfare))
}

\keyword{datasets}

\name{Pairs plot panel functions for off-diagonal cells}
\alias{pairs_strucplot}
\alias{pairs_mosaic}
\alias{pairs_assoc}
\alias{pairs_sieve}
\title{Off-diagonal Panel Functions for Table Pairs Plot}
\description{
  Off-diagonal panel functions for \code{\link{pairs.table}}.
}
\usage{
pairs_strucplot(panel = mosaic, 
  type = c("pairwise", "total", "conditional", "joint"),
  legend = FALSE, margins = c(0, 0, 0, 0), labeling = NULL, \dots)
pairs_assoc(\dots)
pairs_mosaic(\dots)
pairs_sieve(\dots)
}
\arguments{
  \item{panel}{function to be used for the plots in each
    cell, such as \code{\link{pairs_assoc}}, \code{\link{pairs_mosaic}},
    and \code{\link{pairs_sieve}}.}
  \item{type}{character string specifying the type of independence model
    visualized in the cells.}
  \item{legend}{logical specifying whether a legend should be displayed
    in the cells or not.}
  \item{margins}{margins inside each cell (see \code{\link{strucplot}}).}
  \item{labeling}{labeling function or labeling-generating function (see
  \code{\link{strucplot}}).}
\item{\dots}{\code{pairs_mosaic}, \code{\link{pairs_assoc}}, and
  \code{pairs_sieve}: parameters
    passed to \code{pairs_strucplot}. \code{pairs_strucplot}:
    other parameters passed to panel function.}
}
\details{
  These functions really just wrap \code{\link{assoc}}, \code{\link{sieve}}, and
  \code{\link{mosaic}} by basically inhibiting labeling and
  legend-drawing and setting the margins to 0.
}
\value{
  A function with arguments:
  \item{x}{contingency table.}
  \item{i, j}{cell coordinates.}
}
\seealso{
  \code{\link{pairs.table}},
  \code{\link{pairs_text}},
  \code{\link{pairs_barplot}},
  \code{\link{assoc}},
  \code{\link{mosaic}}
}
\references{
  Cohen, A. (1980),
  On the graphical display of the significant components in a two-way
  contingency table.
  \emph{Communications in Statistics---Theory and Methods}, \bold{A9},
  1025--1041.

  Friendly, M. (1992),
  Graphical methods for categorical data.
  \emph{SAS User Group International Conference Proceedings}, \bold{17},
  190--200.
  \url{http://datavis.ca/papers/sugi/sugi17.pdf}
}
\author{
  David Meyer \email{David.Meyer@R-project.org}
}
\examples{
data("UCBAdmissions")
data("PreSex")

pairs(PreSex)
pairs(UCBAdmissions)
pairs(UCBAdmissions, upper_panel_args = list(shade = FALSE))
pairs(UCBAdmissions, lower_panel = pairs_mosaic(type = "conditional"))
pairs(UCBAdmissions, upper_panel = pairs_assoc)
}
\keyword{hplot}

\name{struc_assoc}
\alias{struc_assoc}
\title{Core-generating Function for Association Plots}
\description{
  Core-generating function for \code{strucplot} returning a function
  producing association plots.
}
\usage{
struc_assoc(compress = TRUE, xlim = NULL, ylim = NULL,
  yspace = unit(0.5, "lines"), xscale = 0.9, gp_axis = gpar(lty = 3))
}
\arguments{
  \item{compress}{logical; if \code{FALSE}, the space between the rows
    (columns) are chosen such that the \emph{total} heights (widths) of
    the rows (column) are all equal.  If \code{TRUE}, the space between
    the rows and columns is fixed and hence the plot is more
    \dQuote{compressed}.} 
  \item{xlim}{either a \eqn{2 \times k}{2 x k} matrix of doubles,
    \eqn{k} the number of total columns of the plot, or a recycled
    vector from which such a matrix will be constructed.
    The columns of \code{xlim} correspond to the columns of the
    association plot, the rows describe the column ranges (minimums in
    the first row, maximums in the second row).
    If \code{xlim} is \code{NULL}, the ranges are determined from the
    residuals according to \code{compress} (if \code{TRUE}: widest range
    from each column, if \code{FALSE}: from the whole association plot
    matrix).}
  \item{ylim}{either a \eqn{2 \times k}{2 x k} matrix of doubles,
    \eqn{k} the number of total rows of the plot, or a recycled vector
    from which such a matrix will be constructed.
    The columns of \code{ylim} correspond to the rows of the association
    plot, the rows describe the column ranges (minimums in the first
    row, maximums in the second row).
    If \code{ylim} is \code{NULL}, the ranges are determined from the
    residuals according to \code{compress} (if \code{TRUE}: widest range
    from each row, if \code{FALSE}: from the whole association plot
    matrix).}
  \item{xscale}{scale factor resizing the tile's width, thus adding
    additional space between the tiles. }
  \item{yspace}{object of class \code{"unit"} specifying additional
    space separating the rows.}
  \item{gp_axis}{object of class \code{"gpar"} specifying the visual
    aspects of the tiles' baseline.}
}
\details{
  This function is usually called by \code{strucplot} (typically when
  called by \code{assoc}) and returns a function used by
  \code{strucplot} to produce association plots. 
}
\value{
  A function with arguments:
  \item{residuals}{table of residuals.}
  \item{observed}{not used by \code{struc_assoc}.}
  \item{expected}{table of expected frequencies.}
  \item{spacing}{object of class \code{"unit"} specifying the space
    between the tiles.}
  \item{gp}{list of \code{gpar} objects used for the drawing the tiles.}
  \item{split_vertical}{vector of logicals indicating, for each
    dimension of the table, the split direction.}
}
\author{
  David Meyer \email{David.Meyer@R-project.org}
}
\seealso{
  \code{\link{assoc}},
  \code{\link{strucplot}},
  \code{\link{structable}}
}
\references{
  Cohen, A. (1980),
  On the graphical display of the significant components in a two-way
  contingency table.
  \emph{Communications in Statistics---Theory and Methods}, \bold{A9},
  1025--1041.

  Friendly, M. (1992),
  Graphical methods for categorical data.
  \emph{SAS User Group International Conference Proceedings}, \bold{17},
  190--200.
  \url{http://datavis.ca/papers/sugi/sugi17.pdf}

  Meyer, D., Zeileis, A., and Hornik, K. (2006),
  The strucplot framework: Visualizing multi-way contingency tables with
  \pkg{vcd}.
  \emph{Journal of Statistical Software}, \bold{17(3)}, 1-48.
  \doi{10.18637/jss.v017.i03} and available as 
  \code{vignette("strucplot")}.
}
\examples{
## UCB Admissions
data("UCBAdmissions")
ucb <- aperm(UCBAdmissions)

## association plot for conditional independence
strucplot(ucb, expected = ~ Dept * (Admit + Gender),
  core = struc_assoc(ylim = c(-4, 4)), labeling_args = list(abbreviate = c(Admit = 3)))
}
\keyword{hplot}

\name{hls}
\alias{hls}
\title{HLS Color Specification}
\description{
  Create a HLS color from specifying hue, luminance and saturation.
}
\usage{
hls(h = 1, l = 0.5, s = 1)
}
\arguments{
  \item{h}{hue value in [0, 1].}
  \item{l}{luminance value in [0, 1].}
  \item{s}{saturation value in [0, 1].}
}
\details{
  HLS colors are a similar specification of colors as HSV
  colors, but using hue/luminance/saturation rather that
  hue/saturation/value.
}
\author{
  Achim Zeileis \email{Achim.Zeileis@R-project.org}
}
\seealso{
  \code{\link{hsv}},
  \code{\link{hcl2hex}},
  \code{\link[colorspace]{polarLUV}}
}
\examples{
## an HLS color wheel
pie(rep(1, 12), col = sapply(1:12/12, function(x) hls(x)))
}
\keyword{hplot}

\name{SexualFun}
\alias{SexualFun}
\docType{data}
\title{Sex is Fun}
\description{
  Data from Hout et al. (1987) given by Agresti (1990) summarizing the
  responses of married couples to the questionnaire item: Sex is fun for
  me and my partner: (a) never or occasionally, (b) fairly often, (c)
  very often, (d) almost always.
}
\usage{
data("SexualFun")
}
\format{
  A 2-dimensional array resulting from cross-tabulating the ratings of
  91 married couples.  The variables and their levels are as follows:

  \tabular{rll}{
    No \tab Name \tab Levels \cr
    1 \tab Husband \tab Never Fun, Fairly Often, Very Often, Always Fun \cr
    2 \tab Wife \tab Never Fun, Fairly Often, Very Often, Always Fun
  }
}

\references{
  A. Agresti (1990),
  \emph{Categorical Data Analysis}.
  Wiley-Interscience, New York.

  M. Hout, O. D. Duncan, M. E. Sobel (1987),
  Association and heterogeneity: Structural models of similarities and
  differences,
  \emph{Sociological Methodology}, \bold{17}, 145-184.

  M. Friendly (2000),
  \emph{Visualizing Categorical Data}.
  SAS Institute, Cary, NC.
}

\source{
  M. Friendly (2000),
  Visualizing Categorical Data, page 91.
}

\examples{
data("SexualFun")

## Kappa statistics
Kappa(SexualFun)

## Agreement Chart
agreementplot(t(SexualFun), weights = 1)
## Partial Agreement Chart and B-Statistics
agreementplot(t(SexualFun),
              xlab = "Husband's Rating",
              ylab = "Wife's Rating",
              main = "Husband's and Wife's Sexual Fun")
}

\keyword{datasets}

\name{assocstats}
\alias{assocstats}
\alias{summary.assocstats}
\alias{print.assocstats}
\alias{print.summary.assocstats}
\title{Association Statistics}
\description{
  Computes the Pearson chi-Squared test, the Likelihood Ratio chi-Squared test,
  the phi coefficient, the contingency coefficient and Cramer's V for
  possibly stratified contingency tables.
}
\usage{
assocstats(x)
}
\arguments{
  \item{x}{a contingency table, with possibly more than 2 dimensions. In
  this case, all dimensions except the first two ones are considered as strata.}
}
\value{
  In case of a 2-dimensional table, a list with components:
  \item{chisq_tests}{a \eqn{2 \times 3}{2 x 3} table with the
    chi-squared statistics.}
  \item{phi}{The \emph{absolute value} of the phi coefficient (only
    defined for \eqn{2 \times 2}{2 x 2} tables).}
  \item{cont}{The contingency coefficient.}
  \item{cramer}{Cramer's V.}
  In case of higher-dimensional tables, a list of the above mentioned
  structure, each list component representing one stratum defined by the
  combinations of all levels of the stratum dimensions.
}
\references{
  Michael Friendly (2000),
  \emph{Visualizing Categorical Data}.
  SAS Institute, Cary, NC.

  Fleiss, J. L. (1981).
  \emph{Statistical methods for rates and proportions}
  (2nd ed). New York: Wiley 
}
\author{
  David Meyer \email{David.Meyer@R-project.org}
}
\examples{
data("Arthritis")
tab <- xtabs(~Improved + Treatment, data = Arthritis)
summary(assocstats(tab))

assocstats(UCBAdmissions)
}
\keyword{category}

\name{Lifeboats}
\alias{Lifeboats}
\docType{data}
\title{Lifeboats on the Titanic}
\description{
  Data from Mersey (1912) about the 18 (out of 20) lifeboats launched
  before the sinking of the S. S. Titanic.
}
\usage{data("Lifeboats")}
\format{
  A data frame with 18 observations and 8 variables.
  \describe{
    \item{launch}{launch time in \code{"\link{POSIXt}"} format.}
    \item{side}{factor.  Side of the boat.}
    \item{boat}{factor indicating the boat.}
    \item{crew}{number of male crew members on board.}
    \item{men}{number of men on board.}
    \item{women}{number of women (including female crew) on board.}
    \item{total}{total number of passengers.}
    \item{cap}{capacity of the boat.}
  }
}
\references{
  L. Mersey (1912),
  Report on the loss of the \dQuote{Titanic} (S. S.).
  Parliamentary command paper 6452.

  M. Friendly (2000),
  \emph{Visualizing Categorical Data}.
  SAS Institute, Cary, NC.
}

\source{
  M. Friendly (2000),
  Visualizing Categorical Data:
  \url{http://euclid.psych.yorku.ca/ftp/sas/vcd/catdata/lifeboat.sas}
}

\examples{
data("Lifeboats")
attach(Lifeboats)
ternaryplot(
  Lifeboats[,4:6],
  pch = ifelse(side == "Port", 1, 19),
  col = ifelse(side == "Port", "red", "blue"),
  id  = ifelse(men / total > 0.1, as.character(boat), NA),
  prop_size = 2,
  dimnames_position = "edge",
  main = "Lifeboats on the Titanic"
)
grid_legend(0.8, 0.9, c(1, 19), c("red", "blue"),
  c("Port", "Starboard"), title = "SIDE")
detach(Lifeboats)
}

\keyword{datasets}

\name{VisualAcuity}
\alias{VisualAcuity}
\docType{data}
\title{Visual Acuity in Left and Right Eyes}
\description{
  Data from Kendall & Stuart (1961) on unaided vision among 3,242 men
  and 7,477 women, all aged 30-39 and employed in the U.K. Royal
  Ordnance factories 1943-1946.
}
\usage{
data("VisualAcuity")
}
\format{
  A data frame with 32 observations and 4 variables.
  \describe{
    \item{Freq}{frequency of visual acuity measurements.}
    \item{right}{visual acuity on right eye.}
    \item{left}{visual acuity on left eye.}
    \item{gender}{factor indicating gender of patient.}
  }
}
\references{
  M. G. Kendall & A. Stuart (1961),
  \emph{The Advanced Theory of Statistics}, Vol. 2.
  Griffin, London.

  M. Friendly (2000),
  \emph{Visualizing Categorical Data}.
  SAS Institute, Cary, NC.
}
\source{
  M. Friendly (2000),
  Visualizing Categorical Data:
  \url{http://euclid.psych.yorku.ca/ftp/sas/vcd/catdata/vision.sas}
}
\examples{
data("VisualAcuity")
structable(~ gender + left + right, data = VisualAcuity)
sieve(Freq ~ left + right | gender, data = VisualAcuity, shade = TRUE)
cotabplot(Freq ~ left + right | gender, data = VisualAcuity,
          panel = cotab_agreementplot)
}

\keyword{datasets}

\name{Butterfly}
\alias{Butterfly}
\docType{data}
\title{Butterfly Species in Malaya}
\description{
  Data from Fisher et al. (1943) giving the number of tokens found for
  each of 501 species of butterflies collected in Malaya.
}
\usage{
data("Butterfly")
}
\format{
  A 1-way table giving the number of tokens for 501 species of
  butterflies. The variable and its levels are
  \tabular{rll}{
    No \tab Name \tab Levels \cr
    1 \tab nTokens \tab 0, 1, \dots, 24 \cr
  }
}

\references{
  R. A. Fisher, A. S. Corbet, C. B. Williams (1943),
  The relation between the number of species and the number of
  individuals,
  \emph{Journal of Animal Ecology}, \bold{12}, 42--58.

  M. Friendly (2000),
  \emph{Visualizing Categorical Data}.
  SAS Institute, Cary, NC.
}

\source{
  Michael Friendly (2000),
  Visualizing Categorical Data, pages 21--22.
}
\examples{
data("Butterfly")
Ord_plot(Butterfly)
}

\keyword{datasets}

\name{RepVict}
\alias{RepVict}
\docType{data}
\title{Repeat Victimization Data}
\description{
  Data from Reiss (1980) given by Fienberg (1980) about instances of
  repeat victimization for households in the U.S. National Crime
  Survey.
}
\usage{
data("RepVict")
}
\format{
  A 2-dimensional array resulting from cross-tabulating victimization.
  The variables and their levels are as follows:

  \tabular{rll}{
    No \tab Name \tab Levels \cr
    1 \tab First Victimization \tab Rape, Assault, Robbery, Pickpocket,
    Personal Larceny, \cr
    \tab \tab Burglary, Household Larceny, Auto Theft \cr
    2 \tab Second Victimization \tab Rape, Assault, Robbery, Pickpocket,
    Personal Larceny,\cr
    \tab \tab Burglary, Household Larceny, Auto Theft
  }
}

\references{
  S. E. Fienberg (1980),
  \emph{The Analysis of Cross-Classified Categorical Data},
  MIT Press, Cambridge, 2nd edition.

  A. J. J. Reiss (1980),
  Victim proneness by type of crime in repeat victimization.
  In S. E. Fienberg & A. J. J. Reiss (eds.),
  \emph{Indicators of Crime and Criminal Justice}.
  U.S. Government Printing Office, Washington, DC.

  M. Friendly (2000),
  \emph{Visualizing Categorical Data}.
  SAS Institute, Cary, NC.
}
\source{
  Michael Friendly (2000),
  Visualizing Categorical Data, page 113.
}
\examples{
data("RepVict")

mosaic(RepVict[-c(4,7),-c(4,7)], gp = shading_max,
       main = "Repeat Victimization Data")
}
\keyword{datasets}

\name{goodfit}
\alias{goodfit}
\alias{summary.goodfit}
\alias{plot.goodfit}
\alias{predict.goodfit}
\alias{fitted.goodfit}
\alias{residuals.goodfit}
\alias{print.goodfit}
\title{Goodness-of-fit Tests for Discrete Data}
\description{
  Fits a discrete (count data) distribution for goodness-of-fit tests.
}
\usage{
goodfit(x, type = c("poisson", "binomial", "nbinomial"),
  method = c("ML", "MinChisq"), par = NULL)
\method{predict}{goodfit}(object, newcount = NULL, type = c("response", "prob"), \dots)
\method{residuals}{goodfit}(object, type = c("pearson", "deviance",
"raw"), \dots)
\method{print}{goodfit}(x, residuals_type = c("pearson", "deviance",
"raw"), \dots)

}
\arguments{
  \item{x}{either a vector of counts, a 1-way table of frequencies of
    counts or a data frame or matrix with frequencies in the first
    column and the corresponding counts in the second column.}
  \item{type}{character string indicating: for \code{goodfit}, which
    distribution should be fit; for \code{predict}, the
    type of prediction (fitted response or probabilities); for
    \code{residuals}, either \code{"pearson"}, \code{"deviance"} or
    \code{"raw"}.}
  \item{residuals_type}{character string indicating the type of
    residuals: either \code{"pearson"}, \code{"deviance"} or
    \code{"raw"}.}
  \item{method}{a character string indicating whether the distribution
    should be fit via ML (Maximum Likelihood) or Minimum Chi-squared.}
  \item{par}{a named list giving the distribution parameters (named as
    in the corresponding density function), if set to \code{NULL}, the
    default, the parameters are estimated.  If the parameter \code{size}
    is not specified if \code{type} is \code{"binomial"} it is taken to
    be the maximum count. If \code{type} is \code{"nbinomial"}, then
    parameter \code{size} can be specified to fix it so that only the
    parameter \code{prob} will be estimated (see the examples below).}
  \item{object}{an object of class \code{"goodfit"}.}
  \item{newcount}{a vector of counts.  By default the counts stored in
    \code{object} are used, i.e., the fitted values are computed.  These
    can also be extracted by \code{fitted(object)}.}
  \item{\dots}{\emph{currently not used}.}
}
\details{
  \code{goodfit} essentially computes the fitted values of a discrete
  distribution (either Poisson, binomial or negative binomial) to the
  count data given in \code{x}.  If the parameters are not specified
  they are estimated either by ML or Minimum Chi-squared. 

  To fix parameters,
  \code{par} should be a named list specifying the parameters \code{lambda}
  for \code{"poisson"} and \code{prob} and \code{size} for
  \code{"binomial"} or \code{"nbinomial"}, respectively.
  If for \code{"binomial"}, \code{size} is not specified it is not
  estimated but taken as the maximum count.

  The corresponding Pearson Chi-squared or likelihood ratio statistic,
  respectively, is computed and given with their \eqn{p} values by the
  \code{summary} method. The \code{summary} method always prints this
  information and returns a matrix  with the printed information
  invisibly. The \code{plot} method produces a
  \code{\link{rootogram}} of the observed and fitted values.
  
  In case of count distribtions (Poisson and negative binomial), the
  minimum Chi-squared approach is somewhat ad hoc. Strictly speaking,
  the Chi-squared asymptotics would only hold if the number of cells
  were fixed or did not increase too quickly with the sample size. However,
  in \code{goodfit} the number of cells is data-driven: Each count is
  a cell of its own. All counts larger than the maximal count are merged
  into the cell with the last count for computing the test statistic.
}
\value{
  A list of class \code{"goodfit"} with elements:
  \item{observed}{observed frequencies.}
  \item{count}{corresponding counts.}
  \item{fitted}{expected frequencies (fitted by ML).}
  \item{type}{a character string indicating the distribution fitted.}
  \item{method}{a character string indicating the fitting method (can
    be either \code{"ML"}, \code{"MinChisq"} or \code{"fixed"} if the
    parameters were specified).}
  \item{df}{degrees of freedom.}
  \item{par}{a named list of the (estimated) distribution parameters.}
}
\author{
  Achim Zeileis \email{Achim.Zeileis@R-project.org}
}
\references{
  M. Friendly (2000),
  \emph{Visualizing Categorical Data}.
  SAS Institute, Cary, NC.
}
\seealso{\code{\link{rootogram}}}

\examples{
## Simulated data examples:
dummy <- rnbinom(200, size = 1.5, prob = 0.8)
gf <- goodfit(dummy, type = "nbinomial", method = "MinChisq")
summary(gf)
plot(gf)

dummy <- rbinom(100, size = 6, prob = 0.5)
gf1 <- goodfit(dummy, type = "binomial", par = list(size = 6))
gf2 <- goodfit(dummy, type = "binomial", par = list(prob = 0.6, size = 6))
summary(gf1)
plot(gf1)
summary(gf2)
plot(gf2)

## Real data examples:
data("HorseKicks")
HK.fit <- goodfit(HorseKicks)
summary(HK.fit)
plot(HK.fit)

data("Federalist")
## try geometric and full negative binomial distribution
F.fit <- goodfit(Federalist, type = "nbinomial", par = list(size = 1))
F.fit2 <- goodfit(Federalist, type = "nbinomial")
summary(F.fit)
summary(F.fit2)
plot(F.fit)
plot(F.fit2)
}
\keyword{category}

\name{pairs.table}
\alias{pairs.table}
\alias{pairs.structable}
\title{Pairs Plot for Contingency Tables}
\description{
  Produces a matrix of strucplot displays.
}
\usage{
\method{pairs}{table}(x, upper_panel = pairs_mosaic, upper_panel_args = list(),
  lower_panel = pairs_mosaic, lower_panel_args = list(),
  diag_panel = pairs_diagonal_mosaic, diag_panel_args = list(),
  main = NULL, sub = NULL, main_gp = gpar(fontsize = 20),
  sub_gp = gpar(fontsize = 15), space = 0.3,
  newpage = TRUE, pop = TRUE, return_grob = FALSE,
  margins = unit(1, "lines"), \dots)
}
\arguments{

  \item{x}{a contingency table in array form, with optional category
    labels specified in the \code{dimnames(x)} attribute.}
  \item{upper_panel}{function for the upper triangle of the matrix, or
    corresponding generating function. If \code{NULL}, no panel is drawn.}
  \item{upper_panel_args}{list of arguments for the generating
    function, if specified.}
  \item{lower_panel}{function for the lower triangle of the matrix, or
    corresponding generating function. If \code{NULL}, no panel is drawn.}
  \item{lower_panel_args}{list of arguments for the panel-generating
    function, if specified.}
  \item{diag_panel}{function for the diagonal of the matrix, or
    corresponding generating function. If \code{NULL}, no panel is drawn.}
  \item{diag_panel_args}{list of arguments for the generating
    function, if specified.}
  \item{main}{either a logical, or a character string used for plotting
    the main title.  If \code{main} is a logical and \code{TRUE}, the
    name of the object supplied as \code{x} is used.}
  \item{sub}{a character string used for plotting the subtitle.
    If \code{sub} is a logical and \code{TRUE} and \code{main} is unspecified, the
    name of the object supplied as \code{x} is used.}
  \item{main_gp, sub_gp}{object of class \code{"gpar"} containing the graphical
    parameters used for the main (sub) title, if specified.}
  \item{space}{double specifying the distance between the cells.}
  \item{newpage}{logical controlling whether a new grid page should be
    created.}
  \item{pop}{logical indicating whether all viewports should be popped
    after the plot has been drawn.}
  \item{return_grob}{logical. Should a snapshot of the display be
    returned as a grid grob?}
  \item{margins}{either an object of class \code{"unit"} of length 4, or
    a numeric vector of length 4. The elements are recycled as needed.
    The four components specify the top, right,
    bottom, and left margin of the plot, respectively.
    When a numeric vector is supplied, the numbers are interpreted as
    \code{"lines"} units.  In addition, the unit or numeric vector
    may have named arguments
    (\samp{top}, \samp{right}, \samp{bottom}, and \samp{left}), in which
    case the non-named arguments specify the default values (recycled as
    needed), overloaded by the named arguments.}
  \item{\dots}{For convenience,
    list of arguments for the panel-generating
    functions of upper and lower panels, if specified.}
}
\details{
  This is a \code{\link[graphics]{pairs}} method for objects inheriting
  from class \code{"table"} or \code{"structable"}.
  It plots a matrix of pairwise mosaic plots.
  
  Four independence types are distinguished: \code{"pairwise"},
  \code{"total"}, \code{"conditional"} and \code{"joint"}.
  The pairwise mosaic matrix shows bivariate marginal relations,
  collapsed over all other variables.
  The total independence mosaic matrix shows mosaic plots for mutual
  independence, i.e., for marginal and conditional independence among
  all pairs of variables.
  The conditional independence mosaic matrix shows mosaic plots for
  conditional independence for each pair of variables, given all other variables.
  The joint independence mosaic matrix shows mosaic plots for joint
  independence of all pairs of variables from the others.

  This method uses panel functions called for each cell of the
  matrix which can be different for upper matrix, lower matrix, and
  diagonal cells.  Correspondingly, for each panel parameter \var{foo}
  (= \samp{upper}, \samp{lower}, or \samp{diag}), \code{pairs.table} takes
  two arguments: \var{foo_panel} and \var{foo_panel_args}, which can
  be used to specify the parameters as follows:

  \enumerate{
    \item Passing a suitable panel function to \var{foo_panel} which
    subsequently is called for each cell with the corresponding
    coordinates.
    \item Passing a corresponding \emph{generating function} (of class
    \code{"panel_generator"}) to \var{foo_panel}, along with parameters passed to
    \var{foo_panel_args}, that generates such a function. 
  }
Hence, the second approach is equivalent to the first if
\var{foo_panel(foo_panel_args)} is passed to
\var{foo_panel}.

  
}
\seealso{
  \code{\link{pairs_mosaic}},
  \code{\link{pairs_assoc}},
  \code{\link{pairs_sieve}},
  \code{\link{pairs_diagonal_text}},
  \code{\link{pairs_diagonal_mosaic}},
  \code{\link{pairs_text}},
  \code{\link{pairs_barplot}},
  \code{\link{assoc}},
  \code{\link{sieve}},
  \code{\link{mosaic}}
}
\references{
  Cohen, A. (1980),
  On the graphical display of the significant components in a two-way
  contingency table.
  \emph{Communications in Statistics---Theory and Methods}, \bold{A9},
  1025--1041.

  Friendly, M. (1992),
  Graphical methods for categorical data.
  \emph{SAS User Group International Conference Proceedings}, \bold{17},
  190--200.
  \url{http://datavis.ca/papers/sugi/sugi17.pdf}

  Meyer, D., Zeileis, A., and Hornik, K. (2006),
  The strucplot framework: Visualizing multi-way contingency tables with
  \pkg{vcd}.
  \emph{Journal of Statistical Software}, \bold{17(3)}, 1-48.
  \doi{10.18637/jss.v017.i03} and available as 
  \code{vignette("strucplot")}.
  
}
\author{
  David Meyer \email{David.Meyer@R-project.org}
}
\examples{
data("UCBAdmissions")
data("PreSex")
data(HairEyeColor)
hec = structable(Eye ~ Sex + Hair, data = HairEyeColor)

pairs(PreSex)
pairs(UCBAdmissions)
pairs(UCBAdmissions, upper_panel_args = list(shade = TRUE))
pairs(UCBAdmissions, lower_panel = pairs_mosaic(type = "conditional"))
pairs(UCBAdmissions, diag_panel = pairs_text)
pairs(UCBAdmissions, upper_panel = pairs_assoc, shade = TRUE)
pairs(hec, highlighting = 2, diag_panel_args = list(fill = grey.colors))
pairs(hec, highlighting = 2, diag_panel = pairs_diagonal_mosaic,
           diag_panel_args = list(fill = grey.colors, alternate_labels =TRUE))

}
\keyword{hplot}

\name{distplot}
\alias{distplot}
\title{Diagnostic Distribution Plots}
\description{
  Diagnostic distribution plots: poissonness, binomialness and
  negative binomialness plots.
}
\usage{
distplot(x, type = c("poisson", "binomial", "nbinomial"),
  size = NULL, lambda = NULL, legend = TRUE, xlim = NULL, ylim = NULL,
  conf_int = TRUE, conf_level = 0.95, main = NULL,
  xlab = "Number of occurrences", ylab = "Distribution metameter",
  gp = gpar(cex = 0.8), lwd=2, gp_conf_int = gpar(lty = 2),
  name = "distplot", newpage = TRUE,
  pop =TRUE, return_grob = FALSE, \dots)
}
\arguments{
  \item{x}{either a vector of counts, a 1-way table of frequencies of
    counts or a data frame or matrix with frequencies in the first
    column and the corresponding counts in the second column.}
  \item{type}{a character string indicating the distribution.}
  \item{size}{the size argument for the binomial and negative binomial
    distribution.
    If set to \code{NULL} and \code{type} is \code{"binomial"}, then
    \code{size} is taken to be the maximum count. 
    If set to \code{NULL} and \code{type} is \code{"nbinomial"}, then
    \code{size} is estimated from the data.}
  \item{lambda}{parameter of the poisson distribution.
    If type is \code{"poisson"} and \code{lambda} is specified a leveled
    poissonness plot is produced.}
  \item{legend}{logical.  Should a legend be plotted?}
  \item{xlim}{limits for the x axis.}
  \item{ylim}{limits for the y axis.}
  \item{conf_int}{logical.  Should confidence intervals be plotted?}
  \item{conf_level}{confidence level for confidence intervals.}
  \item{main}{a title for the plot.}
  \item{xlab}{a label for the x axis.}
  \item{ylab}{a label for the y axis.}
  \item{gp}{a \code{"gpar"} object controlling the grid graphical
    parameters of the points.}  
  \item{gp_conf_int}{a \code{"gpar"} object controlling the grid graphical
    parameters of the confidence intervals.}  
  \item{lwd}{line width for the fitted line}
  \item{name}{name of the plotting viewport.}
  \item{newpage}{logical. Should \code{\link{grid.newpage}} be called 
    before plotting?}
  \item{pop}{logical. Should the viewport created be popped?}
  \item{return_grob}{logical. Should a snapshot of the display be
    returned as a grid grob?}
  \item{\dots}{further arguments passed to \code{\link{grid.points}}.}
}
\details{
  \code{distplot} plots the number of occurrences (counts) against the
  distribution metameter of the specified distribution.  If the
  distribution fits the data, the plot should show a straight line.
  See Friendly (2000) for details.
  
  In these plots, the open points show the observed count metameters;
  the filled points show the confidence interval centers, and the
  dashed lines show the \code{conf_level} confidence intervals for
  each point.
}

\value{
Returns invisibly a data frame containing the counts (\code{Counts}), 
frequencies (\code{Freq}) and other details of the computations used
to construct the plot.
}

\author{
  Achim Zeileis \email{Achim.Zeileis@R-project.org}
}
\references{
  D. C. Hoaglin (1980),
  A poissonness plot,
  \emph{The American Statistican}, \bold{34}, 146--149.

  D. C. Hoaglin & J. W. Tukey (1985),
  Checking the shape of discrete distributions.
  In D. C. Hoaglin, F. Mosteller, J. W. Tukey (eds.),
  \emph{Exploring Data Tables, Trends and Shapes}, chapter 9.
  John Wiley & Sons, New York.

  M. Friendly (2000),
  \emph{Visualizing Categorical Data}.
  SAS Institute, Cary, NC.
}
\examples{
## Simulated data examples:
dummy <- rnbinom(1000, size = 1.5, prob = 0.8)
distplot(dummy, type = "nbinomial")

## Real data examples:
data("HorseKicks")
data("Federalist")
data("Saxony")
distplot(HorseKicks, type = "poisson")
distplot(HorseKicks, type = "poisson", lambda = 0.61)
distplot(Federalist, type = "poisson")
distplot(Federalist, type = "nbinomial", size = 1)
distplot(Federalist, type = "nbinomial")
distplot(Saxony, type = "binomial", size = 12)
}
\keyword{category}

\name{Rochdale}
\alias{Rochdale}
\docType{data}
\title{Rochdale Data}
\description{
  Information on 665 households of Rochdale, Lancashire, UK.
  The study was conducted to identify influence factors on economical
  activity of wives. 
}
\usage{
data("Rochdale")
}
\format{
  A 8-dimensional array resulting from cross-tabulating 665 observations
  on 8 variables.  The variables and their levels are as follows:

  \tabular{rll}{
    No \tab Name \tab Levels \cr
    1 \tab EconActive \tab yes, no \cr
    2 \tab Age \tab <38, >38 \cr
    3 \tab HusbandEmployed \tab yes, no \cr
    4 \tab Child \tab yes, no \cr
    5 \tab Education \tab yes, no \cr
    6 \tab HusbandEducation \tab yes, no \cr
    7 \tab Asian \tab yes, no \cr
    8 \tab HouseholdWorking \tab yes, no \cr
  }
}

\note{
  Many observations are missing: only 91 out of all 256 combinations
  contain information.
}
\source{
  Whittaker (1990).
}
\references{
  H. Hofmann (2003).
  Constructing and reading mosaicplots.
  \emph{Computational Statistics & Data Analysis},
  \bold{43}, 4, 565--580.

  J. Whittaker (1990),
  \emph{Graphical Models on Applied Multivariate Statistics},
  Wiley, New York.
}
\examples{
data("Rochdale")
mosaic(Rochdale)
}
\keyword{datasets}

\name{CoalMiners}
\alias{CoalMiners}

\title{Breathlessness and Wheeze in Coal Miners}
\description{
  Data from Ashford & Sowden (1970) given by Agresti (1990) on the
  association between two pulmonary conditions, breathlessness and
  wheeze, in a large sample of coal miners who were smokers with no
  radiological evidence of pneumoconlosis, aged between 20--64
  when examined.
  This data is frequently used as an example of fitting models for
  bivariate, binary responses.
}
\usage{
data("CoalMiners")
}
\format{
  A 3-dimensional table of size 2 x 2 x 9
  resulting from cross-tabulating variables for
  18,282 coal miners.  The variables and their levels are as follows:

  \tabular{rll}{
    No \tab Name \tab Levels \cr
    1 \tab Breathlessness \tab B, NoB \cr
    2 \tab Wheeze \tab W, NoW \cr
    3 \tab Age \tab 20-24, 25-29, 30-34, \dots, 60-64
  }
}

\details{
In an earlier version of this data set, the first group, aged 20-24, was
inadvertently omitted from this data table and the breathlessness variable
was called wheeze and vice versa.
}

\references{
  A. Agresti (1990),
  \emph{Categorical Data Analysis}.
  Wiley-Interscience, New York, Table 7.11, p. 237

  J. R. Ashford and R. D. Sowdon (1970),
  Multivariate probit analysis,
  \emph{Biometrics}, \bold{26}, 535--546.

  M. Friendly (2000),
  \emph{Visualizing Categorical Data}.
  SAS Institute, Cary, NC.
}

\source{
  Michael Friendly (2000),
  Visualizing Categorical Data, pages 82--83, 319--322.
}

\examples{
data("CoalMiners")

ftable(CoalMiners, row.vars = 3)

## Fourfold display, both margins equated
fourfold(CoalMiners[,,2:9], mfcol = c(2,4))

## Fourfold display, strata equated
fourfold(CoalMiners[,,2:9], std = "ind.max", mfcol = c(2,4))


## Log Odds Ratio Plot
lor_CM <- loddsratio(CoalMiners)
summary(lor_CM)
plot(lor_CM)
lor_CM_df <- as.data.frame(lor_CM)

# fit linear models using WLS
age <- seq(20, 60, by = 5)
lmod <- lm(LOR ~ age, weights = 1 / ASE^2, data = lor_CM_df)
grid.lines(age, fitted(lmod), gp = gpar(col = "blue"))
qmod <- lm(LOR ~ poly(age, 2), weights = 1 / ASE^2, data = lor_CM_df)
grid.lines(age, fitted(qmod), gp = gpar(col = "red"))
}

\keyword{datasets}

\name{Bundestag2005}
\alias{Bundestag2005}
\title{Votes in German Bundestag Election 2005}
\description{
Number of votes by province in the German Bundestag election
2005 (for the parties that eventually entered the parliament).
}
\usage{
data("Bundestag2005")
}
\format{
  A 2-way \code{"table"} giving the number of votes for each
  party (\code{Fraktion}) in each of the 16 German provinces
  (\code{Bundesland}):
  
  \tabular{rll}{
    No \tab Name \tab Levels \cr
    1 \tab Bundesland \tab Schleswig-Holstein, Mecklenburg-Vorpommern, \dots \cr
    2 \tab Fraktion \tab SPD, CDU/CSU, Gruene, FDP, Linke
  }
}

\details{
In the election for the German parliament \dQuote{Bundestag}, 
five parties obtained enough votes to enter the parliament:
the social democrats SPD, the conservative CDU/CSU, the liberal FDP,
the green party \dQuote{Die Gruenen} and the leftist party 
\dQuote{Die Linke}. The table \code{Bundestag2005} gives the
number of votes for each party (\code{Fraktion}) in each of the
16 German provinces (\code{Bundesland}). The provinces are ordered
from North to South.

The data have been obtained from the German statistical office (Statistisches
Bundesamt) from the Web page given below.

Note that the number of seats in the parliament cannot be computed from
the number of votes alone. The examples below show the distribution of seats
that resulted from the election.
}

\source{
  Der Bundeswahlleiter, Statistisches Bundesamt.  
  \url{https://www.bundeswahlleiter.de/bundestagswahlen/2005.html}
}

\examples{
library(colorspace)
## The outcome of the election in terms of seats in the
## parliament was:
seats <- structure(c(226, 61, 54, 51, 222),
  .Names = c("CDU/CSU", "FDP",  "Linke", "Gruene", "SPD"))

## Hues are chosen as metaphors for the political parties
## CDU/CSU: blue, FDP: yellow, Linke: purple, Gruene: green, SPD: red
## using the respective hues from a color wheel with
## chroma = 60 and luminance = 75
parties <- rainbow_hcl(6, c = 60, l = 75)[c(5, 2, 6, 3, 1)]
names(parties) <- names(seats)
parties

## The pie chart shows that neither the SPD+Gruene coalition nor
## the opposition of CDU/CSU+FDP could assemble a majority.
## No party would enter a coalition with the leftists, leading to a
## big coalition.
pie(seats, clockwise = TRUE, col = parties)

## The regional distribution of the votes, stratified by province,
## is shown in a mosaic display: first for the 10 Western then the
## 6 Eastern provinces.
data("Bundestag2005")
votes <- Bundestag2005[c(1, 3:5, 9, 11, 13:16, 2, 6:8, 10, 12),
                       c("CDU/CSU", "FDP", "SPD", "Gruene", "Linke")]
mosaic(votes, gp = gpar(fill = parties[colnames(votes)]),
  spacing = spacing_highlighting, labeling = labeling_left,
  labeling_args = list(rot_labels = c(0, 90, 0, 0), pos_labels = "center",
  just_labels = c("center","center","center","right"), varnames = FALSE),
  margins = unit(c(2.5, 1, 1, 12), "lines"),
  keep_aspect_ratio = FALSE)
}

\keyword{datasets}

\name{mar_table}
\alias{mar_table}
\title{Table with Marginal Sums}
\description{
  Adds row and column sums to a two-way table.
}
\usage{
mar_table(x)
}
\arguments{
  \item{x}{a two-way table.}
}
\value{
  A table with row and column totals added.
}
\author{
  David Meyer \email{David.Meyer@R-project.org}
}
\examples{
data("SexualFun")
mar_table(SexualFun)
}
\keyword{category}

\name{VonBort}
\alias{VonBort}
\docType{data}
\title{Von Bortkiewicz Horse Kicks Data}
\description{
  Data from von Bortkiewicz (1898), given by Andrews & Herzberg (1985),
  on number of deaths by horse or mule kicks in 14 corps of the Prussian
  army.
}
\usage{
data("VonBort")
}
\format{
  A data frame with 280 observations and 4 variables.
  \describe{
    \item{deaths}{number of deaths.}
    \item{year}{year of the deaths.}
    \item{corps}{factor indicating the corps.}
    \item{fisher}{factor indicating whether the corresponding corps was
      considered by Fisher (1925) or not.}
  }
}
\references{
  D. F. Andrews & A. M. Herzberg (1985),
  \emph{Data: A Collection of Problems from Many Fields for the Student
    and Research Worker}.
  Springer-Verlag, New York, NY.

  R. A. Fisher (1925),
  \emph{Statistical Methods for Research Workers}.
  Oliver & Boyd, London.

  L. von Bortkiewicz (1898),
  \emph{Das Gesetz der kleinen Zahlen}.
  Teubner, Leipzig.

  M. Friendly (2000),
  \emph{Visualizing Categorical Data}.
  SAS Institute, Cary, NC.
}

\source{
  Michael Friendly (2000),
  Visualizing Categorical Data:
  \url{http://euclid.psych.yorku.ca/ftp/sas/vcd/catdata/vonbort.sas}
}
\seealso{
  \code{\link{HorseKicks}} for a popular subsample.
}
\examples{
data("VonBort")
## HorseKicks data
xtabs(~ deaths, data = VonBort, subset = fisher == "yes")
}

\keyword{datasets}

\name{doubledecker}
\alias{doubledecker.default}
\alias{doubledecker.formula}
\alias{doubledecker}
\title{Doubledecker Plot}
\description{
  This function creates a doubledecker plot visualizing a classification
  rule.
}
\usage{
\method{doubledecker}{formula}(formula, data = NULL, \dots, main = NULL)
\method{doubledecker}{default}(x, depvar = length(dim(x)),
  margins = c(1,4, length(dim(x)) + 1, 1),
  gp = gpar(fill = rev(gray.colors(tail(dim(x), 1)))),
  labeling = labeling_doubledecker,
  spacing = spacing_highlighting,
  main = NULL, keep_aspect_ratio = FALSE, \dots)
}
\arguments{
  \item{formula}{a formula specifying the variables used to create a 
    contingency table from \code{data}. The dependent variable is used
    last for splitting.}
  \item{data}{either a data frame, or an object of class \code{"table"}
    or \code{"ftable"}.}
  \item{x}{a contingency table in array form, with optional category
    labels specified in the \code{dimnames(x)} attribute.}
  \item{depvar}{dimension index or character string specifying the
    dependent variable.  That will be sorted last in the table.}
  \item{margins}{margins of the plot.  Note that by default, all factor
    names (except the last one) and their levels are visualized \emph{as
      a block} under the plot.}
  \item{gp}{object of class \code{"gpar"} used for the tiles of the last variable.}
  \item{labeling}{labeling function or corresponding generating
    generating function (see \code{\link{strucplot}} for details).}
  \item{spacing}{spacing object, spacing function or corresponding
    generating function (see \code{\link{strucplot}} for details).}
  \item{main}{either a logical, or a character string used for plotting
    the main title.  If \code{main} is \code{TRUE}, the
    name of the \code{data} object is used.}
  \item{keep_aspect_ratio}{logical indicating whether the aspect ratio should be
    maintained or not.}
  \item{\dots}{Further parameters passed to \code{mosaic}.}
}
\details{
  Doubledecker plots visualize the the dependence of one
  categorical (typically binary) variable on further categorical
  variables.  Formally, they are mosaic plots with vertical splits for
  all dimensions (antecedents) except the last one, which represents the
  dependent variable (consequent).  The last variable is visualized by
  horizontal splits, no space between the tiles, and separate colors for
  the levels.
}
\value{
  The \code{"structable"} visualized is returned invisibly.
}
\references{
  H. Hoffmann (2001),
  Generalized odds ratios for visual modeling.
  \emph{Journal of Computational and Graphical Statistics},
  \bold{10}, 4, 628--640.

  Meyer, D., Zeileis, A., and Hornik, K. (2006),
  The strucplot framework: Visualizing multi-way contingency tables with
  \pkg{vcd}.
  \emph{Journal of Statistical Software}, \bold{17(3)}, 1-48.
  \doi{10.18637/jss.v017.i03} and available as 
  \code{vignette("strucplot")}.
}
\author{
  David Meyer \email{David.Meyer@R-project.org}
}

\seealso{
  \code{\link{strucplot}},
  \code{\link{mosaic}}
}
\examples{
data("Titanic")
doubledecker(Titanic)
doubledecker(Titanic, depvar = "Survived")
doubledecker(Survived ~ ., data = Titanic)
}
\keyword{hplot}

\name{table2d_summary}
\alias{table2d_summary}
\alias{print.table2d_summary}
\title{Summary of a 2-way Table}
\description{
  Prints a 2-way contingency table along with percentages, marginal,
  and conditional distributions.
}
\usage{
table2d_summary(object, margins = TRUE, percentages = FALSE,
  conditionals = c("none", "row", "column"), chisq.test = TRUE, \dots)
}
\arguments{
  \item{object}{a \eqn{r \times c}{r x c}-contingency table}
  \item{margins}{if \code{TRUE}, marginal distributions are computed.}
  \item{percentages}{if \code{TRUE}, relative frequencies are computed.}
  \item{conditionals}{if not \code{"none"}, the conditional
    distributions, given the row/column factor, are computed.}
  \item{chisq.test}{if \code{TRUE}, a chi-squared test of independence
    is carried out.}
  \item{\dots}{currently not used.}
}
\value{
  Returns invisibly a \eqn{r \times c \times k}{r x c x k}
  table, \eqn{k} depending on the amount of choices (at most 3).
}
\author{
  David Meyer \email{David.Meyer@R-project.org}
}
\seealso{
  \code{\link{mar_table}},
  \code{\link{prop.table}},
  \code{\link{independence_table}}
}
\examples{
data("UCBAdmissions")
table2d_summary(margin.table(UCBAdmissions, 1:2))
}
\keyword{category}

\name{JobSatisfaction}
\alias{JobSatisfaction}
\docType{data}
\title{Job Satisfaction Data}
\description{
  Data from Petersen (1968) about the job satisfaction of 715 blue
  collar workers, selected from Danish Industry in 1968.
}
\usage{
data("JobSatisfaction")
}
\format{
  A data frame with 8 observations and 4 variables.
  \describe{
    \item{Freq}{frequency.}
    \item{management}{factor indicating quality of management (bad,
      good).} 
    \item{supervisor}{factor indicating supervisor's job satisfaction
      (low, high).}
    \item{own}{factor indicating worker's own job satisfaction (low,
      high).}
  }
}

\references{
  E. B. Andersen (1991),
  \emph{The Statistical Analysis of Categorical Data}.
  2nd edition.
  Springer-Verlag, Berlin.

  E. Petersen (1968),
  \emph{Job Satisfaction in Denmark}.
  (In Danish).
  Mentalhygiejnisk Forlag, Copenhagen.
}

\source{
  E. B. Andersen (1991),
  The Statistical Analysis of Categorical Data, Table 5.4.
}

\examples{
data("JobSatisfaction")
structable(~ ., data = JobSatisfaction)

mantelhaen.test(xtabs(Freq ~ own + supervisor + management,
                      data = JobSatisfaction))

}

\keyword{datasets}

\name{woolf_test}
\alias{woolf_test}
\title{Woolf Test}
\description{
  Test for homogeneity on \eqn{2 \times 2 \times k}{2 x 2 x k} tables
  over strata (i.e., whether the log odds ratios are the same in all
  strata).
}
\usage{
woolf_test(x)
}
\arguments{
  \item{x}{A \eqn{2 \times 2 \times k}{2 x 2 x k} table.}
}
\value{
  A list of class \code{"htest"} containing the following
  components: 
  \item{statistic}{the chi-squared test statistic.}
  \item{parameter}{degrees of freedom of the approximate chi-squared
    distribution of the test statistic.}
  \item{p.value}{\eqn{p}-value for the test.}
  \item{method}{a character string indicating the type of test
    performed.}
  \item{data.name}{a character string giving the name(s) of the data.}
  \item{observed}{the observed counts.}
  \item{expected}{the expected counts under the null hypothesis.}
}
\seealso{
  \code{\link{mantelhaen.test}}
}
\references{
Woolf, B. 1955. On estimating the relation between blood group and 
disease. \emph{Ann. Human Genet.} (London) \bold{19}, 251-253.
}
\examples{
data("CoalMiners")
woolf_test(CoalMiners)
}
\keyword{htest}

\name{cotab_panel}
\alias{cotab_mosaic}
\alias{cotab_assoc}
\alias{cotab_sieve}
\alias{cotab_loddsratio}
\alias{cotab_agreementplot}
\alias{cotab_coindep}
\alias{cotab_fourfold}
\title{Panel-generating Functions for Contingency Table Coplots}
\description{
  Panel-generating functions visualizing contingency tables that
  can be passed to \code{cotabplot}.
}
\usage{
cotab_mosaic(x = NULL, condvars = NULL, \dots)
cotab_assoc(x = NULL, condvars = NULL, ylim = NULL, \dots)
cotab_sieve(x = NULL, condvars = NULL, \dots)
cotab_loddsratio(x = NULL, condvars = NULL, \dots)
cotab_agreementplot(x = NULL, condvars = NULL, \dots)
cotab_fourfold(x = NULL, condvars = NULL, \dots)
cotab_coindep(x, condvars,
  test = c("doublemax", "maxchisq", "sumchisq"),
  level = NULL, n = 1000, interpolate = c(2, 4),
  h = NULL, c = NULL, l = NULL, lty = 1,
  type = c("mosaic", "assoc"), legend = FALSE, ylim = NULL, \dots)
}
\arguments{
  \item{x}{a contingency tables in array form.}
  \item{condvars}{margin name(s) of the conditioning variables.}
  \item{ylim}{y-axis limits for \code{assoc} plot. By default this
    is computed from \code{x}.}
  \item{test}{character indicating which type of statistic
    should be used for assessing conditional independence.}
  \item{level,n,h,c,l,lty,interpolate}{variables controlling the HCL shading of the
    residuals, see \code{\link{shadings}} for more details.}
  \item{type}{character indicating which type of plot should be produced.}
  \item{legend}{logical. Should a legend be produced in each panel?}
  \item{\dots}{further arguments passed to the plotting function (such as
    \code{\link{mosaic}} or \code{\link{assoc}} or \code{\link{sieve}}
    respectively).}
}
\details{
  These functions of class \code{"panel_generator"} are panel-generating
  functions for use with \code{\link{cotabplot}}, i.e., they return functions
  with the interface
  
    \code{panel(x, condlevels)}

  required for \code{cotabplot}. The functions produced by \code{cotab_mosaic},
  \code{cotab_assoc} and \code{cotab_sieve} essentially only call \code{co_table}
  to produce the conditioned table and then call \code{\link{mosaic}}, \code{\link{assoc}}
  or \code{\link{sieve}} respectively with the arguments specified.
  
  The function \code{cotab_coindep} is similar but additionally chooses an appropriate
  residual-based shading visualizing the associated conditional independence
  model. The conditional independence test is carried out via \code{\link{coindep_test}}
  and the shading is set up via \code{\link{shading_hcl}}.
    
  A description of the underlying ideas is given in Zeileis, Meyer, Hornik
  (2005).
}
\seealso{
  \code{\link{cotabplot}},
  \code{\link{mosaic}},
  \code{\link{assoc}},
  \code{\link{sieve}},
  \code{\link{co_table}},
  \code{\link{coindep_test}},
  \code{\link{shading_hcl}}
}
\references{
  Meyer, D., Zeileis, A., and Hornik, K. (2006),
  The strucplot framework: Visualizing multi-way contingency tables with
  \pkg{vcd}.
  \emph{Journal of Statistical Software}, \bold{17(3)}, 1-48.
  \doi{10.18637/jss.v017.i03} and available as 
  \code{vignette("strucplot")}.

  Zeileis, A., Meyer, D., Hornik K. (2007), \emph{Residual-based shadings for visualizing
  (conditional) independence}, \emph{Journal of Computational and Graphical Statistics}, \bold{16},
  507--525.
}
\author{
  Achim Zeileis \email{Achim.Zeileis@R-project.org}
}
\examples{
data("UCBAdmissions")

cotabplot(~ Admit + Gender | Dept, data = UCBAdmissions)
cotabplot(~ Admit + Gender | Dept, data = UCBAdmissions, panel = cotab_assoc)
cotabplot(~ Admit + Gender | Dept, data = UCBAdmissions, panel = cotab_fourfold)

ucb <- cotab_coindep(UCBAdmissions, condvars = "Dept", type = "assoc",
                     n = 5000, margins = c(3, 1, 1, 3))
cotabplot(~ Admit + Gender | Dept, data = UCBAdmissions, panel = ucb)
}
\keyword{hplot}

\name{NonResponse}
\alias{NonResponse}
\docType{data}
\title{Non-Response Survey Data}
\description{
  Data about non-response for a Danish survey in 1965.
}
\usage{
data("NonResponse")
}
\format{
  A data frame with 12 observations and 4 variables.
  \describe{
    \item{Freq}{frequency.}
    \item{residence}{factor indicating whether residence was in
      Copenhagen, in a city outside Copenhagen or at the countryside
      (Copenhagen, City, Country).}
    \item{response}{factor indicating whether a response was given (yes,
      no).}
    \item{gender}{factor indicating gender (male, female).}
  }
}

\references{
  E. B. Andersen (1991),
  \emph{The Statistical Analysis of Categorical Data}.
  2nd edition.
  Springer-Verlag, Berlin.
}
\source{
  E. B. Andersen (1991),
  The Statistical Analysis of Categorical Data, Table 5.17.
}
\examples{
data("NonResponse")
structable(~ ., data = NonResponse)
}
\keyword{datasets}

\name{Kappa}
\alias{Kappa}
\alias{print.Kappa}
\alias{confint.Kappa}
\alias{summary.Kappa}
\alias{print.summary.Kappa}
\title{Cohen's Kappa and Weighted Kappa}
\description{
  Computes two agreement rates: Cohen's kappa and weighted kappa, and
  confidence bands.
}
\usage{
Kappa(x, weights = c("Equal-Spacing", "Fleiss-Cohen"))
\S3method{print}{Kappa}(x, digits=max(getOption("digits") - 3, 3),
CI=FALSE, level=0.95, ...)
\S3method{confint}{Kappa}(object, parm, level = 0.95, ...)
\S3method{summary}{Kappa}(object, ...)
\S3method{print}{summary.Kappa}(x, ...)
}
\arguments{
  \item{x}{For \code{Kappa}: a confusion matrix. For the print methods:
    object of class \code{"Kappa"} or \code{"summary.Kappa"}}
  \item{weights}{either one of the character strings given in the
    default value, or a user-specified matrix with same dimensions as
    \code{x}.}
  \item{digits}{minimal number of significant digits.}
  \item{CI}{logical; shall confidence limits be added to the output?}
  \item{level}{confidence level between 0 and 1 used for the confidence
    interval.}
  \item{object}{object of class \code{"Kappa"}.}
  \item{parm}{Currently, ignored.}
  \item{\dots}{Further arguments passed to the default print method.}
}
\details{
  Cohen's kappa is the diagonal sum of the (possibly weighted) relative
  frequencies, corrected for expected values and standardized by its
  maximum value.
  The equal-spacing weights are defined by \eqn{1 - |i - j| / (r -
    1)}{1 - abs(i - j) / (r - 1)}, \eqn{r} number of columns/rows, and
  the Fleiss-Cohen weights by \eqn{1 - |i - j|^2 / (r - 1)^2}{1 -
    abs(i - j)^2 / (r - 1)^2}.
  The latter one attaches greater importance to near disagreements.
    
}
\value{
  An object of class \code{"Kappa"} with three components:
  \item{Unweighted}{numeric vector of length 2 with the kappa statistic
    (\code{value} component), along with Approximate Standard Error
    (\code{ASE} component)}
  \item{Weighted}{idem for the weighted kappa.}
  \item{Weights}{numeric matrix with weights used.}
}
\note{
  The \code{summary} method also prints the weights.

  There is a \code{confint} method for computing approximate confidence
  intervals.
}
\references{
  Cohen, J. (1960),
  A coefficient of agreement for nominal scales.
  \emph{Educational and Psychological Measurement},
  \bold{20}, 37--46.

  Everitt, B.S. (1968),
  Moments of statistics kappa and weighted kappa.
  \emph{The British Journal of Mathematical and Statistical Psychology},
  \bold{21}, 97--103.

  Fleiss, J.L., Cohen, J., and Everitt, B.S. (1969),
  Large sample standard errors of kappa and weighted kappa.
  \emph{Psychological Bulletin},
  \bold{72}, 332--327.

}
\author{
  David Meyer \email{David.Meyer@R-project.org}
}
\seealso{
  \code{\link{agreementplot}},
  \code{\link{confint}}
}
\examples{
data("SexualFun")
K <- Kappa(SexualFun)
K
confint(K)
summary(K)
print(K, CI = TRUE)

}
\keyword{category}

\name{MSPatients}
\alias{MSPatients}
\docType{data}
\title{Diagnosis of Multiple Sclerosis}
\description{
  Data from Westlund & Kurland (1953) on the diagnosis of multiple
  sclerosis (MS): two samples of patients, one from Winnipeg and one
  from New Orleans, were each rated by two neurologists (one from each
  city) in four diagnostic categories.
}
\usage{
data("MSPatients")
}
\format{
  A 3-dimensional array resulting from cross-tabulating 218 observations
  on 3 variables.  The variables and their levels are as follows:

  \tabular{rll}{
    No \tab Name \tab Levels \cr
    1 \tab New Orleans Neurologist \tab Certain, Probable, Possible, Doubtful \cr
    2 \tab Winnipeg Neurologist \tab Certain, Probable, Possible, Doubtful \cr
    3 \tab Patients \tab Winnipeg, New Orleans
  }
}

\references{
  K. B. Westlund & L. T. Kurland (1953),
  Studies on multiple sclerosis in Winnipeg, Manitoba and New Orleans,
  Louisiana,
  \emph{American Journal of Hygiene}, \bold{57}, 380--396.

  M. Friendly (2000),
  \emph{Visualizing Categorical Data}.
  SAS Institute, Cary, NC.
}

\source{M. Friendly (2000),
  Visualizing Categorical Data:
  \url{http://euclid.psych.yorku.ca/ftp/sas/vcd/catdata/msdiag.sas}
}

\examples{
data("MSPatients")
\dontrun{
## best visualized using a resized device, e.g. using:
## get(getOption("device"))(width = 12)
pushViewport(viewport(layout = grid.layout(ncol = 2)))
pushViewport(viewport(layout.pos.col = 1))
popViewport()
pushViewport(viewport(layout.pos.col = 2))
popViewport(2)
dev.off()
}

## alternative, more convenient way
mplot(
  agreementplot(t(MSPatients[,,1]), return_grob = TRUE,
    main = "Winnipeg Patients"),
  agreementplot(t(MSPatients[,,2]), return_grob = TRUE,
    main = "New Orleans Patients")
)

## alternatively, use cotabplot:
cotabplot(MSPatients, panel = cotab_agreementplot)
}
\keyword{datasets}

\name{HorseKicks}
\alias{HorseKicks}
\docType{data}
\title{Death by Horse Kicks}
\description{
  Data from von Bortkiewicz (1898), given by Andrews & Herzberg (1985),
  on number of deaths by horse or mule kicks in 10 (of 14 reported)
  corps of the Prussian army.  4 corps were not considered by Fisher
  (1925) as they had a different organization. This data set is a
  popular subset of the \code{\link{VonBort}} data.
}
\usage{
data("HorseKicks")
}
\format{
  A 1-way table giving the number of deaths in 200 corps-years.  The
  variable and its levels are
  \tabular{rll}{
    No \tab Name \tab Levels \cr
    1 \tab nDeaths \tab 0, 1, \dots, 4 \cr
  }
}

\references{
  D. F. Andrews & A. M. Herzberg (1985),
  \emph{Data: A Collection of Problems from Many Fields for the Student
    and Research Worker}.
  Springer-Verlag, New York, NY.

  R. A. Fisher (1925),
  \emph{Statistical Methods for Research Workers}.
  Oliver & Boyd, London.

  L. von Bortkiewicz (1898),
  \emph{Das Gesetz der kleinen Zahlen}.
  Teubner, Leipzig.

  M. Friendly (2000),
  \emph{Visualizing Categorical Data}.
  SAS Institute, Cary, NC.
}

\source{
  Michael Friendly (2000),
  Visualizing Categorical Data, page 18.
}
\seealso{
  \code{\link{VonBort}}
}
\examples{
data("HorseKicks")
gf <- goodfit(HorseKicks)
summary(gf)
plot(gf)
}

\keyword{datasets}

\name{UKSoccer}
\alias{UKSoccer}
\docType{data}
\title{UK Soccer Scores}
\description{
  Data from Lee (1997), on the goals scored by Home and Away teams in
  the Premier Football League, 1995/6 season.
}
\usage{
data("UKSoccer")
}
\format{
  A 2-dimensional array resulting from cross-tabulating the number of
  goals scored in 380 games.  The variables and their levels are as
  follows:

  \tabular{rll}{
    No \tab Name \tab Levels \cr
    1 \tab Home \tab 0, 1, \dots, 4 \cr
    2 \tab Away \tab 0, 1, \dots, 4
  }
}

\references{
  A. J. Lee (1997),
  Modelling scores in the Premier League: Is Manchester United really
  the best?,
  \emph{Chance}, \bold{10}(1), 15--19.

  M. Friendly (2000),
  \emph{Visualizing Categorical Data}.
  SAS Institute, Cary, NC.
}
\source{
  M. Friendly (2000),
  Visualizing Categorical Data, page 27.
}

\seealso{
  \code{\link{Bundesliga}}
}
\examples{
data("UKSoccer")
mosaic(UKSoccer, gp = shading_max, main = "UK Soccer Scores")
}

\keyword{datasets}

\name{mplot}
\alias{mplot}
\title{Multiple Grid plots}
\description{
  combines severals grid-based plots in a multi-panel-layout.
}
\usage{
mplot(..., .list = list(),
      layout = NULL, cex = NULL,
      main = NULL, gp_main = gpar(fontsize = 20),
      sub = NULL, gp_sub = gpar(fontsize = 15),
      keep_aspect_ratio = TRUE)
}
\arguments{
  \item{\dots, .list}{A list of objects inheriting from class \code{"grob"},
    or having a \code{"grob"} attribute containing such an object.}
  \item{layout}{integer vector of length 2 giving the number of rows and
    columns. If \code{NULL}, the values will be
    guessed using some heuristics from the number of objects supplied in
    \dots.}
  \item{cex}{Scaling factor for the fonts in the subplots. If
    \code{NULL}, the value is calculated as the inverse square root of
    the row number.}
  \item{main, sub}{Optional main and sub title, respectively.}
  \item{gp_main, gp_sub}{Optional objects of class \code{"gpar"} specifying
    the graphical parameters for the main and sub title, respectively.}
  \item{keep_aspect_ratio}{logical; should the aspect ratio of the plots
  be fixed?}
}
\value{
  None.
}
\details{
  This is a convenience function for producing multi-panel plots from
  grid-based displays, especially those produced by the vcd methods. The
  layout (number of rows and columns) is guessed from the amount of
  supplied objects, if not supplied. Currently, the vcd plotting
  functions do not return grob objects by default---this might change in
  the future. Also, some of them will return the grob object as a
  \code{"grob"} attribute, attached to the currently returned object.
}
\author{
  David Meyer \email{David.Meyer@R-project.org}
}
\examples{
mplot(mosaic(Titanic, return_grob = TRUE),
      assoc(Titanic), return_grob = TRUE)

A = mosaic(Titanic, return_grob = TRUE)
B = mosaic(Titanic, type = "expected", return_grob = TRUE)
mplot(A, B)

mplot(sieve(SexualFun, return_grob = TRUE),
      agreementplot(SexualFun, return_grob = TRUE),
      main = "Sexual Fun")

mplot(A, grid.circle())
}

\name{PreSex}
\alias{PreSex}
\docType{data}
\title{Pre-marital Sex and Divorce}
\description{
  Data from Thornes & Collard (1979), reported in Gilbert (1981), on
  pre- and extra-marital sex and divorce.
}
\usage{
data("PreSex")
}
\format{
  A 4-dimensional array resulting from cross-tabulating 1036
  observations on 4 variables.  The variables and their levels are as
  follows:

  \tabular{rll}{
    No \tab Name \tab Levels \cr
    1 \tab MaritalStatus \tab Divorced, Married \cr
    2 \tab ExtramaritalSex \tab Yes, No \cr
    3 \tab PremaritalSex \tab Yes, No \cr
    4 \tab Gender \tab Women, Men
  }
}

\references{
  G. N. Gilbert (1981),
  \emph{Modelling Society: An Introduction to Loglinear Analysis for
    Social Researchers}.
  Allen and Unwin, London.

  B. Thornes & J. Collard (1979),
  \emph{Who Divorces?}.
  Routledge & Kegan, London.

  M. Friendly (2000),
  \emph{Visualizing Categorical Data}.
  SAS Institute, Cary, NC.
}

\source{
  Michael Friendly (2000),
  Visualizing Categorical Data:
  \url{http://euclid.psych.yorku.ca/ftp/sas/vcd/catdata/marital.sas}
}

\examples{
data("PreSex")

## Mosaic display for Gender and Premarital Sexual Experience
## (Gender Pre)
mosaic(margin.table(PreSex, c(3,4)), 
                main = "Gender and Premarital Sex")

## (Gender Pre)(Extra)
mosaic(margin.table(PreSex, c(2,3,4)), 
       expected = ~Gender * PremaritalSex + ExtramaritalSex ,
	   main = "PreMaritalSex*Gender +Sex")

## (Gender Pre Extra)(Marital)
mosaic(PreSex,
       expected = ~Gender*PremaritalSex*ExtramaritalSex + MaritalStatus,
       main = "PreMarital*ExtraMarital + MaritalStatus")

## (GPE)(PEM)
mosaic(PreSex, 
       expected = ~ Gender * PremaritalSex * ExtramaritalSex
                    + MaritalStatus * PremaritalSex * ExtramaritalSex,
       main = "G*P*E + P*E*M")
}

\keyword{datasets}

\name{spine}
\alias{spine}
\alias{spine.default}
\alias{spine.formula}
\title{Spine Plots and Spinograms}
\description{
  Spine plots are a special cases of mosaic plots, and can be seen as
  a generalization of stacked (or highlighted) bar plots. Analogously,
  spinograms are an extension of histograms.
}
\usage{
spine(x, \dots)
\method{spine}{default}(x, y = NULL,
  breaks = NULL, ylab_tol = 0.05, off = NULL,
  main = "", xlab = NULL, ylab = NULL, ylim = c(0, 1), margins = c(5.1, 4.1, 4.1, 3.1),
  gp = gpar(), name = "spineplot", newpage = TRUE, pop = TRUE,
  \dots)
\method{spine}{formula}(formula, data = list(),
  breaks = NULL, ylab_tol = 0.05, off = NULL,
  main = "", xlab = NULL, ylab = NULL, ylim = c(0, 1), margins = c(5.1, 4.1, 4.1, 3.1),
  gp = gpar(), name = "spineplot", newpage = TRUE, pop = TRUE,
  \dots)
}
\arguments{
  \item{x}{an object, the default method expects either a single variable
    (interpreted to be the explanatory variable) or a 2-way table. See details.}
  \item{y}{a \code{"factor"} interpreted to be the dependent variable}
  \item{formula}{a \code{"formula"} of type \code{y ~ x} with a single dependent \code{"factor"}
    and a single explanatory variable.}    
  \item{data}{an optional data frame.}
  \item{breaks}{if the explanatory variable is numeric, this controls how
    it is discretized. \code{breaks} is passed to \code{\link{hist}} and can
    be a list of arguments.}
  \item{ylab_tol}{convenience tolerance parameter for y-axis annotation.
    If the distance between two labels drops under this threshold, they are
    plotted equidistantly.}
  \item{off}{vertical offset between the bars (in per cent). It is fixed to
    \code{0} for spinograms and defaults to \code{2} for spine plots.}
  \item{main, xlab, ylab}{character strings for annotation}
  \item{ylim}{limits for the y axis}
  \item{margins}{margins when calling \code{\link{plotViewport}}}
  \item{gp}{a \code{"gpar"} object controlling the grid graphical
    parameters of the rectangles. It should specify in particular a vector of
    \code{fill} colors of the same length as \code{levels(y)}. The default is
    to call \code{\link{gray.colors}}.}  
  \item{name}{name of the plotting viewport.}
  \item{newpage}{logical. Should \code{\link{grid.newpage}} be called 
    before plotting?}
  \item{pop}{logical. Should the viewport created be popped?}
  \item{\dots}{additional arguments passed to \code{\link{plotViewport}}.}
}
\details{
  \code{spine} creates either a spinogram or a spine plot. It can be called
  via \code{spine(x, y)} or \code{spine(y ~ x)} where \code{y} is interpreted
  to be the dependent variable (and has to be categorical) and \code{x}
  the explanatory variable. \code{x} can be either categorical (then a spine
  plot is created) or numerical (then a spinogram is plotted).
  Additionally, \code{spine} can also be called with only a single argument
  which then has to be a 2-way table, interpreted to correspond to \code{table(x, y)}.
  
  Spine plots are a generalization of stacked bar plots where not the heights
  but the widths of the bars corresponds to the relative frequencies of \code{x}.
  The heights of the bars then correspond to the conditional relative frequencies
  of \code{y} in every \code{x} group. This is a special case of a mosaic plot
  with specific spacing and shading.
  
  Analogously, spinograms extend stacked histograms. As for the histogram,
  \code{x} is first discretized (using \code{\link{hist}}) and then for the
  discretized data a spine plot is created.
}
\value{
  The table visualized is returned invisibly.
}
\seealso{
  \code{\link{cd_plot}}, \code{\link{mosaic}}, \code{\link{hist}}
}
\references{
  Hummel, J. (1996), Linked bar charts: Analysing categorical data graphically.
  \emph{Computational Statistics}, \bold{11}, 23--33.

  Hofmann, H., Theus, M. (2005), \emph{Interactive graphics for visualizing
  conditional distributions}, Unpublished Manuscript.
}
\author{
  Achim Zeileis \email{Achim.Zeileis@R-project.org}
}
\examples{
## Arthritis data (dependence on a categorical variable)
data("Arthritis")
(spine(Improved ~ Treatment, data = Arthritis))

## Arthritis data (dependence on a numerical variable)
(spine(Improved ~ Age, data = Arthritis, breaks = 5))
(spine(Improved ~ Age, data = Arthritis, breaks = quantile(Arthritis$Age)))
(spine(Improved ~ Age, data = Arthritis, breaks = "Scott"))

## Space shuttle data (dependence on a numerical variable)
data("SpaceShuttle")
(spine(Fail ~ Temperature, data = SpaceShuttle, breaks = 3))
}
\keyword{hplot}

\name{tile}
\alias{tile}
\alias{tile.default}
\alias{tile.formula}
\title{Tile Plot}
\description{
  Plots a tile display.
}
\usage{
\method{tile}{default}(x,
         tile_type = c("area", "squaredarea", "height", "width"),
         halign = c("left", "center", "right"),
         valign = c("bottom", "center", "top"),
         split_vertical = NULL,
         shade = FALSE,
         spacing = spacing_equal(unit(1, "lines")),
         set_labels = NULL,
         margins = unit(3, "lines"),
         keep_aspect_ratio = FALSE,
         legend = NULL,
         legend_width = NULL,
         squared_tiles = TRUE,
         main = NULL, sub = NULL, ...)
\method{tile}{formula}(formula, data,
  \dots, main = NULL, sub = NULL, subset = NULL, na.action = NULL)
}
\arguments{
  \item{x}{a contingency table, or an object coercible to one.}
  \item{formula}{a formula specifying the variables used to create a 
    contingency table from \code{data}}.
  \item{data}{either a data frame, or an object of class \code{"table"}
    or \code{"ftable"}.}
  \item{subset}{an optional vector specifying a subset of observations to be used.}
  \item{na.action}{a function which indicates what should happen when
    the data contain \code{NA}s. Ignored if \code{data} is a contingency
    table.}
  \item{tile_type}{character string indicating how the tiles should reflect
    the table frequencies (see details).}
  \item{halign, valign}{character string specifying the horizontal and
    vertical alignment of the tiles.}
  \item{split_vertical}{vector of logicals of length \eqn{k}, where
    \eqn{k} is the number of margins of \code{x} (values are recycled as needed).
    A \code{TRUE} component indicates that the tile(s) of the
    corresponding dimension should be split vertically, \code{FALSE}
    means horizontal splits. Default is \code{FALSE.}}
  \item{spacing}{spacing object, spacing function, or corresponding
    generating function (see \code{\link{strucplot}} for more
    information).}
  \item{set_labels}{An optional character vector with named components
    replacing the so-specified variable names. The component names must
    exactly match the variable names to be replaced.}
  \item{shade}{logical specifying whether shading should be enabled or
    not (see \code{\link{strucplot}}).}
  \item{margins}{either an object of class \code{"unit"} of length 4, or
    a numeric vector of length 4. The elements are recycled as needed.
    The four components specify the top, right,
    bottom, and left margin of the plot, respectively.
    When a numeric vector is supplied, the numbers are interpreted as
    \code{"lines"} units.  In addition, the unit or numeric vector
    may have named arguments
    (\samp{top}, \samp{right}, \samp{bottom}, and \samp{left}), in which
    case the non-named arguments specify the default values (recycled as
    needed), overloaded by the named arguments.}
  \item{legend}{either a legend-generating function, or a legend
    function (see details and \code{\link{legends}}), or a logical.
    If \code{legend} is \code{NULL} or \code{TRUE} and \code{gp} is a
    function or missing, legend defaults to \code{\link{legend_resbased}}.
  }
  \item{legend_width}{An object of class \code{"unit"} of length
    1 specifying the width of the legend (if any). Default: 5 lines.}
  \item{keep_aspect_ratio}{logical indicating whether the aspect ratio should be
    fixed or not. The default is \code{FALSE} to enable the creation of
    squared tiles.}
  \item{squared_tiles}{logical indicating whether white space should be
    added as needed to rows or columns to obtain squared tiles in case of an
    unequal number of row and column labels.}
  \item{main, sub}{either a logical, or a character string used for plotting
    the main (sub) title.  If logical and \code{TRUE}, the
    name of the \code{data} object is used.}
  \item{\dots}{Other arguments passed to \code{\link{strucplot}}}
}
\details{
  A tile plot is a matrix of tiles. For each tile, either the \code{"width"},
  \code{"height"}, \code{"area"}, or squared area is proportional to the
  corresponding entry. The first three options allow
  column-wise, row-wise and overall comparisons, respectively. The last
  variant allows to compare the tiles both column-wise and row-wise,
  considering either the width or the height, respectively.

  In contrast to other high-level strucplot functions, \code{tile}
  also accepts a table with duplicated levels (see examples). In this
  case, artificial dimnames will be created, and the actual ones are
  drawn using \code{set_labels}.
  
  Note that multiway-tables are first \dQuote{flattened} using
  \code{structable}.
}
\value{
  The \code{"structable"} visualized is returned invisibly.
}
\author{
  David Meyer \email{David.Meyer@R-project.org}
}

\seealso{
  \code{\link{assoc}},
  \code{\link{strucplot}},
  \code{\link{mosaic}},
  \code{\link{structable}},
}
\examples{
data("Titanic")

## default plot
tile(Titanic)
tile(Titanic, type = "expected")
tile(Titanic, shade = TRUE)

## some variations
tile(Titanic, tile_type = "squaredarea")
tile(Titanic, tile_type = "width", squared_tiles = FALSE)
tile(Titanic, tile_type = "height", squared_tiles = FALSE)
tile(Titanic, tile_type = "area", halign = "center", valign = "center")

## repeat levels
tile(Titanic[,,,c(1,2,1,2)])
}
\keyword{hplot}

\name{labeling_cells_list}
\alias{labeling_list}
\alias{labeling_cells}
\title{Labeling Functions for Strucplots}
\description{
  These functions generate labeling functions that produce labels for
  strucplots.
}
\usage{
labeling_cells(labels = TRUE, varnames = TRUE,
  abbreviate_labels = FALSE, abbreviate_varnames = FALSE,
  gp_text = gpar(), lsep = ": ", lcollapse = "\n",
  just = "center", pos = "center", rot = 0,
  margin = unit(0.5, "lines"), clip_cells = TRUE,
  text = NULL, \dots)
labeling_list(gp_text = gpar(), just = "left", pos = "left", lsep = ": ",
  sep = " ", offset = unit(c(2, 2), "lines"),
  varnames = TRUE, cols = 2, \dots) 
}
\arguments{
  \item{labels}{vector of logicals indicating, for each dimension,
    whether labels for the factor levels should be drawn or not.
    Values are recycled as needed.} 
  \item{varnames}{vector of logicals indicating, for each dimension,
    whether variable names should be drawn.
    Values are recycled as needed.}
  \item{abbreviate_labels}{vector of integers or logicals indicating,
    for each dimension, the number of characters the labels should be
    abbreviated to.
    \code{TRUE} means 1 character, \code{FALSE} causes no abbreviation.
    Values are recycled as needed.}
  \item{abbreviate_varnames}{vector of integers or logicals indicating,
    for each dimension, the number of characters the variable
    (i.e., dimension) names should be abbreviated to.
    \code{TRUE} means 1 character, \code{FALSE} causes no abbreviation.
    Values are recycled as needed.}
  \item{gp_text}{object of class \code{"gpar"} used for the text drawn.}
  \item{lsep}{character that separates variable names from the factor
    levels.}
  \item{sep}{character that separates the factor levels (only used for
    \code{labeling_list}).} 
  \item{offset}{object of class \code{"unit"} of length 2 specifying the
    offset in x- and y-direction of the text block drawn under the
    strucplot (only used for \code{labeling_list}).}
  \item{cols}{number of text columns (only used for
    \code{labeling_list}).}
  \item{lcollapse}{character that separates several variable name/factor
    level-combinations.  Typically a line break.
    (Only used for \code{labeling_cells}.)}
  \item{just, pos}{character string of length 1 (\code{labeling_list})
    or at most 2 (\code{labeling_cells}) specifying the labels'
    horizontal position and justification (horizontal and vertical for
    \code{labeling_cells}).}
  \item{rot}{rotation angle in degrees, used for all labels (only used
    for \code{labeling_cells}).}
  \item{margin}{object of class \code{"unit"} (a numeric value is
    converted to \code{"lines"}) specifying an offset from the cell
    borders (only used for \code{labeling_cells}).}
  \item{clip_cells}{logical indicating whether text should be clipped at
    the cell borders (only used for \code{labeling_cells}).}
  \item{text}{Optionally, a character table of the same dimensions than
    the contingency table whose entries will then be used instead of
    the labels.  \code{NA} entries are not drawn.
    This allows custom cell annotations (see examples).
    Only used for \code{labeling_cells}.}
  \item{\dots}{Currently not used.}
}
\details{
  These functions generate labeling functions that can add different
  kinds of labels to an existing plot. Typically they are 
  supplied to \code{\link{strucplot}} which then generates and calls
  the labeling function. They assume that a strucplot has been drawn
  and the corresponding viewport structure is pushed, so that by 
  navigating through the viewport tree the labels can be positioned
  appropriately.
  
  This help page only documents \code{labeling_list} and
  \code{labeling_cells}; more functions are described on the help page
  for \code{\link{labeling_border}}.
  
  The functions can also be used \sQuote{stand-alone} as shown in the
  examples.

  Using \code{labeling_list} will typically necessitate a bottom margin
  adjustment.
}
\value{
  A function with arguments:
  \item{d}{\code{"dimnames"} attribute from the visualized contingency
    table, or the visualized table itself from which the
    \code{"dimnames"} attributes will then be extracted.}
  \item{split_vertical}{vector of logicals indicating the split
    directions.}
  \item{condvars}{integer vector of conditioning dimensions}
}
\author{
  David Meyer \email{David.Meyer@R-project.org}
}
\references{
  Meyer, D., Zeileis, A., and Hornik, K. (2006),
  The strucplot framework: Visualizing multi-way contingency tables with
  \pkg{vcd}.
  \emph{Journal of Statistical Software}, \bold{17(3)}, 1-48.
  \doi{10.18637/jss.v017.i03} and available as 
  \code{vignette("strucplot")}.
}
\seealso{
  \code{\link{labeling_border}}, 
  \code{\link{structable}}, 
  \code{\link[grid]{grid.text}}
}
\examples{
data("Titanic")

mosaic(Titanic, labeling = labeling_cells)
mosaic(Titanic, labeling = labeling_list)

## A more complex example, adding the observed frequencies
## to a mosaic plot:
tab <- ifelse(Titanic < 6, NA, Titanic)
mosaic(Titanic, pop = FALSE)
labeling_cells(text = tab, margin = 0)(Titanic)
}
\keyword{hplot}

\name{assoc}
\alias{assoc}
\alias{assoc.default}
\alias{assoc.formula}
\title{Extended Association Plots}
\description{
  Produce an association plot indicating deviations from a
  specified independence model in a possibly high-dimensional
  contingency table.
}
\usage{
\method{assoc}{default}(x, row_vars = NULL, col_vars = NULL, compress = TRUE,
  xlim = NULL, ylim = NULL,
  spacing = spacing_conditional(sp = 0), spacing_args = list(),
  split_vertical = NULL, keep_aspect_ratio = FALSE, 
  xscale = 0.9, yspace = unit(0.5, "lines"), main = NULL, sub = NULL,
  \dots, residuals_type = "Pearson", gp_axis = gpar(lty = 3))
\method{assoc}{formula}(formula, data = NULL, \dots, subset = NULL, na.action = NULL, main = NULL, sub = NULL)
}
\arguments{
  \item{x}{a contingency table in array form with optional category
    labels specified in the \code{dimnames(x)} attribute, or an object
    inheriting from the \code{"ftable"} class (such as
    \code{"structable"} objects).}
  \item{row_vars}{a vector of integers giving the indices, or a
    character vector giving the names of the variables to be used for
    the rows of the association plot.}
  \item{col_vars}{a vector of integers giving the indices, or a
    character vector giving the names of the variables to be used for
    the columns of the association plot.}
  \item{compress}{logical; if \code{FALSE}, the space between the rows
    (columns) are chosen such that the \emph{total} heights (widths) of
    the rows (columns) are all equal.  If \code{TRUE}, the space between
    rows and columns is fixed and hence the plot is more
    \dQuote{compressed}.}
  \item{xlim}{a \eqn{2 \times k}{2 x k} matrix of doubles, \eqn{k}
    number of total columns of the plot.  The columns of \code{xlim}
    correspond to the columns of the association plot, the rows describe
    the column ranges (minimums in the first row, maximums in the second
    row).  If \code{xlim} is \code{NULL}, the ranges are determined from
    the residuals according to \code{compress} (if \code{TRUE}: widest
    range from each column, if \code{FALSE}: from the whole association
    plot matrix).}
  \item{ylim}{a \eqn{2 \times k}{2 x k} matrix of doubles, \eqn{k}
    number of total rows of the plot.  The columns of \code{ylim}
    correspond to the rows of the association plot, the rows describe
    the column ranges (minimums in the first row, maximums in the second
    row).  If \code{ylim} is \code{NULL}, the ranges are determined from
    the residuals according to \code{compress} (if \code{TRUE}: widest
    range from each row, if \code{FALSE}: from the whole association
    plot matrix).}
  \item{spacing}{a spacing object, a spacing function, or a
    corresponding generating function (see \code{\link{strucplot}} for
    more information).  The default is the spacing-generating function
    \code{\link{spacing_conditional}} that is (by default) called with the
    argument list \code{spacing_args} (see \code{spacings} for more details).}
  \item{spacing_args}{list of arguments for the spacing-generating function, if
    specified (see \code{\link{strucplot}} for more information).}
  \item{split_vertical}{vector of logicals of length \eqn{k}, where \eqn{k}
    is the number of margins of \code{x} (default: \code{FALSE}).
    Values are recycled as needed.
    A \code{TRUE} component indicates that the corresponding dimension
    is folded into the columns, \code{FALSE} folds the dimension into the
    rows.}
  \item{keep_aspect_ratio}{logical indicating whether the aspect ratio should be
    fixed or not.}
  \item{residuals_type}{a character string indicating the type of
    residuals to be computed. Currently, only Pearson residuals are
    supported.}
  \item{xscale}{scale factor resizing the tile's width, thus adding
    additional space between the tiles. }
  \item{yspace}{object of class \code{"unit"} specifying additional
    space separating the rows.}
  \item{gp_axis}{object of class \code{"gpar"} specifying the visual
    aspects of the tiles' baseline.}
  \item{formula}{a formula object with possibly both left and right hand
    sides specifying the column and row variables of the flat table.}
  \item{data}{a data frame, list or environment containing the variables
    to be cross-tabulated, or an object inheriting from class \code{table}.}
  \item{subset}{an optional vector specifying a subset of observations
    to be used. Ignored if \code{data} is a contingency table.}
  \item{na.action}{an optional function which indicates what should happen when
    the data contain \code{NA}s. Ignored if \code{data} is a contingency table.}
  \item{main, sub}{either a logical, or a character string used for plotting
    the main (sub) title.  If logical and \code{TRUE}, the
    name of the \code{data} object is used.}
  \item{\dots}{other parameters passed to \code{\link{strucplot}}}
}
\details{
  Association plots have been suggested by Cohen (1980) and extended
  by Friendly (1992) and provide a means for visualizing the residuals
  of an independence model for a contingency table.

  \code{assoc} is a generic function and currently has a default method and a
  formula interface.  Both are high-level interfaces to the
  \code{\link{strucplot}} function, and produce (extended) association
  plots.  Most of the functionality is described there, such as
  specification of the independence model, labeling, legend, spacing,
  shading, and other graphical parameters.

  For a contingency table, the signed contribution
  to Pearson's \eqn{\chi^2}{chi^2} for cell \eqn{\{ij\ldots k\}} is
  
  \deqn{d_{ij\ldots k} = \frac{(f_{ij\ldots k} - e_{ij\ldots k})}{
      \sqrt{e_{ij\ldots k}}}}{d_\{ij\ldotsk\} = (f_\{ij\ldotsk\} -
    e_\{ij\ldotsk\}) / sqrt(e_\{ij\ldotsk\})}
  
  where \eqn{f_{ij\ldots k}}{f_\{ij\ldotsk\}} and \eqn{e_{ij\ldots
      k}}{e_\{ij\ldotsk\}} 
  are the observed and  expected counts corresponding to the cell.  In
  the association plot, each cell is represented by a
  rectangle that has (signed) height proportional to \eqn{d_{ij\ldots
      k}}{d_\{ij\ldotsk\}} 
  and width proportional to
  \eqn{\sqrt{e_{ij\ldots k}}}{sqrt(e_\{ij...k\})},
  so that the area of the box is proportional to the difference in
  observed and expected frequencies.  The rectangles in each row are
  positioned relative to a baseline indicating independence
  (\eqn{d_{ij\ldots k} = 0}{d_\{ij\ldotsk\} = 0}). 
  If the observed frequency of a cell is greater than the expected one,
  the box rises above the baseline, and falls below otherwise.

  Additionally, the residuals can be colored depending on a specified
  shading scheme (see Meyer et al., 2003).  Package \pkg{vcd} offers a range of
  \emph{residual-based} shadings (see the shadings help page). Some of
  them allow, e.g., the visualization of test statistics.

  Unlike the \code{\link[graphics]{assocplot}} function in the
  \pkg{graphics} package, this function allows the visualization of
  contingency tables with more than two dimensions. Similar to the
  construction of \sQuote{flat} tables (like objects of class \code{"ftable"} or
  \code{"structable"}), the dimensions are folded into rows and columns.
 
  The layout is very flexible: the specification of shading, labeling,
  spacing, and legend is modularized (see \code{\link{strucplot}} for
  details).

}
\value{
  The \code{"structable"} visualized is returned invisibly.
}
\seealso{
  \code{\link{mosaic}},
  \code{\link{strucplot}},
  \code{\link{structable}}
}
\references{
  Cohen, A. (1980),
  On the graphical display of the significant components in a two-way
  contingency table.
  \emph{Communications in Statistics---Theory and Methods}, \bold{A9},
  1025--1041.

  Friendly, M. (1992),
  Graphical methods for categorical data.
  \emph{SAS User Group International Conference Proceedings}, \bold{17},
  190--200.
  \url{http://datavis.ca/papers/sugi/sugi17.pdf}
  
  Meyer, D., Zeileis, A., Hornik, K. (2003), Visualizing independence using
  extended association plots. \emph{Proceedings of the 3rd International
  Workshop on Distributed Statistical Computing}, K. Hornik, F. Leisch,
  A. Zeileis (eds.), ISSN 1609-395X.
  \url{https://www.R-project.org/conferences/DSC-2003/Proceedings/}

  Meyer, D., Zeileis, A., and Hornik, K. (2006),
  The strucplot framework: Visualizing multi-way contingency tables with
  \pkg{vcd}.
  \emph{Journal of Statistical Software}, \bold{17(3)}, 1-48.
  \doi{10.18637/jss.v017.i03} and available as \code{vignette ("strucplot", package = "vcd")}.

}
\author{
  David Meyer \email{David.Meyer@R-project.org}
}
\examples{
data("HairEyeColor")
## Aggregate over sex:
(x <- margin.table(HairEyeColor, c(1, 2)))

## Ordinary assocplot:
assoc(x)
## and with residual-based shading (of independence)
assoc(x, main = "Relation between hair and eye color", shade = TRUE)

## Aggregate over Eye color:
(x <- margin.table(HairEyeColor, c(1, 3)))
chisq.test(x)
assoc(x, main = "Relation between hair color and sex", shade = TRUE)

# Visualize multi-way table
assoc(aperm(HairEyeColor), expected = ~ (Hair + Eye) * Sex,
      labeling_args = list(just_labels = c(Eye = "left"),
                           offset_labels = c(right = -0.5),
                           offset_varnames = c(right = 1.2),
                           rot_labels = c(right = 0),
                           tl_varnames = c(Eye = TRUE))
)

assoc(aperm(UCBAdmissions), expected = ~ (Admit + Gender) * Dept, compress = FALSE,
      labeling_args = list(abbreviate = c(Gender = TRUE), rot_labels = 0)
)
}
\keyword{hplot}
